/*
 * @(#)BaumHilfsFunktionen.java   17.02.10
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.gtm.datvew.lib;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeGroupUsage;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.dav.daf.main.impl.config.DafConfigurationObjectType;
import de.bsvrz.dav.daf.main.impl.config.DafDynamicObjectType;

import de.inovat.buv.gtm.datvew.Activator;
import de.inovat.buv.gtm.datvew.aktionen.AktionenGTM;
import de.inovat.buv.gtm.datvew.menu.IMenuPunkt;
import de.inovat.buv.gtm.datvew.menu.MenuVew;
import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.projektlib.dav.DavDatenVew;
import de.inovat.buv.projektlib.rechtevew.aktionen.AktionenVew;
import de.inovat.buv.projektlib.rechtevew.lib.RechteVew;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

//~ KLASSEN ===================================================================

/**
 * Die Klasse stellt Hilfsfunktionen fr die Baumdarstellung zur Verfgung.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2010-01-01 00:00:00 +0200 (So, 27 Mai 2007)
 *          $ / ($Author: LG $)
 */
public class BaumHilfsFunktionen {
    private static final String SICHT_ID_TABELLENVERWALTUNG = "de.inovat.buv.plugin.gtm.navigation.tabellendarstellung.view";

    /**
     * Enthlt den Namen der TabellenDarstellungsVorauswahl oder null,
     * falls keine Auswahl beim Viewffnen vorgenommen werden soll.
     */
    private static String TABELLEN_DARSTELLUNGS_VORAUSWAHL = null;

    /**
     * Liste der Objekte fr die Tabellendarstellung.
     * Wird erst vor der ffnung der Tabellendarstellung initialisiert.
     */
    private static List<Object> _listeObjekte = new ArrayList<Object>();

    //~ METHODEN ==============================================================

    /**
     * Die Methode bestimmeAspekte liefert zu einer Attributgruppe, abhngig davon ob es sich um konfigurierende
     * Daten handelt oder nicht die zugehrigen Aspekte.
     * @param atg Attributgruppe fr die die Apsekte bestimmt werden.
     * @param konfDaten boolesche Variable die angibt ob es sich um eine konfigurierende Attributgruppe handelt oder nicht. True
     * bedeutet, dass es sich um konfigurierende Daten handelt.
     * @return Collection die die Aspekte enthlt.
     */
    public static List<Aspect> bestimmeAspekte(AttributeGroup atg, boolean konfDaten) {
        Collection<Aspect> aspects       = atg.getAspects();
        List<Aspect>       konfAspekte   = new ArrayList<Aspect>();
        List<Aspect>       onlineAspekte = new ArrayList<Aspect>();

        for (Aspect asp : aspects) {
            AttributeGroupUsage atgUsage = atg.getAttributeGroupUsage(asp);

            if (atgUsage.isConfigurating() && konfDaten) {
                konfAspekte.add(asp);
            } else if (!(atgUsage.isConfigurating()) &&!konfDaten) {
                onlineAspekte.add(asp);
            }
        }

        if (konfDaten) {
            return konfAspekte;
        } else {
            return onlineAspekte;
        }
    }

    /**
     * Die Methode bestimmeAttributgruppen bestimmt abhngig von dem Parameter die
     * Attributgruppen, der Parameter gibt an, ob die Attributgruppen konfigurierend
     * oder nicht konfigurierend sein sollen.
     * @param mengeObjekte die Menge der Objekte fr die die Attributgruppen bestimmt werden.
     * @param konfDaten boolesche Variable die angibt ob es sich um konfigurierende oder
     * nicht konfigurierende Attributgruppen handelt.
     * @return Liste der Attributgruppen.
     */
    public static List<AttributeGroup> bestimmeAttributgruppen(Collection<Object> mengeObjekte, boolean konfDaten) {
        List<AttributeGroup>                    atgGroupsKonfDaten           = new ArrayList<AttributeGroup>();
        List<AttributeGroup>                    atgGroupsOnlineDaten         = new ArrayList<AttributeGroup>();
        Map<AttributeGroup, Collection<Aspect>> atgGroupAspectMapKonfDaten   = new HashMap<AttributeGroup, Collection<Aspect>>();
        Map<AttributeGroup, Collection<Aspect>> atgGroupAspectMapOnlineDaten = new HashMap<AttributeGroup, Collection<Aspect>>();
        List<AttributeGroup>                    atgGroups                    = new ArrayList<AttributeGroup>();

        if (!(mengeObjekte == null)) {
            for (Object obj : mengeObjekte) {
                if (obj != null) {
                    if (obj instanceof DafConfigurationObjectType) {
                        DafConfigurationObjectType dafObj = (DafConfigurationObjectType) obj;

                        atgGroups = dafObj.getAttributeGroups();
                    } else if (obj instanceof DafDynamicObjectType) {
                        DafDynamicObjectType dafObj = (DafDynamicObjectType) obj;

                        atgGroups = dafObj.getAttributeGroups();
                    } else if (obj instanceof SystemObject) {
                        SystemObject systemObj = (SystemObject) obj;

                        atgGroups = systemObj.getType().getAttributeGroups();
                    } else if (obj instanceof ISystemObjekt) {
                        atgGroups = ((ISystemObjekt) obj).getSystemObjekt().getType().getAttributeGroups();
                    } else {

                        // System.out.println("keine ATG  " + obj.toString());
                    }
                }
            }
        }

        if (!(atgGroups == null)) {
            for (AttributeGroup atg : atgGroups) {
                Collection<AttributeGroupUsage> usedAtg = atg.getAttributeGroupUsages();

                for (AttributeGroupUsage used : usedAtg) {
                    if (konfDaten) {
                        if (used.isConfigurating()) {
                            atgGroupAspectMapKonfDaten.put(atg, atg.getAspects());
                        }
                    } else {
                        if (!(used.isConfigurating())) {
                            atgGroupAspectMapOnlineDaten.put(atg, atg.getAspects());
                        }
                    }
                }
            }
        }

        if (konfDaten) {
            for (AttributeGroup atgKonf : atgGroupAspectMapKonfDaten.keySet()) {
                atgGroupsKonfDaten.add(atgKonf);
            }

            return atgGroupsKonfDaten;
        } else {
            for (AttributeGroup atgOnline : atgGroupAspectMapOnlineDaten.keySet()) {
                atgGroupsOnlineDaten.add(atgOnline);
            }

            return atgGroupsOnlineDaten;
        }
    }

    /**
     * Die Methode bestimmeAttributgruppen bestimmt abhngig von dem Parameter die
     * Attributgruppen, der Parameter gibt an, ob die Attributgruppen konfigurierend
     * oder nicht konfigurierend sein sollen.
     * @param objekt Objekt fr das die Attributgruppen bestimmt werden.
     * @param konfDaten boolesche Variable die angibt ob es sich um konfigurierende oder
     * nicht konfigurierende Attributgruppen handelt.
     * @return Liste der Attributgruppen.
     */
    public static List<AttributeGroup> bestimmeAttributgruppen(Object objekt, boolean konfDaten) {
        Collection<Object> mengeObjekte = new ArrayList<Object>();

        mengeObjekte.add(objekt);

        return bestimmeAttributgruppen(mengeObjekte, konfDaten);
    }

    /**
     * Bestimme den Typ der Liste.
     *
     * @param liste Liste der Objekte (Objekte sind eigentlich vom gleichen Typ).
     *
     * @return Typ der Liste (als erster ermittelbarer Typ)
     * oder null.
     */
    public static SystemObjectType bestimmeTypDerListe(List<Object> liste) {
        SystemObjectType typ = null;

        for (Object obj : liste) {
            typ = bestimmeTypDesObjektes(obj);

            if (typ != null) {
                break;
            }
        }

        return typ;
    }

    /**
     * Bestimme den Typ der Liste.
     *
     * @param listePid Liste der Objekten-PIDs.
     *
     * @return Typ der Liste (als erster ermittelbarer Typ)
     * oder null.
     */
    public static SystemObjectType bestimmeTypDerListePid(List<String> listePid) {
        List<Object> liste = new ArrayList<Object>();

        for (String pid : listePid) {
            liste.add(DavDatenVew.getInstanz().getDav().getDataModel().getObject(pid));
        }

        return bestimmeTypDerListe(liste);
    }

    /**
     * Die Methode bestimmt zu einem Objekt den Typ.
     *
     * @param obj Objekt
     *
     * @return Typ des Objektes als String.
     * Wenn das Objekt nicht spezifiziert werden kann wird null zurckgegeben.
     */
    public static SystemObjectType bestimmeTypDesObjektes(Object obj) {
        if (obj instanceof SystemObject) {
            SystemObject systemObj = (SystemObject) obj;

            return systemObj.getType();
        } else if (obj instanceof ISystemObjekt) {
            return ((ISystemObjekt) obj).getSystemObjekt().getType();
        } else {
            return null;
        }
    }

    /**
     * Erstelle KontextMenu fr die selektierten Objekte. Das Men wird abhngig
     * vom Typ des erst selektierten Objekts aufgebaut.
     *
     * @param menu Men, wohin die erstellten Menpunkte eingefgt werden.
     * @param arTabellenVerwaltung Das Array der gespeicherten TabellenVerwaltung.
     */
    public static void erstelleKontextMenu(Menu menu, String[] arTabellenVerwaltung) {
        List<Object> mengeObjekte        = SelektionVew.getInstanz().ermittleSelektierteObjekteVonTypDesErstenObjekts();
        final Object erstSelektiertesObj = SelektionVew.getInstanz().ermittleErstSelektiertesObjekt();

        if (erstSelektiertesObj != null) {
            erstelleKontextMenu(menu, mengeObjekte, erstSelektiertesObj, arTabellenVerwaltung);
        }
    }

    /**
     * Erstelle KontextMenu fr die selektierten Objekte. Das Men wird abhngig
     * vom Typ des im Parameter bergebenen erst selektierten Objekts aufgebaut.
     *
     * @param menu Men, wohin die erstellten Menpunkte eingefgt werden.
     * @param mengeObjekte Menge der Objekte fr die Men-Aktionen.
     * @param erstSelektiertesObj Das erst selektierte Objekt.
     * @param arTabellenVerwaltung Das Array der gespeicherten TabellenVerwaltung.
     */
    public static void erstelleKontextMenu(Menu menu, final List<Object> mengeObjekte, final Object erstSelektiertesObj, String[] arTabellenVerwaltung) {

        // ---------------------------------------------------------------
        // ---- TabellenDarstellung:
        erstelleKontextMenuTabellenDarstellung(menu, mengeObjekte, erstSelektiertesObj, arTabellenVerwaltung);

        // ---------------------------------------------------------------
        // --- Erweiterte KontextMenu:
        SystemObject so = null;

        if (erstSelektiertesObj instanceof SystemObject) {
            so = (SystemObject) erstSelektiertesObj;
        } else if (erstSelektiertesObj instanceof ISystemObjekt) {
            so = ((ISystemObjekt) erstSelektiertesObj).getSystemObjekt();
        }

        if (so != null) {

            // Ermittle alle zum Typ passende erweiterte Men-Eintrge:
            String           typ;
            List<IMenuPunkt> listeMp = new ArrayList<IMenuPunkt>();

            for (IMenuPunkt mp : MenuVew.getInstanz().getListeMenuPunkte()) {
                typ = mp.getTyp();

                if (so.isOfType(typ)) {
                    listeMp.add(mp);
                }
            }

            // Menpunkte:
            if (!listeMp.isEmpty()) {

                // --- Separator:
                new MenuItem(menu, SWT.SEPARATOR);

                // --- Erstelle erweiterte Menpunkte:
                for (final IMenuPunkt mp : listeMp) {
                    MenuItem miErweitert = new MenuItem(menu, SWT.PUSH);

                    miErweitert.setText(mp.getName());
                    miErweitert.addListener(SWT.Selection, new Listener() {
                        @Override
                        public void handleEvent(Event event) {
                            mp.getAktion().run();
                        }
                    });
                    miErweitert.setEnabled(mp.getAktion().isEnabled());
                }
            }
        }

        // ---------------------------------------------------------------
        // ---- Dialog mit Info:
        if ((mengeObjekte.size() == 1) && (erstSelektiertesObj instanceof IInformation)) {
            final IInformation info = ((IInformation) erstSelektiertesObj);
            final String       text = info.getInfo();

            if ((text != null) && (!text.isEmpty())) {

                // --- Separator:
                new MenuItem(menu, SWT.SEPARATOR);

                MenuItem miInfo = new MenuItem(menu, SWT.PUSH);

                // --- MenPunkt:
                miInfo.setText("Information");
                miInfo.addListener(SWT.Selection, new Listener() {
                    @Override
                    public void handleEvent(Event event) {

                        // Definiere den Info-Dialog:
                        InputDialog dialog = new InputDialog(Display.getCurrent().getActiveShell(), "Info", info.getTitel(), text, null) {
                            @Override
                            protected Control createDialogArea(Composite parent) {
                                Control ctrl = super.createDialogArea(parent);

                                getText().setEditable(false);

                                return ctrl;
                            }
                            @Override
                            protected void createButtonsForButtonBar(Composite parent) {
                                super.createButtonsForButtonBar(parent);
                                getOkButton().setText("Kopieren");
                            }
                        };

                        // Kopiere den Inhalt in Buffer:
                        if (dialog.open() == Dialog.OK) {
                            final Clipboard cb           = new Clipboard(Display.getCurrent());
                            TextTransfer    textTransfer = TextTransfer.getInstance();

                            cb.setContents(new Object[] { dialog.getValue() }, new Transfer[] { textTransfer });
                            cb.dispose();
                        }
                    }
                });
            }
        }
    }

    /**
     * Erstelle KontextMenu mit dem Aufruf von Views 'Tabellendarstellung' und
     * 'Tabellendarstellung mit Vorauswahl'.
     * </p>
     * Das Men wird abhngig vom Typ des im Parameter bergebenen erst selektierten
     * Objekts aufgebaut. Dafr werden die Namen aller gespeicherter TabellenVerwaltungen
     * untersucht. Falls der Name dem Muster sys_typPid_name entspricht und das erst
     * selektierte Objekt von Typ typPid ist, wird der entsprechende MenPunkt erstellt.
     * </p>
     * @param menu Men, wohin die erstellten Menpunkte eingefgt werden.
     * @param mengeObjekte Menge der Objekte fr die Men-Aktionen.
     * @param erstSelektiertesObj Das erst selektierte Objekt.
     * @param arTabellenVerwaltung Das Array der gespeicherten TabellenVerwaltung.
     */
    private static void erstelleKontextMenuTabellenDarstellung(Menu menu, final List<Object> mengeObjekte, final Object erstSelektiertesObj, String[] arTabellenVerwaltung) {

        // ---------------------------------------------------------------
        boolean istErlaubt = RechteVew.getInstanz().istFunktionFuerAngemeldetenBenutzerErlaubt(AktionenGTM.FUNKTION_ID_TABELLENDARSTELLUNG);

        // ---------------------------------------------------------------
        // ---- TabellenDarstellung:
        MenuItem miTd = new MenuItem(menu, SWT.PUSH);

        miTd.setText("Tabellendarstellung");
        miTd.addListener(SWT.Selection, new Listener() {
            @Override
            public void handleEvent(Event event) {
                oeffneTabellendarstellung(mengeObjekte, erstSelektiertesObj);
            }
        });
        miTd.setEnabled(istErlaubt);

        // ---------------------------------------------------------------
        // ---- TabellenDarstellung mit Vorauswahl:
        // Map <MenuPunktName, TabellenName>:
        Map<String, String> mapNamen = new HashMap<String, String>();
        SystemObject        so       = null;

        if (erstSelektiertesObj instanceof SystemObject) {
            so = (SystemObject) erstSelektiertesObj;
        } else if (erstSelektiertesObj instanceof ISystemObjekt) {
            so = ((ISystemObjekt) erstSelektiertesObj).getSystemObjekt();
        }

        if (so != null) {

            // Prfe, ob der Name dem Muster sys_typPid_name entspricht:
            String   prefix = "sys_";
            String[] ar;
            String   doppel;

            for (String name : arTabellenVerwaltung) {
                if (name.startsWith(prefix)) {
                    ar = name.split("_", 3);

                    if ((ar.length == 3) && (so.isOfType(ar[1]))) {
                        doppel = mapNamen.put(ar[2], name);

                        if (doppel != null) {
                            Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID,                                                               //
                                                         String.format("Fr den Menpunkt %s exestieren mehrere Tabellendarstellungen: <%s> und <%s>)",  //
                                                                       ar[2], doppel, name)));
                        }
                    }
                }
            }
        }

        List<String> listeMenuPunktNamen = new ArrayList<String>();

        listeMenuPunktNamen.addAll(mapNamen.keySet());
        Collections.sort(listeMenuPunktNamen);

        // ---------------------------------------------------------------
        // Men:
        if (listeMenuPunktNamen.size() > 0) {

            // --- Separator:
            new MenuItem(menu, SWT.SEPARATOR);

            for (String menuPunktName : listeMenuPunktNamen) {

                // ---- TabellenDarstellung mit Vorauswahl:
                MenuItem miTdv = new MenuItem(menu, SWT.PUSH);

                miTdv.setText(String.format("Tabellendarstellung: %s", menuPunktName));

                final String tabellenName = mapNamen.get(menuPunktName);

                miTdv.addListener(SWT.Selection, new Listener() {
                    @Override
                    public void handleEvent(Event event) {
                        oeffneTabellendarstellung(mengeObjekte, erstSelektiertesObj, tabellenName);
                    }
                });
                miTdv.setEnabled(istErlaubt);
            }
        }
    }

    /**
     * Die Methode erzeugt eine Pid-Liste der bergebenen Objekte.
     * @param collection Collection der Objekte.
     * @return Liste mit den Pid's der Objekte.
     */
    private static List<String> erzeugePidListeSelektierterObj(Collection<Object> collection) {
        List<String> pidSelektierteObj = new ArrayList<String>();

        for (Object obj : collection) {
            String pidObj = "";

            if (obj instanceof SystemObject) {
                SystemObject systemObj = (SystemObject) obj;

                pidObj = systemObj.getPidOrId();
            } else if (obj instanceof ISystemObjekt) {
                pidObj = ((ISystemObjekt) obj).getSystemObjekt().getPidOrId();
            }

            pidSelektierteObj.add(pidObj);
        }

        return pidSelektierteObj;
    }

    /**
     * Ermittle gefilterte Liste aus der angegebenen Liste.
     * Neue Liste enthlt Elemente von Typ des angegebenen Objekts.
     *
     * @param liste Zu filternde Liste.
     * @param objektFuerTyp Objekt, deren Typ als Filter-Kriterium dient.
     *
     * @return Gefilterte Liste mit Objekten vom angegebenen Typ.
     */
    public static List<Object> filtereListeMitTypDesObjekts(List<Object> liste, Object objektFuerTyp) {
        List<Object>     listeGefiltert = new ArrayList<Object>();
        SystemObjectType typ1           = bestimmeTypDesObjektes(objektFuerTyp);

        if (typ1 != null) {
            for (Object objekt : liste) {
                if (typ1.equals(bestimmeTypDesObjektes(objekt))) {
                    listeGefiltert.add(objekt);
                }
            }
        }

        return listeGefiltert;
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft '_listeObjekte'.
     *
     * @return Wert der Eigenschaft '_listeObjekte'.
     *
     * @see #_listeObjekte
     */
    public static List<Object> getListeObjekte() {
        return _listeObjekte;
    }

    /**
     * Getter fr Eigenschaft 'pidSelektierterObj'.
     *
     * @param collection Menge der gewhlten Objekte.
     * @return Wert der Eigenschaft 'pidSelektierterObj'.
     */
    public static List<String> getPidSelektierterObj(Collection<Object> collection) {
        return erzeugePidListeSelektierterObj(collection);
    }

    /**
     *     Getter fr Eigenschaft 'tABELLEN_DARSTELLUNGS_VORAUSWAHL'.
     *
     *     @return Wert der Eigenschaft 'tABELLEN_DARSTELLUNGS_VORAUSWAHL'.
     *
     *     @see #TABELLEN_DARSTELLUNGS_VORAUSWAHL
     */
    public static String getTABELLEN_DARSTELLUNGS_VORAUSWAHL() {
        return TABELLEN_DARSTELLUNGS_VORAUSWAHL;
    }

    //~ METHODEN ==============================================================

    /**
     * ffne die Menge der selektierten Objekte.
     *
     * @return true, falls die Objekte geffnet werden, false - sonst.
     */
    public static boolean oeffneTabellendarstellung() {
        List<Object> mengeObjekte        = SelektionVew.getInstanz().ermittleSelektierteObjekteVonTypDesErstenObjekts();
        final Object erstSelektiertesObj = SelektionVew.getInstanz().ermittleErstSelektiertesObjekt();

        if (erstSelektiertesObj == null) {
            return false;
        }

        return oeffneTabellendarstellung(mengeObjekte, erstSelektiertesObj);
    }

    /**
     * ffne die Menge der Objekte.
     *
     * @param mengeObjekte
     *            Menge der geffneten Objekte im Baum.
     *            Stellt fest, ob die Fehlermeldung angezeigt werden soll, falls
     *            der Benutzer keine Berechtigung hat, sich die Objekte
     *            anzusehen.
     * @param ersteWahl Parameter gibt die erste Knotenwahl an.
     * @return true, falls die Sicht geffnet werden, false - sonst.
     */
    public static boolean oeffneTabellendarstellung(List<Object> mengeObjekte, Object ersteWahl) {
        _listeObjekte = filtereListeMitTypDesObjekts(mengeObjekte, ersteWahl);

        return AktionenVew.getInstanz().zeigeSichtMulti(SICHT_ID_TABELLENVERWALTUNG);
    }

    // ~ METHODEN ==============================================================

    /**
     * ffne die Menge der Objekte mit der TabellenDarstellungsVorauswahl.
     *
     * @param mengeObjekte Menge der Objekte fr die Men-Aktionen.
     *            Stellt fest, ob die Fehlermeldung angezeigt werden soll, falls
     *            der Benutzer keine Berechtigung hat, sich die Objekte
     *            anzusehen.
     * @param ersteWahl Parameter gibt die erste Knotenwahl an.
     * @param tabellenDarstellungsVorauswahl TabellenDarstellungsVorauswahl,
     * die, falls existiert, im Combo selektiert wird.
     *
     * @return true, falls die Objekte geffnet werden, false - sonst.
     */
    public static boolean oeffneTabellendarstellung(List<Object> mengeObjekte, Object ersteWahl, String tabellenDarstellungsVorauswahl) {
        TABELLEN_DARSTELLUNGS_VORAUSWAHL = tabellenDarstellungsVorauswahl;

        boolean erfolgreich = oeffneTabellendarstellung(mengeObjekte, ersteWahl);

        TABELLEN_DARSTELLUNGS_VORAUSWAHL = null;

        return erfolgreich;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 22.10.13
