/*
 * @(#)DaVMenge.java   20.02.13
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.gtm.datvew.lib;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.impl.config.DafConfigurationObject;
import de.inovat.buv.gtm.datvew.konstanten.KonstantenGTMGUIResource;

import org.eclipse.swt.graphics.Image;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Klasse beschreibt eine DaV-Menge  Objekt als interne Reprsentation.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class DaVMenge implements ISystemObjekt {

    /** ObjectSet das die Menge reprsentiert. */
    private ObjectSet _menge;

    /** Parent DaVSystemObjekt. */
    private DaVSystemObjekt _parentDavSo;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse DaVMenge.
     *
     * @param parentDavSo Parent DaVSystemObjekt.
     * @param menge Menge.
     */
    public DaVMenge(DaVSystemObjekt parentDavSo, ObjectSet menge) {
        _parentDavSo = parentDavSo;
        _menge       = menge;
    }

    //~ GET METHODEN ==========================================================

    @Override
    public Image getImage() {
        return KonstantenGTMGUIResource.ICON_GTM_MENGE;
    }

    /**
     * Ermittle alle Elemente der Menge.
     *
     * @return Alle Elemente der Menge oder leere Liste.
     */
    public List<DaVMengenElement> getMengenElemente() {
        List<DaVMengenElement> liste = new ArrayList<DaVMengenElement>();

        try {
            DafConfigurationObject conf = (DafConfigurationObject) _parentDavSo.getSystemObjekt();

            for (SystemObject so : conf.getObjectSet(_menge.getName()).getElements()) {
                liste.add(new DaVMengenElement(this, so));
            }
        }
        catch (Exception ignored) {}

        return liste;
    }

    /**
     * Getter fr Eigenschaft 'parentDavSo'.
     *
     * @return Wert der Eigenschaft 'parentDavSo'.
     *
     * @see #_parentDavSo
     */
    public DaVSystemObjekt getParent() {
        return _parentDavSo;
    }

    @Override
    public SystemObject getSystemObjekt() {
        return _menge;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 20.02.13
