/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.gtm.datvew.lib;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;

import de.inovat.buv.gtm.datvew.konstanten.KonstantenGTMGUIResource;

import org.eclipse.swt.graphics.Image;

//~ JDK IMPORTE ===============================================================

import java.util.Arrays;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Die Klasse verwaltet einen Messquerschnitt.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class Messquerschnitt implements ISystemObjekt {

    /** Array mit allen Fahrbahenen zu dem Messquerschnitt. */
    private Fahrbahnen[] _arFahrbahnen;

    /** Konfigurationsobjekt. */
    private ConfigurationObject _confObj;

    /** Eltern Auswertequerschnitt des Messquerschnittes. */
    private AuswerteQuerschnitt _parentAuswerteQuerschnitt;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse Messquerschnitt.
     *
     * @param confObj ConfigurationObject
     * @param auswerteQuerschnitt   Auswertequerschnitt.
     */
    public Messquerschnitt(ConfigurationObject confObj, AuswerteQuerschnitt auswerteQuerschnitt) {
        _confObj                   = confObj;
        _parentAuswerteQuerschnitt = auswerteQuerschnitt;
        _arFahrbahnen              = initFahrbahnenZuMQ();
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'arFahrbahnen'.
     *
     * @return Wert der Eigenschaft 'arFahrbahnen'.
     *
     * @see #_arFahrbahnen
     */
    public Fahrbahnen[] getArFahrbahnen() {
        return _arFahrbahnen;
    }

    @Override
    public Image getImage() {
        return KonstantenGTMGUIResource.ICON_GTM_MQ;
    }

    /**
     * Getter Eltern Auswertequerschnitt zu dem Messquerschnitt.
     * @return Eltern Auswertequerschnitt.
     */
    public AuswerteQuerschnitt getParentAuswerteQuerschnittZuMQ() {
        return _parentAuswerteQuerschnitt;
    }

    @Override
    public SystemObject getSystemObjekt() {
        return _confObj;
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode bestimmt ein Array mit allen Fahrbahenen zu dem Messquerschnitt.
     * @return Fahrbahnarray.
     */
    private Fahrbahnen[] initFahrbahnenZuMQ() {

        /** Array Fahrbahnen. */
        Fahrbahnen[]       arFahrbahnen;
        List<SystemObject> listFS;

        try {
            listFS       = _confObj.getObjectSet("FahrStreifen").getElements();
            arFahrbahnen = new Fahrbahnen[listFS.size()];

            for (int fsNr = 0; (fsNr < listFS.size()); fsNr++) {
                try {
                    Fahrbahnen fahrbahn = new Fahrbahnen((ConfigurationObject) listFS.get(fsNr), this);

                    arFahrbahnen[fsNr] = fahrbahn;
                }
                catch (Exception e) {
                    System.err.println(e);
                    e.printStackTrace();
                }
            }

            Arrays.sort(arFahrbahnen, new Fahrbahnen.FSComperatorVonAussenNachInnen());
        }
        catch (Exception e) {
            arFahrbahnen = new Fahrbahnen[0];
        }

        return arFahrbahnen;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 14.03.12
