/**
 *
 */



package de.inovat.buv.plugin.gtm.bast.gui;

//~ NICHT JDK IMPORTE =========================================================

import com.jcraft.jsch.Session;

import de.bsvrz.buv.rw.basislib.listendarstellung.DataTableObjectErweitert;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.TextValue;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;

import de.inovat.buv.gtm.datvew.lib.BaumVerwaltung;
import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.inovat.lib.funktionen.datum.DatumFunktionen;
import de.inovat.buv.inovat.lib.gui.GuiFunktionen;
import de.inovat.buv.plugin.gtm.bast.Activator;
import de.inovat.buv.plugin.gtm.bast.KonstantenGTMBast;
import de.inovat.buv.plugin.gtm.navigation.lib.DateiAuswahlFunktionen;
import de.inovat.buv.projektlib.dav.DavArchivVew;
import de.inovat.buv.projektlib.dav.DavDatenVew;

import org.apache.commons.net.ftp.FTPClient;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import static de.inovat.buv.inovat.lib.konstanten.Konstanten.NEWLINE;

//~ JDK IMPORTE ===============================================================

import java.io.IOException;

import java.text.SimpleDateFormat;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

//~ KLASSEN ===================================================================

/**
 * Die Klasse verwaltet das GUI und bearbeitet die SchaltflaecheAktionen.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 398 $ / $Date: 2013-06-20 11:38:33 +0200 (Do, 20 Jun 2013) $ / ($Author: LG $)
 */
public class ImportGUIVerwaltung implements ClientReceiverInterface, ClientSenderInterface {

    /** Aspekt Anfrage. */
    private static final String ASP_ANFRAGE = "asp.anfrage";

    /** Aspekt Zustand. */
    private static final String ASP_IMPORT_STATUS = "asp.bastImportStatus";

    /** Attributgruppe fr den Empfang des Importstatus. */
    private static final String ATG_BAST_STATUS = "atg.bastStatus";

    /** Attributgruppe zur Steuerung des BASt-Band Imports: */
    private static final String ATG_IMPORT_STEUERUNG = "atg.bastImportSteuerung";
    private static final String DATEI_BEARBEITET     = ".bearbeitet";
    private static final String DATEI_LOCK           = ".lock";

    /** Konstante Importverzeichnis V2004 FTP-Server. */
    private static final String IMPORT_V2004 = "Import/Verkehr_Menge_2004";

    /** Konstante Importverzeichnis V2007 FTP-Server. */
    private static final String IMPORT_V2007 = "Import/Verkehr_Geschwindigkeit_2007";

    //~ FELDER ================================================================

    /** Die im GUI angezeigte Meldung. */
    private String _meldung = "";

    /** Die selektierten Langzeitzaehlstellen. */
    private List<String> _listeDateien = new ArrayList<String>();

    /** Shell */
    private Shell _shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();

    /** Map mit allen ImportBenutzereinstellungen, Schlssel ist der Name der Importbenutzereinstellung, Wert eine ImportBenutzereinstellung. */
    private Map<String, ImportBenutzerEinstellungen> _mapImportBenutzerEinstellungen = new HashMap<String, ImportBenutzerEinstellungen>();

    /** Boolesches Attribut das angibt, ob die Anmeldung als Empfnger schon erfolgt ist. */
    private boolean _anmeldenDaVEmpfaenger;

    /** Boolesches Attribut das angibt, ob die Anemldung als Sender schon erfolgt ist. */
    private boolean _anmeldungDaVSenderImportOK;

    /** Datenbeschreibungsobjekt Import Status. */
    private DataDescription _datenbeschreibungImportStatus;

    /** Datenbeschreibungsobjekt Import Steuerung. */
    private DataDescription _datenbeschreibungImportSteuerung;

    /** ClientDavInteface. */
    private ClientDavInterface _dav;

    /** Instanz der Klasse ImportGUI, wo die Felder definiert sind. */
    private ImportGUI _gui;

    /** ImportBenutzerEinstellungenVerwaltungsinstanz. */
    private ImportBenutzerEinstellungenVerwaltung _ImportVerwaltung;
    protected int                                 _prozentWert;

    /** Systemobjekt fr den BASt-Band Import. */
    private SystemObject _systemObjectBAStImportImportSteuerung;

    /**
     * Typ des Systemobjekte fr den BASt-Band Import.
     */
    private String _typBAStImport;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse ImportGUIVerwaltung.
     *
     * @param guiInstanz Instanz der GUI-Klasse.
     * @param pidKonfigurationsObjBastBand bastBandObj.
     */
    protected ImportGUIVerwaltung(ImportGUI guiInstanz, String pidKonfigurationsObjBastBand) {
        _typBAStImport                  = pidKonfigurationsObjBastBand;
        _gui                            = guiInstanz;
        _ImportVerwaltung               = new ImportBenutzerEinstellungenVerwaltung(_mapImportBenutzerEinstellungen);
        _mapImportBenutzerEinstellungen = _ImportVerwaltung.getMapImportBenutzerEinstellungen();
        anmeldenDaVEmpfaengerBAStImportStatusMeldung();
    }

    //~ METHODEN ==============================================================

    /**
     * Abmelden des Empfngers. Diese Abmelderoutine wird beim erfolgreichen Ende des Importvorganges, wie auch
     * beim Schlieen des GUI aufgerufen.
     */
    protected void abmeldenDaVEmpfaengerImportStatus() {
        if (_anmeldenDaVEmpfaenger) {
            _dav.unsubscribeReceiver(this, _systemObjectBAStImportImportSteuerung, _datenbeschreibungImportStatus);
        }

        _anmeldenDaVEmpfaenger = false;
    }

    /**
     * Abmelden des Senders. Auch wenn der Import gescheitert ist, muss die Abmeldung vollzogen werden.
     */
    protected void abmeldenDaVSenderImportImportSteuerung() {
        if (_anmeldungDaVSenderImportOK) {
            _dav.unsubscribeSender(this, _systemObjectBAStImportImportSteuerung, _datenbeschreibungImportSteuerung);
        }

        _anmeldungDaVSenderImportOK = false;
    }

    /**
     * Die Methode ndert die Werte fr die Fortschrittsanzeige und Meldungen.
     * Die Meldung wird zum alten Text hinzugefgt.
     *
     * @param prozent Der neue Wert der allgemeinen Fortschrittsanzeige oder
     *  leer, falls der Wert nicht geaendert werden soll.
     * @param meldung Neue Meldung oder null, falls der Wert nicht geaendert werden soll.
     * @param ausUpdateMethode gibt an, ob die Methode aus der update Methode heraus aufgerufen worden ist,
     * wenn dies der Fall ist, wird die bergebene Zeitangabe verwendet.
     */
    public void aendereGUIAnzeige(final int prozent, final String meldung, final boolean ausUpdateMethode) {

        // Zeitstempel
        long             zeitstempel          = System.currentTimeMillis();
        SimpleDateFormat sdf                  = new SimpleDateFormat("dd.MM.yyyy hh:mm:ss");
        String           time                 = sdf.format(zeitstempel);
        final String     meldungMitZeitangabe = time + ": " + meldung + NEWLINE;

        if (!_gui.isDisposed()) {
            _gui.getDisplay().syncExec(new Runnable() {
                @Override
                public void run() {
                    if (prozent >= 0) {
                        _gui.getPrgFortschrittsanzeige().setSelection(prozent);
                        berechneRestZeit(prozent);
                    }

                    if ((meldung != null) &&!meldung.equals("")) {
                        if (ausUpdateMethode) {
                            _meldung = _meldung + meldung;
                        } else {
                            _meldung = _meldung + meldungMitZeitangabe;
                        }

                        _gui.getTxtMeldungen().setText(_meldung);

                        // Scrolle die Meldungen zum Ende:
                        _gui.getTxtMeldungen().setSelection(_gui.getTxtMeldungen().getCharCount());
                    }

                    _gui.getCompositeAllgemein().redraw();
                    _gui.getCompositeAllgemein().update();
                }
            });
        }
    }

    /**
     * Die Methode meldet sich zum Empfang des Status BASt-Band Importes an.
     * Diese Anmeldung wird genau dann vollzogen, wenn die Anmeldung zum Senden
     * der Daten zum BASt-Band Import geklappt haben.
     */
    public void anmeldenDaVEmpfaengerBAStImportStatusMeldung() {
        AttributeGroup attributgruppeStatus;
        Aspect         aspektZustand;
        ReceiveOptions anmeldeoptionen = ReceiveOptions.normal();
        ReceiverRole   empfaengerrolle = ReceiverRole.drain();

        _dav                                   = DavDatenVew.getInstanz().getDav();
        _systemObjectBAStImportImportSteuerung = _dav.getDataModel().getObject(_typBAStImport);
        attributgruppeStatus                   = _dav.getDataModel().getAttributeGroup(ATG_BAST_STATUS);
        aspektZustand                          = _dav.getDataModel().getAspect(ASP_IMPORT_STATUS);
        _datenbeschreibungImportStatus         = new DataDescription(attributgruppeStatus, aspektZustand);

        // Anmelden zum Empfang der Daten
        _anmeldenDaVEmpfaenger = subscribe(_systemObjectBAStImportImportSteuerung, _datenbeschreibungImportStatus, anmeldeoptionen, empfaengerrolle);

        // Anmelden zum Empfang der Daten
        long   startZeit = DatumFunktionen.getAktuelleDateTime();
        String zeit      = DatumFunktionen.getDatumZeitAlsString(startZeit);

        if (_anmeldenDaVEmpfaenger) {
            aendereGUIAnzeige(1, zeit + "Anmeldung fr die Statusinformationen Erfolgreich durchgefhrt", false);
        } else {
            aendereGUIAnzeige(1, zeit + "Es sind Probleme bei der Anmeldung fr die Statusinformationen aufgetreten", false);
        }
    }

    /**
     * Anmeldung beim Datenverteiler fr den BASt-Band Daten Import.
     */
    public void anmeldenDaVSenderBAStImport() {
        AttributeGroup attributgruppeImportSteuerung;
        Aspect         aspektAnfrage;
        ReceiveOptions anmeldeoptionen = ReceiveOptions.normal();
        SenderRole     senderRolle     = SenderRole.sender();

        _dav                                   = DavDatenVew.getInstanz().getDav();
        _systemObjectBAStImportImportSteuerung = _dav.getDataModel().getObject(_typBAStImport);
        attributgruppeImportSteuerung          = _dav.getDataModel().getAttributeGroup(ATG_IMPORT_STEUERUNG);
        aspektAnfrage                          = _dav.getDataModel().getAspect(ASP_ANFRAGE);
        _datenbeschreibungImportSteuerung      = new DataDescription(attributgruppeImportSteuerung, aspektAnfrage);

        // Anmelden zum Senden der Daten
        _anmeldungDaVSenderImportOK = subscribe(_systemObjectBAStImportImportSteuerung, _datenbeschreibungImportSteuerung, anmeldeoptionen, senderRolle);

        if (!_anmeldungDaVSenderImportOK) {
            aendereGUIAnzeige(1, "Die Anmeldung beim Datenverteiler konnte nicht erfolgen, bitte Prfen Sie die Datenverteiler-Einstellungen.", false);
        }
    }

    /**
     * Die Methode bearbeitet die Aktionen zu den Radiobuttons. Wenn ein Radiobutton selektiert wurde,
     * mssen die anderen Radiobuttons unselektiert sein.
     * @param radioBASt der selektierte Radiobutton.
     */
    public void bastFormatSelektiert(Button radioBASt) {
        if (radioBASt.equals(_gui.getRadioBASt2004())) {
            if (_gui.getRadioBASt2007().getSelection()) {
                _listeDateien.clear();
                _gui.getRadioBASt2007().setSelection(false);
                _gui.getRadioBASt2004().setSelection(true);
            }
        } else if (radioBASt.equals(_gui.getRadioBASt2007())) {
            if (_gui.getRadioBASt2004().getSelection()) {
                _listeDateien.clear();
                _gui.getRadioBASt2004().setSelection(false);
                _gui.getRadioBASt2007().setSelection(true);
            }
        } else {
            _gui.getRadioBASt2004().setSelection(true);
            _gui.getRadioBASt2007().setSelection(false);
        }

        setzeImportSchaltflaeche();
        zeigeAnzahlSelDateien(_listeDateien);
    }

    /**
     * Berechne die Restzeit fr den Importvorgang
     * und zeige die im GUI.
     *
     * @param prozentGemacht Wie viel Prozent des Imports schon gemacht wurde.
     */
    private void berechneRestZeit(int prozentGemacht) {

        /**
         *       if (prozentGemacht < 100) {
         *
         *           // Wie viel Zeit wurde verbraucht fr das gemachte Prozent des Imports:
         *           long zeit = DatumFunktionen.getAktuelleDateTime() - _startZeit;
         *
         *           // Wie viel Zeit ist geblieben fr den Rest der Arbeit (entspricht 100-prozentGemacht Prozenten).
         *           long restZeit = zeit / prozentGemacht * (100 - prozentGemacht);
         *
         *           rest = "geschaetzte Restzeit   " + DatumFunktionen.getZeitdauerAlsString(restZeit);
         *       }
         *
         *       // Zeige den Restzeit im GUI:
         *       _gui.getLbRestzeit().setText(rest);
         */
    }

    /**
     * Die Taste "Abbruch"  wird selektiert, d.h. ber den Datenverteiler wird
     * dem Prozess BASt-Band die Information bermittelt, dass der aktuelle Importprozess abzubrechen ist.
     * <p>
     * Das GUI wird nicht geschlossen.
     * </p>
     */
    protected void btnAbbruchSelektiert() {
        anmeldenDaVSenderBAStImport();

        // Zeitstempel
        long           zeitstempel              = System.currentTimeMillis();
        AttributeGroup atg                      = _dav.getDataModel().getAttributeGroup(ATG_IMPORT_STEUERUNG);
        Data           datensatzImportSteuerung = _dav.createData(atg);
        TextValue      ftpServer                = datensatzImportSteuerung.getTextValue("FTP-Server");
        StringBuilder  ftpZugangsInformation    = new StringBuilder();

        ftpZugangsInformation.append("IMPORT-ABBRUCH");
        ftpServer.setText(ftpZugangsInformation.toString());

        // Ausgabeformat je nach Wahl im View.
        TextValue bastBandAusgabeFormat = datensatzImportSteuerung.getTextValue("BastBandAusgabeFormat");

        bastBandAusgabeFormat.setText("V2004");

        // Telegramm erstellen
        ResultData datenImportBAStBandResultData = new ResultData(_systemObjectBAStImportImportSteuerung, _datenbeschreibungImportSteuerung, zeitstempel, datensatzImportSteuerung);

        // Versenden
        try {
            _dav.sendData(datenImportBAStBandResultData);
            aendereGUIAnzeige(1, "Der Aktuelle Importvorgang wird abgebrochen.", false);
            aendereGUIAnzeige(1, "Die schon importierten Dateien sind von dem Abbruch nicht mehr betroffen!", false);
        }
        catch (Exception e) {
            aendereGUIAnzeige(1, "Der Abbruch des Importvorganges konnte nicht vollzogen werden!", false);
            aendereGUIAnzeige(1, "Exception Meldung:  " + e, false);
        }

        abmeldenDaVSenderImportImportSteuerung();
        _gui.getBtnAbbrechen().setEnabled(false);
        setzeImportSchaltflaeche();
    }

    /**
     * Die Taste "ImportDateiauswahl"  wird selektiert.
     * <p>
     * Es wird ein Dateiauswahldialog geffnet, der die Dateien auf dem FTP-Server bzgl. des
     * Importformates anzeigt. Die existierenden Dateien werden in einer Liste angezeigt, die
     * mittels mehrfachselektion selektiert werden knnen.
     * ftp:// benutzername : passwort @ hostname
     * </p>
     */
    protected void btnImportDateiauswahlSelektiert() {

        // FTP:
        if (_gui.getRbtnFtp().getSelection()) {
            ftpDateiAuswahl();
        }

        // SFTP:
        else if (_gui.getRbtnSftp().getSelection()) {
            sftpDateiAuswahl();
        }
    }

    /**
     * Die Taste "Importieren"  wird selektiert.
     * <p>
     * Speichere die GUI-Werte, Importiere die selektierten Langzeitzaehlstellen in die Datei,
     * leere das Set der selektierten Langzeitzaehlstellenn und erlaube weitere Funktionen mit der erzeugten Datei.
     * </p>
     */
    protected void btnImportierenSelektiert() {

        // Fortschrittanzeige und Meldungen initialisieren:
        initParameterMeldungUndFortschritt();
        _gui.getBtnImportieren().setEnabled(false);
        aendereGUIAnzeige(1, "Der Import wird gestartet. Bitte warten ....", false);
        davSendeApplikationImportBASt();
        _listeDateien.clear();
        zeigeAnzahlSelDateien(_listeDateien);
        _gui.getCompositeAllgemein().redraw();
        _gui.getCompositeAllgemein().update();
    }

    /**
     * Die Methode lscht den ImporteinstellungsNamen aus der Map aller Importeinstellungen.
     * @param ImporteinstellungsName der zu lschende Eintrag in der Map aller Importeinstellungen.
     */
    public void btnLoeschenImporteinstellung(String ImporteinstellungsName) {
        _mapImportBenutzerEinstellungen.remove(ImporteinstellungsName);
        _ImportVerwaltung.setMapImportBenutzerEinstellungen(_mapImportBenutzerEinstellungen);
        _ImportVerwaltung.speichereImportVerwaltungen(KonstantenGTMBast.ALLE_IMPORT_EINSTELLUNGEN);
        erzeugeComboboxGespeicherteImporteinstellungen("");
        initialisiereGUI();
    }

    /**
     * Die Methode schliet das aktuelle Gui.
     */
    public void btnSchliessenSelektiert() {
        GuiFunktionen.schliesseGUI(_gui.getParent(), _gui.getViewID());
    }

    /**
     * Die Methode sorgt dafr, dass die gettigten Eingaben gespeichert werden.
     */
    public void btnSpeichern() {

        // Definiere den Validator:
        IInputValidator validator = new IInputValidator() {
            @Override
            public String isValid(String text) {
                text = text.trim();

                if (text.length() == 0) {
                    return "";
                } else if (importEinstellungsNameExistiertSchon(text)) {
                    return "Der Name existiert schon !";
                }

                return null;
            }
        };

        // Definiere den Dialog:
        InputDialog inputDialog = new InputDialog(_gui.getShell(), "Speichern", "Bitte geben Sie den Importeinstellungsnamen ein", "", validator);

        // ffne den Dialog:
        if (inputDialog.open() == Dialog.OK) {

            // Merke den eingegebenen Namen:
            String speicherName = inputDialog.getValue().trim();

            // Speichere den Namen:
            speicherAktuelleImportDarstellungsOptionen(speicherName);

            // Passe das Combo an:
            erzeugeComboboxGespeicherteImporteinstellungen(speicherName);
        }
    }

    @Override
    public void dataRequest(SystemObject object, DataDescription dataDescription, byte state) {}

    /**
     * Die Methode ruft alle notwendigen Methoden zum Senden der Importdaten auf.
     */
    private void davSendeApplikationImportBASt() {
        anmeldenDaVSenderBAStImport();
        sendeImportDaten();
        abmeldenDaVSenderImportImportSteuerung();
    }

    /**
     * Due Methode ermglicht genau dann eine Dateiauswahl auf dem FTP-Server, wenn alle
     * notwendigen Angaben zum Zugriff auf den FTP-Server gemacht wurden.
     */
    protected void ermoeglicheDateiAuswahl() {
        if (!_gui.getTxtURL().getText().isEmpty() &&!_gui.getTxtBenutzer().getText().isEmpty() &&!_gui.getTxtPasswd().getText().isEmpty()) {
            _gui.getBtnDateienAuswahl().setEnabled(true);
        } else {
            _gui.getBtnDateienAuswahl().setEnabled(false);
        }
    }

    /**
     * Die Methode fllt die Combobox mit den gespeicherten Importeinstellungsnamen neu und
     * belegt sie.
     */
    public void erzeugeComboboxGespeicherteImporteinstellungen(String auswahl) {
        _gui.getComboGespeicherteImporteinstellungen().removeAll();

        if (_mapImportBenutzerEinstellungen != null) {
            for (String ImportBenutzerEinstellung : _mapImportBenutzerEinstellungen.keySet()) {
                _gui.getComboGespeicherteImporteinstellungen().add(ImportBenutzerEinstellung);
            }

            if ((_gui.getComboGespeicherteImporteinstellungen().getItemCount() > 1) &&!auswahl.equals("")) {
                for (int i = 0; i < _gui.getComboGespeicherteImporteinstellungen().getItemCount(); i++) {
                    if (_gui.getComboGespeicherteImporteinstellungen().getItem(i).equals(auswahl)) {
                        _gui.getComboGespeicherteImporteinstellungen().select(i);
                        _gui.getBtnloeschenImporteinstellung().setEnabled(true);
                    }
                }
            } else if (_gui.getComboGespeicherteImporteinstellungen().getItemCount() == 0) {
                _gui.getComboGespeicherteImporteinstellungen().add("", 0);
                _gui.getComboGespeicherteImporteinstellungen().select(0);
                _gui.getBtnloeschenImporteinstellung().setEnabled(false);
            } else {
                _gui.getComboGespeicherteImporteinstellungen().add("", 0);
                _gui.getComboGespeicherteImporteinstellungen().select(0);
                _gui.getBtnloeschenImporteinstellung().setEnabled(false);
            }
        }
    }

    /**
     * Dateiauswahl ber das FTP-Server.
     */
    private void ftpDateiAuswahl() {
        String       fehlerMeldung   = null;
        List<String> listeFtpDateien = new ArrayList<String>();

        // -----------------------------------------
        // Erstelle die FTP-Verbindung:
        // -----------------------------------------
        int port;

        try {
            port = Integer.parseInt(_gui.getTxtPort().getText());
        }
        catch (Exception e) {
            port = 21;
            aendereGUIAnzeige(1, "FTPServer Port kann nicht in einer Zahl umgewandelt werden ===> wird auf 21 gesetzt. ", false);
        }

        FTPClient ftpClient = new FTPClient();
        Exception exception = verbindeMitFTPServer(ftpClient, _gui.getTxtURL().getText(), port, _gui.getTxtBenutzer().getText(), _gui.getTxtPasswd().getText());

        if (exception == null) {

            // -----------------------------------------
            // Passiv/Aktiv-Modus:
            // -----------------------------------------
            if (_gui.getCbtnPassivmodus().getSelection()) {
                ftpClient.enterLocalPassiveMode();
            } else {
                ftpClient.enterLocalActiveMode();
            }

            // -----------------------------------------
            // ffne den ImportOrdener:
            // -----------------------------------------
            String ordner = null;

            if (_gui.getRadioBASt2004().getSelection()) {
                ordner = IMPORT_V2004;
            } else if (_gui.getRadioBASt2007().getSelection()) {
                ordner = IMPORT_V2007;
            }

            if (ordner != null) {
                try {
                    if (ftpClient.changeWorkingDirectory(ordner)) {

                        // -----------------------------------------
                        // Ermittle Dateien im FTP-Ordner:
                        // -----------------------------------------
                        String[] arFtpDatei = ftpClient.listNames();

                        if ((arFtpDatei != null) && (arFtpDatei.length > 0)) {
                            for (String ftpDatei : arFtpDatei) {

                                // Dateien mit Endung .lock oder .bearbeitet sollen nicht mehr fr den Import zur Verfgung stehen:
                                if (!(ftpDatei.endsWith(DATEI_LOCK) || ftpDatei.endsWith(DATEI_BEARBEITET))) {
                                    listeFtpDateien.add(ftpDatei);
                                }
                            }
                        }

                        if (listeFtpDateien.isEmpty()) {
                            fehlerMeldung = "Es sind keine zu importierte Dateien auf dem FTP-Server vorhanden.";
                        }
                    } else {
                        fehlerMeldung = String.format(  //
                            "Fehler beim Wechsel in das korrekte Verzeichnis <%s> auf dem FTP-Server, kontaktieren Sie den System Administrator des FTP-Servers.", ordner);
                    }
                }
                catch (Exception e) {
                    fehlerMeldung = "Fehler beim Wechsel in das Korrekte Verzeichnis auf dem FTP-Server, kontaktieren Sie den System Administrator des FTP-Servers.";
                    exception     = e;
                }
            } else {
                fehlerMeldung = "Das BASt-Band Import Format ist nicht ausgewhlt!";
            }
        } else {
            fehlerMeldung = "Verbindungsprobleme mit dem FTP-Server, der Import kann nicht gestartet werden. berprfen Sie bitte die FTPServer-Zugangsdaten!";
        }

        // -----------------------------------------
        // ffne den Dateiauswahl-Dialog:
        // -----------------------------------------
        // bergebe die Liste der Dateien weiter:
        // BAST 2004:
        if (_gui.getRadioBASt2004().getSelection()) {
            BaumVerwaltung.getInstanz().setListeAllerImportDateienFTPServer2004(listeFtpDateien, true);
        }

        // BAST 2007:
        else {
            BaumVerwaltung.getInstanz().setListeAllerImportDateienFTPServer2007(listeFtpDateien, false);
        }

        if (!listeFtpDateien.isEmpty()) {
            _listeDateien = DateiAuswahlFunktionen.dateiAuswahlDialogOeffnen(_shell, _listeDateien);
        }

        // -----------------------------------------
        // Anpassung vom GUI:
        // -----------------------------------------
        zeigeAnzahlSelDateien(_listeDateien);
        _gui.getBtnImportieren().setEnabled(istStarteImportOK());

        // -----------------------------------------
        // Fehlermeldung:
        // -----------------------------------------
        if (fehlerMeldung != null) {
            if (exception != null) {
                fehlerMeldung = String.format("%s\n\n%s", fehlerMeldung, exception.toString());
            }

            MessageDialog.openError(_gui.getShell(), "Fehler", fehlerMeldung);
            aendereGUIAnzeige(1, fehlerMeldung, false);
        }
    }

    /**
     * Die Methode berprft ob der Name schon existiert.
     * @param ImportEinstellungsName der geprfte Name.
     * @return true wenn der Importeinstellungsname schon existiert, false sonst.
     */
    private boolean importEinstellungsNameExistiertSchon(String ImportEinstellungsName) {
        return _mapImportBenutzerEinstellungen.containsKey(ImportEinstellungsName);
    }

    /**
     * Die Methode initialisiert das GUI.
     */
    public void initialisiereGUI() {
        _gui.getRadioBASt2004().setSelection(true);
        _gui.getRadioBASt2007().setSelection(false);

        //
        _gui.getRbtnFtp().setSelection(true);
        _gui.getRbtnSftp().setSelection(false);
        rbtnServertypGeaendert();
        _gui.getTxtURL().setText("");
        _gui.getTxtBenutzer().setText("");
        _gui.getTxtPasswd().setText("");
        _gui.getCbtnPassivmodus().setSelection(false);
    }

    /**
     * Initialisiere den Meldungstext und die Fortschrittsanzeige.
     */
    protected void initParameterMeldungUndFortschritt() {
        _gui.getPrgFortschrittsanzeige().setSelection(0);
        _meldung = "";
        _gui.getTxtMeldungen().setText("");
    }

    /**
     * Initialisiert die Zeitbereichsangabe. Dabei wird das Von-Datum auf den
     * ersten Tag des letzten Monats gesetzt, das Bis-Datum auf den letzten Tag
     * des letzten Monats.
     */
    protected void initZeitbereich() {
        _gui.getCompositeAllgemein().pack();
        _gui.getCompositeAllgemein().redraw();
        _gui.getCompositeAllgemein().update();
    }

    //~ GET METHODEN ==========================================================

    @Override
    public boolean isRequestSupported(SystemObject object, DataDescription dataDescription) {
        return false;
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode berprft ob der Importvorgang gestartet werden darf.
     * @return true wenn alle notwendigen Angaben vollzogen und notwendigen Bedingungen erfllt sind, fals sonst.
     * Notwendige Angaben sind die Zugangsinformationen fr den FTP-Server und die Dateien die Importiert werden sollen.
     * Notwendige Bedingungen sind dass der Prozess BASt-Band Import luft.
     */
    private boolean istStarteImportOK() {
        boolean startImportOK;

        if ((DavArchivVew.getInstanz().istArchivVerfuegbar()) && (!_listeDateien.isEmpty()) &&  //
            (_prozentWert == 100) &&                                                            //
            (_gui.getRbtnFtp().getSelection() == !_gui.getRbtnSftp().getSelection()) &&         //
            (!_gui.getTxtURL().getText().isEmpty()) &&                                          //
            (!_gui.getTxtPort().getText().isEmpty()) &&                                         //
            (!_gui.getTxtBenutzer().getText().isEmpty()) &&                                     //
            (!_gui.getTxtPasswd().getText().isEmpty())) {

            //
            startImportOK = true;
        } else {
            startImportOK = false;
        }

        return startImportOK;
    }

    /**
     * Die Methode ldt die gespeicherten Importeinstellungen der ausgewhlten Importeinstellung.
     * @param einstellungsName Importeinstellung.
     */
    public void ladeGespeicherteImporteinstellungen(String einstellungsName) {
        ImportBenutzerEinstellungen einstellung = _mapImportBenutzerEinstellungen.get(einstellungsName);

        if (einstellung != null) {
            _gui.getRadioBASt2004().setSelection(einstellung.isBast2004());
            _gui.getRadioBASt2007().setSelection(einstellung.isBast2007());

            //
            _gui.getTxtURL().setText(einstellung.getFtpServerURL());
            _gui.getTxtPort().setText(einstellung.getFtpServerPort());
            _gui.getTxtBenutzer().setText(einstellung.getFtpServerBenutzer());
            _gui.getTxtPasswd().setText(einstellung.getFtpServerPasswort());
            _gui.getRbtnFtp().setSelection(einstellung.isIstFtp());
            _gui.getRbtnSftp().setSelection(!einstellung.isIstFtp());
            _gui.getCbtnPassivmodus().setSelection(einstellung.isPassivmodus());
        }
    }

    /**
     * Servertyp wurde gendert
     */
    protected void rbtnServertypGeaendert() {

        // Defaultwert fr FTP:
        if (_gui.getRbtnFtp().getSelection()) {
            _gui.getTxtPort().setText("21");
        }

        // Defaultwert fr SFTP:
        else {
            _gui.getTxtPort().setText("22");
        }
    }

    /**
     * Anlegen eines leeren Datensatzes zur Attributgruppe atg.bastImportSteuerung.<p>
     * Die werte des Datensatzes werden mit den Werten aus dem Import-View belegt.
     * Senden der Daten als Sender.
     * <p>
     * DataNotSubscribeException: Ausnahme, die beim Senden von Datenstzen ohne entsprechende Sendeanmeldungen
     * generiert wird.
     * SendSubscriptionNotConfirmed: Ausnahme, die beim Senden von Daten als einfacher Sender generiert wird,
     * wenn noch keine positive Sendesteuerung vom Datenverteiler fr die zu versendenden Daten vorliegt
     */
    public void sendeImportDaten() {
        if (_anmeldungDaVSenderImportOK) {
            StringBuilder meldungOk = new StringBuilder();

            meldungOk.append("Folgende Dateien werden importiert: ").append(NEWLINE);

            StringBuilder meldungFehler = new StringBuilder();

            meldungFehler.append("Folgende Dateien werden NICHT importiert, da sie nicht den Konventionen entsprechen: ").append(NEWLINE);

            List<String> korrekteDateinamen = new ArrayList<String>();

            for (String name : _listeDateien) {
                if (name.contains(".") || name.contains("dat")) {
                    korrekteDateinamen.add(name);
                    meldungOk.append(name).append(NEWLINE);
                } else {
                    meldungFehler.append(name).append(NEWLINE);
                }
            }

            // Zeitstempel
            long zeitstempel = System.currentTimeMillis();

            // Datensatz erstellen
            AttributeGroup atg                      = _dav.getDataModel().getAttributeGroup(ATG_IMPORT_STEUERUNG);
            Data           datensatzImportSteuerung = _dav.createData(atg);

            // Ausgabeformat je nach Wahl im View.
            TextValue bastBandAusgabeFormat = datensatzImportSteuerung.getTextValue("BastBandAusgabeFormat");
            String    ausgabeFormat;

            if (_gui.getRadioBASt2004().getSelection()) {
                ausgabeFormat = "V2004";
            } else {
                ausgabeFormat = "V2007";
            }

            bastBandAusgabeFormat.setText(ausgabeFormat);

            // FTP-Server Angaben.
            // DaV-Datenformat: "-Benutzer:%s-Passwort:%s-URL:%s-Port:%s-Passivmodus:%s-Servertyp:%s"
            //
            // alte Variante (ohne Port und FTP/SFTP):
            // "Benutzer:%sPasswort:%sURL:%sPassivmodus:%s"
            TextValue     ftpServer             = datensatzImportSteuerung.getTextValue("FTP-Server");
            StringBuilder ftpZugangsInformation = new StringBuilder();

            ftpZugangsInformation.append("-Benutzer:");
            ftpZugangsInformation.append(_gui.getTxtBenutzer().getText());
            ftpZugangsInformation.append("-Passwort:");
            ftpZugangsInformation.append(_gui.getTxtPasswd().getText());
            ftpZugangsInformation.append("-URL:");
            ftpZugangsInformation.append(_gui.getTxtURL().getText());
            ftpZugangsInformation.append("-Port:");
            ftpZugangsInformation.append(_gui.getTxtPort().getText());
            ftpZugangsInformation.append("-Passivmodus:");
            ftpZugangsInformation.append(_gui.getCbtnPassivmodus().getSelection() ? "ja" : "nein");
            ftpZugangsInformation.append("-Servertyp:");
            ftpZugangsInformation.append(_gui.getRbtnFtp().getSelection() ? "FTP" : "SFTP");
            ftpServer.setText(ftpZugangsInformation.toString());

            Data.TextArray ar_Dateien = datensatzImportSteuerung.getTextArray("ImportDateien");

            ar_Dateien.setLength(korrekteDateinamen.size());

            Data.TextValue[] dateiNamenListe = ar_Dateien.getTextValues();

            for (int i = 0; i < korrekteDateinamen.size(); i++) {
                dateiNamenListe[i].setText(korrekteDateinamen.get(i));
            }

            // Telegramm erstellen
            ResultData datenImportBAStBandResultData = new ResultData(_systemObjectBAStImportImportSteuerung, _datenbeschreibungImportSteuerung, zeitstempel, datensatzImportSteuerung);

            // Versenden
            try {
                _dav.sendData(datenImportBAStBandResultData);

                if (_listeDateien.size() != korrekteDateinamen.size()) {
                    aendereGUIAnzeige(1, meldungFehler.toString(), false);
                }

                aendereGUIAnzeige(1, meldungOk.toString(), false);
                _gui.getBtnAbbrechen().setEnabled(true);
            }
            catch (Exception e) {
                aendereGUIAnzeige(1, "Die Daten konnten nicht an den Datenverteiler gesendet werden!" + e, false);
            }
        } else {
            aendereGUIAnzeige(1, "Der Import konnte nicht gestartet werden, da eine Anmeldung am Datenverteiler nicht mglich war, bitte berprfen Sie die Datenverteiler-Einstellungen!", false);
        }
    }

    /**
     * Setze die GUI Daten mit den Werten aus der aktuellen Parameter-Instanz.
     */
    protected void setzeGUIDatenMitAktParameter() {
        _gui.getTxtMeldungen().setSelection(_gui.getTxtMeldungen().getCharCount());
        zeigeAnzahlSelDateien(_listeDateien);
        _gui.getCompositeAllgemein().redraw();
        _gui.getCompositeAllgemein().update();
    }

    /**
     * Die Methode berprft die Pflichteingabefelder URL, Benutzer und
     * Passwort auf leerheit und setzt dementsprechend die Importschaltflche.
     */
    public void setzeImportSchaltflaeche() {
        if (istStarteImportOK()) {
            _gui.getBtnImportieren().setEnabled(true);
        } else {
            _gui.getBtnImportieren().setEnabled(false);
        }
    }

    /**
     * Dateiauswahl ber das SFTP-Server.
     */
    private void sftpDateiAuswahl() {
        String       fehlerMeldung    = null;
        List<String> listeSftpDateien = new ArrayList<String>();

        // -----------------------------------------
        // Erstelle die SFTP-Verbindung:
        // -----------------------------------------
        int port;

        try {
            port = Integer.parseInt(_gui.getTxtPort().getText());
        }
        catch (Exception e) {
            port = 22;
            aendereGUIAnzeige(1, "SFTPServer Port kann nicht in einer Zahl umgewandelt werden ===> wird auf 22 gesetzt. ", false);
        }

        Session   session;
        Exception exception = null;

        try {
            session = SFTPFunktionen.verbindeMitSFTPServer(_gui.getTxtURL().getText(), port, _gui.getTxtBenutzer().getText(), _gui.getTxtPasswd().getText());
        }
        catch (Exception e) {
            session   = null;
            exception = e;
        }

        if (exception == null) {

            // -----------------------------------------
            // Definiere den ImportOrdener:
            // -----------------------------------------
            String ordner = null;

            if (_gui.getRadioBASt2004().getSelection()) {
                ordner = IMPORT_V2004;
            } else if (_gui.getRadioBASt2007().getSelection()) {
                ordner = IMPORT_V2007;
            }

            if (ordner != null) {

                // -----------------------------------------
                // Ermittle Dateien im SFTP-Ordner:
                // -----------------------------------------
                List<String> listeDateien = SFTPFunktionen.ermittleDateienImSFTPOrdner(session, ordner);

                if (listeDateien != null) {
                    if (listeDateien.size() > 0) {
                        for (String sftpDatei : listeDateien) {

                            // Dateien mit Endung .lock oder .bearbeitet sollen nicht mehr fr den Import zur Verfgung stehen:
                            if (!(sftpDatei.endsWith(DATEI_LOCK) || sftpDatei.endsWith(DATEI_BEARBEITET))) {
                                listeSftpDateien.add(sftpDatei);
                            }
                        }
                    } else {
                        fehlerMeldung = "Es sind keine zu importierte Dateien auf dem SFTP-Server vorhanden.";
                    }
                } else {
                    fehlerMeldung = String.format(  //
                        "Fehler beim Lesen aus dem SFTP-Verzeichnis <%s>, kontaktieren Sie den System Administrator des SFTP-Servers.", ordner);
                }
            } else {
                fehlerMeldung = "Das BASt-Band Import Format ist nicht ausgewhlt!";
            }

            // -----------------------------------------
            // Terminiere die SFTP-Verbindung:
            // -----------------------------------------
            session.disconnect();
        }

        //
        else {
            fehlerMeldung = "Verbindungsprobleme mit dem SFTP-Server, der Import kann nicht gestartet werden. berprfen Sie bitte die SFTPServer-Zugangsdaten!";
        }

        // -----------------------------------------
        // ffne den Dateiauswahl-Dialog:
        // -----------------------------------------
        // bergebe die Liste der Dateien weiter:
        // BAST 2004:
        if (_gui.getRadioBASt2004().getSelection()) {
            BaumVerwaltung.getInstanz().setListeAllerImportDateienFTPServer2004(listeSftpDateien, true);
        }

        // BAST 2007:
        else {
            BaumVerwaltung.getInstanz().setListeAllerImportDateienFTPServer2007(listeSftpDateien, false);
        }

        if (!listeSftpDateien.isEmpty()) {
            _listeDateien = DateiAuswahlFunktionen.dateiAuswahlDialogOeffnen(_shell, _listeDateien);
        }

        // -----------------------------------------
        // Anpassung vom GUI:
        // -----------------------------------------
        zeigeAnzahlSelDateien(_listeDateien);
        _gui.getBtnImportieren().setEnabled(istStarteImportOK());

        // -----------------------------------------
        // Fehlermeldung:
        // -----------------------------------------
        if (fehlerMeldung != null) {
            if (exception != null) {
                fehlerMeldung = String.format("%s\n\n%s", fehlerMeldung, exception.toString());
            }

            MessageDialog.openError(_gui.getShell(), "Fehler", fehlerMeldung);
            aendereGUIAnzeige(1, fehlerMeldung, false);
        }
    }

    /**
     * Speichern der aktuellen Importeinstellung.
     * @param einstellungsName der Name unter der die aktuelle Sicht gespeichert wird.
     */
    private void speicherAktuelleImportDarstellungsOptionen(String einstellungsName) {
        ImportBenutzerEinstellungen einstellung =                                                            //
            new ImportBenutzerEinstellungen(_gui.getRadioBASt2004().getSelection(),                          //
                                            _gui.getRadioBASt2007().getSelection(),                          //
                                            _gui.getTxtBenutzer().getText(), _gui.getTxtPasswd().getText(),  //
                                            _gui.getTxtURL().getText(), _gui.getTxtPort().getText(),         //
                                            _gui.getCbtnPassivmodus().getSelection(),                        //
                                            _gui.getRbtnFtp().getSelection());

        _mapImportBenutzerEinstellungen.put(einstellungsName, einstellung);
        _ImportVerwaltung.setMapImportBenutzerEinstellungen(_mapImportBenutzerEinstellungen);
        _ImportVerwaltung.speichereImportVerwaltungen(KonstantenGTMBast.ALLE_IMPORT_EINSTELLUNGEN);
    }

    /**
     * Anemldung am Datenverteiler.
     *
     * @param systemObjectBAStImportImportSteuerung
     *            das SystemObjekt das angemeldet wird.
     * @param datenbeschreibungImportSteuerung
     *            die Datenbeschreibung.
     * @param anmeldeoptionen
     *            die Anmeldeoption.
     * @param rolle
     *            dabei handelt es sich entweder um einen Sender oder einen Empfnger.
     */
    public boolean subscribe(SystemObject systemObjectBAStImportImportSteuerung, DataDescription datenbeschreibungImportSteuerung, ReceiveOptions anmeldeoptionen, Object rolle) {
        Shell        shell;
        ReceiverRole empfaengerrolle = ReceiverRole.receiver();
        SenderRole   senderRolle     = SenderRole.sender();
        boolean      anmeldungOK     = false;

        if (systemObjectBAStImportImportSteuerung == null) {
            shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
            MessageDialog.openError(shell, "Fehler", "Das angegebene SystemObjekt fr den BASt-Band Import existiert nicht!");
        } else {
            try {
                if (rolle instanceof ReceiverRole) {
                    _dav.subscribeReceiver(ImportGUIVerwaltung.this, systemObjectBAStImportImportSteuerung, datenbeschreibungImportSteuerung, anmeldeoptionen, empfaengerrolle);
                    anmeldungOK = true;
                } else if (rolle instanceof SenderRole) {
                    _dav.subscribeSender(ImportGUIVerwaltung.this, systemObjectBAStImportImportSteuerung, datenbeschreibungImportSteuerung, senderRolle);
                    anmeldungOK = true;
                }
            }
            catch (Exception e) {
                shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
                MessageDialog.openError(shell, "Fehler", "Die Objekte konnten nicht beim Datenverteiler angemeldet werden! ");
                aendereGUIAnzeige(1, "Die Objekte konnten nicht beim Datenverteiler angemeldet werden!", false);
            }
        }

        return anmeldungOK;
    }

    /**
     * Text 'Port' wurde gendert.
     */
    protected void txtPortGeaendert() {

        // Prfe, ob eine Zahl ist:
        try {
            Integer.parseInt(_gui.getTxtPort().getText());
        }
        catch (Exception e) {
            MessageDialog.openError(_shell, "Fehler", "Port-Angabe muss eine Zahl sein !!!");
        }
    }

    @Override
    public void update(ResultData[] results) {
        if (!_gui.isDisposed()) {
            String        errorText = "Es sind Probleme bei der Aktualisierung der Statusmeldung aufgetreten!";
            final Display display   = Display.getDefault();

            for (ResultData result : results) {
                if (!_gui.isDisposed()) {
                    if (result.hasData()) {
                        Data data = result.getData();

                        // Der bermittelte Prozentwert als Integervalue, so dass dieser direkt fr die Fortschrittsanzeige verwendet werden kann.
                        _prozentWert = data.getScaledValue("BAStBandFortschrittProzent").intValue();

                        // Der Statusmeldungstext.
                        String statusMeldung = data.getItem("StatusMeldung").valueToString();

                        aendereGUIAnzeige(_prozentWert, statusMeldung, true);

                        if (_prozentWert != 100) {
                            try {
                                if (!_gui.isDisposed()) {
                                    display.syncExec(new Runnable() {
                                        @Override
                                        public void run() {
                                            _gui.getBtnImportieren().setEnabled(false);
                                        }
                                    });
                                }
                            }
                            catch (Exception e) {
                                Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Es sind Probleme bei der Aktualisierung aufgetreten!", e));
                            }
                        } else if (_prozentWert == 100) {
                            try {
                                if (!_gui.isDisposed()) {
                                    display.syncExec(new Runnable() {
                                        @Override
                                        public void run() {
                                            _gui.getBtnAbbrechen().setEnabled(false);

                                            if (istStarteImportOK()) {
                                                _gui.getBtnImportieren().setEnabled(true);
                                            }
                                        }
                                    });
                                }
                            }
                            catch (Exception e) {
                                Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, errorText, e));
                            }
                        }
                    } else {
                        DataTableObjectErweitert dto             = new DataTableObjectErweitert(result);
                        DataState                resultDataState = dto.getDataState();
                        String                   statusText      = DavDatenVew.getInstanz().bestimmeStatusText(resultDataState);

                        try {
                            if (!_gui.isDisposed()) {
                                display.syncExec(new Runnable() {
                                    @Override
                                    public void run() {
                                        _gui.getBtnImportieren().setEnabled(false);
                                        _gui.getBtnAbbrechen().setEnabled(false);
                                    }
                                });
                            }
                        }
                        catch (Exception e) {
                            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Es sind Probleme bei der Aktualisierung aufgetreten!", e));
                        }

                        if (resultDataState == DataState.NO_SOURCE) {
                            aendereGUIAnzeige(1, String.format(                                                                                        //
                                "Der Import kann nicht durchgefhrt werden, da der Prozess BASt-Band nicht luft.%sDatenverteiler Information: %s%s",  //
                                NEWLINE, statusText, NEWLINE), true);
                        }
                    }
                }
            }
        }
    }

    /**
     * Stelle die Verbindung mit dem FTP-Server her.
     *
     * @param ftpClient FTP-Verbindung.
     * @param url FTP-Host.
     * @param port FTP-Port.
     * @param benutzer FTP-Benutzer.
     * @param passwort FTP-Passwort
     *
     * @return Exception, falls bei der Verbindung _Fehler aufgetreten sind,
     * null - sonst.
     */
    private Exception verbindeMitFTPServer(FTPClient ftpClient, String url, int port, String benutzer, String passwort) {

        // Verbindung:
        try {
            ftpClient.connect(url, port);
        }
        catch (IOException e) {
            return new Exception(String.format("Es sind Probleme bei der Verbindung mit dem FTP-Server <%s:%s> aufgetreten.", url, port), e);
        }

        // Login:
        String fehlerMeldung = String.format("Der login fr <%s> auf dem FTP-Server <%s> ist gescheitert. Bitte kontrollieren Sie den angegebenen Benutzernamen und das Passwort!", benutzer, url);

        try {
            if (ftpClient.login(benutzer, passwort)) {
                return null;
            } else {
                ftpClient.disconnect();

                return new Exception(fehlerMeldung);
            }
        }
        catch (IOException e) {
            return new Exception(fehlerMeldung, e);
        }
    }

    /**
     * Zeige im GUI, wie viele Dateien werden Importiert.
     */
    private void zeigeAnzahlSelDateien(List<String> listeDateien) {
        _gui.getLbBAStBandDatenDateiAuswahl().setText(DateiAuswahlFunktionen.ermittleAnzahlSelDateien(listeDateien));
    }

    /**
     * Zeige die Exception im Meldungsfenster.
     *
     * @param e Das zu zeigende Exception.
     */
    public void zeigeExceptionMeldung(Exception e) {
        String        trenLinie = "----------------------------------------------------------------------";
        StringBuilder text      = new StringBuilder();

        text.append(NEWLINE).append(NEWLINE).append(trenLinie).append(NEWLINE);
        text.append(e.toString()).append(NEWLINE);

        for (StackTraceElement trace : e.getStackTrace()) {
            text.append("\tat ").append(trace).append(NEWLINE);
        }

        text.append(trenLinie).append(NEWLINE);
        aendereGUIAnzeige(-1, text.toString(), false);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 28.10.13
