/*
 * @(#)HelpTocProvider.java   18.02.11
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.hilfe;

//~ NICHT JDK IMPORTE =========================================================

import org.eclipse.core.expressions.IEvaluationContext;
import org.eclipse.help.*;

import java.util.ArrayList;
import java.util.List;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Klasse verwaltet den Hilfe-Inhalt fr das inovatGTM-Projekt.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class HelpTocProvider extends AbstractTocProvider {

    /**
     * Ermittle Contribution.
     *
     * @param id Contribution-ID.
     *
     * @return Contribution.
     */
    private ITocContribution ermittleContribution(final String id) {
        return new ITocContribution() {
            @Override
            public boolean isPrimary() {
                return true;
            }
            @Override
            public IToc getToc() {
                return ermittleToc();
            }
            @Override
            public String getLocale() {
                return HilfeVew.getInstanz().getLokale();
            }
            @Override
            public String getLinkTo() {
                return null;
            }
            @Override
            public String getId() {
                return id;
            }
            @Override
            public String[] getExtraDocuments() {
                return new String[0];
            }
            @Override
            public String getContributorId() {
                return Activator.PLUGIN_ID;
            }
            @Override
            public String getCategoryId() {
                return null;
            }
        };
    }

    /**
     * Ermittle Toc.
     *
     * @return Toc.
     */
    private IToc ermittleToc() {
        return new IToc() {
            @Override
            public String getLabel() {
                return HilfeVew.getInstanz().getTitelHilfeInhalt();
            }
            @Override
            public String getHref() {
                return null;
            }
            @Override
            public boolean isEnabled(IEvaluationContext context) {
                return true;
            }
            @Override
            public IUAElement[] getChildren() {
                List<ITopic> listeTopic = new ArrayList<ITopic>();

                for (String anker : HilfeVew.getInstanz().ermittleListeAnker()) {
                    listeTopic.add(ermittleTopic(anker));
                }

                return listeTopic.toArray(new ITopic[listeTopic.size()]);
            }
            @Override
            public ITopic[] getTopics() {
                return new ITopic[0];
            }
            @Override
            public ITopic getTopic(String href) {
                return null;
            }
        };
    }

    /**
     * Ermittle Topic.
     *
     * @param anker Anker.
     *
     * @return Topic.
     */
    private ITopic ermittleTopic(final String anker) {
        return new ITopic() {
            @Override
            public String getLabel() {
                return HilfeVew.getInstanz().ermittleNameFuerAnker(anker);
            }
            @Override
            public String getHref() {
                return HilfeVew.getInstanz().ermittleHrefOhnePluginId(anker);
            }
            @Override
            public boolean isEnabled(IEvaluationContext context) {
                return true;
            }
            @Override
            public IUAElement[] getChildren() {
                return new IUAElement[0];
            }
            @Override
            public ITopic[] getSubtopics() {
                return new ITopic[0];
            }
        };
    }

    //~ GET METHODEN ==========================================================

    @Override
    public ITocContribution[] getTocContributions(String locale) {
        ITocContribution contribution = ermittleContribution(Activator.PLUGIN_ID + ".gtmTitel");

        return new ITocContribution[] { contribution };
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 18.02.11
