/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.datenidentifikation;

//~ NICHT JDK IMPORTE =========================================================

import com.swtdesigner.SWTResourceManager;

import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

import de.inovat.buv.gtm.datvew.lib.BaumHilfsFunktionen;
import de.inovat.buv.gtm.datvew.lib.BaumVerwaltung;
import de.inovat.buv.plugin.gtm.navigation.filter.FilterEinstellungen;
import de.inovat.buv.plugin.gtm.navigation.sortierung.SortierungEinstellungen;
import de.inovat.buv.plugin.gtm.navigation.tabellenverwaltung.TabellenVerwaltungEinstellungen;

import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.PlatformUI;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

//~ KLASSEN ===================================================================

/**
 * Diese Klasse erzeugt und verwaltet das GUI fr die Datenidentifikation.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class DatenidentifikationGUIVew {

    /** 
     * Modus gibt an, ob die Combobox fr den Aspekt angezeigt werden soll.
     * Falls Wert gleich 0 ist, wird das Combo nicht angezeigt, beim anderen Wert
     * wird das Combo fr Aspekt angezeigt. 
     */
    private int _modus = 0;

    /** FormToolkit. */
    private final FormToolkit formToolkit = new FormToolkit(Display.getDefault());

    /** Checkbox ob es sich um Archivdaten handelt. */
    private Button _checkArchivDaten;

    /** Checkbox ob es sich um konfigurierende Daten handelt. */
    private Button _checkKonfDaten;

    /** Comob Aspekte. */
    private Combo _comboAsp;

    /** Combo Attributgruppen. */
    private Combo _comboAtg;

    /** Combo Objekttyp. */
    private Combo _comboObjTyp;

    /** Filter. */
    private FilterEinstellungen _filterEinstellungen;

    /** Gewhlte Attributgruppenliste. */
    private List<AttributeGroup> _listeAtg;

    /** Liste der Objekttypen. */
    private List<SystemObject> _listeTyp;

    /** PidAttributgruppe Attributgruppe in HashMap. */
    private Map<String, AttributeGroup> _mapAtg;

    /** Objekttyp . */
    private Map<String, SystemObject> _mapTyp;

    /** Instanz Soritereinstellungen. */
    private SortierungEinstellungen _sortierungEinstellungen;

    /** Instanz Tabellenverwaltungseinstellungen. */
    private TabellenVerwaltungEinstellungen _tabellenVerwaltungEinstellungen;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     *
     * Erzeugt eine Instanz der Klasse DatenidentifikationGUIVew.
     *
     * @param compFilterParent Parent-_composite.
     * @param filterEinstellungen   Filtereinstellungen.
     */
    public DatenidentifikationGUIVew(Composite compFilterParent, FilterEinstellungen filterEinstellungen) {
        _filterEinstellungen = filterEinstellungen;
        initialisiereGUIBereich(compFilterParent);
    }

    /**
     *
     * Erzeugt eine Instanz der Klasse DatenidentifikationGUIVew.
     *
     * @param compSortParent Parent- composite.
     * @param sortierungEinstellungen Sortiereinstellungen.
     */
    public DatenidentifikationGUIVew(Composite compSortParent, SortierungEinstellungen sortierungEinstellungen) {
        _sortierungEinstellungen = sortierungEinstellungen;
        initialisiereGUIBereich(compSortParent);
    }

    /**
     *
     * Erzeugt eine Instanz der Klasse DatenidentifikationGUIVew.
     *
     * @param compTabellenV Parent- composite.
     * @param tabellenVerwaltungEinstellungen Tabellenverwaltungseinstellungen.
     * @param modus Modus.
     * Falls Wert gleich 0 ist, wird das Combo Aspekt nicht angezeigt,
     * beim anderen Wert wird das Combo angezeigt. 
     */
    public DatenidentifikationGUIVew(Composite compTabellenV, TabellenVerwaltungEinstellungen tabellenVerwaltungEinstellungen, int modus) {
        _tabellenVerwaltungEinstellungen = tabellenVerwaltungEinstellungen;
        _modus                           = modus;
        initialisiereGUIBereich(compTabellenV);
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode bestimmeObjectList bestimmt die Liste aller
     * Objekttypen.
     * @return   Collektion der selektierten Systemobjekte.
     */
    private Collection<Object> bestimmeObjectList() {
        Collection<Object> selektierteObjListe = new ArrayList<Object>();
        SystemObject       syObj               = _mapTyp.get(_comboObjTyp.getText());

        if (syObj != null) {
            selektierteObjListe.add(syObj);
        }

        return selektierteObjListe;
    }

    /**
     * Initialisiere eine Combobox.
     *
     * @param combo    Combobox.
     */
    private void erzeugeCombo(Combo combo) {
        erzeugeCombo(combo, null);
    }

    /**
     * Initialisiere eine Combobox.
     *
     * @param combo    Combobox.
     * @param text    Text zu selektieren oder null.
     */
    private void erzeugeCombo(Combo combo, String text) {
    	
    	// Aspekt:
    	if(combo==null){
    		return;
    	}

        // ----------------------------------------------------
        // Definiere die Elemente von combo:
        ArrayList<String> list = new ArrayList<String>();

        // Typ:
        if (combo.equals(_comboObjTyp)) {
            for (SystemObject syObj : _listeTyp) {
                list.add(syObj.getPid());
            }
        }

        // Atg:
        else if (combo.equals(_comboAtg)) {
            if (_comboAsp != null) {
                _comboAsp.removeAll();
            }

            _listeAtg = BaumHilfsFunktionen.bestimmeAttributgruppen(bestimmeObjectList(), _checkKonfDaten.getSelection());
            _mapAtg   = new HashMap<String, AttributeGroup>();

            for (AttributeGroup atg : _listeAtg) {
                list.add(atg.getPid());
                _mapAtg.put(atg.getPid(), atg);
            }
        }

        // Aspekt:
        else if (combo.equals(_comboAsp)) {
            if (_modus != 0) {
                AttributeGroup atg = _mapAtg.get(_comboAtg.getText());

                if (atg != null) {
                    List<Aspect> aspListe = BaumHilfsFunktionen.bestimmeAspekte(atg, _checkKonfDaten.getSelection());

                    for (Aspect asp : aspListe) {
                        list.add(asp.getPid());
                    }
                }
            }
        }

        Collections.sort(list);
        combo.setItems(list.toArray(new String[list.size()]));

        // ----------------------------------------------------
        // Selektiere ein Element:
        if (text != null) {
            combo.setText(text);
        } else {
            if ((combo.equals(_comboObjTyp)) && (!list.isEmpty())) {
                _comboObjTyp.select(0);
            }
        }
    }

    /**
     * Die Methode fuelleClientGUI sorgt dafr, das je nachdem um welchen Clienten es sich handelt (Filter, Sortierung, oder
     * Spaltenauswahl) die GUI's mit Daten gefllt werden.
     */
    private void fuelleClientGUI() {
        if (!(_filterEinstellungen == null)) {
            _filterEinstellungen.fuelleTabelle();
        } else if (!(_sortierungEinstellungen == null)) {
            _sortierungEinstellungen.setzeInitialDaten();
        } else if (!(_tabellenVerwaltungEinstellungen == null)) {
            _tabellenVerwaltungEinstellungen.fuelleAlleComboBoxen();
        }
    }

    //~ GET METHODEN ==========================================================

    /**
     *     Ermittle 'selektierteAsp'.
     *
     *     @return 'selektierteAsp'.
     */
    public String getSelektierteAsp() {
        if (_comboAsp.getSelectionIndex() == -1) {
            return "";
        } else {
            return _comboAsp.getItem(_comboAsp.getSelectionIndex());
        }
    }

    /**
     * Ermittle 'selektierteAtg'.
     *
     * @return 'selektierteAtg'.
     */
    public String getSelektierteAtg() {
        return _comboAtg.getText();
    }

    /**
     * Ermittle 'selektierteObjType'.
     *
     * @return Wert der Eigenschaft 'selektierteObjType'.
     */
    public String getSelektierteObjType() {
        return _comboObjTyp.getText();
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode initialisiereGUI sorgt dafr, dass alles auf Initialwerte gesetzt wird.
     */
    public void initialisiereGUI() {
        if (!(_comboAtg == null)) {
            erzeugeCombo(_comboAtg);
        }

        if (!(_comboAsp == null)) {
            erzeugeCombo(_comboAsp);
        }

        if (!(_comboObjTyp == null)) {
            erzeugeCombo(_comboObjTyp);
        }

        if (!(_checkKonfDaten == null)) {
            _checkKonfDaten.setSelection(false);
        }

        if (!(_checkArchivDaten == null)) {
            _checkArchivDaten.setSelection(false);
        }
    }

    /**
     * Die Methode initialisiert den GUI Bereich.
     * @param parent Parent- composite.
     *
     * @wbp.parser.entryPoint
     */
    private void initialisiereGUIBereich(Composite parent) {

        //
        _listeTyp = BaumVerwaltung.getInstanz().bestimmeSystemObjektTypen();
        _mapTyp   = new HashMap<String, SystemObject>();

        for (SystemObject syObj : _listeTyp) {
            _mapTyp.put(syObj.getPid(), syObj);
        }

        //
        Composite        compDatenidentifikation       = new Composite(parent, SWT.NONE);
        final GridLayout gridLayoutDatenIdentifikation = new GridLayout();

        gridLayoutDatenIdentifikation.marginHeight = 0;
        gridLayoutDatenIdentifikation.marginWidth  = 0;
        compDatenidentifikation.setLayout(gridLayoutDatenIdentifikation);

        GridData gridData = new GridData();

        gridData.horizontalAlignment       = GridData.FILL;
        gridData.grabExcessHorizontalSpace = true;
        compDatenidentifikation.setLayoutData(gridData);

        // -----------
        Section sectionDatenidentifikation = formToolkit.createSection(compDatenidentifikation, Section.TWISTIE | Section.TITLE_BAR);

        sectionDatenidentifikation.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        formToolkit.paintBordersFor(sectionDatenidentifikation);
        sectionDatenidentifikation.setText("Datenidentifikation");

        Composite composite = new Composite(sectionDatenidentifikation, SWT.NONE);

        composite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
        formToolkit.adapt(composite);
        formToolkit.paintBordersFor(composite);
        sectionDatenidentifikation.setClient(composite);
        composite.setLayout(new GridLayout(2, false));

        //
        _checkArchivDaten = new Button(composite, SWT.CHECK);
        _checkArchivDaten.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                if (_checkArchivDaten.getSelection()) {
                    _checkKonfDaten.setSelection(false);
                }

                erzeugeCombo(_comboAtg);
                fuelleClientGUI();
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });
        _checkArchivDaten.setText("Archivdaten");
        formToolkit.adapt(_checkArchivDaten, true, true);

        //
        _checkKonfDaten = new Button(composite, SWT.CHECK);
        _checkKonfDaten.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                if (_checkKonfDaten.getSelection()) {
                    _checkArchivDaten.setSelection(false);
                }

                erzeugeCombo(_comboAtg);
                fuelleClientGUI();
            }
        });
        _checkKonfDaten.setToolTipText("Wenn Sie diese Checkbox ausw\u00E4hlen, werden die angezeigten Attributgruppen auf Konfigurierende Attributgruppen eingeschr\u00E4nkt.");
        _checkKonfDaten.setText("Konfigurierende Daten");
        formToolkit.adapt(_checkKonfDaten, true, true);

        //
        Label _lbTyp = new Label(composite, SWT.NONE);

        _lbTyp.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        _lbTyp.setText("Typ:");
        formToolkit.adapt(_lbTyp, true, true);

        //
        _comboObjTyp = new Combo(composite, SWT.READ_ONLY);
        _comboObjTyp.setToolTipText("Bitte w\u00E4hlen Sie den Objekttyp aus.");
        _comboObjTyp.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
            @Override
            public void widgetSelected(SelectionEvent e) {
                erzeugeCombo(_comboAtg);
                fuelleClientGUI();
            }
        });
        erzeugeCombo(_comboObjTyp);

        //
        GridData gd__comboObjTyp = new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1);

        gd__comboObjTyp.widthHint = 470;
        _comboObjTyp.setLayoutData(gd__comboObjTyp);
        formToolkit.adapt(_comboObjTyp);
        formToolkit.paintBordersFor(_comboObjTyp);

        //
        Label _lbAttGr = new Label(composite, SWT.NONE);

        _lbAttGr.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        _lbAttGr.setText("Attributgruppe:");
        _lbAttGr.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
        formToolkit.adapt(_lbAttGr, true, true);

        //
        _comboAtg = new Combo(composite, SWT.READ_ONLY);
        _comboAtg.setToolTipText("Bitte w\u00E4hlen Sie die Attributgruppe aus, zu der Sie die Informationen angezeigt bekommen m\u00F6chten.");
        _comboAtg.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
            @Override
            public void widgetSelected(SelectionEvent e) {
                erzeugeCombo(_comboAsp);
                fuelleClientGUI();
            }
        });

        GridData gd__comboAtg = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);

        gd__comboAtg.widthHint = 470;
        _comboAtg.setLayoutData(gd__comboAtg);
        formToolkit.adapt(_comboAtg);
        formToolkit.paintBordersFor(_comboAtg);

        if (!(_modus == 0)) {
            Label _lbAsp = new Label(composite, SWT.NONE);

            _lbAsp.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
            _lbAsp.setText("Aspekt:");
            _lbAsp.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
            formToolkit.adapt(_lbAsp, true, true);
            _comboAsp = new Combo(composite, SWT.READ_ONLY);
            _comboAsp.setToolTipText("Bitte w\u00E4hlen Sie den Aspekt aus, zu dem Sie die Informationen angezeigt bekommen m\u00F6chten.");
            _comboAsp.addSelectionListener(new SelectionListener() {
                @Override
                public void widgetDefaultSelected(SelectionEvent e) {}
                @Override
                public void widgetSelected(SelectionEvent e) {
                    fuelleClientGUI();
                }
            });
        }

        GridData gd__comboAsp = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);

        gd__comboAsp.widthHint = 470;

        if (!(_modus == 0)) {
            _comboAsp.setLayoutData(gd__comboAsp);
            formToolkit.adapt(_comboAsp);
            formToolkit.paintBordersFor(_comboAsp);
        }

        sectionDatenidentifikation.setExpanded(true);
        PlatformUI.getWorkbench().getHelpSystem().setHelp(sectionDatenidentifikation, "de.inovat.buv.plugin.gtm.hilfe.datenidentifikation");
    }

    /**
     * Stellt fest, ob "Archiv" selektiert ist.
     *
     * @return true, falls "Archiv" selektiert ist.
     */
    public boolean istArchivDaten() {
        return _checkArchivDaten.getSelection();
    }

    /**
     * Stellt fest, ob "Konfiguration" selektiert ist.
     *
     * @return true, falls "Konfiguration" selektiert ist.
     */
    public boolean istKonfigurierendeDaten() {
        return _checkKonfDaten.getSelection();
    }

    /**
     * Setze die GUI-Daten.
     *
     * @param archivDaten Archiv.
     * @param konfigurierendeDaten Konfiguration.
     * @param selektierteObjType Typ.
     * @param selektierteAtg Atg.
     * @param selektierteAsp Aspekt.
     */
    public void setzeDaten(boolean archivDaten, boolean konfigurierendeDaten,  //
                           String selektierteObjType, String selektierteAtg, String selektierteAsp) {

        //
        _checkArchivDaten.setSelection(archivDaten);
        _checkKonfDaten.setSelection(konfigurierendeDaten);

        //
        erzeugeCombo(_comboObjTyp, selektierteObjType);

        //
        erzeugeCombo(_comboAtg, selektierteAtg);

        //
        if (!(_modus == 0)) {
            erzeugeCombo(_comboAsp, selektierteAsp);
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 05.10.11
