/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.gtmdiag;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.Data;

//~ KLASSEN ===================================================================

/**
 * Die Klasse RingBufferObjekt erzeugt ein Objekt das innerhalb des RingBuffers gespeichert wird.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class RingBufferObjekt {
    

    /** Aspekt. */
    private Aspect _aspect;

    /** Attributgruppe. */
    private AttributeGroup _attributeGroup;

    /** Dataobjekt. */
    private Data _data;

    /** Datum. */
    private long _dateDaten;

    /** Datum Archiv. */
    private long _dateDatenArchiv;

    /** Datenzhler. */
    private int _datenZaehler;

    /** Pid oder Name oder Id des Objektes. */
    private String _pidOrNameOrId;

    /** Statustext. */
    private String _statusText;

	private ArchiveDataKind _dataKind;

	private int _archivDatenZeit;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Konstruktor initialisiert das Ringbufferobjekt.
     * @param dataKind archivDatenKind
     * @param data  Daten.
     * @param attributeGroup  Attributgruppe.
     * @param aspect Aspekt.
     * @param pidOrNameOrId  Pid oder Name oder Id.
     * @param datenZaehler  Zhlvariable.
     * @param datenZeit Datum.
     * @param archivDatenZeit 
     */
    public RingBufferObjekt(ArchiveDataKind dataKind, Data data, AttributeGroup attributeGroup, Aspect aspect, int datenZaehler, String pidOrNameOrId, long datenZeit, int archivDatenZeit) {
       _dataKind = dataKind;
    	_data           = data;
        _attributeGroup = attributeGroup;
        _aspect         = aspect;
        _datenZaehler   = datenZaehler;
        _pidOrNameOrId  = pidOrNameOrId;
        _dateDaten      = datenZeit;
        _statusText     = "";
        _archivDatenZeit = archivDatenZeit;
    }

    /**
	 * @return the dateDaten
	 */
	public long getDateDaten() {
		return _dateDaten;
	}

	/**
	 * @param dateDaten the dateDaten to set
	 */
	public void setDateDaten(long dateDaten) {
		_dateDaten = dateDaten;
	}

	/**
	 * @return the datenZaehler
	 */
	public int getDatenZaehler() {
		return _datenZaehler;
	}

	/**
	 * @param datenZaehler the datenZaehler to set
	 */
	public void setDatenZaehler(int datenZaehler) {
		_datenZaehler = datenZaehler;
	}

	/**
	 * @return the dataKind
	 */
	public ArchiveDataKind getDataKind() {
		return _dataKind;
	}

	/**
	 * @param dataKind the dataKind to set
	 */
	public void setDataKind(ArchiveDataKind dataKind) {
		_dataKind = dataKind;
	}

	/**
	 * @return the archivDatenZeit
	 */
	public int getArchivDatenZeit() {
		return _archivDatenZeit;
	}

	/**
	 * @param archivDatenZeit the archivDatenZeit to set
	 */
	public void setArchivDatenZeit(int archivDatenZeit) {
		_archivDatenZeit = archivDatenZeit;
	}

	/**
	 * @param aspect the aspect to set
	 */
	public void setAspect(Aspect aspect) {
		_aspect = aspect;
	}

	/**
	 * @param attributeGroup the attributeGroup to set
	 */
	public void setAttributeGroup(AttributeGroup attributeGroup) {
		_attributeGroup = attributeGroup;
	}

	/**
	 * @param data the data to set
	 */
	public void setData(Data data) {
		_data = data;
	}

	/**
	 * @param pidOrNameOrId the pidOrNameOrId to set
	 */
	public void setPidOrNameOrId(String pidOrNameOrId) {
		_pidOrNameOrId = pidOrNameOrId;
	}

	/**
	 * @param statusText the statusText to set
	 */
	public void setStatusText(String statusText) {
		_statusText = statusText;
	}

	/**
     *
     * Erzeugt eine Instanz der Klasse RingBufferObjekt.
     *
     * @param statusText Statustext.
     * @param attributeGroup Attributgruppe.
     * @param aspect Aspekt.
     * @param datenZaehler Datenzhler.
     * @param pidOrNameOrId Name oder Pid oder Id des Objektes.
     * @param date Datum.
     */
    public RingBufferObjekt(String statusText, AttributeGroup attributeGroup, Aspect aspect, int datenZaehler, String pidOrNameOrId, long date) {
        _data           = null;
        _attributeGroup = attributeGroup;
        _aspect         = aspect;
        _datenZaehler   = datenZaehler;
        _pidOrNameOrId  = pidOrNameOrId;
        _dateDaten      = date;
        _statusText     = statusText;
    }

    //~ GET METHODEN ==========================================================

  

    public Aspect getAspect() {
        return _aspect;
    }

    public AttributeGroup getAttributeGroup() {
        return _attributeGroup;
    }

    /**
     * Getter fr Eigenschaft 'data'.
     *
     * @return Wert der Eigenschaft 'data'.
     *
     * @see #_data
     */
    public Data getData() {
        return _data;
    }

    /**
     * Getter fr Eigenschaft 'date'.
     *
     * @return Wert der Eigenschaft 'date'.
     *
     * @see #_dateDaten
     */
    public long getDate() {
        return _dateDaten;
    }

    /**
     *     Getter fr Eigenschaft 'dateDatenArchiv'.
     *
     *     @return Wert der Eigenschaft 'dateDatenArchiv'.
     *
     *     @see #_dateDatenArchiv
     */
    public long getDateDatenArchiv() {
        return _dateDatenArchiv;
    }

    /**
     * Getter fr Eigenschaft 'pidOrNameOrId'.
     *
     * @return Wert der Eigenschaft 'pidOrNameOrId'.
     *
     * @see #_pidOrNameOrId
     */
    public String getPidOrNameOrId() {
        return _pidOrNameOrId;
    }

    /**
     * Getter fr Eigenschaft 'statusText'.
     *
     * @return Wert der Eigenschaft 'statusText'.
     *
     * @see #_statusText
     */
    public String getStatusText() {
        return _statusText;
    }

    //~ SET METHODEN ==========================================================

  

    /**
     * Setter fr Eigenschaft 'dateDatenArchiv'.
     *
     * @param dateDatenArchiv Wert zum Setzen der Eigenschaft 'dateDatenArchiv'.
     *
     * @see #_dateDatenArchiv
     */
    public void setDateDatenArchiv(long dateDatenArchiv) {
        _dateDatenArchiv = dateDatenArchiv;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 27.04.10
