/*
 * @(#)IDarstellung.java   28.02.12
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.gui.lib;

//~ NICHT JDK IMPORTE =========================================================

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;

//~ INTERFACES ================================================================

/**
 * Interface definiert eine Darstellung fr die inovatGTM-Navigation.
 * </p>
 * Interface wird von Klassen implementiert, die im ExtensionPoint
 * de.inovat.buv.plugin.gtm.navigation.NavigationDarstellung angegeben werden.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public interface IDarstellung {

    /**
     * Ermittle ContentProvider fr die Darstellung.
     *
     * @return ITreeContentProvider fr die Darstellung.
     */
    public ITreeContentProvider getContentProvider();

    /**
     * Ermittle das Input: Liste der Root-Elemente.
     *
     * @return Input: Liste der Root-Elemente.
     * Muss vom Typ List sein.
     */
    public Object getInput();

    /**
     * Ermittle ILabelProvider fr die Darstellung.
     *
     * @return ILabelProvider fr die Darstellung.
     */
    public ILabelProvider getLabelProvider();

    /**
     * Ermittle den Namen der Darstellung.
     *
     * @return Name der Darstellung.
     */
    public String getName();

    //~ METHODEN ==============================================================

    /**
     * Stellt fest, ob das Modell dynamisch aufgebaut ist,
     * damit man die Aktualisierung in das GUI einbaut.
     *
     * @return true, falls das Modell dynamisch aufgebaut ist,
     * false - sonst.
     */
    public boolean istDynamisch();

    /**
     * Stellt fest, ob die nderung im dynamischen Modell berwacht wird,
     * damit man die automatische Aktualisierung in das GUI einschalten kann.
     * </p>
     * Bei der Modell-nderung muss man dann die Methode
     * {@link DarstellungenVew#benachrichtigeModellGeaendert(IDarstellung)} aufrufen.
     * </p>
     * Die Methode wird bewertet, nur falls die Methode{@link #istDynamisch()}
     * true zurck liefert.
     * </p>
     *
     * @return true, falls die nderung im dynamischen Modell berwacht wird,
     * false - sonst.
     */
    public boolean istModellAenderungUeberwacht();

    /**
     * Stellt fest, ob in der Navigation die Suche-Funktion eingeschaltet werden soll.
     * Suche wird nicht funktionieren, falls IContentProvider immer neu
     * dynamische Elemente erzeugt.
     *
     * @return true, falls die Such-Funktion eingeschaltet werden soll,
     * false - sonst.
     */
    public boolean istSucheEingeschaltet();
}


//~Formatiert mit 'inovat Kodierkonvention' am 15.03.12
