/*
 * @(#)TreeContentProviderKonfiguration.java   20.02.13
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.gui.sichte;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.gtm.datvew.lib.DatenVew;
import de.inovat.buv.gtm.datvew.lib.DaVKonfigurationsbereichsObjekt;
import de.inovat.buv.gtm.datvew.lib.DaVMenge;
import de.inovat.buv.gtm.datvew.lib.DaVMengenElement;
import de.inovat.buv.gtm.datvew.lib.DaVObjektTyp;
import de.inovat.buv.gtm.datvew.lib.DaVSystemObjekt;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

//~ JDK IMPORTE ===============================================================

import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Klasse definiert die Baum-Struktur fr Konfiguration
 * (mit und ohne KB).
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class TreeContentProviderKonfiguration implements ITreeContentProvider {

    /** Sicht, fr die das ContentProvider implementiert wird. */
    private Sicht _sicht;

    //~ KONSTANTE AUFZAEHLUNGSTYPEN ===========================================

    /**
     * Darstellungsicht.
     */
    enum Sicht { KONF, KONF_MIT_KB }

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse TreeContentProviderKonfiguration.
     *
     * @param sicht Sicht, fr die das ContentProvider implementiert wird.
     */
    public TreeContentProviderKonfiguration(Sicht sicht) {
        _sicht = sicht;
    }

    //~ METHODEN ==============================================================

    @Override
    public void dispose() {}

    //~ GET METHODEN ==========================================================

    @Override
    public Object[] getChildren(Object parentElement) {
        return treeKonfigurationGetChildren(parentElement);
    }

    @SuppressWarnings("unchecked")
    @Override
    public Object[] getElements(Object inputElement) {
        if (inputElement instanceof List) {
            List root = (List) inputElement;

            return root.toArray();
        }

        return null;
    }

    @Override
    public Object getParent(Object element) {
        return treeKonfigurationGetParent(element);
    }

    @Override
    public boolean hasChildren(Object element) {
        return treeKonfigurationHasChildren(element);
    }

    //~ METHODEN ==============================================================

    @Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}

    /**
     * Ermittle Kinder fr das Element aus dem Baum "Konfiguration".
     *
     * @param parentElement parentElement.
     *
     * @return Array von Kindern.
     */
    private Object[] treeKonfigurationGetChildren(Object parentElement) {
        if (parentElement instanceof DaVKonfigurationsbereichsObjekt) {
            DaVKonfigurationsbereichsObjekt kb = (DaVKonfigurationsbereichsObjekt) parentElement;

            return DatenVew.getInstanz().ermittleKBKinder(kb.getSystemObjekt().getPidOrId()).toArray();
        } else if (parentElement instanceof DaVObjektTyp) {
            DaVObjektTyp davTyp = (DaVObjektTyp) parentElement;

            if (_sicht == Sicht.KONF) {
                return davTyp.getObjekteListeDesSystemObjTyps().toArray();
            } else if (_sicht == Sicht.KONF_MIT_KB) {
                return davTyp.getObjekteListeDesSystemObjTypsImKB().toArray();
            }
        } else if (parentElement instanceof DaVSystemObjekt) {
            DaVSystemObjekt davSo = (DaVSystemObjekt) parentElement;

            return davSo.getMengen().toArray();
        } else if (parentElement instanceof DaVMenge) {
            DaVMenge menge = (DaVMenge) parentElement;

            return menge.getMengenElemente().toArray();
        }

        return null;
    }

    /**
     * Ermittle Parent fr das Element aus dem Baum "Konfiguration".
     *
     * @param element Element.
     *
     * @return Parent-Element.
     */
    private Object treeKonfigurationGetParent(Object element) {
        if (element instanceof DaVKonfigurationsbereichsObjekt) {
            return null;
        } else if (element instanceof DaVObjektTyp) {
            DaVObjektTyp davTyp = (DaVObjektTyp) element;

            return davTyp.getParentKb();
        } else if (element instanceof DaVSystemObjekt) {
            DaVSystemObjekt davSo = (DaVSystemObjekt) element;

            return davSo.getParentDavTyp();
        } else if (element instanceof DaVMenge) {
            DaVMenge menge = (DaVMenge) element;

            return menge.getParent();
        } else if (element instanceof DaVMengenElement) {
            DaVMengenElement el = (DaVMengenElement) element;

            return el.getParent();
        }

        return null;
    }

    /**
     * Ermittle, ob das Element aus dem Baum "Konfiguration" Kinder hat.
     *
     * @param element Element.
     *
     * @return true, falls das Element aus dem Baum "Konfiguration" Kinder hat,
     * false - sonst.
     */
    private boolean treeKonfigurationHasChildren(Object element) {
        if (element instanceof DaVKonfigurationsbereichsObjekt) {
            return true;
        } else if (element instanceof DaVObjektTyp) {
            DaVObjektTyp davTyp = (DaVObjektTyp) element;

            if (_sicht == Sicht.KONF) {
                return !davTyp.getObjekteListeDesSystemObjTyps().isEmpty();
            } else if (_sicht == Sicht.KONF_MIT_KB) {
                return !davTyp.getObjekteListeDesSystemObjTypsImKB().isEmpty();
            }
        } else if (element instanceof DaVSystemObjekt) {
            DaVSystemObjekt davSo = (DaVSystemObjekt) element;

            return !davSo.getMengen().isEmpty();
        } else if (element instanceof DaVMenge) {
            DaVMenge menge = (DaVMenge) element;

            return !menge.getMengenElemente().isEmpty();
        }

        return false;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 20.02.13
