/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.sortierung;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.gui.GuiFunktionen;
import de.inovat.buv.inovat.lib.gui.vewdlg.IRahmenVerwaltungsDialogGUI;
import de.inovat.buv.inovat.lib.gui.vewdlg.RahmenVerwaltungsDialogGUI;
import de.inovat.buv.plugin.gtm.navigation.datenidentifikation.Datenidentifikation;
import de.inovat.buv.plugin.gtm.navigation.datenidentifikation.DatenidentifikationGUIVew;
import de.inovat.buv.plugin.gtm.navigation.tabellen.SpaltenObjekt;
import de.inovat.buv.plugin.gtm.navigation.ViewSortierung;
import de.inovat.buv.plugin.gtm.navigation.ViewSpaltenauswahl;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.PlatformUI;

import static de.inovat.buv.projektlib.konstanten.KonstantenGUIResource.FARBE_WEIS;
import static de.inovat.buv.projektlib.konstanten.KonstantenGUIResource.ICON_VEW_PFEIL_HOCH;
import static de.inovat.buv.projektlib.konstanten.KonstantenGUIResource.ICON_VEW_PFEIL_LINKS;
import static de.inovat.buv.projektlib.konstanten.KonstantenGUIResource.ICON_VEW_PFEIL_RECHTS;
import static de.inovat.buv.projektlib.konstanten.KonstantenGUIResource.ICON_VEW_PFEIL_RUNTER;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Die Klasse verwaltet den Dialog fr die Sortierungen.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class SortierungEinstellungen implements IRahmenVerwaltungsDialogGUI {

    /** Farbe fr die Statusinformation 'Keine Daten'. */
    public static final Color BACKGROUND = FARBE_WEIS;

    //~ FELDER ================================================================

    /** Array der festen Tabellenspalten, wenn es sich um eine Archivanfrage handelt. */
    private final String[] _archivSpaltenAuswahl = { "Art", "Datenzeit", "Archivzeit", "Objekt" };

    /** Legt fest, ob die GUI-Daten dem GUI-Bereich equivalent sind. */
    private boolean _guiDatenEqualGUIBereich = true;

    /** Array der festen Tabellenspalten, wenn es sich um keine Archivanfrage handelt. */
    private final String[] _normalSpaltenAuswahl = { "Art", "Datenzeit", "Objekt" };

    /** FormToolkit. */
    private final FormToolkit formToolkit = new FormToolkit(Display.getDefault());

    /** Liste rechts: enthlt fr die Sortierung ausgewhlte sortierbaren Spalten. */
    private List<String> _listeRechts = new ArrayList<String>();

    /** Liste links: enthlt alle sortierbare Spalten. */
    private List<String> _listeLinks = new ArrayList<String>();

    /**
     * Instanz der Klasse AlleSortierungen.
     */
    private AlleSortierungen _alleSortierungenSpaltenInstanz;

    /** GUI Datenidentifikation. */
    private DatenidentifikationGUIVew _datenIdentifikationGUIVew;

    /** Instanz RahmenVerwaltungsDialogGUI. */
    private RahmenVerwaltungsDialogGUI _guiVew;

    /**
     * Liste rechts: enthlt fr die Sortierung ausgewhlte sortierbare Objekte.
     * Ein sortierbares Objekt besteht aus dem Spaltennamen wie der Sortierinformation aufsteigend oder
     * absteigend.
     */
    private List<SortierObjekt> _listeRechtsSortierObj;

    /** ListViewer links: enthlt alle sortierbare Variable. */
    private ListViewer _listViewerSVlinks;

    /**
     * Modus
     * 0: Sortierung
     * 1: Spaltenauswahl.
     */
    private int _modus;

    /** Die aktuell im EinstelungsDialogausgewhlte Sortierung. */
    private Sortierung _sortierungElement;

    /** TableViewer rechts: enthlt fr die Sortierung ausgewhlte sortierbare Spalte. */
    private TableViewer _tableViewerSVrechts;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse SortierungEinstellungen.
     *
     * @param modus   Modus ob es sich um eine Sortierinstanz oder eine Spaltenauswahlinstanz handelt.
     */
    public SortierungEinstellungen(int modus) {
        _modus = modus;
        ladeAusXML();
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode aenderungErfolgt wird immer aufgerufen, wenn Daten ggf. modifiziert worden sind.
     * Die Methode prft, basierend auf den equals Methoden der Objekte, ob wirklich nderungen vollzogen worden sind und
     * setzt dementsprechend die Schaltflchen bernehmen und Zurcksetzten.
     *
     */
    private void aenderungErfolgt() {
        boolean aenderung = true;

        ladeAusXML();

        Sortierung sortierung = _alleSortierungenSpaltenInstanz.getSortierungObjekt(_sortierungElement.getSortierungName());

        if (sortierung != null) {
            List<SortierungEintrag> sortierEintraegeListe          = sortierung.getSortierungEintraege();
            SortierungEintrag       guiSortierEintragVergleichsObj = guiVergleichsObjFuellen();

            for (SortierungEintrag gespeicherterSortierEintrag : sortierEintraegeListe) {
                aenderung                = !guiSortierEintragVergleichsObj.equals(gespeicherterSortierEintrag);
                _guiDatenEqualGUIBereich = !aenderung;
            }

            if (sortierEintraegeListe.size() == 0) {
                guiInitialObjFuellen();
                aenderung                = false;
                _guiDatenEqualGUIBereich = !aenderung;
            }
        }

        if (_guiVew != null) {
            _guiVew.getBtnUebernehmen().setEnabled(aenderung);
            _guiVew.getBtnZuruecksetzen().setEnabled(aenderung);
        }
    }

    /**
     * Die Methode btnObjVonSortierungEntfernen entfernt die ausgewhlten Objekte aus der Sortierauswahl.
     * @param viewerLinks Viewer links indem die Spalten dargestellt werden.
     * @param listLinks Liste links, in der sich die Eintrge befinden, die im Viewer dargestellt werden.
     * @param viewerRechts Viewer rechts, indem die auserwhlten Spalten dargestellt werden.
     * @param listRechts Liste rechts, in der sich die Eintrge befinden, die im Viewer rechts dargestellt werden.
     */
    private void btnObjVonSortierungEntfernen(StructuredViewer viewerLinks, List<String> listLinks, StructuredViewer viewerRechts, List<String> listRechts) {
        _guiDatenEqualGUIBereich = false;

        IStructuredSelection    selection         = (IStructuredSelection) viewerRechts.getSelection();
        Iterator<SortierObjekt> it                = selection.iterator();
        List<String>            selectirteObjekte = new ArrayList<String>();
        int                     indexZuEntfernen  = 0;

        while (it.hasNext()) {
            String gewaehlteObjName = it.next().getSortierWahl();

            selectirteObjekte.add(gewaehlteObjName);

            for (SortierObjekt sortierObj : _listeRechtsSortierObj) {
                if (sortierObj.getSortierWahl().equals(gewaehlteObjName)) {
                    indexZuEntfernen = _listeRechtsSortierObj.indexOf(sortierObj);
                }
            }

            _listeRechtsSortierObj.remove(indexZuEntfernen);
        }

        // Entferne die selektierten Objekte aus der GUI Liste:
        entferneElementeAusListe(selectirteObjekte, listRechts);

        // Fge die selektierten Objekte in die andere GUI Liste:
        einfgenElementeInListe(selectirteObjekte, listLinks);

        //
        viewerLinks.setInput(listLinks);
        viewerRechts.setInput(_listeRechtsSortierObj);

        //
        aenderungErfolgt();
    }

    /**
     * Die Methode entfernt das selektierte Element aus der Liste links und fgt es in die Liste rechts hinzu.
     * D.h. dass die Elemente zur Sortierung hinzugefgt werden.
     * Zudem wird ein neues SortierObjekt in der _listeRechtsSortierObj hinzugefgt mit dem Namen.
     * @param viewerLinks Viewer links indem die Spalten dargestellt werden.
     * @param listLinks Liste links, in der sich die Eintrge befinden, die im Viewer dargestellt werden.
     * @param viewerRechts Viewer rechts, indem die auserwhlten Spalten dargestellt werden.
     * @param listRechts Liste rechts, in der sich die Eintrge befinden, die im Viewer rechts dargestellt werden.
     */
    private void btnObjZurSortierungHinzufuegen(StructuredViewer viewerLinks, List<String> listLinks, StructuredViewer viewerRechts, List<String> listRechts) {
        _guiDatenEqualGUIBereich = false;

        IStructuredSelection selection         = (IStructuredSelection) viewerLinks.getSelection();
        Iterator<?>          it                = selection.iterator();
        List<String>         selectirteObjekte = new ArrayList<String>();

        while (it.hasNext()) {
            String gewaehlteObjName = it.next().toString();

            selectirteObjekte.add(gewaehlteObjName);
            _listeRechtsSortierObj.add(new SortierObjekt(gewaehlteObjName, true));
        }

        // Entferne die selektierten Objekte aus der GUI Liste:
        entferneElementeAusListe(selectirteObjekte, listLinks);

        // Fge die selektierten Objekte in die andere GUI Liste:
        einfgenElementeInListe(selectirteObjekte, listRechts);

        //
        viewerLinks.setInput(listLinks);
        viewerRechts.setInput(_listeRechtsSortierObj);

        //
        aenderungErfolgt();
    }

    /**
     * Entferne die selektierten Objekte aus einer GUI Liste
     * und fge die selektierten Objekte in die andere GUI Liste.
     *
     * @param viewerMinus Viewer, von dem die selektierten Objekte entfernt werden.
     * @param listMinus Liste, von der die selektierten Objekte entfernt werden.
     * @param viewerPlus Viewer, zu dem die selektierten Objekte eingefgt werden.
     * @param listPlus Liste, zu der die selektierten Objekte eingefgt werden.
     */
    private void btnRechtsLinksMouseDown(StructuredViewer viewerMinus, List<String> listMinus, StructuredViewer viewerPlus, List<String> listPlus) {
        _guiDatenEqualGUIBereich = false;

        // Definiere die Liste von selektierten Objekten:
        IStructuredSelection selection         = (IStructuredSelection) viewerMinus.getSelection();
        Iterator<?>          it                = selection.iterator();
        List<String>         selectirteObjekte = new ArrayList<String>();

        while (it.hasNext()) {
            selectirteObjekte.add(it.next().toString());
        }

        // Entferne die selektierten Objekte aus der GUI Liste:
        entferneElementeAusListe(selectirteObjekte, listMinus);
        viewerMinus.setInput(listMinus);

        // Fge die selektierten Objekte in die andere GUI Liste:
        einfgenElementeInListe(selectirteObjekte, listPlus);
        viewerPlus.setInput(listPlus);

        //
        aenderungErfolgt();
    }

    /**
     * Die Methode doubleClickTableViewerSVrechts reagiert auf einen Doppelklick in der rechten Liste.
     * Durch einen Doppelklick in der rechten Liste wird die Sortierung umgekehrt und das Icon das dieses andeutet.
     */
    private void doubleClickTableViewerSVrechts() {
        if (_modus == 0) {
            StructuredSelection selection            = (StructuredSelection) _tableViewerSVrechts.getSelection();
            Object[]            ausgewaehlteElemente = selection.toArray();

            for (Object anAusgewaehlteElemente : ausgewaehlteElemente) {
                if (anAusgewaehlteElemente instanceof SortierObjekt) {
                    SortierObjekt sortObj = (SortierObjekt) anAusgewaehlteElemente;

                    if (sortObj.isAufsteigend()) {
                        sortObj.setAufsteigend(false);
                    } else {
                        sortObj.setAufsteigend(true);
                    }
                }
            }

            _tableViewerSVrechts.setInput(_listeRechtsSortierObj);
        }

        _guiDatenEqualGUIBereich = false;
        aenderungErfolgt();
    }

    @Override
    public void einfuegenNeuesElement(String objektName) {
        _sortierungElement = new Sortierung(objektName, new ArrayList<SortierungEintrag>());
        _alleSortierungenSpaltenInstanz.addSortierung(_sortierungElement);
        speichereMapInXML();
        _guiDatenEqualGUIBereich = true;
    }

    /**
     * Einfge ein Element in die Liste.
     *
     * @param elemente Das einzufgende Element.
     * @param list Liste, in die das Element eingefgt wird.
     */
    private void einfgenElementeInListe(List<String> elemente, List<String> list) {
        list.addAll(elemente);
    }

    /**
     * Entferne ein Element aus der Liste.
     *
     * @param elemente Das zu entfernte Element.
     * @param list Liste, aus der das element entfernt wird.
     */
    private void entferneElementeAusListe(List<String> elemente, List<String> list) {
        for (String el : elemente) {
            list.remove(el);
        }
    }

    @Override
    public String[] ermittleInputArray() {
        return _alleSortierungenSpaltenInstanz.getSortierungArray();
    }

    //~ GET METHODEN ==========================================================

    /**
     * Erstelle eine Liste von SortierbarenVaribleNamen.
     *
     * @return Liste von Namen.
     */
    private List<String> getListeNamenSortierbarenSpalten() {
        ArrayList<String> arName = new ArrayList<String>();
        SpaltenObjekt     spObj  = new SpaltenObjekt(0);

        if (_datenIdentifikationGUIVew != null) {
            if (!(_datenIdentifikationGUIVew.getSelektierteAtg().equals(""))) {
                arName.addAll(spObj.getSpaltenNamen(_datenIdentifikationGUIVew.getSelektierteAtg()));
            }

            if (_datenIdentifikationGUIVew.istArchivDaten()) {
                Collections.addAll(arName, _archivSpaltenAuswahl);
            } else {
                Collections.addAll(arName, _normalSpaltenAuswahl);
            }
        }

        return arName;
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode fllt das initiale GUI Objekt je nachdem ob es sich um eine Sortierinstanz oder
     * Spaltenauswahl handelt.
     */
    private void guiInitialObjFuellen() {
        List<String>        vergleichsObjListe      = new ArrayList<String>();
        List<String>        vergleichsObjListeLinks = new ArrayList<String>();
        List<SortierObjekt> sortObjListe            = new ArrayList<SortierObjekt>();

        vergleichsObjListe.addAll(_listeRechts);
        vergleichsObjListeLinks.addAll(_listeLinks);

        if (_modus == 0) {
            _sortierungElement.getSortierungEintraege().add(new SortierungEintrag(new Datenidentifikation(_datenIdentifikationGUIVew.getSelektierteObjType(), _datenIdentifikationGUIVew.getSelektierteAtg(), null, false, false), vergleichsObjListeLinks, vergleichsObjListe, sortObjListe));
        } else if (_modus == 1) {
            _sortierungElement.getSortierungEintraege().add(new SortierungEintrag(new Datenidentifikation(_datenIdentifikationGUIVew.getSelektierteObjType(), _datenIdentifikationGUIVew.getSelektierteAtg(), null, false, false), vergleichsObjListeLinks, vergleichsObjListe));
        }

        speichereMapInXML();
        _guiDatenEqualGUIBereich = true;
    }

    /**
     * Die Methode guiVergleichsObjFuellen fllt das Vergleichsobjekt mit den aktuellen Daten des GUI.
     *
     * @return Vergleichsobjekt.
     */
    private SortierungEintrag guiVergleichsObjFuellen() {
        List<String>        vergleichsObjListe      = new ArrayList<String>();
        List<String>        vergleichsObjListeLinks = new ArrayList<String>();
        List<SortierObjekt> sortObjListe            = new ArrayList<SortierObjekt>();

        vergleichsObjListe.addAll(_listeRechts);
        vergleichsObjListeLinks.addAll(_listeLinks);

        for (SortierObjekt sortObj : _listeRechtsSortierObj) {
            SortierObjekt sortierObjGUI = new SortierObjekt(sortObj.getSortierWahl(), sortObj.isAufsteigend());

            sortObjListe.add(sortierObjGUI);
        }

        SortierungEintrag guiSortierEintragVergleichsObj;

        if (_modus == 0) {
            guiSortierEintragVergleichsObj = new SortierungEintrag(new Datenidentifikation(_datenIdentifikationGUIVew.getSelektierteObjType(), _datenIdentifikationGUIVew.getSelektierteAtg(), null, _datenIdentifikationGUIVew.istArchivDaten(), _datenIdentifikationGUIVew.istKonfigurierendeDaten()), vergleichsObjListeLinks, vergleichsObjListe, sortObjListe);
        } else {
            guiSortierEintragVergleichsObj = new SortierungEintrag(new Datenidentifikation(_datenIdentifikationGUIVew.getSelektierteObjType(), _datenIdentifikationGUIVew.getSelektierteAtg(), null, _datenIdentifikationGUIVew.istArchivDaten(), _datenIdentifikationGUIVew.istKonfigurierendeDaten()), vergleichsObjListeLinks, vergleichsObjListe);
        }

        return guiSortierEintragVergleichsObj;
    }

    @Override
    public void initialisiereElement() {
        _sortierungElement = new Sortierung("", new ArrayList<SortierungEintrag>());
        _listeLinks        = getListeNamenSortierbarenSpalten();
        _listeRechts       = new ArrayList<String>();
    }

    /**
     * @wbp.parser.entryPoint
     */
    @Override
    public void initialisiereGUIBereich(Composite parent) {
        {
            _listeRechtsSortierObj = new ArrayList<SortierObjekt>();

            ScrolledComposite scrolledComposite = new ScrolledComposite(parent, SWT.H_SCROLL | SWT.V_SCROLL);

            scrolledComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
            scrolledComposite.setExpandHorizontal(true);
            scrolledComposite.setExpandVertical(true);
            scrolledComposite.setBackground(BACKGROUND);

            Composite sortierAuswahl = new Composite(scrolledComposite, SWT.NONE);

            sortierAuswahl.setLayout(new GridLayout(1, false));
            sortierAuswahl.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
            sortierAuswahl.setBackground(BACKGROUND);
            _datenIdentifikationGUIVew = new DatenidentifikationGUIVew(sortierAuswahl, this);

            Section _sectionSortierung = formToolkit.createSection(sortierAuswahl, Section.TWISTIE | Section.TITLE_BAR);

            _sectionSortierung.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
            formToolkit.paintBordersFor(_sectionSortierung);
            _sectionSortierung.setText("Sortierung");

            Group _goupSortierung = new Group(_sectionSortierung, SWT.NONE);

            _sectionSortierung.setClient(_goupSortierung);
            formToolkit.adapt(_goupSortierung);
            formToolkit.paintBordersFor(_goupSortierung);
            _goupSortierung.setLayout(new GridLayout(1, false));

            GridData listSortierbareVariableLData = new GridData();

            listSortierbareVariableLData.grabExcessVerticalSpace   = true;
            listSortierbareVariableLData.verticalAlignment         = GridData.FILL;
            listSortierbareVariableLData.grabExcessHorizontalSpace = true;
            listSortierbareVariableLData.widthHint                 = 200;

            Composite composite = new Composite(_goupSortierung, SWT.NONE);

            composite.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, true, 1, 1));
            formToolkit.adapt(composite);
            formToolkit.paintBordersFor(composite);
            composite.setLayout(new GridLayout(3, false));
            _listViewerSVlinks = new ListViewer(composite, SWT.H_SCROLL | SWT.V_SCROLL | SWT.EMBEDDED | SWT.BORDER | SWT.MULTI);
            _listViewerSVlinks.getControl().setLayoutData(listSortierbareVariableLData);
            _listViewerSVlinks.setContentProvider(new ArrayContentProvider());
            _listViewerSVlinks.setLabelProvider(new LabelProvider());
            _listViewerSVlinks.setInput(_listeLinks);

            GridLayout compRechtsLinksBtnLayout = new GridLayout();

            compRechtsLinksBtnLayout.makeColumnsEqualWidth = true;
            {
                Composite compRechtsLinksBtn = new Composite(composite, SWT.NONE);

                compRechtsLinksBtn.setLayout(compRechtsLinksBtnLayout);
                {
                    Button btnRechts = new Button(compRechtsLinksBtn, SWT.PUSH | SWT.CENTER);

                    btnRechts.setImage(ICON_VEW_PFEIL_RECHTS);
                    btnRechts.addSelectionListener(new SelectionListener() {
                        @Override
                        public void widgetSelected(SelectionEvent e) {
                            if (_modus == 0) {

                                // Elemente aus der Liste links werden in die Liste Rechts verschoben,
                                // d.h. es muss genau ein neues Objekt erzeugt werden.
                                btnObjZurSortierungHinzufuegen(_listViewerSVlinks, _listeLinks, _tableViewerSVrechts, _listeRechts);
                            } else {
                                btnRechtsLinksMouseDown(_listViewerSVlinks, _listeLinks, _tableViewerSVrechts, _listeRechts);
                            }
                        }
                        @Override
                        public void widgetDefaultSelected(SelectionEvent e) {}
                    });
                }
                {
                    Button btnLinks = new Button(compRechtsLinksBtn, SWT.PUSH | SWT.CENTER);

                    btnLinks.setImage(ICON_VEW_PFEIL_LINKS);
                    btnLinks.addSelectionListener(new SelectionListener() {
                        @Override
                        public void widgetSelected(SelectionEvent e) {
                            if (_modus == 0) {
                                btnObjVonSortierungEntfernen(_listViewerSVlinks, _listeLinks, _tableViewerSVrechts, _listeRechts);
                            } else {
                                btnRechtsLinksMouseDown(_tableViewerSVrechts, _listeRechts, _listViewerSVlinks, _listeLinks);
                            }
                        }
                        @Override
                        public void widgetDefaultSelected(SelectionEvent e) {}
                    });
                }
            }

            //
            _tableViewerSVrechts = new TableViewer(composite, SWT.H_SCROLL | SWT.V_SCROLL | SWT.EMBEDDED | SWT.BORDER | SWT.MULTI | SWT.FULL_SELECTION);

            if (_modus == 0) {
                TableViewerColumn tableViewerColumnIcon = new TableViewerColumn(_tableViewerSVrechts, SWT.NONE);

                tableViewerColumnIcon.getColumn().setWidth(20);

                TableViewerColumn tableViewerColumnSortierung = new TableViewerColumn(_tableViewerSVrechts, SWT.NONE);

                tableViewerColumnSortierung.getColumn().setWidth(197);
            }

            GridData tableSortVariableLData = new GridData();

            tableSortVariableLData.verticalAlignment         = GridData.FILL;
            tableSortVariableLData.grabExcessVerticalSpace   = true;
            tableSortVariableLData.grabExcessHorizontalSpace = true;
            tableSortVariableLData.horizontalAlignment       = GridData.CENTER;
            tableSortVariableLData.widthHint                 = 200;
            _tableViewerSVrechts.getControl().setLayoutData(tableSortVariableLData);
            _sectionSortierung.setExpanded(true);

            //
            scrolledComposite.setContent(sortierAuswahl);
            scrolledComposite.setMinSize(sortierAuswahl.computeSize(SWT.DEFAULT, SWT.DEFAULT));

            if (_modus == 1) {
                _tableViewerSVrechts.setContentProvider(new ArrayContentProvider());
                _tableViewerSVrechts.setLabelProvider(new LabelProvider());
                _tableViewerSVrechts.setInput(_listeRechts);
            } else {
                _tableViewerSVrechts.setContentProvider(new SortierungContentProvider());
                _tableViewerSVrechts.setLabelProvider(new SortierungLabelProvider());

                for (String auswahl : _listeRechts) {
                    SortierObjekt sortObj = new SortierObjekt(auswahl, true);

                    _listeRechtsSortierObj.add(sortObj);
                }

                _tableViewerSVrechts.addDoubleClickListener(new IDoubleClickListener() {
                    @Override
                    public void doubleClick(DoubleClickEvent event) {
                        doubleClickTableViewerSVrechts();
                    }
                });
                _tableViewerSVrechts.setInput(_listeRechtsSortierObj);
            }

            if (_modus == 1) {
                final TextTransfer textTransfer = TextTransfer.getInstance();
                Transfer[]         dragType     = new Transfer[] { textTransfer };

                _tableViewerSVrechts.addDragSupport(SWT.Move, dragType, new DragSourceAdapter() {
                    public void dragSetData(DragSourceEvent event) {
                        if (textTransfer.isSupportedType(event.dataType)) {
                            IStructuredSelection selection = (IStructuredSelection) _tableViewerSVrechts.getSelection();

                            event.data = selection.getFirstElement();
                        }
                    }
                });
                _tableViewerSVrechts.addDropSupport(SWT.Move, dragType, new ViewerDropAdapter(_listViewerSVlinks) {
                    @Override
                    public boolean performDrop(Object data) {
                        Object target = getCurrentTarget();

                        verschiebeElement((String) data, (String) target);
                        _tableViewerSVrechts.setInput(_listeRechts);

                        return true;
                    }
                    @Override
                    public boolean validateDrop(Object target, int operation, TransferData transferType) {
                        return textTransfer.isSupportedType(transferType);
                    }
                    @Override
                    public boolean getFeedbackEnabled() {
                        return true;
                    }
                });
            }
        }
    }

    @Override
    public void kopiereElement(String objektName) {
        List<SortierungEintrag> se = new ArrayList<SortierungEintrag>();

        se.addAll(_sortierungElement.getSortierungEintraege());
        _sortierungElement = new Sortierung(objektName, se);
        _alleSortierungenSpaltenInstanz.addSortierung(_sortierungElement);
        speichereMapInXML();
        _guiDatenEqualGUIBereich = true;
    }

    /**
     * Die Methode ladeAusXML liest die gespeicherten Daten aus dem XML ein.
     * Wenn der Modus == 0 ist, handelt es sich um eine Sortierung, wenn der Modus == 1 ist, handelt es sich
     * um eine Spaltenauswahl, dementsprechend werden die Instanzen geladen.
     */
    private void ladeAusXML() {
        if (_modus == 0) {
            _alleSortierungenSpaltenInstanz = AlleSortierungen.getSortierInstanz();
        } else if (_modus == 1) {
            _alleSortierungenSpaltenInstanz = AlleSortierungen.getSpaltenAuswahlInstanz();
        }
    }

    @Override
    public void loescheElement(String objektName) {
        _alleSortierungenSpaltenInstanz.loescheSortierung(objektName);
        speichereMapInXML();
        _guiDatenEqualGUIBereich = true;
    }

    @Override
    public void selektiereElement(String objektName) {
        _sortierungElement       = _alleSortierungenSpaltenInstanz.getSortierungObjekt(objektName);
        _guiDatenEqualGUIBereich = true;
    }

    //~ SET METHODEN ==========================================================

    /**
     * Setter fr die Eigenschaft 'guiVew'.
     * @param guiVew RahmenVerwaltungsDialogGUI.
     */
    public void setGuiVew(RahmenVerwaltungsDialogGUI guiVew) {
        _guiVew = guiVew;
    }

    //~ METHODEN ==============================================================

    @Override
    public void setzeGUIbereichMitElementDaten(String objektName) {

        // erstelle listeRechts=Liste von Eintrgen in sortierungElement
        // und listeLinks=Liste von SortierbarenVariablen - listeRechts.
        List<String> listeEintraege = new ArrayList<String>();

        _listeLinks            = new ArrayList<String>();
        _listeRechts           = new ArrayList<String>();
        _listeRechtsSortierObj = new ArrayList<SortierObjekt>();

        if (_sortierungElement != null) {
            if (_sortierungElement.getSortierungEintraege().isEmpty()) {
                _datenIdentifikationGUIVew.initialisiereGUI();
            } else {
                for (SortierungEintrag se : _sortierungElement.getSortierungEintraege()) {
                    listeEintraege.addAll(se.getGewaehlteZeilen());
                    _datenIdentifikationGUIVew.setzeDaten(se.getDatenidentifikation().isArchivDaten(), se.getDatenidentifikation().isKonfigurierendeDaten(),  //
                                                          se.getDatenidentifikation().getObjTyp(), se.getDatenidentifikation().getAtgGroup(), "");

                    for (SortierObjekt sortObj : se.getGewaehlteSortierObj()) {
                        SortierObjekt lokaleSortierObj = new SortierObjekt(sortObj.getSortierWahl(), sortObj.isAufsteigend());

                        _listeRechtsSortierObj.add(lokaleSortierObj);
                    }
                }

                _listeLinks.addAll(getListeNamenSortierbarenSpalten());

                if (_modus == 0) {
                    for (SortierObjekt sortierObj : _listeRechtsSortierObj) {
                        _listeRechts.add(sortierObj.getSortierWahl());
                    }
                } else {
                    _listeRechts.addAll(listeEintraege);
                }

                _listeLinks.removeAll(_listeRechts);
            }
        }

        _listViewerSVlinks.setInput(_listeLinks);

        if (_modus == 1) {
            _tableViewerSVrechts.setInput(_listeRechts);
        } else {
            _tableViewerSVrechts.setInput(_listeRechtsSortierObj);
        }

        _guiDatenEqualGUIBereich = true;
        aenderungErfolgt();
    }

    /**
     * Die Methode setzteInitialDaten wird von der Datenidentifikation aufgerufen.
     */
    public void setzeInitialDaten() {

        //
        _listeLinks = new ArrayList<String>();
        _listeLinks.addAll(getListeNamenSortierbarenSpalten());
        _listViewerSVlinks.setInput(_listeLinks);

        //
        _listeRechts           = new ArrayList<String>();
        _listeRechtsSortierObj = new ArrayList<SortierObjekt>();

        if (_modus == 1) {
            _tableViewerSVrechts.setInput(_listeRechts);
        } else {
            _tableViewerSVrechts.setInput(_listeRechtsSortierObj);
        }

        //
        aenderungErfolgt();
    }

    @Override
    public boolean sindGuiDatenMitElementDatenGleich() {
        return _guiDatenEqualGUIBereich;
    }

    @Override
    public void speichereElement() {

        // speichere die rechte Liste als SortierungEintrag im sortierungElement:
        _sortierungElement.getSortierungEintraege().clear();

        Datenidentifikation datId            = new Datenidentifikation(_datenIdentifikationGUIVew.getSelektierteObjType(), _datenIdentifikationGUIVew.getSelektierteAtg(), null, _datenIdentifikationGUIVew.istArchivDaten(), _datenIdentifikationGUIVew.istKonfigurierendeDaten());
        List<SortierObjekt> speicherObjListe = new ArrayList<SortierObjekt>();

        if (_modus == 0) {
            for (SortierObjekt speicherndesSortObj : _listeRechtsSortierObj) {
                SortierObjekt sortObjNeu = new SortierObjekt(speicherndesSortObj.getSortierWahl(), speicherndesSortObj.isAufsteigend());

                speicherObjListe.add(sortObjNeu);
            }

            _sortierungElement.getSortierungEintraege().add(new SortierungEintrag(datId, _listeLinks, _listeRechts, speicherObjListe));
        } else if (_modus == 1) {
            _sortierungElement.getSortierungEintraege().add(new SortierungEintrag(datId, _listeLinks, _listeRechts));
        }

        speichereMapInXML();
        _guiDatenEqualGUIBereich = true;
        setzeGUIbereichMitElementDaten(_sortierungElement.getSortierungName());
    }

    /**
     * Speichere das Map der Sortierungen.
     */
    private void speichereMapInXML() {
        _alleSortierungenSpaltenInstanz.speichereAlleSortierungen();
    }

    /**
     * Element wird in Array arSortVariable nach elementStart verschoben.
     *
     * @param element Element, das verschoben werden soll.
     * @param elementStart Element, nach dessen Position das Element verschoben wird.
     */
    private void verschiebeElement(String element, String elementStart) {
        int      startPosition   = -1;
        int      elementPosition = -1;
        String[] arSortVariable  = _listeRechts.toArray(new String[_listeRechts.size()]);
        int      groesse         = arSortVariable.length;
        int      i               = 0;

        while (i < groesse) {
            if (arSortVariable[i].equals(element)) {
                elementPosition = i;
            } else if (arSortVariable[i].equals(elementStart)) {
                startPosition = i;
            }

            i++;
        }

        if ((startPosition > -1) && (elementPosition > -1)) {
            if (startPosition < elementPosition) {
                int j;

                for (j = elementPosition; j > startPosition + 1; j--) {
                    arSortVariable[j] = arSortVariable[j - 1];
                }

                arSortVariable[j] = element;
            } else if (startPosition > elementPosition) {
                int j;

                for (j = elementPosition; j < startPosition; j++) {
                    arSortVariable[j] = arSortVariable[j + 1];
                }

                arSortVariable[j] = element;
            }
        }

        List<String> ar = Arrays.asList(arSortVariable);

        _listeRechts.clear();
        _listeRechts.addAll(ar);
        _guiDatenEqualGUIBereich = false;
        aenderungErfolgt();
    }

    //~ INNERE KLASSEN ========================================================

    /**
     * ContentProvider fr die Sortierdarstellung.
     *
     * @author inovat
     * @author Dagmar Rostek (DR)
     * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
     */
    private class SortierungContentProvider implements IStructuredContentProvider {
        @Override
        public void dispose() {}

        //~ GET METHODEN ======================================================

        @Override
        public Object[] getElements(Object inputElement) {
            if (inputElement instanceof List) {
                List<SortierObjekt> liste = (List<SortierObjekt>) inputElement;
                SortierObjekt[]     arr   = new SortierObjekt[liste.size()];

                for (int i = 0; i < liste.size(); i++) {
                    arr[i] = liste.get(i);
                }

                return arr;
            }

            return null;
        }

        //~ METHODEN ==========================================================

        @Override
        public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}
    }


    /**
     *
     * LabelProvider fr die Sortierung.
     *
     * @author inovat
     * @author Dagmar Rostek (DR)
     * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
     */
    private class SortierungLabelProvider implements ITableLabelProvider {
        @Override
        public void addListener(ILabelProviderListener listener) {}

        @Override
        public void dispose() {}

        //~ GET METHODEN ======================================================

        @Override
        public Image getColumnImage(Object element, int columnIndex) {
            if (element instanceof SortierObjekt) {
                if (columnIndex == 0) {
                    SortierObjekt sortierObj = (SortierObjekt) element;

                    if (sortierObj.isAufsteigend()) {
                        return ICON_VEW_PFEIL_HOCH;
                    } else {
                        return ICON_VEW_PFEIL_RUNTER;
                    }
                }
            }

            return null;
        }

        @Override
        public String getColumnText(Object element, int columnIndex) {
            if (element instanceof SortierObjekt) {
                if (columnIndex == 1) {
                    SortierObjekt sortierObj = (SortierObjekt) element;

                    return sortierObj.getSortierWahl();
                }
            }

            return null;
        }

        @Override
        public boolean isLabelProperty(Object element, String property) {
            return false;
        }

        //~ METHODEN ==========================================================

        @Override
        public void removeListener(ILabelProviderListener listener) {}
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 16.10.13
