/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.sortierung;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.plugin.gtm.navigation.datenidentifikation.Datenidentifikation;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

//~ KLASSEN ===================================================================

/**
 * Beschreibung eines Eintrags im SortierungObjekt.
 * Diese Eingaben werden als Sortierungskriterium benutzt.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class SortierungEintrag {

    /** Gewhlte SortierObjekte. */
    private List<SortierObjekt> _gewaehlteSortierObj = new ArrayList<SortierObjekt>();

    /** Alle Tabellezeilen. */
    private List<String> _alleZeilen = new ArrayList<String>();

    /**
     * HashMap Zeilen mit wert true oder false, wobei true fr aufsteigend steht, false fr absteigend. Diese
     * Map wird bei der Sortierung der Tabelle verwendet.
     */
    private Map<String, Boolean> _sortierAngabeAlleZeilen = new HashMap<String, Boolean>();

    /** Datenidentifikation des Filtereintrages. */
    private Datenidentifikation _datenidentifikation;

    /** Gewhlte Tabellenzeilen. */
    private List<String> _gewaehlteZeilen;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * WICHTIG_: Die Methode darf nicht aufgerufen werden. Die wird von XMLDecoder bentigt.
     *
     * Erzeugt eine Instanz der Klasse SortierungEintrag.
     */
    public SortierungEintrag() {}

    /**
     *
     * Erzeugt eine Instanz der Klasse SortierungEintrag.
     *
     * @param datId Datenidentifikation.
     * @param alleZeilen alle Zeilen.
     * @param gewaehlteZeilen   gewhlte Zeilen.
     */
    public SortierungEintrag(Datenidentifikation datId, List<String> alleZeilen, List<String> gewaehlteZeilen) {
        _gewaehlteZeilen     = new ArrayList<String>();
        _datenidentifikation = datId;
        _alleZeilen          = alleZeilen;
        _gewaehlteZeilen     = gewaehlteZeilen;
    }

    /**
     * Erzeugt eine Instanz der Klasse Sortiereintrag.
     * @param datId Datenidentifikation.
     * @param alleZeilen alle Zeilen.
     * @param gewaehlteZeilen   gewhlte Zeilen.
     * @param listeSortierObj Liste der Sortierobjekte.
     */
    public SortierungEintrag(Datenidentifikation datId, List<String> alleZeilen, List<String> gewaehlteZeilen, List<SortierObjekt> listeSortierObj) {
        _gewaehlteZeilen     = new ArrayList<String>();
        _datenidentifikation = datId;
        _alleZeilen          = alleZeilen;
        _gewaehlteZeilen     = gewaehlteZeilen;
        _gewaehlteSortierObj = listeSortierObj;
    }

    //~ METHODEN ==============================================================

    /*
     *  (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }

        if (obj == null) {
            return false;
        }

        if (!(obj instanceof SortierungEintrag)) {
            return false;
        }

        SortierungEintrag other = (SortierungEintrag) obj;

        if (_datenidentifikation == null) {
            if (other._datenidentifikation != null) {
                return false;
            }
        } else if (!_datenidentifikation.equals(other._datenidentifikation)) {
            return false;
        }

        if (_gewaehlteZeilen == null) {
            if (other._gewaehlteZeilen != null) {
                return false;
            }
        } else if (!_gewaehlteZeilen.equals(other._gewaehlteZeilen)) {
            return false;
        }

        if (_gewaehlteSortierObj == null) {
            if (other._gewaehlteSortierObj != null) {
                return false;
            }
        } else if (!_gewaehlteSortierObj.equals(other._gewaehlteSortierObj)) {
            return false;
        }

        if (_sortierAngabeAlleZeilen == null) {
            if (other._sortierAngabeAlleZeilen != null) {
                return false;
            }
        } else if (!_sortierAngabeAlleZeilen.equals(other._sortierAngabeAlleZeilen)) {
            return false;
        }

        return true;
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'alleZeilen'.
     *
     * @return Wert der Eigenschaft 'alleZeilen'.
     *
     * @see #_alleZeilen
     */
    public List<String> getAlleZeilen() {
        return _alleZeilen;
    }

    /**
     * Getter fr Eigenschaft 'datenidentifikation'.
     *
     * @return Wert der Eigenschaft 'datenidentifikation'.
     *
     * @see #_datenidentifikation
     */
    public Datenidentifikation getDatenidentifikation() {
        return _datenidentifikation;
    }

    /**
     *     Getter fr Eigenschaft 'gewhlteSortierObj'.
     *    
     *     @return Wert der Eigenschaft 'gewhlteSortierObj'.
     *    
     *     @see #_gewaehlteSortierObj
     */
    public List<SortierObjekt> getGewaehlteSortierObj() {
        return _gewaehlteSortierObj;
    }

    /**
     * Getter fr Eigenschaft 'gewaehlteZeilen'.
     *
     * @return Wert der Eigenschaft 'gewaehlteZeilen'.
     *
     * @see #_gewaehlteZeilen
     */
    public List<String> getGewaehlteZeilen() {
        return _gewaehlteZeilen;
    }

    /**
     * Getter fr Eigenschaft 'sortierAngabeAlleZeilen'.
     *
     * @return Wert der Eigenschaft 'sortierAngabeAlleZeilen'.
     *
     * @see #_sortierAngabeAlleZeilen
     */
    public Map<String, Boolean> getSortierAngabeAlleZeilen() {
        return _sortierAngabeAlleZeilen;
    }

    //~ METHODEN ==============================================================

    /*
     *  (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        final int prime  = 31;
        int       result = 1;

        result = prime * result + ((_alleZeilen == null) ? 0 : _alleZeilen.hashCode());
        result = prime * result + ((_datenidentifikation == null) ? 0 : _datenidentifikation.hashCode());
        result = prime * result + ((_gewaehlteZeilen == null) ? 0 : _gewaehlteZeilen.hashCode());
        result = prime * result + ((_gewaehlteSortierObj == null) ? 0 : _gewaehlteSortierObj.hashCode());
        result = prime * result + ((_sortierAngabeAlleZeilen == null) ? 0 : _sortierAngabeAlleZeilen.hashCode());

        return result;
    }

    //~ SET METHODEN ==========================================================

    /**
     * Setter fr Eigenschaft 'alleZeilen'.
     *
     * @param alleZeilen Wert zum Setzen der Eigenschaft 'alleZeilen'.
     *  WICHTIG! Verwendung fr die Speicherung im XML.
     * @see #_alleZeilen
     */
    public void setAlleZeilen(List<String> alleZeilen) {
        _alleZeilen = alleZeilen;
    }

    /**
     * Setter fr Eigenschaft 'datenidentifikation'.
     *
     * @param datenidentifikation Wert zum Setzen der Eigenschaft 'datenidentifikation'.
     *  WICHTIG! Verwendung fr die Speicherung im XML.
     * @see #_datenidentifikation
     */
    public void setDatenidentifikation(Datenidentifikation datenidentifikation) {
        _datenidentifikation = datenidentifikation;
    }

    /**
     *     Setter fr Eigenschaft 'gewaehlteSortierObj'.
     *    
     *     @param gewaehlteSortierObj Wert zum Setzen der Eigenschaft 'gewaehlteSortierObj'.
     *    
     *     @see #_gewaehlteSortierObj
     */
    public void setGewaehlteSortierObj(List<SortierObjekt> gewaehlteSortierObj) {
        _gewaehlteSortierObj = gewaehlteSortierObj;
    }

    /**
     * Setter fr Eigenschaft 'gewaehlteZeilen'.
     *
     * @param gewaehlteZeilen Wert zum Setzen der Eigenschaft 'gewaehlteZeilen'.
     *  WICHTIG! Verwendung fr die Speicherung im XML.
     * @see #_gewaehlteZeilen
     */
    public void setGewaehlteZeilen(List<String> gewaehlteZeilen) {
        _gewaehlteZeilen = gewaehlteZeilen;
    }

    /**
     * Setter fr Eigenschaft 'sortierAngabeAlleZeilen'.
     *
     * @param sortierAngabeAlleZeilen Wert zum Setzen der Eigenschaft 'sortierAngabeAlleZeilen'.
     *
     * @see #_sortierAngabeAlleZeilen
     */
    public void setSortierAngabeAlleZeilen(Map<String, Boolean> sortierAngabeAlleZeilen) {
        _sortierAngabeAlleZeilen = sortierAngabeAlleZeilen;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 03.09.10
