/*
 * @(#)GanglinienGrafik.java   17.09.10
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.visualisierung.ganglinien;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.plugin.gtm.visualisierung.Activator;
import de.inovat.buv.projektlib.konstanten.KonstantenGUIResource;

import org.eclipse.birt.chart.device.IDeviceRenderer;
import org.eclipse.birt.chart.exception.ChartException;
import org.eclipse.birt.chart.factory.GeneratedChartState;
import org.eclipse.birt.chart.factory.Generator;
import org.eclipse.birt.chart.model.attribute.Anchor;
import org.eclipse.birt.chart.model.attribute.AxisType;
import org.eclipse.birt.chart.model.attribute.Bounds;
import org.eclipse.birt.chart.model.attribute.ColorDefinition;
import org.eclipse.birt.chart.model.attribute.impl.BoundsImpl;
import org.eclipse.birt.chart.model.attribute.impl.ColorDefinitionImpl;
import org.eclipse.birt.chart.model.attribute.impl.JavaNumberFormatSpecifierImpl;
import org.eclipse.birt.chart.model.attribute.IntersectionType;
import org.eclipse.birt.chart.model.attribute.LineAttributes;
import org.eclipse.birt.chart.model.attribute.LineStyle;
import org.eclipse.birt.chart.model.attribute.Position;
import org.eclipse.birt.chart.model.attribute.TickStyle;
import org.eclipse.birt.chart.model.Chart;
import org.eclipse.birt.chart.model.ChartWithAxes;
import org.eclipse.birt.chart.model.component.Axis;
import org.eclipse.birt.chart.model.component.impl.SeriesImpl;
import org.eclipse.birt.chart.model.component.Scale;
import org.eclipse.birt.chart.model.component.Series;
import org.eclipse.birt.chart.model.data.impl.NumberDataElementImpl;
import org.eclipse.birt.chart.model.data.impl.NumberDataSetImpl;
import org.eclipse.birt.chart.model.data.impl.SeriesDefinitionImpl;
import org.eclipse.birt.chart.model.data.NumberDataSet;
import org.eclipse.birt.chart.model.data.SeriesDefinition;
import org.eclipse.birt.chart.model.impl.ChartWithAxesImpl;
import org.eclipse.birt.chart.model.layout.Legend;
import org.eclipse.birt.chart.model.type.impl.LineSeriesImpl;
import org.eclipse.birt.chart.model.type.LineSeries;
import org.eclipse.birt.chart.util.PluginSettings;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;

//~ JDK IMPORTE ===============================================================

import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Klasse erstellt die Grafik fr die Sicht 'Ganglinien'.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 88 $ / $Date: 2012-03-09 16:20:38 +0100 (Fr, 09 Mrz 2012) $ / ($Author: LG $)
 */
public class GanglinienGrafik extends Canvas {
    private static final String          FONT_NAME           = "Microsoft Sans Serif";
    private static final ColorDefinition FARBE_MITTEL_GRAU   = ColorDefinitionImpl.create(160, 160, 160);
    private static final ColorDefinition FARBE_DUNKEL_GRAU   = ColorDefinitionImpl.create(127, 127, 127);
    private static ColorDefinition[]     AR_FARBE_GANGLINIEN = {  //
        ColorDefinitionImpl.create(0, 255, 128),                  //
        ColorDefinitionImpl.create(0, 128, 255),                  //
        ColorDefinitionImpl.create(255, 128, 0),                  //
        ColorDefinitionImpl.create(172, 89, 255),                 //
        ColorDefinitionImpl.create(0, 128, 0),                    //
        ColorDefinitionImpl.create(128, 0, 0),                    //
        ColorDefinitionImpl.create(0, 0, 128),                    //
    };

    //~ FELDER ================================================================

    private IDeviceRenderer     _render = null;
    private GeneratedChartState _state  = null;

    /** X-Achse Zeitraum. */
    private Axis _xAxis = null;

    /** Y-Achse QKfz- und QSgv-Werte. */
    private Axis _yAxisQ = null;

    /** Grafik. */
    private Chart _grafik;

    /** Instanz GUI-Verwaltung. */
    private GanglinienGUIVew _guiVew;

    /** Image, wo die Grafik gespeichert wird. */
    private Image _image;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse GanglinienGrafik.
     *
     * @param parent Parent-Composite.
     * @param guiVew GUI-Verwaltung.
     */
    public GanglinienGrafik(Composite parent, GanglinienGUIVew guiVew) {
        super(parent, SWT.NO_BACKGROUND);
        setBackground(KonstantenGUIResource.FARBE_WEIS);

        //
        _guiVew = guiVew;

        //
        initRenderer();

        //
        build();

        //
        addPaintListener(new PaintListener() {
            @Override
            public void paintControl(PaintEvent e) {
                final GC gc = e.gc;

                gc.fillRectangle(0, 0, 1800, 800);

                try {
                    _render.setProperty(IDeviceRenderer.GRAPHICS_CONTEXT, gc);

                    final Point  size2      = getSize();
                    final Bounds bo         = BoundsImpl.create(0, 0, size2.x, size2.y);
                    final int    resolution = _render.getDisplayServer().getDpiResolution();

                    bo.scale(72d / resolution);

                    final Generator gr = Generator.instance();

                    _state = gr.build(_render.getDisplayServer(), _grafik, bo, null, null, null);
                    gr.render(_render, _state);

                    // Erstelle Image:
                    if (_image != null) {
                        _image.dispose();
                    }

                    _image = new Image(null, size2.x, size2.y);
                    gc.copyArea(_image, 0, 0);

                    // Variante: in der Datei speichern:
                    // speichereInDatei();
                }
                catch (final ChartException e1) {

                    // e1.printStackTrace();
                    System.err.println(e1);
                    gc.drawText("Bitte die Ganglinienangaben selektieren und auf 'Aktualisieren' drcken !", 40, 40);
                }
            }
        });
    }

    //~ METHODEN ==============================================================

    /**
     * Baue die Grafik.
     */
    public final void build() {
        createChart();
        buildPlot();
        buildLegend();
        buildTitle();
        buildXAxis();
        buildYAxis();
        buildXSeries();
        buildYSeries();
    }

    /**
     * Erstelle die Legende.
     */
    private void buildLegend() {
        final Legend lg = _grafik.getLegend();

        lg.setVisible(true);

        //
        _grafik.getLegend().setTitlePosition(Position.ABOVE_LITERAL);
        _grafik.getLegend().getTitle().setVisible(true);
        _grafik.getLegend().getTitle().getCaption().setValue(_guiVew.ermittleLegendenTitel());
        _grafik.getLegend().getTitle().getCaption().getFont().setSize(12);

        //
        _grafik.getLegend().getText().getFont().setSize(8);

        //
        _grafik.getLegend().setAnchor(Anchor.NORTH_LITERAL);
    }

    /**
     * Erstelle Grundriss der Grafik.
     */
    private void buildPlot() {
        _grafik.getPlot().getClientArea().getOutline().setVisible(true);
        _grafik.getPlot().getClientArea().getOutline().setColor(FARBE_MITTEL_GRAU.copyInstance());
    }

    /**
     * Erstelle den Titel.
     */
    private void buildTitle() {
        _grafik.getTitle().setVisible(false);
    }

    /**
     * Erstelle die X-Achse.
     */
    private void buildXAxis() {
        _xAxis = ((ChartWithAxes) _grafik).getPrimaryBaseAxes()[0];

        //
        _xAxis.getTitle().setVisible(true);

        if (_guiVew.getGanglinienArtBerechnet().istTag()) {
            _xAxis.getTitle().getCaption().setValue("Zeit");
        } else {
            _xAxis.getTitle().getCaption().setValue("Datum");
        }

        _xAxis.getTitle().getCaption().getFont().setBold(false);
        _xAxis.getTitle().getCaption().getFont().setSize(8);
        _xAxis.getTitle().getCaption().getFont().setName(FONT_NAME);

        //
        _xAxis.getLabel().setVisible(true);
        _xAxis.getLabel().getCaption().getFont().setRotation(90);
        _xAxis.getLabel().getCaption().getFont().setSize(8);
        _xAxis.getLabel().getCaption().getFont().setName(FONT_NAME);

        //
        _xAxis.getMajorGrid().getLineAttributes().setVisible(true);
        _xAxis.getMajorGrid().getLineAttributes().setStyle(LineStyle.SOLID_LITERAL);
        _xAxis.getMajorGrid().getLineAttributes().setColor(FARBE_MITTEL_GRAU.copyInstance());
        _xAxis.getMajorGrid().setTickStyle(TickStyle.ACROSS_LITERAL);
        _xAxis.getMajorGrid().getTickAttributes().setColor(FARBE_DUNKEL_GRAU.copyInstance());

        // _xAxis.setType(AxisType.LINEAR_LITERAL); <--- funktioniert nicht
        _xAxis.setType(AxisType.TEXT_LITERAL);
        _xAxis.getOrigin().setType(IntersectionType.MIN_LITERAL);

        //
        _xAxis.setFormatSpecifier(new MyZeitJavaNumberFormatSpecifierImpl(_guiVew.getGanglinienArtBerechnet()));

        //
        final Scale scale = _xAxis.getScale();

        scale.setStepNumber(_guiVew.ermittleSchrittXSkala());
        scale.setMajorGridsStepNumber(_guiVew.ermittleSchrittHauptlinienXSkala());
        scale.setTickBetweenCategories(false);
    }

    /**
     * Definiere den Inhalt der X-Achse.
     */
    private void buildXSeries() {
        final NumberDataSet dataSetX = NumberDataSetImpl.create(_guiVew.getListeZeitPunkte());
        final Series        serieX   = SeriesImpl.create();

        serieX.setDataSet(dataSetX);

        final SeriesDefinition sdX = SeriesDefinitionImpl.create();

        sdX.getSeries().add(serieX);
        _xAxis.getSeriesDefinitions().add(sdX);
    }

    /**
     * Erstelle die Y-Achse.
     */
    private void buildYAxis() {

        // ******************************
        // Y-Achse Q fr die Angabe 'Fahrzeuge pro Stunde':
        _yAxisQ = ((ChartWithAxes) _grafik).getPrimaryOrthogonalAxis(_xAxis);

        //
        _yAxisQ.getTitle().setVisible(true);

        if (_guiVew.getGanglinienArtBerechnet().istTag()) {
            _yAxisQ.getTitle().getCaption().setValue("Fz/h");
        } else {
            _yAxisQ.getTitle().getCaption().setValue("Fz/Tag");
        }

        _yAxisQ.getTitle().getCaption().getFont().setBold(false);
        _yAxisQ.getTitle().getCaption().getFont().setRotation(90);
        _yAxisQ.getTitle().getCaption().getFont().setSize(8);
        _yAxisQ.getTitle().getCaption().getFont().setName(FONT_NAME);

        //
        _yAxisQ.getLabel().setVisible(true);
        _yAxisQ.getLabel().getCaption().getFont().setSize(8);
        _yAxisQ.getLabel().getCaption().getFont().setName(FONT_NAME);

        //
        _yAxisQ.getMajorGrid().getLineAttributes().setVisible(true);
        _yAxisQ.getMajorGrid().getLineAttributes().setStyle(LineStyle.SOLID_LITERAL);
        _yAxisQ.getMajorGrid().getLineAttributes().setColor(FARBE_MITTEL_GRAU.copyInstance());
        _yAxisQ.getMajorGrid().setTickStyle(TickStyle.ACROSS_LITERAL);
        _yAxisQ.getMajorGrid().getTickAttributes().setColor(FARBE_DUNKEL_GRAU.copyInstance());

        //
        _yAxisQ.setType(AxisType.LINEAR_LITERAL);

        //
        _yAxisQ.setTitlePosition(Position.LEFT_LITERAL);
        _yAxisQ.getOrigin().setType(IntersectionType.MIN_LITERAL);
        _yAxisQ.setLabelPosition(Position.LEFT_LITERAL);

        //
        _yAxisQ.setFormatSpecifier(JavaNumberFormatSpecifierImpl.create("###,###"));

        //
        final Scale scaleQ = _yAxisQ.getScale();
        int         min    = _guiVew.ermittleQMin();
        int         max    = _guiVew.ermittleQMax();

        if (min > -1) {
            scaleQ.setMin(NumberDataElementImpl.create(min));
        }

        if (max > -1) {
            scaleQ.setMax(NumberDataElementImpl.create(max));
        }
    }

    /**
     * Definiere den Inhalt der Y-Achsen.
     */
    private void buildYSeries() {

        // *************************************************************************************
        // Y-Achse QFz
        // *************************************************************************************
        SeriesDefinition serDefQ = SeriesDefinitionImpl.create();

        _yAxisQ.getSeriesDefinitions().add(serDefQ);

        // *************************************************************************************
        // Ganglinien Q:
        for (int q = 0; q < 2; q++) {
            boolean             zeigen;
            String              qText;
            List<List<Integer>> listenQ;

            // Q-KFZ:
            if (q == 0) {
                zeigen  = _guiVew.istQKfzSelektiert();
                listenQ = _guiVew.getListenQKfz();
                qText   = "Q KFZ ";
            }

            // Q-SGV:
            else {
                zeigen  = _guiVew.istQSgvSelektiert();
                qText   = "Q SGV ";
                listenQ = _guiVew.getListenQSgv();
            }

            if (zeigen) {
                for (int i = 0; i < GanglinienGUIVew.ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS; i++) {
                    if (_guiVew.istDarzustellenderWertSelektiert(i)) {
                        final NumberDataSet dataSetQ;

                        dataSetQ = NumberDataSetImpl.create(listenQ.get(i));

                        final LineSeries lsQ = (LineSeries) LineSeriesImpl.create();

                        lsQ.setSeriesIdentifier(qText + _guiVew.ermittleLegende(i));
                        lsQ.setDataSet(dataSetQ);
                        lsQ.setConnectMissingValue(false);
                        lsQ.getMarkers().clear();

                        final LineAttributes lineAttributesQ = lsQ.getLineAttributes();

                        lineAttributesQ.setColor(AR_FARBE_GANGLINIEN[i].copyInstance());
                        lineAttributesQ.setThickness(2);

                        if (q == 1) {
                            lineAttributesQ.setStyle(LineStyle.DASHED_LITERAL);
                        }

                        lsQ.getLabel().setVisible(false);

                        //
                        serDefQ.getSeries().add(lsQ);
                    }
                }
            }
        }
    }

    /**
     * Erstelle Grafik-Instanz.
     */
    private void createChart() {
        _grafik = ChartWithAxesImpl.create();

        //
        ChartWithAxes g = (ChartWithAxes) _grafik;

        g.setUnitSpacing(69);
    }

    //~ GET METHODEN ==========================================================

    /**
     *     Getter fr Eigenschaft 'image'.
     *
     *     @return Wert der Eigenschaft 'image'.
     *
     *     @see #_image
     */
    public Image getImage() {
        return _image;
    }

    //~ METHODEN ==============================================================

    /**
     * Initialisiere Renderer.
     */
    private void initRenderer() {
        final PluginSettings ps = PluginSettings.instance();

        try {
            _render = ps.getDevice("dv.SWT");
        }
        catch (ChartException e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }
    }

    /**
     * Speichere die Grafik in die Datei.
     */
    private void speichereInDatei() {

        // In der Datei speichern:

        /* zustzliche Birt-Plugins in plugin.xml einfgen!
        try {

                final PluginSettings ps = PluginSettings.instance();
                IDeviceRenderer renderImg = ps.getDevice("dv.PNG");


                Bounds bo = BoundsImpl.create(0, 0, 800, 600);

            final Generator gr = Generator.instance();

            GeneratedChartState  state = gr.build(renderImg.getDisplayServer(), _grafik, null, bo, null);



            java.awt.Image img = new BufferedImage(

                        (int) bo.getWidth(), (int) bo.getHeight(), BufferedImage.TYPE_INT_ARGB

                        );

            renderImg.setProperty(IDeviceRenderer.FILE_IDENTIFIER, "chartProbe.png");

            gr.render(renderImg, state);




                } catch (Exception e) {
                        System.err.println(e);
                }
                */
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 30.10.13
