/*
 * Copyright (c) 2010 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.buv.plugin.gtm.visualisierung.karte;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.buv.plugin.selektion.SelektionsTransfer;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;

import de.inovat.buv.gtm.datvew.lib.BaumHilfsFunktionen;
import de.inovat.buv.gtm.datvew.lib.DatenVew;
import de.inovat.buv.gtm.datvew.lib.Fahrbahnen;
import de.inovat.buv.gtm.datvew.lib.Messquerschnitt;
import de.inovat.buv.gtm.datvew.lib.ZaehlStelle;
import de.inovat.buv.gtm.datvew.tls.De;
import de.inovat.buv.gtm.datvew.tls.Geraet;
import de.inovat.buv.gtm.datvew.tls.Kri2;
import de.inovat.buv.gtm.datvew.tls.Kri2Vew;
import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.inovat.lib.funktionen.datum.DatumFunktionen;
import de.inovat.buv.inovat.lib.io.CsvLeser;
import de.inovat.buv.plugin.gtm.navigation.tabellenverwaltung.AlleTabellenVerwaltungen;
import de.inovat.buv.plugin.gtm.visualisierung.Activator;
import de.inovat.buv.projektlib.bundesland.BundeslandVew;
import de.inovat.buv.projektlib.dav.DavDatenVew;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.internal.misc.StringMatcher;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;

import static de.inovat.buv.inovat.lib.konstanten.Konstanten.FILESEPARATOR;
import static de.inovat.buv.inovat.lib.konstanten.Konstanten.NEWLINE;
import static de.inovat.buv.projektlib.konstanten.KonstantenGUIResource.FARBE_BLAU;
import static de.inovat.buv.projektlib.konstanten.KonstantenGUIResource.FARBE_GELB;
import static de.inovat.buv.projektlib.konstanten.KonstantenGUIResource.FARBE_GRAU;
import static de.inovat.buv.projektlib.konstanten.KonstantenGUIResource.FARBE_GRUEN;
import static de.inovat.buv.projektlib.konstanten.KonstantenGUIResource.FARBE_GRUEN_DUNKEL2;
import static de.inovat.buv.projektlib.konstanten.KonstantenGUIResource.FARBE_MAGENDA;
import static de.inovat.buv.projektlib.konstanten.KonstantenGUIResource.FARBE_ORANGE;
import static de.inovat.buv.projektlib.konstanten.KonstantenGUIResource.FARBE_ROT;
import static de.inovat.buv.projektlib.konstanten.KonstantenGUIResource.FARBE_SCHWARZ;
import static de.inovat.buv.projektlib.konstanten.KonstantenGUIResource.FARBE_VIOLET;

//~ JDK IMPORTE ===============================================================

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

//~ KLASSEN ===================================================================

/**
 * Klasse verwaltet das GUI 'bersichtskarte'.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 407 $ / $Date: 2013-08-01 13:20:45 +0200 (Do, 01 Aug 2013) $ / ($Author: LG $)
 */
public class KarteGUIVew {

    /** Werte fr den mglichen Mastab der Karte. */
    private static final float[] AR_MASSSTAB = new float[] { 0.5f, 1, 1.5f, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 20 };

    /** Die Beschriftungen der Radiobuttons fr den Mastab. */
    public static final String[] AR_MASSSTAB_NAMEN = new String[] { "50%", "100%", "150%", "200%", "300%", "400%", "500%", "600%", "700%", "800%", "900%", "1000%", "1100%", "2000%" };

    /**
     * Array definiert fr jeden Mastab aus AR_MASSSTAB die Breite der Fahrstreifen-Linie.
     */
    private static final int[] AR_MASSTAB_FS_LINIE_BRIETE = new int[] { 3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 7, 7, 7, 7 };

    /**
     * Array definiert fr jeden Mastab aus AR_MASSSTAB die Lnge der Fahrstreifen-Linie.
     */
    private static final int[] AR_MASSTAB_FS_LINIE_LAENGE = new int[] { 12, 12, 12, 12, 12, 12, 20, 20, 20, 20, 30, 30, 30, 30 };

    /**
     * Array definiert fr jeden Mastab aus AR_MASSSTAB das zugehrige Bild
     * (Index) aus AR_IMAGE_KARTE.
     */
    private static final int[] AR_MASSTAB_IMAGE_INDEX = new int[] { 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2 };

    /**
     * Array der originalen Bilder fr die Kartendarstellung.
     */
    private static final Image[] AR_IMAGE_KARTE = new Image[] {                                                    //
        new Image(null, BundeslandVew.getInstanz().ermittleDateiImOrdnerBundeslandResource("Karte/karte_1.bmp")),  //
        new Image(null, BundeslandVew.getInstanz().ermittleDateiImOrdnerBundeslandResource("Karte/karte_2.bmp")),  //
        new Image(null, BundeslandVew.getInstanz().ermittleDateiImOrdnerBundeslandResource("Karte/karte_3.bmp"))   //
    };

    /** Hhe der Zeichnung in Pixel fr Mastab == 100%. */
    private final static int KARTE_HOEHE_IN_PIXEL = 500;

    /** Status-Text fr die Fahrstreifen ohne Quelle. */
    private static final String STATUS_FAHRSTREIFE_OHHNE_DE = "nicht ermittelbar";

    /**
     * Farbe fr die Begrenzung der Zhlstellen.
     */
    private static final Color FARBE_BEGRENZUNG_ZS = FARBE_SCHWARZ;

    /**
     * Farbe fr die Begrenzung der Kri2b.
     */
    private static final Color FARBE_BEGRENZUNG_KRI2B = FARBE_GRUEN_DUNKEL2;

    /**
     * Farbe fr die Begrenzung der Kri2_b.
     */
    private static final Color FARBE_BEGRENZUNG_KRI2_B = FARBE_BLAU;

    /** Breite der Zeichnungsrahmen in Pixel. */
    private static final int ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL = 10;

    /**
     * Anzahl der X-UTM-Koordinaten in einem Pixel fr
     * jedes Bild aus AR_IMAGE_KARTE.
     */
    private static double[] AR_ANZAHL_X_UTM_PRO_PIXEL;

    /**
     * Anzahl der X-WGS84-Koordinaten in einem Pixel fr
     * jedes Bild aus AR_IMAGE_KARTE.
     */
    private static double[] AR_ANZAHL_X_WGS84_PRO_PIXEL;

    /**
     * Anzahl der Y-UTM-Koordinaten in einem Pixel fr
     * jedes Bild aus AR_IMAGE_KARTE.
     */
    private static double[] AR_ANZAHL_Y_UTM_PRO_PIXEL;

    /**
     * Anzahl der Y-WGS84-Koordinaten in einem Pixel fr
     * jedes Bild aus AR_IMAGE_KARTE.
     */
    private static double[] AR_ANZAHL_Y_WGS84_PRO_PIXEL;

    /**
     * Array UTM-Koordinaten zu jedem Bild aus AR_IMAGE_KARTE.
     *
     * Kartengre wird durch 2 Punkten bestimmt:
     * linken unteren und rechten oberen.
     */
    private static double[][] AR_IMAGE_KARTE_GROESSE_IN_UTM;

    /**
     * Array WGS84-Koordinaten zu jedem Bild aus AR_IMAGE_KARTE.
     *
     * Kartengre wird durch 2 Punkten bestimmt:
     * linken unteren und rechten oberen.
     */
    private static double[][] AR_IMAGE_KARTE_GROESSE_IN_WGS84;

    /**
     * Array der KartenLnge in Pixel fr Mastab == 100%
     * fr jedes Bild aus AR_IMAGE_KARTE.
     */
    private static int[] AR_KARTE_LAENGE_IN_PIXEL;

    /**
     * TestPunkte fr das Kalibrieren.
     * Die Koordinate von zwei TestPunkten, um die Positionierung auf der Karte zu prfen.
     */
    private static double[] AR_TEST_PUNKTE_IN_WGS84;

    //~ FELDER ================================================================

    /** X-WGS84Dec-Koordinate der letzte Positionierung des Kreuzes in der Karte. */
    private double _xKreuzWgs84 = -1;

    /** Y-WGS84Dec-Koordinate der letzte Positionierung des Kreuzes in der Karte. */
    private double _yKreuzWgs84 = -1;

    /** Kri2-Verwaltung-Instanz. */
    private Kri2Vew _kri2Vew = Kri2Vew.getInstanz();

    /** Stellt fest, ob die Karte gescrollt werden soll. */
    private boolean _istScrollen = false;

    /**
     * Array aller ZhlStellen_Namen plus leeres String
     * fr das Initialisieren von Combo 'Zhlstellen' ohne Filter.
     */
    private String[] _arComboZsAlleElemente;

    /**
     * Array aller ZhlStellen-Tooltip (Name (PID)) plus leeres String
     * fr das Initialisieren von Combo 'Zhlstellen' ohne Filter.
     */
    private String[] _arComboZsAlleElementeToolTip;

    /** Datenverteilerverbindung. */
    private ClientDavInterface _dav;

    /** Die Datenverteiler-Konfiguration. */
    private DataModel _davKonf;

    /** GUI-Instanz. */
    private KarteGUI _gui;

    /** Index des selektierten Mastab der Karte-Zeichnung. */
    private int _indexMassstab;

    /** Kri2_b, die sich in der Mausposition befindet, oder null. */
    private Kri2 _kri2_bInMausPostion;

    /** Kri2b, die sich in der Mausposition befindet, oder null. */
    private Kri2 _kri2bInMausPostion;

    /** Alle Systemobjekte vom Typ De. */
    private List<SystemObject> _listeDeSystemObj;

    /**
     * Liste der KartenImage fr jeden Mastab aus AR_MASSSTAB.
     */
    private List<Image> _listeKartenImageProMassstab;

    /**
     * Liste enthlt fr jedes Image aus AR_IMAGE_KARTE
     * die Liste der Pixel Koordinaten aller Kri2_b
     * beim Mastab==1 ohne die Bercksichtigung der ZeichnungRahmen.
     */
    private List<List<int[]>> _listePixelKoordAllerKri2_bFuerImage;

    /**
     * Liste enthlt fr jedes Image aus AR_IMAGE_KARTE
     * die Liste der Pixel Koordinaten aller Kri2b
     * beim Mastab==1 ohne die Bercksichtigung der ZeichnungRahmen.
     */
    private List<List<int[]>> _listePixelKoordAllerKri2bFuerImage;

    /**
     * Liste enthlt fr jedes Image aus AR_IMAGE_KARTE
     * die Liste der Pixel Koordinaten aller Zhlstellen
     * beim Mastab==1 ohne die Bercksichtigung der ZeichnungRahmen.
     */
    private List<List<int[]>> _listePixelKoordAllerZsFuerImage;

    /**
     * Liste enthlt fr jedes Image aus AR_IMAGE_KARTE
     * die Liste der Pixel Koordinaten der TestPunkten
     * beim Mastab==1 ohne die Bercksichtigung der ZeichnungRahmen.
     */
    private List<List<int[]>> _listePixelKoordTestPunktenFuerImage;

    /**
     * Menge der FehlerStatus fr alle De (Datenendgertekanle).
     * Als Key dient PID von DE.
     */
    private Map<String, DE_FEHLER_STATUS> _mapDeFehlerStatus;

    /** HTML-ToolTip. */
    private ToolTipHtml _toolTip;

    /** X-WGS84Dec-Koordinate der MausPosition in der Karte. */
    private double _xMausWgs84;

    /** Y-WGS84Dec-Koordinate der MausPosition in der Karte. */
    private double _yMausWgs84;

    /** Die ZaehlStelle, die sich in der Mausposition befindet, oder null. */
    private ZaehlStelle _zsbInMausPostion;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse KarteGUIVew.
     *
     * @param gui GUI-Instanz.
     */
    public KarteGUIVew(KarteGUI gui) {
        _gui = gui;

        // ------------------------------------------------------------------
        // Datenverteiler Zugriff holen:
        _dav     = DavDatenVew.getInstanz().getDav();
        _davKonf = _dav.getDataModel();

        // ------------------------------------------------------------------
        // Initialisiere die Karten-Daten aus Dateien:
        initialisiereArrayKartenGroessen();
        initialisiereArrayTestPunkte();

        // ------------------------------------------------------------------
        // Berechne die Korrektur Faktor fr WGS-Koordinaten und die Lnge
        // der Zeichnung in Pixel fr jedes Bild aus AR_IMAGE_KARTE:
        double laengeWGS84Dec;
        double hoeheWGS84Dec;
        int    laengeImage;
        int    hoeheImage;
        int    anz = AR_IMAGE_KARTE.length;

        AR_KARTE_LAENGE_IN_PIXEL    = new int[anz];
        AR_ANZAHL_X_WGS84_PRO_PIXEL = new double[anz];
        AR_ANZAHL_Y_WGS84_PRO_PIXEL = new double[anz];

        for (int i = 0; i < anz; i++) {
            laengeWGS84Dec = AR_IMAGE_KARTE_GROESSE_IN_WGS84[i][2] - AR_IMAGE_KARTE_GROESSE_IN_WGS84[i][0];
            hoeheWGS84Dec  = AR_IMAGE_KARTE_GROESSE_IN_WGS84[i][3] - AR_IMAGE_KARTE_GROESSE_IN_WGS84[i][1];
            laengeImage    = AR_IMAGE_KARTE[i].getBounds().width;
            hoeheImage     = AR_IMAGE_KARTE[i].getBounds().height;

            //
            AR_KARTE_LAENGE_IN_PIXEL[i]    = Math.round(1f * KARTE_HOEHE_IN_PIXEL * laengeImage / hoeheImage);
            AR_ANZAHL_Y_WGS84_PRO_PIXEL[i] = hoeheWGS84Dec / KARTE_HOEHE_IN_PIXEL;
            AR_ANZAHL_X_WGS84_PRO_PIXEL[i] = laengeWGS84Dec / AR_KARTE_LAENGE_IN_PIXEL[i];
        }

        // ------------------------------------------------------------------
        // Berechne die Korrektur Faktor fr UTM-Koordinaten und die Lnge
        // der Zeichnung in Pixel fr jedes Bild aus AR_IMAGE_KARTE:
        double laengeUTM;
        double hoeheUTM;

        AR_ANZAHL_X_UTM_PRO_PIXEL = new double[anz];
        AR_ANZAHL_Y_UTM_PRO_PIXEL = new double[anz];

        for (int i = 0; i < anz; i++) {
            laengeUTM = AR_IMAGE_KARTE_GROESSE_IN_UTM[i][2] - AR_IMAGE_KARTE_GROESSE_IN_UTM[i][0];
            hoeheUTM  = AR_IMAGE_KARTE_GROESSE_IN_UTM[i][3] - AR_IMAGE_KARTE_GROESSE_IN_UTM[i][1];

            //
            AR_ANZAHL_Y_UTM_PRO_PIXEL[i] = hoeheUTM / KARTE_HOEHE_IN_PIXEL;
            AR_ANZAHL_X_UTM_PRO_PIXEL[i] = laengeUTM / AR_KARTE_LAENGE_IN_PIXEL[i];
        }

        // ------------------------------------------------------------------
        // Initialisiere die Liste der KartenImage fr jeden Mastab:
        _listeKartenImageProMassstab = new ArrayList<Image>();

        ImageData id;
        Image     img;
        int       indexImage;

        for (int i = 0; i < AR_MASSSTAB.length; i++) {

            // Fr jeden Mastab hole das OriginalBild und skaliere das:
            indexImage = AR_MASSTAB_IMAGE_INDEX[i];
            id         = AR_IMAGE_KARTE[indexImage].getImageData();
            id         = id.scaledTo(Math.round(AR_KARTE_LAENGE_IN_PIXEL[indexImage] * AR_MASSSTAB[i]), Math.round(KARTE_HOEHE_IN_PIXEL * AR_MASSSTAB[i]));
            img        = new Image(null, id);
            _listeKartenImageProMassstab.add(img);
        }
    }

    //~ AUFZAEHLUNGSTYPEN =====================================================

    /**
     * Fehlerstatus des Datenendgertekanals (De).
     */
    private enum DE_FEHLER_STATUS {
        OK("ok", "ok", FARBE_GRUEN), STOERUNG_EAK("Strung von EAK erkannt", "StrEAK", FARBE_GELB),                             //
        STOERUNG_SM("Strung von SM erkannt", "StrSM", FARBE_ROT), KEINE_QUELLE("keine Quelle", "keineQuelle", FARBE_MAGENDA),  //
        KEINE_DATEN("keine Daten", "keineDaten", FARBE_GRAU), UNBEKANNT("-", "", FARBE_ORANGE);

        private final Color  farbe;
        private final String status;
        private final String statusDav;

        //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==========

        private DE_FEHLER_STATUS(String status, String statusDav, Color farbe) {
            this.status    = status;
            this.statusDav = statusDav;
            this.farbe     = farbe;
        }

        //~ METHODEN ==========================================================

        /**
         * Ermittle den De-FehlerStatus fr angegebenen DaV-Status.
         *
         * @param deFehlerStatusDav De-FehlerStatus in DaV.
         *
         * @return De-FehlerStatus fr angegebenen DaV-Status.
         */
        private static DE_FEHLER_STATUS ermittleStatusFuerDav(String deFehlerStatusDav) {
            for (DE_FEHLER_STATUS st : values()) {
                if (st.statusDav.equals(deFehlerStatusDav)) {
                    return st;
                }
            }

            return UNBEKANNT;
        }
    }

    //~ METHODEN ==============================================================

    /**
     * Melde Objekte, die am Datenverteiler angemeldet sind von diesem ab.
     */
    private void abmeldenDav() {
        boolean istFehler = false;

        // Definiere DataDescription:
        AttributeGroup  atg = _davKonf.getAttributeGroup("atg.tlsGloDeFehler");
        Aspect          asp = _davKonf.getAspect("asp.tlsAntwort");
        DataDescription dd  = new DataDescription(atg, asp);

        // Abmeldung:
        for (SystemObject so : _listeDeSystemObj) {
            try {
                _dav.unsubscribeReceiver(_gui, so, dd);
            }
            catch (Exception e) {
                istFehler = true;
                Log.zeigeInterneMultiMeldung(Activator.PLUGIN_ID, "Probleme beim Abmelden der Systemobejkte vom Datenverteiler.", e);
            }
        }

        // Fehlermeldung:
        if (istFehler) {
            MessageDialog.openError(_gui.getShell(), "Fehler", "Probleme beim Abmelden der Systemobejkte vom Datenverteiler.");
        }
    }

    /**
     * Melde das GUI auf die nderungen von DeFehlerStatus an.
     */
    protected void anmeldenDavFuerDeFehlerStatusUeberwachung() {

        // ------------------------------------------------------------------
        // Bestimme aller Systemobjekte vom Typ De:
        try {
            _listeDeSystemObj = _davKonf.getType("typ.de").getElements();
        }
        catch (Exception e) {
            _listeDeSystemObj = new ArrayList<SystemObject>();
        }

        // ------------------------------------------------------------------
        // Initialisiere die Menge der DeFehlerStatus:
        _mapDeFehlerStatus = new HashMap<String, DE_FEHLER_STATUS>();

        for (SystemObject so : _listeDeSystemObj) {
            _mapDeFehlerStatus.put(so.getPidOrId(), DE_FEHLER_STATUS.UNBEKANNT);
        }

        // ------------------------------------------------------------------
        // Anmelden zum Empfang der Daten:
        AttributeGroup  atg = _davKonf.getAttributeGroup("atg.tlsGloDeFehler");
        Aspect          asp = _davKonf.getAspect("asp.tlsAntwort");
        DataDescription dd  = new DataDescription(atg, asp);

        _dav.subscribeReceiver(_gui, _listeDeSystemObj, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
    }

    /**
     * Ein der CheckButtons fr die darzustellenden Elemente wird selektiert.
     */
    protected void cbtnDarzustellendeElementeSelektiert() {

        // Zeichne die Karte neu:
        zeichneKarteNeu();
    }

    /**
     * Ein der Combo 'Element zentrieren' wird selektiert.
     *
     * @param comboObjekt Das selektierte Combo.
     */
    protected void comboZentierenSelektiert(Object comboObjekt) {
        Combo             combo                                 = null;
        List<List<int[]>> listePixelKoordAllerElementeFuerImage = null;
        int               indexGefiltert                        = -1;

        if (comboObjekt == _gui.getComboZs()) {
            combo                                 = _gui.getComboZs();
            listePixelKoordAllerElementeFuerImage = _listePixelKoordAllerZsFuerImage;

            // Falls Filter fr Combo 'Zhlstelle' nicht leer ist,
            // definiere das Index des selektierten Elementes:
            if (_gui.getTxtZsFilter().getText().length() > 0) {
                String zsSelektietr = _gui.getComboZs().getText();

                indexGefiltert = 0;

                for (String zs : _arComboZsAlleElemente) {
                    if (zs.equals(zsSelektietr)) {
                        break;
                    }

                    indexGefiltert++;
                }
            }
        } else if (comboObjekt == _gui.getComboKri2_b()) {
            combo                                 = _gui.getComboKri2_b();
            listePixelKoordAllerElementeFuerImage = _listePixelKoordAllerKri2_bFuerImage;
        } else if (comboObjekt == _gui.getComboKri2b()) {
            combo                                 = _gui.getComboKri2b();
            listePixelKoordAllerElementeFuerImage = _listePixelKoordAllerKri2bFuerImage;
        }

        if (combo != null) {
            int index;

            if (indexGefiltert < 0) {
                index = combo.getSelectionIndex() - 1;  // (-1): erstes leeres ComboElement wird nicht bercksichtigt.
            } else {
                index = indexGefiltert - 1;
            }

            // ToolTip:
            if ((comboObjekt == _gui.getComboZs()) && (index >= 0)) {
                combo.setToolTipText(_arComboZsAlleElementeToolTip[index + 1]);
            } else {
                combo.setToolTipText(combo.getText());
            }

            if (index >= 0) {

                // Ermittle die PixelKoordinaten des selektierten Elements:
                int[] koord         = listePixelKoordAllerElementeFuerImage.get(AR_MASSTAB_IMAGE_INDEX[_indexMassstab]).get(index);
                float massstabKarte = AR_MASSSTAB[_indexMassstab];
                float x             = massstabKarte * koord[0] + ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL - _gui.getScompKarte().getClientArea().width / 2;
                float y             = massstabKarte * koord[1] + ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL - _gui.getScompKarte().getClientArea().height / 2;

                // Scrolle zum Element:
                _gui.getScompKarte().setOrigin(Math.round(x), Math.round(y));

                // Setze die Koordinaten fr den Kreuz:
                double[] koordWgs = umwandelePixelKoordinatenInWGS84DecKoordinaten(Math.round(massstabKarte * koord[0] + ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL), Math.round(massstabKarte * koord[1] + ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL));

                _xKreuzWgs84 = koordWgs[0];
                _yKreuzWgs84 = koordWgs[1];
            } else {

                // Lsche die Koordinaten fr den Kreuz:
                _xKreuzWgs84 = -1;
            }
        }

        zeichneKarteNeu();
    }

    /**
     * Deaktiviere alle unntigen Daten nach dem Schlieen der Sicht.
     */
    public void deaktiviereDaten() {

        // Lsche ToolTip:
        try {
            _toolTip.deactivate();
        }
        catch (Exception e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }

        // Melde Objekte, die am Datenverteiler angemeldet sind von diesem ab:
        try {
            abmeldenDav();
        }
        catch (Exception e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }

        // Lsche alle Image:
        for (Image img : _listeKartenImageProMassstab) {
            if (img != null) {
                try {
                    img.dispose();
                }
                catch (Exception e) {
                    Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
                }
            }
        }

        // Lsche doubleBufferImage:
        try {
            Image imageDb = (Image) _gui.getCanvasKarte().getData("double-buffer-image");

            if (imageDb != null) {
                imageDb.dispose();
            }
        }
        catch (Exception e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }
    }

    /**
     * Ermittle die Farbe fr De abhngig vom DeFehlerStatus.
     *
     * @param dePid PID von De, deren Farbe ermittelt werden soll
     * oder null (falls keine Quelle fr die Fahrstreife vorhanden ist).
     *
     * @return Ermittle die Farbe fr De abhngig vom DeFehlerStatus.
     */
    private Color ermittleDeFarbe(String dePid) {

        // Farbe fr die PID null (d.h. keine Quelle fr die Fahrstreife vorhanden ist):
        if (dePid == null) {
            return FARBE_VIOLET;
        }

        //
        DE_FEHLER_STATUS deFs = _mapDeFehlerStatus.get(dePid);

        if (deFs != null) {
            return deFs.farbe;
        } else {

            // Falls kein Status gefunden wird (soll aber niemals passieren):
            return FARBE_VIOLET;
        }
    }

    /**
     * Ermittle den Status fr De abhngig vom DeFehlerStatus.
     *
     * @param dePid PID von De, deren Status ermittelt werden soll
     * oder null (falls keine Quelle fr die Fahrstreife vorhanden ist).
     *
     * @return Ermittle den Status fr De abhngig vom DeFehlerStatus.
     */
    private String ermittleDeStatus(String dePid) {

        // Status fr die PID null (d.h. keine Quelle fr die Fahrstreife vorhanden ist):
        if (dePid == null) {
            return STATUS_FAHRSTREIFE_OHHNE_DE;
        }

        //
        DE_FEHLER_STATUS deFs = _mapDeFehlerStatus.get(dePid);

        if (deFs != null) {
            return deFs.status;
        } else {

            // Falls kein Status gefunden wird (soll aber niemals passieren):
            return "???";
        }
    }

    /**
     * Ermittle die HtmlBeschreibung fr das Objekt, das sich in der MausPosition befindet.
     *
     * @return HtmlBeschreibung fr das Objekt, das sich in der MausPosition befindet.
     */
    public String ermittleHtmlBeschreibung() {
        StringBuilder toolTip = new StringBuilder();

        // --- ZhlstellenBeschreibung:
        if (_zsbInMausPostion != null) {
            toolTip.append(_zsbInMausPostion.ermittleHtmlBeschreibung());

            //
            if (toolTip.length() > 0) {
                toolTip.insert(0, "<html><body style='overflow:auto;'>");
                toolTip.append("</br></br>");
                toolTip.append(ermittleTabelleFehlerStatusAlsHtml(_zsbInMausPostion));
                toolTip.append("</html></body>");
            }
        }

        // --- Kri2_b:
        else if (_kri2_bInMausPostion != null) {
            toolTip.append(ermittleHtmlBeschreibung(_kri2_bInMausPostion));

            //
            if (toolTip.length() > 0) {
                toolTip.insert(0, "<html><body style='overflow:auto;'>");
                toolTip.append("</br>");
                toolTip.append(ermittleTabelleFehlerStatusAlsHtml(_kri2_bInMausPostion));
                toolTip.append("</html></body>");
            }
        }

        // --- Kri2b:
        else if (_kri2bInMausPostion != null) {
            toolTip.append(ermittleHtmlBeschreibung(_kri2bInMausPostion));

            //
            if (toolTip.length() > 0) {
                toolTip.insert(0, "<html><body style='overflow:auto;'>");
                toolTip.append("</br>");
                toolTip.append(ermittleTabelleFehlerStatusAlsHtml(_kri2bInMausPostion));
                toolTip.append("</html></body>");
            }
        }

        return toolTip.toString();
    }

    /**
     * Ermittle die HTML-Beschreibung der angegebenen Kri2.
     *
     * @param kri2 Kri2.
     *
     * @return HTML-Beschreibung der Kri2 als Text.
     */
    private String ermittleHtmlBeschreibung(Kri2 kri2) {
        StringBuilder text = new StringBuilder();

        if (kri2 != null) {
            text.append(String.format("<font size=2><nobr><b>&lt%s&gt &nbsp %s</b></nobr></font></br>", kri2.getId(), kri2.getName()));
            text.append(String.format("<font size=2><nobr>%s</nobr></font></br>", kri2.getEakId()));
        }

        return text.toString();
    }

    /**
     * Ermittle alle De zur ZaehlStelle.
     *
     * @param zsb ZaehlStelle.
     *
     * @return Liste aller De zur ZaehlStelle.
     */
    private ArrayList<Object> ermittleListeDe(ZaehlStelle zsb) {
        ArrayList<Object> listeDe = new ArrayList<Object>();

        if (zsb != null) {
            Fahrbahnen[] arFs;
            String       quelle;

            for (int i = 0; i < 2; i++) {

                // in Richtung:
                if (i == 0) {
                    arFs = zsb.ermittleArFsInRichtung();
                }

                // gegen Richtung:
                else {
                    arFs = zsb.ermittleArFsGegenRichtung();
                }

                for (Fahrbahnen fs : arFs) {
                    quelle = fs.getFsQuellePid();

                    if (quelle != null) {
                        De de = DatenVew.getInstanz().ermittleDe(quelle);

                        if (de != null) {
                            listeDe.add(de);
                        } else {

                            // FehlerMeldung:
                            Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, String.format(  //
                                "Es existiert kein De-Objekt mit dem PID <%s> in DaV!", quelle)));
                        }
                    }
                }
            }
        }

        return listeDe;
    }

    /**
     * Ermittle die sichtbare Breite der Karte.
     *
     * @return Sichtbare Breite der Karte.
     */
    private int ermittleSichtbareBreiteDerKarte() {
        return Math.min(_gui.getScompKarte().getClientArea().width, _gui.getCanvasKarte().getSize().x);
    }

    /**
     * Ermittle die sichtbare Hhe der Karte.
     *
     * @return Sichtbare Hhe der Karte.
     */
    private int ermittleSichtbareHoeheDerKarte() {
        return Math.min(_gui.getScompKarte().getClientArea().height, _gui.getCanvasKarte().getSize().y);
    }

    /**
     * Ermittle die Tabelle der FehlerStatus als Html-Text.
     *
     * @param kri2 Kri2.
     *
     * @return Tabelle der FehlerStatus als Html-Text.
     */
    private String ermittleTabelleFehlerStatusAlsHtml(Kri2 kri2) {
        StringBuilder tabelle = new StringBuilder();

        tabelle.append("<table border='1'><thead><tr><th>De</th><th>Status</th></tr></thead>");

        String farbeHtml;
        Color  farbe;

        //
        tabelle.append("<tbody>");

        for (SystemObject de : kri2.getListeDeSystemObjOhne255()) {

            // De:
            tabelle.append("<tr><td align='left'>");
            tabelle.append("<font size=2>");
            tabelle.append(de.getPidOrId());
            tabelle.append("</font>");

            // Farbe:
            farbe     = ermittleDeFarbe(de.getPidOrId());
            farbeHtml = String.format("'#%s%s%s'", Integer.toHexString(farbe.getRed()), Integer.toHexString(farbe.getGreen()), Integer.toHexString(farbe.getBlue()));
            tabelle.append("<td align='center' bgcolor=");
            tabelle.append(farbeHtml);
            tabelle.append(">");
            tabelle.append("<font size=2>");

            // De-FehlerStatus:
            tabelle.append(ermittleDeStatus(de.getPidOrId()));
            tabelle.append("</font>");
            tabelle.append("</td>");
            tabelle.append("</tr>");
        }

        tabelle.append("</tbody>");
        tabelle.append("</table>");

        return tabelle.toString();
    }

    /**
     * Ermittle die Tabelle der FehlerStatus als Html-Text.
     *
     * @param zsb ZaehlStelle.
     *
     * @return Tabelle der FehlerStatus als Html-Text.
     */
    private String ermittleTabelleFehlerStatusAlsHtml(ZaehlStelle zsb) {
        StringBuilder tabelle = new StringBuilder();

        tabelle.append("<table border='1'><thead><tr><th>Fahrstreifen</th><th>Quelle</th><th>Status</th></tr></thead>");

        String farbeHtml;
        Color  farbe;

        //
        Fahrbahnen[] arFs;
        String       ri;
        String[]     arZiele = _zsbInMausPostion.ermittleNahFernZiele();
        String       quelle;

        for (int i = 0; i < 2; i++) {

            // in Richtung:
            if (i == 0) {
                arFs = zsb.ermittleArFsInRichtung();
                ri   = String.format("in Richtung %s (%s)", arZiele[0], arZiele[1]);
            }

            // gegen Richtung:
            else {
                arFs = zsb.ermittleArFsGegenRichtung();
                ri   = String.format("gegen Richtung %s (%s)", arZiele[2], arZiele[3]);
            }

            tabelle.append("<tr><th colspan=3 align='left'>");
            tabelle.append(ri);
            tabelle.append("</th></tr>");
            tabelle.append("<tbody>");

            for (Fahrbahnen fs : arFs) {

                // FS-Name:
                tabelle.append("<tr><td align='left'>");
                tabelle.append("<font size=2>");
                tabelle.append(fs.getSystemObjekt().getName());
                tabelle.append("</font>");

                // FS-Quelle:
                quelle = fs.getFsQuellePid();
                tabelle.append("<td align='left'>");
                tabelle.append("<font size=2>");

                if (quelle == null) {
                    tabelle.append(STATUS_FAHRSTREIFE_OHHNE_DE);
                } else {
                    tabelle.append(quelle);
                }

                tabelle.append("</font>");

                // Farbe:
                farbe     = ermittleDeFarbe(quelle);
                farbeHtml = String.format("'#%s%s%s'", Integer.toHexString(farbe.getRed()), Integer.toHexString(farbe.getGreen()), Integer.toHexString(farbe.getBlue()));
                tabelle.append("<td align='center' bgcolor=");
                tabelle.append(farbeHtml);
                tabelle.append(">");
                tabelle.append("<font size=2>");

                // FS-FehlerStatus:
                tabelle.append(ermittleDeStatus(quelle));
                tabelle.append("</font>");
                tabelle.append("</td>");
                tabelle.append("</tr>");
            }

            tabelle.append("</tbody>");
        }

        tabelle.append("</table>");

        return tabelle.toString();
    }

    /**
     * Erstelle das KontextMenu fr die Karte.
     */
    protected void erstelleKontextMenu() {
        Menu menu = new Menu(_gui.getShell(), SWT.POP_UP);

        // ---------------------------------------------------------------
        // Bereich zeigen:
        MenuItem miBereichZeigen = new MenuItem(menu, SWT.CASCADE);

        miBereichZeigen.setText("Bereich zentriert im Mastab zeigen...");

        Menu     menuMasssstab = new Menu(miBereichZeigen);
        MenuItem miMassstab;

        // MenItem mit Mastben definieren:
        for (int i = 0; i < AR_MASSSTAB_NAMEN.length; i++) {
            miMassstab = new MenuItem(menuMasssstab, SWT.PUSH);
            miMassstab.setText(AR_MASSSTAB_NAMEN[i]);

            final int index = i;

            miMassstab.addListener(SWT.Selection, new Listener() {
                @Override
                public void handleEvent(Event event) {
                    scrolleKarte(index);
                }
            });
            miMassstab.setEnabled(i != _indexMassstab);
        }

        miBereichZeigen.setMenu(menuMasssstab);

        // ---------------------------------------------------------------
        // Men fr die ausgewhlte ZhlStelle:
        if (_zsbInMausPostion != null) {

            // ---------------------------------------------------------------
            // Separator:
            new MenuItem(menu, SWT.SEPARATOR);

            String[] arTv = AlleTabellenVerwaltungen.getTabellenVerwaltungInstanz().getTabellenVerwaltungArray();

            // ---------------------------------------------------------------
            // ZhlStelle:
            MenuItem miZaehlStelle = new MenuItem(menu, SWT.CASCADE);

            miZaehlStelle.setText("Zhlstelle");

            Menu              menuZaehlStelle = new Menu(miZaehlStelle);
            ArrayList<Object> mengeObjekte    = new ArrayList<Object>();

            mengeObjekte.add(_zsbInMausPostion);

            // selektionSpeichern:
            ArrayList<Object> listeSelektierteObjekte = new ArrayList<Object>();

            listeSelektierteObjekte.add(_zsbInMausPostion);

            IWorkbenchPart sicht = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActivePart();

            SelektionsTransfer.getInstanz().selectionChanged(sicht, new StructuredSelection(listeSelektierteObjekte));

            //
            BaumHilfsFunktionen.erstelleKontextMenu(menuZaehlStelle, mengeObjekte, _zsbInMausPostion, arTv);
            miZaehlStelle.setMenu(menuZaehlStelle);

            // ---------------------------------------------------------------
            // Messquerschnitt:
            mengeObjekte = new ArrayList<Object>();

            for (Messquerschnitt mq : _zsbInMausPostion.ermittleArMq()) {
                mengeObjekte.add(mq);
            }

            if (mengeObjekte.size() > 0) {
                MenuItem miMessquerschnitt = new MenuItem(menu, SWT.CASCADE);

                miMessquerschnitt.setText("Messquerschnitte");

                Menu menuMessquerschnitt = new Menu(miMessquerschnitt);

                BaumHilfsFunktionen.erstelleKontextMenu(menuMessquerschnitt, mengeObjekte, mengeObjekte.get(0), arTv);
                miMessquerschnitt.setMenu(menuMessquerschnitt);
            } else {
                Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, String.format(  //
                    "Die Zhlstelle <%s> enthlt keine Messquerschnitte!", _zsbInMausPostion.getId())));
            }

            // ---------------------------------------------------------------
            // Fahrstreifen:
            mengeObjekte = new ArrayList<Object>();

            for (Fahrbahnen fs : _zsbInMausPostion.ermittleArFsInRichtung()) {
                mengeObjekte.add(fs);
            }

            for (Fahrbahnen fs : _zsbInMausPostion.ermittleArFsGegenRichtung()) {
                mengeObjekte.add(fs);
            }

            if (mengeObjekte.size() > 0) {
                MenuItem miFahrstreifen = new MenuItem(menu, SWT.CASCADE);

                miFahrstreifen.setText("Fahrstreifen");

                Menu menuFahrstreifen = new Menu(miFahrstreifen);

                BaumHilfsFunktionen.erstelleKontextMenu(menuFahrstreifen, mengeObjekte, mengeObjekte.get(0), arTv);
                miFahrstreifen.setMenu(menuFahrstreifen);
            } else {
                Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, String.format(  //
                    "Die Zhlstelle <%s> enthlt keine Fahrstreifen!", _zsbInMausPostion.getId())));
            }

            // ---------------------------------------------------------------
            // De:
            mengeObjekte = ermittleListeDe(_zsbInMausPostion);

            if (mengeObjekte.size() > 0) {
                MenuItem miDe = new MenuItem(menu, SWT.CASCADE);

                miDe.setText("De");

                Menu menuDe = new Menu(miDe);

                BaumHilfsFunktionen.erstelleKontextMenu(menuDe, mengeObjekte, mengeObjekte.get(0), arTv);
                miDe.setMenu(menuDe);
            } else {
                Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, String.format(  //
                    "Die Zhlstelle <%s> enthlt keine De!", _zsbInMausPostion.getId())));
            }

            // ---------------------------------------------------------------
            // Separator:
            new MenuItem(menu, SWT.SEPARATOR);

            // ---------------------------------------------------------------
            // ZhlstellenDokumentation:
            MenuItem miZsDok = new MenuItem(menu, SWT.PUSH);

            miZsDok.setText("Zhlstellendokumentation");
            miZsDok.addListener(SWT.Selection, new Listener() {
                @Override
                public void handleEvent(Event event) {
                    zeigeZsDokumentation();
                }
            });
        }

        // ---------------------------------------------------------------
        _gui.getCanvasKarte().setMenu(menu);
    }

    /**
     * Initialisiere das Array der KartenGren in WGS.
     */
    private void initialisiereArrayKartenGroessen() {

        // -------------------------------------------------------------
        // Datei einlesen...
        // -------------------------------------------------------------
        String       dateiName = "Karte/KartenGrenzKoordinaten.csv";
        String       spalte1   = "X_WGS84Dec_LINKS";
        String       spalte2   = "Y_WGS84Dec_UNTEN";
        String       spalte3   = "X_WGS84Dec_RECHTS";
        String       spalte4   = "Y_WGS84Dec_OBEN";
        String       dateiPfad = BundeslandVew.getInstanz().ermittleDateiImOrdnerBundeslandResource(dateiName);
        File         datei     = new File(dateiPfad);
        CsvLeser     csvDatei;
        List<Double> liste    = new ArrayList<Double>();
        int          zeilenNr = 1;
        String       k1;
        String       k2;
        String       k3;
        String       k4;

        try {

            // Datei einlesen...
            csvDatei = new CsvLeser(new FileReader(datei), ';');

            Map<String, String> zeile;

            do {
                zeile = csvDatei.getNaechsteZeile();
                k1    = zeile.get(spalte1).trim();
                k2    = zeile.get(spalte2).trim();
                k3    = zeile.get(spalte3).trim();
                k4    = zeile.get(spalte4).trim();
                liste.add(Double.valueOf(k1));
                liste.add(Double.valueOf(k2));
                liste.add(Double.valueOf(k3));
                liste.add(Double.valueOf(k4));
                zeilenNr++;
            } while (csvDatei.hasNochMehrZeilen());

            csvDatei.close();
        }
        catch (FileNotFoundException e) {
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Datei <" + datei + "> kann nicht gefunden werden!", e));
        }
        catch (NumberFormatException e) {
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Fehler beim Einlesen der Datei <" + datei + ">  in der Zeile " + zeilenNr + ".", e));
        }
        catch (Exception e) {
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Fehler beim Einlesen der Datei <" + datei + ">  in der Zeile " + zeilenNr + ".", e));
        }

        // -------------------------------------------------------------
        // Initialisiere das WGS-Array:
        // -------------------------------------------------------------
        if (liste.size() < AR_IMAGE_KARTE.length * 4) {
            MessageDialog.openError(_gui.getShell(), "Fehler",                                                 //
                                    "Die folgende Datei sind fehlerhaft: " + NEWLINE +                         //
                                    dateiPfad + NEWLINE + NEWLINE +                                            //
                                    "Diese mssen jeweils 4 Koordinaten fr alle 3 Kartenbilder enthalten." +  //
                                    NEWLINE + NEWLINE +                                                        //
                                    "Bitte, korrigieren Sie die Datei und starten Sie die Anwendung neu.");
        } else {
            AR_IMAGE_KARTE_GROESSE_IN_WGS84 = new double[AR_IMAGE_KARTE.length][];

            for (int i = 0; i < AR_IMAGE_KARTE.length; i++) {
                AR_IMAGE_KARTE_GROESSE_IN_WGS84[i] = new double[]                                              //
                                { liste.get(i * 4), liste.get(i * 4 + 1), liste.get(i * 4 + 2), liste.get(i * 4 + 3) };
            }
        }

        // -------------------------------------------------------------
        // Initialisiere das UTM-Array:
        // -------------------------------------------------------------
        AR_IMAGE_KARTE_GROESSE_IN_UTM = new double[AR_IMAGE_KARTE_GROESSE_IN_WGS84.length][];

        double[] punktLinksUnten;
        double[] punktRechtsOben;

        for (int i = 0; i < AR_IMAGE_KARTE_GROESSE_IN_WGS84.length; i++) {
            punktLinksUnten                  = KoordinatenKonvertor.wgs2utm(AR_IMAGE_KARTE_GROESSE_IN_WGS84[i][0], AR_IMAGE_KARTE_GROESSE_IN_WGS84[i][1]);
            punktRechtsOben                  = KoordinatenKonvertor.wgs2utm(AR_IMAGE_KARTE_GROESSE_IN_WGS84[i][2], AR_IMAGE_KARTE_GROESSE_IN_WGS84[i][3]);
            AR_IMAGE_KARTE_GROESSE_IN_UTM[i] = new double[] { punktLinksUnten[0], punktLinksUnten[1], punktRechtsOben[0], punktRechtsOben[1] };
        }
    }

    /**
     * Initialisiere das Array der TestPunkte in WGS
     * fr das Kalibrieren.
     */
    private void initialisiereArrayTestPunkte() {

        // -------------------------------------------------------------
        // Datei einlesen...
        // -------------------------------------------------------------
        String       dateiName = "Karte/TestPunkte.csv";
        String       spalte1   = "X_WGS84Dec";
        String       spalte2   = "Y_WGS84Dec";
        String       dateiPfad = BundeslandVew.getInstanz().ermittleDateiImOrdnerBundeslandResource(dateiName);
        File         datei     = new File(dateiPfad);
        CsvLeser     csvDatei;
        List<Double> liste    = new ArrayList<Double>();
        int          zeilenNr = 1;
        String       xWgs;
        String       yWgs;

        try {

            // Datei einlesen...
            csvDatei = new CsvLeser(new FileReader(datei), ';');

            Map<String, String> zeile;

            while (csvDatei.hasNochMehrZeilen()) {
                zeile = csvDatei.getNaechsteZeile();
                xWgs  = zeile.get(spalte1).trim();
                yWgs  = zeile.get(spalte2).trim();
                liste.add(Double.valueOf(xWgs));
                liste.add(Double.valueOf(yWgs));
                zeilenNr++;
            }

            csvDatei.close();
        }
        catch (FileNotFoundException e) {
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Datei <" + datei + "> kann nicht gefunden werden!", e));
        }
        catch (NumberFormatException e) {
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Fehler beim Einlesen der Datei <" + datei + ">  in der Zeile " + zeilenNr + ".", e));
        }
        catch (Exception e) {
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Fehler beim Einlesen der Datei <" + datei + ">  in der Zeile " + zeilenNr + ".", e));
        }

        // -------------------------------------------------------------
        // Initialisiere das Array:
        // -------------------------------------------------------------
        if (liste.size() % 2 != 0) {
            liste.remove(liste.size() - 1);
        }

        AR_TEST_PUNKTE_IN_WGS84 = new double[liste.size()];

        for (int i = 0; i < liste.size(); i++) {
            AR_TEST_PUNKTE_IN_WGS84[i] = liste.get(i);
        }
    }

    /**
     * Initialisiere alle Combo 'Element zentrieren'.
     */
    private void initialisiereComboElementZentrieren() {

        // -------------------------------------------------------
        // ZhlStellen:
        // Erstelle Array alle ZhlStellenPIDs plus leeres String:
        String[] arElemente        = new String[DatenVew.getInstanz().getListeZaehlStellen().size() + 1];
        String[] arElementeTooltip = new String[arElemente.length];

        arElemente[0]        = "";
        arElementeTooltip[0] = "";

        int i = 1;

        for (ZaehlStelle zsb : DatenVew.getInstanz().getListeZaehlStellen()) {
            arElemente[i]        = zsb.getSystemObjekt().getNameOrPidOrId();
            arElementeTooltip[i] = String.format("%s (%s)", zsb.getSystemObjekt().getName(), zsb.getSystemObjekt().getPidOrId());
            i++;
        }

        _arComboZsAlleElemente        = arElemente;
        _arComboZsAlleElementeToolTip = arElementeTooltip;

        // Setze Combo:
        _gui.getComboZs().setItems(arElemente);

        // -------------------------------------------------------
        // Kri2_b:
        arElemente    = new String[_kri2Vew.getListeKri2_b().size() + 1];
        arElemente[0] = "";
        i             = 1;

        for (Kri2 kri2 : _kri2Vew.getListeKri2_b()) {
            arElemente[i] = kri2.getId();
            i++;
        }

        _gui.getComboKri2_b().setItems(arElemente);

        // -------------------------------------------------------
        // Kri2b:
        arElemente    = new String[_kri2Vew.getListeKri2b().size() + 1];
        arElemente[0] = "";
        i             = 1;

        for (Kri2 kri2 : _kri2Vew.getListeKri2b()) {
            arElemente[i] = kri2.getId();
            i++;
        }

        _gui.getComboKri2b().setItems(arElemente);
    }

    /**
     * Initialisiere das GUI.
     */
    protected void initialisiereGui() {

        // ------------------------------------------------------------------------
        // Initialisiere ToolTip:
        _toolTip = new ToolTipHtml(_gui.getCanvasKarte());
        _toolTip.setHideOnMouseDown(false);

        // ------------------------------------------------------------------------
        // Initialisiere den Mastab:
        setzeMassstab(1);

        // ------------------------------------------------------------------------
        // Initialisiere alle Combo 'Element zentrieren':
        initialisiereComboElementZentrieren();

        // ------------------------------------------------------------------------
        // Initialisiere CheckButtons fr die darzustellenden Elemente:
        _gui.getCbtnZaehlstellen().setSelection(true);
        _gui.getCbtnKri2_b().setSelection(false);
        _gui.getCbtnKri2b().setSelection(false);

        // ------------------------------------------------------------------------
        // Initialisiere die Listen mit den Pixel-Koordinaten:
        initialisiereListenKoord();
    }

    /**
     * Initialisiere die Listen  der Pixel-Koordinaten aller Zhlstellen, Kri2_b und Kri2b
     * fr jedes Bild aus AR_IMAGE_KARTE.
     */
    private void initialisiereListenKoord() {
        _listePixelKoordAllerZsFuerImage     = new ArrayList<List<int[]>>();
        _listePixelKoordAllerKri2_bFuerImage = new ArrayList<List<int[]>>();
        _listePixelKoordAllerKri2bFuerImage  = new ArrayList<List<int[]>>();
        _listePixelKoordTestPunktenFuerImage = new ArrayList<List<int[]>>();

        List<int[]> listeKoordPixel;

        for (int i = 0; i < AR_IMAGE_KARTE.length; i++) {

            // ------------------------------------------------------
            // ZhlStellen:
            listeKoordPixel = new ArrayList<int[]>();

            for (ZaehlStelle zsb : DatenVew.getInstanz().getListeZaehlStellen()) {
                listeKoordPixel.add(umwandeleWGS84DecKoordinatenInPixelKoordinaten(zsb.getxWgs84(), zsb.getyWgs84(), i));
            }

            _listePixelKoordAllerZsFuerImage.add(listeKoordPixel);

            // ------------------------------------------------------
            // Kri2_b:
            listeKoordPixel = new ArrayList<int[]>();

            int k = 0;

            for (Kri2 kri2 : _kri2Vew.getListeKri2_b()) {

                // Kri-Koord:
                if (kri2.getxWgs84() > -1) {
                    listeKoordPixel.add(umwandeleWGS84DecKoordinatenInPixelKoordinaten(kri2.getxWgs84(), kri2.getyWgs84(), i));
                } else {
                    listeKoordPixel.add(umwandeleWGS84DecKoordinatenInPixelKoordinaten(7.0 + 0.2 * k, 51.50, i));
                    k++;
                }
            }

            _listePixelKoordAllerKri2_bFuerImage.add(listeKoordPixel);

            // ------------------------------------------------------
            // Kri2b:
            listeKoordPixel = new ArrayList<int[]>();
            k               = 0;

            for (Kri2 kri2 : _kri2Vew.getListeKri2b()) {

                // Kri-Koord:
                if (kri2.getxWgs84() > -1) {
                    listeKoordPixel.add(umwandeleWGS84DecKoordinatenInPixelKoordinaten(kri2.getxWgs84(), kri2.getyWgs84(), i));
                } else {
                    listeKoordPixel.add(umwandeleWGS84DecKoordinatenInPixelKoordinaten(7.0 + 0.2 * k, 51.40, i));
                    k++;
                }
            }

            _listePixelKoordAllerKri2bFuerImage.add(listeKoordPixel);

            // ------------------------------------------------------
            // TestPunkte:
            listeKoordPixel = new ArrayList<int[]>();

            for (int j = 0; j < AR_TEST_PUNKTE_IN_WGS84.length / 2; j++) {
                listeKoordPixel.add(umwandeleWGS84DecKoordinatenInPixelKoordinaten(AR_TEST_PUNKTE_IN_WGS84[j * 2], AR_TEST_PUNKTE_IN_WGS84[j * 2 + 1], i));
            }

            _listePixelKoordTestPunktenFuerImage.add(listeKoordPixel);
        }
    }

    /**
     * Initialisiere die Objekte (ZaehlStelle, Kri2_b, Kri2b) in der Maus-Position.
     *
     * Nur das Objekt mit der min. Distanz (falls berhaupt gefunden wird) wird initialisiert.
     * Alle andere werden auf null gesetzt.
     *
     * @param x X-Koordinate der Maus.
     * @param y Y-Koordinate der Maus.
     */
    private void initialisiereObjekteInMausPosition(int x, int y) {

        // ------------------------------------------------------------------
        // Suche die drei Objekte (ZaehlStelle, Kri2_b, Kri2b),
        // die sich in den  angegeben Koordinaten (oder nah genug) befindet.
        double minDistanz0 = Double.MAX_VALUE;
        double minDistanz1 = Double.MAX_VALUE;
        double minDistanz2 = Double.MAX_VALUE;

        // objektN: 0 - ZaehlStelle, 1 - Kri2_b, 2 - Kri2b
        for (int objektN = 0; objektN < 3; objektN++) {
            List<int[]> listeKoordPixel;

            // Initialisiere die Liste der Koordinaten abhngig von Typ des zu suchenden Objekt:
            if (objektN == 0) {
                listeKoordPixel = _listePixelKoordAllerZsFuerImage.get(AR_MASSTAB_IMAGE_INDEX[_indexMassstab]);
            } else if (objektN == 1) {
                listeKoordPixel = _listePixelKoordAllerKri2_bFuerImage.get(AR_MASSTAB_IMAGE_INDEX[_indexMassstab]);
            } else if (objektN == 2) {
                listeKoordPixel = _listePixelKoordAllerKri2bFuerImage.get(AR_MASSTAB_IMAGE_INDEX[_indexMassstab]);
            } else {
                listeKoordPixel = new ArrayList<int[]>();
            }

            // Suche das Objekt mit minimaler Distanz:
            double min = Double.MAX_VALUE;
            double distanz;
            int    iMin          = -1;
            int    i             = 0;
            float  massstabKarte = AR_MASSSTAB[_indexMassstab];

            for (int[] koord : listeKoordPixel) {
                Math.pow(x - koord[0], 2);
                distanz = Math.sqrt(Math.pow(x - ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL - koord[0] * massstabKarte, 2) + Math.pow(y - ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL - koord[1] * massstabKarte, 2));

                if (distanz < min) {
                    min  = distanz;
                    iMin = i;
                }

                i++;
            }

            int passt = AR_MASSTAB_FS_LINIE_BRIETE[_indexMassstab] * 2;

            // Initialisiere das gesuchte Objekt, falls das nah genug
            // zu angegebenen Koordinaten liegt:
            if (objektN == 0) {
                if (min < passt) {
                    minDistanz0       = min;
                    _zsbInMausPostion = DatenVew.getInstanz().getListeZaehlStellen().get(iMin);
                } else {
                    minDistanz0       = Double.MAX_VALUE;
                    _zsbInMausPostion = null;
                }
            } else if (objektN == 1) {
                if (min < passt) {
                    minDistanz1          = min;
                    _kri2_bInMausPostion = _kri2Vew.getListeKri2_b().get(iMin);
                } else {
                    minDistanz1          = Double.MAX_VALUE;
                    _kri2_bInMausPostion = null;
                }
            } else if (objektN == 2) {
                if (min < passt) {
                    minDistanz2         = min;
                    _kri2bInMausPostion = _kri2Vew.getListeKri2b().get(iMin);
                } else {
                    minDistanz2         = Double.MAX_VALUE;
                    _kri2bInMausPostion = null;
                }
            }
        }

        // ------------------------------------------------------------------
        // Aus drei gefundenen Objekten lsche nicht angezeigte,
        // und lasse nur ein mit minimaler Distanz.
        if (!_gui.getCbtnZaehlstellen().getSelection()) {
            minDistanz0       = Double.MAX_VALUE;
            _zsbInMausPostion = null;
        }

        if (!_gui.getCbtnKri2_b().getSelection()) {
            minDistanz1          = Double.MAX_VALUE;
            _kri2_bInMausPostion = null;
        }

        if (!_gui.getCbtnKri2b().getSelection()) {
            minDistanz2         = Double.MAX_VALUE;
            _kri2bInMausPostion = null;
        }

        double[] arMin   = new double[] { minDistanz0, minDistanz1, minDistanz2 };
        int      objektN = 0;

        for (int i = 1; i < arMin.length; i++) {
            if (arMin[i] < arMin[objektN]) {
                objektN = i;
            }
        }

        if (objektN != 0) {
            _zsbInMausPostion = null;
        }

        if (objektN != 1) {
            _kri2_bInMausPostion = null;
        }

        if (objektN != 2) {
            _kri2bInMausPostion = null;
        }
    }

    /**
     * Neuer Mastab wird selektiert.
     */
    protected void massstabSelektiert() {

        // -------------------------
        // Setze neuen Mastab:
        int indexMassstabAlt = _indexMassstab;

        for (int i = 0; i < AR_MASSSTAB.length; i++) {
            if (_gui.getArRbtnMassstab()[i].getSelection()) {
                _indexMassstab = i;

                break;
            }
        }

        // -------------------------
        // Berechne neue Werte und zeichne Karte neu:
        if (_indexMassstab != indexMassstabAlt) {

            // -------------------------
            // Berechne die Mitte der angezeigten Karte im alten Mastab,
            // um die Karte im neuen Mastab zu diesem Mittelpunkt zu scrollen::
            int x = _gui.getScompKarte().getOrigin().x + (ermittleSichtbareBreiteDerKarte() / 2);
            int y = _gui.getScompKarte().getOrigin().y + (ermittleSichtbareHoeheDerKarte() / 2);

            // Umwandle die alten MausKoordinaten in WGS84-Dec und speichere die:
            int indexMassstabNeu = _indexMassstab;

            _indexMassstab = indexMassstabAlt;

            double[] koordWgs = umwandelePixelKoordinatenInWGS84DecKoordinaten(x, y);

            _indexMassstab = indexMassstabNeu;
            _xMausWgs84    = koordWgs[0];
            _yMausWgs84    = koordWgs[1];

            // -------------------------
            // Berechne die neuen Werte fr das Scrollen:
            _istScrollen = true;

            float massstabKarte = AR_MASSSTAB[_indexMassstab];
            int   breite        = Math.round(massstabKarte * AR_KARTE_LAENGE_IN_PIXEL[AR_MASSTAB_IMAGE_INDEX[_indexMassstab]]) + 2 * ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL;
            int   hoehe         = Math.round(massstabKarte * KARTE_HOEHE_IN_PIXEL) + 2 * ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL;

            _gui.getCanvasKarte().setSize(breite, hoehe);
            _gui.getCanvasKarte().setLocation(0, 0);
            _gui.getScompKarte().setMinSize(breite, hoehe);
            _gui.getScompKarte().setOrigin(0, 0);
        }
    }

    /**
     * Die Maus ist ber der Karte:
     *
     * zeige ToolTip.
     *
     * @param e MouseEvent.
     */
    protected void mausUeberDerKarte(MouseEvent e) {

        // Umwandle die MausKoordinaten in WGS84-Dec und speichere die:
        double[] koordWgs = umwandelePixelKoordinatenInWGS84DecKoordinaten(e.x, e.y);

        _xMausWgs84 = koordWgs[0];
        _yMausWgs84 = koordWgs[1];

        //
        initialisiereObjekteInMausPosition(e.x, e.y);

        // Ermittle den HTML-ToolText:
        String toolTip = ermittleHtmlBeschreibung();

        // Zeige den HTML - ToolTip:
        if (toolTip.length() > 0) {
            _toolTip.activate();
            _toolTip.setzeText(toolTip);

            //
            _gui.getCanvasKarte().setToolTipText(null);
        }

        // Zeige den einfachen ToolTip:
        else {
            _toolTip.deactivate();

            // ToolText mit Koordinaten:
            // String koord = "" + e.x + " " + e.y + Konstanten.NEWLINE + koordWgs[0] + " " + koordWgs[1];
            // _gui.getCanvasKarte().setToolTipText(koord);
            //
            _gui.getCanvasKarte().setToolTipText(null);
        }
    }

    /**
     * Doppel-Click mit der Maus auf der Karte
     * (ffne die TabellenDarstellung fr die entsprechenden De).
     */
    protected void mouseDoppelClickAufDerKarte() {

        // ----------------------------------------------------------
        // ZaehlStelle:
        // Ermittle alle De zur ZaehlStelle:
        if (_zsbInMausPostion != null) {
            ArrayList<Object> listeDe = ermittleListeDe(_zsbInMausPostion);

            // ffne die TabbelenDarstellung fr die ausgewhlten De:
            if (listeDe.size() > 0) {
                BaumHilfsFunktionen.oeffneTabellendarstellung(listeDe, listeDe.get(0));
            } else {
                MessageDialog.openWarning(_gui.getShell(), "Warnung", "Die Zhlstelle enthlt keine De!");
            }
        }

        // ----------------------------------------------------------
        // Kri2_b:
        else if (_kri2_bInMausPostion != null) {
            ArrayList<Object> liste  = new ArrayList<Object>();
            Geraet            geraet = DatenVew.getInstanz().ermittleTlsGeraet(_kri2_bInMausPostion.getSoKri2().getPidOrId());

            if (geraet != null) {
                liste.add(geraet);
            } else {
                Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, String.format(  //
                    "Objekt KRI2_b <%s> wurde nicht gefunden", _kri2_bInMausPostion.getSoKri2().getPidOrId())));
            }

            BaumHilfsFunktionen.oeffneTabellendarstellung(liste, liste.get(0));
        }

        // ----------------------------------------------------------
        // Kri2b:
        else if (_kri2bInMausPostion != null) {
            ArrayList<Object> liste  = new ArrayList<Object>();
            Geraet            geraet = DatenVew.getInstanz().ermittleTlsGeraet(_kri2bInMausPostion.getSoKri2().getPidOrId());

            if (geraet != null) {
                liste.add(geraet);
            } else {
                Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, String.format(  //
                    "Objekt KRI2 <%s> wurde nicht gefunden", _kri2bInMausPostion.getSoKri2().getPidOrId())));
            }

            BaumHilfsFunktionen.oeffneTabellendarstellung(liste, liste.get(0));
        }
    }

    /**
     * Scrolle die Karte zur MausPosition im angegeben Mastab.
     *
     * @param indexMassstabNeu Neuen Mastab.
     */
    private void scrolleKarte(int indexMassstabNeu) {
        if (_indexMassstab != indexMassstabNeu) {

            // Setze den neuen Mastab:
            setzeMassstab(indexMassstabNeu);

            // Berechne die neuen Werte fr das Scrollen:
            _istScrollen = true;

            float massstabKarte = AR_MASSSTAB[_indexMassstab];
            int   breite        = Math.round(massstabKarte * AR_KARTE_LAENGE_IN_PIXEL[AR_MASSTAB_IMAGE_INDEX[_indexMassstab]]) + 2 * ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL;
            int   hoehe         = Math.round(massstabKarte * KARTE_HOEHE_IN_PIXEL) + 2 * ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL;

            _gui.getCanvasKarte().setSize(breite, hoehe);
            _gui.getCanvasKarte().setLocation(0, 0);
            _gui.getScompKarte().setMinSize(breite, hoehe);
            _gui.getScompKarte().setOrigin(0, 0);

            // Setze die Koordinaten fr den Kreuz:
            _xKreuzWgs84 = _xMausWgs84;
            _yKreuzWgs84 = _yMausWgs84;
        }
    }

    /**
     * Setze den Mastab.
     *
     * @param indexMassstab Index des Mastabes im Array AR_MASSSTAB.
     */
    private void setzeMassstab(int indexMassstab) {
        for (Button rbtn : _gui.getArRbtnMassstab()) {
            rbtn.setSelection(false);
        }

        _indexMassstab = indexMassstab;
        _gui.getArRbtnMassstab()[_indexMassstab].setSelection(true);
    }

    /**
     * Filter fr Zhlstelle-Combo ist gendert.
     */
    protected void txtZsFilterGeaendert() {
        String filterText = _gui.getTxtZsFilter().getText();

        // Falls Filter leer ist, setze alle Combo-Elemente:
        if (filterText.length() == 0) {
            _gui.getComboZs().setItems(_arComboZsAlleElemente);
            _gui.getComboZs().setToolTipText("");
        }

        // Filtere die ComboElemente:
        else {
            StringMatcher filter         = new StringMatcher("*" + filterText.replaceAll("\\.", "?").replaceAll("_", "?") + "*", true, false);
            List<String>  listeGefiltert = new ArrayList<String>();

            for (String zs : _arComboZsAlleElemente) {
                if (filter.match(zs)) {
                    listeGefiltert.add(zs);
                }
            }

            // Setze die gefilterten Elemente und selektiere das erste:
            _gui.getComboZs().setItems(listeGefiltert.toArray(new String[listeGefiltert.size()]));

            if (listeGefiltert.size() > 0) {
                _gui.getComboZs().select(0);
            }

            comboZentierenSelektiert(_gui.getComboZs());
        }
    }

    /**
     * Ermittle die WGS84Dec-Koordinaten aus den Pixel-Koordinaten mit dem im GUI selektierten Mastab
     * und mit der Bercksichtigung der ZeichnungRahmen.
     *
     * @param xInPixel X-Koordinate in Pixel mit der ZeichnungRahmen.
     * @param yInPixel Y-Koordinate in Pixel mit der ZeichnungRahmen.
     *
     * @return Koordinaten in WGS84Dec.
     */
    private double[] umwandelePixelKoordinatenInWGS84DecKoordinaten(int xInPixel, int yInPixel) {

        // Lineale Konvertierung von Pixel in UTM-Koordinaten:
        int    indexImage = AR_MASSTAB_IMAGE_INDEX[_indexMassstab];
        double massstab   = AR_MASSSTAB[_indexMassstab];

        //
        double x = (1d * xInPixel - ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL) / massstab * AR_ANZAHL_X_UTM_PRO_PIXEL[indexImage] + AR_IMAGE_KARTE_GROESSE_IN_UTM[indexImage][0];

        //
        double y = 1d * KARTE_HOEHE_IN_PIXEL - (1d * yInPixel - ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL) / massstab;

        y = y * AR_ANZAHL_Y_UTM_PRO_PIXEL[indexImage] + AR_IMAGE_KARTE_GROESSE_IN_UTM[indexImage][1];

        // Konvertierung von UTM in WGS-Koordinaten:
        return KoordinatenKonvertor.utm2wgs(x, y);
    }

    /**
     * Ermittle die Pixel-Koordinaten aus den WGS84Dec-Koordinaten mit dem Mastab 1
     * ohne die Bercksichtigung der ZeichnungRahmen.
     *
     * Bei der Umrechnung kann Exception auftreten, falls die Ergebnis-Koordinate zu gro ist
     * und nicht von long nach int umgewandelt werden kann.
     *
     * @param xWGS84Dec X-Koordinate in WGS84Decimal.
     * @param yWGS84Dec Y-Koordinate in WGS84Decimal.
     * @param indexImage Index des Bildes aus AR_IMAGE_KARTE, fr welches
     * die Umrechnungen durchgefhrt werden soll.
     *
     * @return Koordinaten in Pixel.
     */
    private int[] umwandeleWGS84DecKoordinatenInPixelKoordinaten(Double xWGS84Dec, Double yWGS84Dec, int indexImage) {

        // Konvertierung von WGS in UTM-Koordinaten:
        double[] utm = KoordinatenKonvertor.wgs2utm(xWGS84Dec, yWGS84Dec);

        // Lineale Konvertierung von UTM-Koordinaten in Pixel:
        double x = (utm[0] - AR_IMAGE_KARTE_GROESSE_IN_UTM[indexImage][0]) / AR_ANZAHL_X_UTM_PRO_PIXEL[indexImage];
        double y = (utm[1] - AR_IMAGE_KARTE_GROESSE_IN_UTM[indexImage][1]) / AR_ANZAHL_Y_UTM_PRO_PIXEL[indexImage];

        //
        int xInPixel = (int) Math.round(x);
        int yInPixel = (int) Math.round((KARTE_HOEHE_IN_PIXEL - y));

        return new int[] { xInPixel, yInPixel };
    }

    /**
     * Reagiere auf die nderungen von DeFehlerStatus.
     *
     * @param results Neue Werte.
     */
    protected void update(ResultData[] results) {
        String  pid;
        String  statusDav;
        boolean istGeaendert = false;

        for (ResultData ergebnis : results) {
            pid = ergebnis.getObject().getPidOrId();

            if (ergebnis.hasData()) {

                // Lese die gelieferten Werte:
                statusDav = ergebnis.getData().getTextValue("DEFehlerStatus").getText();
            } else {

                // keine Quelle:
                if (ergebnis.isNoSourceAvailable()) {
                    statusDav = DE_FEHLER_STATUS.KEINE_QUELLE.statusDav;
                }

                // keine Daten:
                else if (ergebnis.isNoDataAvailable()) {
                    statusDav = DE_FEHLER_STATUS.KEINE_DATEN.statusDav;
                }

                // unbekannt:
                else {
                    statusDav = DE_FEHLER_STATUS.UNBEKANNT.statusDav;
                }
            }

            // ndere den DeFehlerStatus, falls der Wert gendert wurde:
            if (!_mapDeFehlerStatus.get(pid).statusDav.equals(statusDav)) {
                _mapDeFehlerStatus.put(pid, DE_FEHLER_STATUS.ermittleStatusFuerDav(statusDav));
                istGeaendert = true;
            }
        }

        // Aktualisiere die Karte:
        if (istGeaendert) {
            PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run() {
                    if (!_gui.isDisposed()) {

                        // Zeige die Zeit der letzte Statusnderung:
                        _gui.getLbLetzteStatusAenderung().setText(DatumFunktionen.getUhrzeitAlsString(System.currentTimeMillis()));

                        //
                        zeichneKarteNeu();
                    }
                }
            });
        }
    }

    /**
     * Zeichne die gesamte Karte.
     *
     * @param e PaintEvent.
     */
    protected void zeichneKarte(PaintEvent e) {

        // Zeichne die Bundesland-Karte:
        float massstabKarte = AR_MASSSTAB[_indexMassstab];

        // Definiere die Gre der Zeichnung:
        int breite = Math.round(massstabKarte * AR_KARTE_LAENGE_IN_PIXEL[AR_MASSTAB_IMAGE_INDEX[_indexMassstab]]) + 2 * ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL;
        int hoehe  = Math.round(massstabKarte * KARTE_HOEHE_IN_PIXEL) + 2 * ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL;

        _gui.getCanvasKarte().setSize(breite, hoehe);
        _gui.getCanvasKarte().setLocation(0, 0);
        _gui.getScompKarte().setMinSize(breite, hoehe);

        // ---
        // Benutze die doubleBufferImage-Methode
        // (zuerst zeichne die Karte auf dem lokalen Image imageDb
        // und zeige das erstellte Image auf Canvas):
        String dbi     = "double-buffer-image";
        Image  imageDb = (Image) _gui.getCanvasKarte().getData(dbi);

        // Image erstelle new, nur wenn ntig ist:
        if ((imageDb == null) || (imageDb.getBounds().width != breite) || (imageDb.getBounds().height != hoehe)) {
            imageDb = new Image(null, breite, hoehe);
            _gui.getCanvasKarte().setData(dbi, imageDb);
        }

        GC gcDb = new GC(imageDb);

        // Zeichne Hintergrund:
        gcDb.fillRectangle(0, 0, breite, hoehe);

        // Zeichne die Karte:
        int startX = ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL;
        int startY = ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL;

        gcDb.drawImage(_listeKartenImageProMassstab.get(_indexMassstab), startX, startY);

        // Zeichne die Zhlstellen:
        if (_gui.getCbtnZaehlstellen().getSelection()) {
            zeichneZaehlstellen(gcDb);
        }

        // Zeichne die Kri2_b:
        if (_gui.getCbtnKri2_b().getSelection()) {
            zeichneKri2(gcDb, false);
        }

        // Zeichne die Kri2b:
        if (_gui.getCbtnKri2b().getSelection()) {
            zeichneKri2(gcDb, true);
        }

        // Scrolle zur MausPosition:
        if (_istScrollen) {
            int[] koord = umwandeleWGS84DecKoordinatenInPixelKoordinaten(_xMausWgs84, _yMausWgs84, AR_MASSTAB_IMAGE_INDEX[_indexMassstab]);
            float x     = massstabKarte * koord[0] + ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL - _gui.getScompKarte().getClientArea().width / 2;
            float y     = massstabKarte * koord[1] + ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL - _gui.getScompKarte().getClientArea().height / 2;

            _gui.getScompKarte().setOrigin(Math.round(x), Math.round(y));
            _istScrollen = false;
        }

        // Zeichne das Kreuz:
        if (_xKreuzWgs84 > 0) {
            int   lKreuz = 130;
            int[] koord  = umwandeleWGS84DecKoordinatenInPixelKoordinaten(_xKreuzWgs84, _yKreuzWgs84, AR_MASSTAB_IMAGE_INDEX[_indexMassstab]);
            int   xK     = Math.round(massstabKarte * koord[0]) + ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL;
            int   yK     = Math.round(massstabKarte * koord[1]) + ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL;

            gcDb.drawLine(xK - lKreuz, yK, xK + lKreuz, yK);
            gcDb.drawLine(xK, yK - lKreuz, xK, yK + lKreuz);
        }

        // Zeige das erstellte Image auf Canvas:
        e.gc.drawImage(imageDb, 0, 0);
        gcDb.dispose();
    }

    /**
     * Zeichne die Karte neu.
     */
    private void zeichneKarteNeu() {
        try {
            _gui.getCanvasKarte().redraw(_gui.getScompKarte().getOrigin().x, _gui.getScompKarte().getOrigin().y, ermittleSichtbareBreiteDerKarte(), ermittleSichtbareHoeheDerKarte(), false);
        }
        catch (Exception e) {
            Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, "Fehler beim Zeichnen der bersichtskarte", e));
        }
    }

    /**
     * Zeichne alle Kri2.
     *
     * @param gc Grafik-Kontext.
     * @param istKri2b true, falls Kri2b gezeichnet werden sollen,
     * false - Kri2_b.
     */
    private void zeichneKri2(GC gc, boolean istKri2b) {

        // Merke die alte Grafik-Werte:
        Color farbeAltF = gc.getForeground();
        Color farbeAltB = gc.getBackground();
        int   lbAlt     = gc.getLineWidth();

        //
        float massstabKarte = AR_MASSSTAB[_indexMassstab];
        int   x;
        int   y;

        // ------------------------------------------------------------------
        int fsLinienBreite = AR_MASSTAB_FS_LINIE_BRIETE[_indexMassstab];
        int fsLinienLaenge = AR_MASSTAB_FS_LINIE_LAENGE[_indexMassstab];

        //
        Kri2        kri2;
        Color       farbe;
        int         xNeu;
        int         anzDe;
        List<int[]> listeKoordPixel;
        Color       farbeGrenz;

        if (istKri2b) {
            listeKoordPixel = _listePixelKoordAllerKri2bFuerImage.get(AR_MASSTAB_IMAGE_INDEX[_indexMassstab]);
            farbeGrenz      = FARBE_BEGRENZUNG_KRI2B;
        } else {
            listeKoordPixel = _listePixelKoordAllerKri2_bFuerImage.get(AR_MASSTAB_IMAGE_INDEX[_indexMassstab]);
            farbeGrenz      = FARBE_BEGRENZUNG_KRI2_B;
        }

        // Zeichne alle Kri2:
        for (int i = 0; i < listeKoordPixel.size(); i++) {
            if (istKri2b) {
                kri2 = _kri2Vew.getListeKri2b().get(i);
            } else {
                kri2 = _kri2Vew.getListeKri2_b().get(i);
            }

            //
            x = ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL + Math.round(listeKoordPixel.get(i)[0] * massstabKarte);
            y = ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL + Math.round(listeKoordPixel.get(i)[1] * massstabKarte);
            y = y - fsLinienLaenge / 2;
            gc.setLineWidth(fsLinienBreite);

            // De:
            anzDe = kri2.getListeDeSystemObjOhne255().size();

            int xKorr = 0;

            if (anzDe % 2 == 0) {
                xKorr = fsLinienBreite / 2;
            }

            for (int j = 0; j < anzDe; j++) {
                farbe = ermittleDeFarbe(kri2.getListeDeSystemObjOhne255().get(j).getPidOrId());
                gc.setForeground(farbe);
                xNeu = x - (anzDe / 2 - j) * fsLinienBreite + xKorr;
                gc.drawLine(xNeu, y, xNeu, y + fsLinienLaenge);
            }

            // Rahmen:
            gc.setLineWidth(1);
            gc.setForeground(farbeGrenz);

            int x0 = x - anzDe / 2 * fsLinienBreite + xKorr - fsLinienBreite + fsLinienBreite / 2;

            gc.drawRectangle(x0, y, anzDe * fsLinienBreite + 1, fsLinienLaenge);

            // Dach:
            gc.setBackground(farbeGrenz);

            int[] arKoord = new int[] { x0 - fsLinienBreite, y, x, y - fsLinienLaenge / 2, x0 + (anzDe + 1) * fsLinienBreite, y };

            gc.fillPolygon(arKoord);
        }

        // Setze die Grafik-Werte zurck:
        if (farbeAltF != null) {
            gc.setForeground(farbeAltF);
        }

        if (farbeAltB != null) {
            gc.setBackground(farbeAltB);
        }

        gc.setLineWidth(lbAlt);
    }

    /**
     * Zeichne alle Zhlstellen.
     *
     * @param gc Grafik-Kontext.
     */
    private void zeichneZaehlstellen(GC gc) {

        // Merke die alte Grafik-Werte:
        Color farbeAlt = gc.getForeground();
        int   lbAlt    = gc.getLineWidth();

        //
        float massstabKarte = AR_MASSSTAB[_indexMassstab];
        int   x;
        int   y;

        // ------------------------------------------------------------------
        // Die TestPunkte auf der Karte:
        gc.setForeground(FARBE_ROT);
        gc.setLineWidth(4);

        int         radius                    = 12;
        List<int[]> listeTestPunkteKoordPixel = _listePixelKoordTestPunktenFuerImage.get(AR_MASSTAB_IMAGE_INDEX[_indexMassstab]);

        for (int[] koord : listeTestPunkteKoordPixel) {
            x = ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL + Math.round(koord[0] * massstabKarte);
            y = ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL + Math.round(koord[1] * massstabKarte);
            gc.drawOval(x - radius / 2, y - radius / 2, radius, radius);
            gc.drawPoint(x, y);
        }

        // ------------------------------------------------------------------
        int fsLinienBreite = AR_MASSTAB_FS_LINIE_BRIETE[_indexMassstab];
        int fsLinienLaenge = AR_MASSTAB_FS_LINIE_LAENGE[_indexMassstab];

        //
        ZaehlStelle zsb;
        Color       farbe;
        int         xNeu;
        int         anzInRi;
        int         anzGegenRi;
        List<int[]> listeZsKoordPixel = _listePixelKoordAllerZsFuerImage.get(AR_MASSTAB_IMAGE_INDEX[_indexMassstab]);

        // Zeichne alle Zhlstellen:
        for (int i = 0; i < listeZsKoordPixel.size(); i++) {
            zsb = DatenVew.getInstanz().getListeZaehlStellen().get(i);

            //
            x = ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL + Math.round(listeZsKoordPixel.get(i)[0] * massstabKarte);
            y = ZEICHNUNG_RAHMEN_BREITE_IN_PIXEL + Math.round(listeZsKoordPixel.get(i)[1] * massstabKarte);
            y = y - fsLinienLaenge / 2;
            gc.setLineWidth(fsLinienBreite);

            // Mitte:
            gc.setForeground(FARBE_BEGRENZUNG_ZS);
            gc.drawLine(x, y, x, y + fsLinienLaenge);

            // in Richtung:
            anzInRi = zsb.ermittleArFsInRichtung().length;

            for (int j = 0; j < anzInRi; j++) {
                farbe = ermittleDeFarbe(zsb.ermittleArFsInRichtung()[j].getFsQuellePid());
                gc.setForeground(farbe);
                xNeu = x - (anzInRi - j) * fsLinienBreite;
                gc.drawLine(xNeu, y, xNeu, y + fsLinienLaenge);
            }

            // gegen Richtung:
            anzGegenRi = zsb.ermittleArFsGegenRichtung().length;

            for (int j = 0; j < anzGegenRi; j++) {
                farbe = ermittleDeFarbe(zsb.ermittleArFsGegenRichtung()[j].getFsQuellePid());
                gc.setForeground(farbe);
                xNeu = x + (anzGegenRi - j) * fsLinienBreite;
                gc.drawLine(xNeu, y, xNeu, y + fsLinienLaenge);
            }

            // Rahmen:
            gc.setLineWidth(1);
            gc.setForeground(FARBE_BEGRENZUNG_ZS);
            gc.drawRectangle(x - anzInRi * fsLinienBreite - fsLinienBreite + fsLinienBreite / 2, y, (anzInRi + anzGegenRi + 1) * fsLinienBreite + 1, fsLinienLaenge);
        }

        // Setze die Grafik-Werte zurck:
        if (farbeAlt != null) {
            gc.setForeground(farbeAlt);
        }

        gc.setLineWidth(lbAlt);
    }

    /**
     * Zeige die Zhlstellendokumentation.
     */
    private void zeigeZsDokumentation() {
        if (_zsbInMausPostion != null) {
            String ordnerName = "Zaehlstellendokumentation";
            String dateiName  = _zsbInMausPostion.getId() + ".pdf";
            String dateiPfad  = BundeslandVew.getInstanz().ermittleDateiImOrdnerBundeslandResource(String.format("%s%s%s", ordnerName, FILESEPARATOR, dateiName));

            if (new File(dateiPfad).exists()) {
                dateiPfad = "file://" + dateiPfad;

                Shell                 shell  = new Shell(_gui.getShell(), SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
                ZsDokumentationDialog dialog = new ZsDokumentationDialog(shell);

                dialog.setUrl(dateiPfad);
                dialog.open();
            } else {
                MessageDialog.openWarning(_gui.getShell(), "Warnung",                                         //
                                          "Die Datei " + NEWLINE + NEWLINE + dateiPfad + NEWLINE + NEWLINE +  //
                                          "mit der Zhlstellendokumentation existiert nicht!");
            }
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 13.06.14
