/*
 * @(#)DavArchivVew.java   08.10.13
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.projektlib.dav;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;
import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.dav.daf.main.archive.ArchiveAvailabilityListener;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestManager;

import de.inovat.buv.projektlib.rwservice.RWService;

//~ JDK IMPORTE ===============================================================

import java.util.HashSet;
import java.util.Set;

//~ KLASSEN ===================================================================

/**
 * Klasse realisiert den Listener fr die berwachung vom Status des Archives.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 0 $ / $Date: 2008-01-01 00:00:00 +0200 (So, 27 Mai 2007) $ / ($Author: LG $)
 */
public class DavArchivVew implements ArchiveAvailabilityListener, DavVerbindungsListener {

    /** Einzige Instanz der Klasse. */
    private static DavArchivVew _instanz = new DavArchivVew();

    //~ FELDER ================================================================

    /** Menge der fr nderungen vom Archiv-Status registrierten Klassen. */
    private final Set<IDavArchiv> _mengeArchivListener = new HashSet<IDavArchiv>();
    private ArchiveRequestManager _archiv;
    private Rahmenwerk            _rw;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt einzige Instanz der Klasse DavDatenVew.
     */
    private DavArchivVew() {
        _rw = RWService.getService().getRw();

        if (_rw.isOnline()) {
            _archiv = _rw.getDavVerbindung().getArchive();
            _archiv.addArchiveAvailabilityListener(this);
        }

        _rw.addDavVerbindungsListener(this);
    }

    //~ METHODEN ==============================================================

    /**
     * Registriert Klasse als Listener auf {@link IDavArchiv}.
     *
     * @param klasse Klasse, die das Interface {@link IDavArchiv}
     * implementiert und ber nderungen vom Archiv-Status informiert werden will.
     */
    public void addArchivListener(IDavArchiv klasse) {
        synchronized (_mengeArchivListener) {
            _mengeArchivListener.add(klasse);
        }
    }

    @Override
    public void archiveAvailabilityChanged(ArchiveRequestManager archiv) {
        benachrichtigeArchivListener();
    }

    /**
     * Benachrichtige jede als Listener auf {@link IDavArchiv} registrierte
     * Klasse ber die nderungen vom Archiv-Status.
     */
    private void benachrichtigeArchivListener() {

        // Fr die Iteration erzeuge das Array, weil man whrend der Benachrichtigung
        // die Menge mit ad/removeListener() ndern kann:
        IDavArchiv[] ar;

        synchronized (_mengeArchivListener) {
            ar = _mengeArchivListener.toArray(new IDavArchiv[_mengeArchivListener.size()]);
        }

        for (IDavArchiv klasse : ar) {
            klasse.archivStatusGeaendert(istArchivVerfuegbar());
        }
    }

    //~ GET METHODEN ==========================================================

    /**
     * Ermittle einzige Instanz der Klasse.
     *
     * @return Einzige Instanz der Klasse.
     */
    public static DavArchivVew getInstanz() {
        return _instanz;
    }

    //~ METHODEN ==============================================================

    /**
     * Ermittle, ob das Archiv verfgbar ist.
     *
     * @return true, das Archiv verfgbar ist,
     * false - sonst.
     *
     */
    public boolean istArchivVerfuegbar() {
        if (_archiv != null) {
            return _archiv.isArchiveAvailable();
        }

        return false;
    }

    /**
     * Lscht die Registrierung einer Klasse als Listener auf
     * {@link IDavArchiv}.
     *
     * @param klasse Klasse, die sich zuvor mittels
     * {@link #addArchivListener(IDavArchiv)} registriert hat.
     */
    public void removeArchivListener(IDavArchiv klasse) {
        synchronized (_mengeArchivListener) {
            _mengeArchivListener.remove(klasse);
        }
    }

    @Override
    public void verbindungGetrennt(DavVerbindungsEvent event) {
        if (_archiv != null) {
            _archiv.removeArchiveAvailabilityListener(this);
            _archiv = null;
        }

        benachrichtigeArchivListener();
    }

    @Override
    public boolean verbindungHalten(DavVerbindungsEvent event) {
        return false;
    }

    @Override
    public void verbindungHergestellt(DavVerbindungsEvent event) {
        if (_rw.isOnline()) {
            _archiv = _rw.getDavVerbindung().getArchive();
            _archiv.addArchiveAvailabilityListener(this);
        }

        benachrichtigeArchivListener();
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 17.10.13
