/*
 * @(#)AbstractView.java   30.09.13
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.projektlib.rechtevew.aktionen;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.buv.rw.basislib.berechtigung.BerechtigungEreignis;

import de.inovat.buv.projektlib.rechtevew.lib.IRechteAenderung;
import de.inovat.buv.projektlib.rechtevew.lib.RechteVew;

import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.PlatformUI;

//~ KLASSEN ===================================================================

/**
 * Klasse wird von Sichten implementiert, die mit einer Berechtigungsfunktion verbunden sind.
 * Die Berechtigungsfunktion reguliert das Recht auf die Sicht.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 0 $ / $Date: 2008-01-01 00:00:00 +0200 (So, 27 Mai 2007) $ / ($Author: LG $)
 */
public abstract class AbstrakteSichtMitBerechtigung extends ViewPart implements IRechteAenderung {

    /** ID der BerechtigungsFunktion, die reguliert ob die Sicht gezeigt werden darf. */
    private final String _funktion = getBerechtigungsFunktionId();

    /** Parent-Composite der Sicht. */
    private Composite _parent;

    //~ METHODEN ==============================================================

    @Override
    public void createPartControl(Composite parent) {
        _parent = parent;

        // -----------------------------------------------------------------
        // Prfe das Recht. Falls der Benutzer kein Recht auf die Sicht hat,
        // wird leeres GUI erstellt:
        boolean erlaubt = AktionenVew.getInstanz().pruefeRechtFuerSicht(_funktion, _parent, false, getPartName());

        // Fall das Recht vorhanden ist:
        if (erlaubt) {

            // -----------------------------------------------------------------
            // Hilfe
            String hilfeId = getHilfeContextId();

            if (hilfeId != null) {
                PlatformUI.getWorkbench().getHelpSystem().setHelp(parent, hilfeId);
            }

            // -----------------------------------------------------------------
            // GUI:
            Control gui = initGUI(parent);

            // -----------------------------------------------------------------
            // Melde die Sicht als Listener auf Rechtenderungen:
            RechteVew.addRechteListener(this, _funktion);

            // -----------------------------------------------------------------
            // DisposeListener:
            final AbstrakteSichtMitBerechtigung klasse = this;

            try {
                parent.addDisposeListener(new DisposeListener() {
                    public void widgetDisposed(final DisposeEvent e) {

                        // Lscht die Registrierung der Klasse als Listener auf RechteAenderung:
                        RechteVew.removeRechteListener(klasse);
                    }
                });
            }
            catch (Exception ignored) {}
        }
    }

    @Override
    public void freigabe(BerechtigungEreignis e) {}

    //~ GET METHODEN ==========================================================

    /**
     * Ermittle die ID der BerechtigungsFunktion der Sicht.
     *
     * @return ID der BerechtigungsFunktion (darf nicht null sein).
     */
    abstract public String getBerechtigungsFunktionId();

    /**
     * Ermittle die ID der ContextHilfe.
     *
     * @return ID der ContextHilfe oder null.
     */
    public String getHilfeContextId() {
        return null;
    }

    /**
     * Ermittle die Second-ID der Sicht.
     *
     * @return Second-ID der Sicht oder null, falls es keine Multi-Sicht ist.
     */
    public String getSecondViewId() {
        return getViewSite().getSecondaryId();
    }

    /**
     * Ermittle die ID der Sicht.
     *
     * @return ID der Sicht.
     */
    public String getViewId() {
        return getViewSite().getId();
    }

    //~ METHODEN ==============================================================

    /**
     * Initialisiere das GUI.
     *
     * @param parent Parent-Composite der Sicht.
     */
    abstract public Control initGUI(Composite parent);

    //~ SET METHODEN ==========================================================

    @Override
    public void setFocus() {}

    //~ METHODEN ==============================================================

    @Override
    public void sperrung(BerechtigungEreignis e) {

        // Zeige leeres GUI mit Meldung:
        AktionenVew.getInstanz().pruefeRechtFuerSicht(_funktion, _parent, true, getPartName());
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 28.10.13
