/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.projektlib.speicher.einstellungen;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.projektlib.speicher.einstellungen.SpeicherFunktionen.ArtOrt;

//~ KLASSEN ===================================================================

/**
 * Klasse beschreibt die Art und Ort von Speicher.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 9 $ / $Date: 2010-11-05 13:20:42 +0100 (Fr, 05. Nov 2010) $ / ($Author: LG $)
 */
public class SpeicherArtUndOrt {

    /**
     * Array von Art (allgemein/Benutzer) und Ort (DaV/lokal) der Einstellungen.
     * Falls der Speicher nicht synchronisiert wird,
     * definiert auch die Reihenfolge beim Lesen.
     *
     * WICHTIG_: Array darf nicht leer sein.
     */
    private ArtOrt[] _arArtOrt;

    /**
     * Legt fest, ob der Speicher synchronisiert werden soll.
     * Beim Synchronisieren wird auch das Datum mitgespeichert.
     */
    private boolean _synchronisieren;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse SpeicherArtUndOrt.
     *
     * @param arArtOrt Array von Art und Ort-Aufzhlungen.
     * WICHTIG_: Array darf nicht leer sein.
     *
     * @param synchronisieren Legt fest, ob der Speicher synchronisiert werden soll.
     */
    public SpeicherArtUndOrt(ArtOrt[] arArtOrt, boolean synchronisieren) {
        _arArtOrt        = arArtOrt;
        _synchronisieren = synchronisieren;
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'arArtOrt'.
     *
     * @return Wert der Eigenschaft 'arArtOrt'.
     *
     * @see #_arArtOrt
     */
    public ArtOrt[] getArArtOrt() {
        return _arArtOrt;
    }

    /**
     * Getter fr Eigenschaft 'synchronisieren'.
     *
     * @return Wert der Eigenschaft 'synchronisieren'.
     *
     * @see #_synchronisieren
     */
    public boolean isSynchronisieren() {
        return _synchronisieren;
    }

    //~ SET METHODEN ==========================================================

    /**
     * Setter fr Eigenschaft 'arArtOrt'.
     *
     * @param arArtOrt Wert zum Setzen der Eigenschaft 'arArtOrt'.
     *
     * @see #_arArtOrt
     */
    public void setArArtOrt(ArtOrt[] arArtOrt) {
        _arArtOrt = arArtOrt;
    }

    /**
     * Setter fr Eigenschaft 'synchronisieren'.
     *
     * @param synchronisieren Wert zum Setzen der Eigenschaft 'synchronisieren'.
     *
     * @see #_synchronisieren
     */
    public void setSynchronisieren(boolean synchronisieren) {
        _synchronisieren = synchronisieren;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 07.10.13
