/*
 * Copyright 2008 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kni, Kln
 * <p>
 * This file is part of de.inovat.pat.datkat2html
 * <p>
 * de.inovat.pat.datkat2html is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * <p>
 * de.inovat.pat.datkat2html is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.pat.datkat2html; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * <p>
 * Contact Information:<br>
 * inovat, Dipl.-Ing. H. C. Kni<br>
 * Klner Strae 30<br>
 * 50859 Kln<br>
 * Phone: +49 2234 4301 800<br>
 * mailto: info@inovat.de
 */
package de.inovat.pat.datkat2html.ausgabe.ascii;

import de.inovat.pat.datkat2html.Activator;
import de.inovat.pat.datkat2html.ausgabe.IAusgabe;
import de.inovat.pat.datkat2html.ausgabe.ReferenzInformation;
import de.inovat.pat.datkat2html.ausgabe.beschreibung.*;
import de.inovat.pat.datkat2html.ausgabe.beschreibung.Inhalt.InhaltTyp;
import org.eclipse.core.runtime.*;

import java.util.ArrayList;
import java.util.List;


/**
 * Testausgabeklasse des Datenkatalogs in ASCII
 * 
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 408 $ / $Date: 2010-04-06 08:14:07 +0200 (Di, 06. Apr 2010) $ / ($Author: HCK $)
 */

public class ASCIIAusgabe implements IAusgabe{
// ------------------------------------------------------------------------
// VARIABLENDEKLARATIONEN
// ------------------------------------------------------------------------

// Private ----------------------------------------------------------------
	private final String TABELLE_TRENNZEICHEN="-";
	private final String SPALTEN_TRENNZEICHEN=" |";
	private final String NIVEAU_TRENNZEICHEN="- ";

	/**
	 * Text fr die Tabelle wird ausgegeben, wenn keine Definition (Referenzierung) gefunden ist.
	 */
	//private final String UNBEKANNT="unbekannte Definition";

	/**
	 * Maximale Anzahl der Symbolen in der Tabellenspalte mit Info.
	 */
	private final int MAX_INFO=63;

	private ReferenzInformation _refInfo;

// ------------------------------------------------------------------------
// KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
// ------------------------------------------------------------------------

	public ASCIIAusgabe(String zielOrdner, ReferenzInformation refInfo) {
		_refInfo=refInfo;
	}

// ------------------------------------------------------------------------
// INTERFACE METHODEN
// ------------------------------------------------------------------------


// --------------------- Interface IAusgabe ---------------------



	@Override
	public void loescheAlteDatei() {
		// TODO Auto-generated method stub
	}

	@Override
	public void initialisieren(String KBPid) {
	}

	@Override
	public void abschliessen() {
		System.out.println();
		System.out.println("********************************************************************");
		System.out.println("********************************************************************");
		System.out.println();
	}

	@Override
	public String ermittleStartDatei() {
		// TODO Auto-generated method stub
		return "C:/Dokumente und Einstellungen/LG/Desktop/HTML-Buch/Datenkatalog/start.html";
	}

	@Override
	public void ausgebenKopfbereich(String kommentar, String langBezeichung) {
		// TODO Auto-generated method stub
	}

	@Override
	public IStatus ausgebenTabelleKvKb(
			List<KonfigurationsBereichBeschreibung> listeKbBeschreibungen) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public IStatus ausgebenAbhaengigkeitsmatrixKB(List<KonfigurationsBereichBeschreibung> listeKbBeschreibungen) {
		// TODO Auto-generated method stub
		System.out.println("***********************");
		for(String pidKB1: _refInfo.getMapDateiPfad().keySet()){
			System.out.println(pidKB1);
			for(String pidKB2: _refInfo.getMapDateiPfad().keySet()){
				if(!pidKB1.equals(pidKB2)){
					System.out.println("   "+pidKB2+"  "+_refInfo.istKBvomAnderenKBabhaengig(pidKB1, pidKB2));
				}
			}
		}
		System.out.println("***********************");
		return null;
	}

	@Override
	public IStatus ausgebenUebersichtListe(List<Uebersicht> listeUebersicht, String listenName) {
		if(!listeUebersicht.isEmpty()){
			System.out.println(" *************************************************************");
			System.out.println("bersicht: "+listenName);
			System.out.println();

			List<Object[]> listeZeilen=new ArrayList<Object[]>();

			for(Uebersicht zeile: listeUebersicht){
				Inhalt kurzInfo=null;
				if(zeile.getInfo()!=null){
					kurzInfo=zeile.getInfo().getKurzinfo();
				}
				listeZeilen.add(new Object[]{zeile.getPid(), zeile.getName(), kurzInfo});
				//listeZeilen.add(new Object[]{zeile.getPid(), zeile.getName(), zeile.getInfo()});
			}

			erzeugeTabelle(new String[] {"pid", "name", "kurzInfo"}, listeZeilen);

			System.out.println();
		}
		return null;
	}

	@Override
	public IStatus ausgebenKBTitel(KonfigurationsBereichBeschreibung kbB) {
		System.out.println("4 *************************************************************");
		System.out.println("Konfigurationverantwortliche: "+kbB.getVerantwortlich());
		System.out.println("KonfigurationBereich:");
		System.out.println();
		System.out.println("   PID:  "+kbB.getUebersicht().getPid());
		System.out.println("   Name: "+kbB.getUebersicht().getName());
		System.out.println();
		ausgebenInfo(kbB.getUebersicht().getInfo());
		return null;
	}

	@Override
	public IStatus ausgebenModellTitel() {
		System.out.println("5 *************************************************************");
		System.out.println("Model:");
		System.out.println();
		return null;
	}

	@Override
	public IStatus ausgebenObjekteTitel() {
		System.out.println("8 *************************************************************");
		System.out.println("Objekte:");
		System.out.println();

		return null;
	}

	@Override
	public IStatus ausgebenKonfigurationsAenderungenTabelle(List<KonfigurationsAenderungBeschreibung> listeKAB) {
		if(!listeKAB.isEmpty()){
			List<Object[]> listeZeilen=new ArrayList<Object[]>();

			/**
			 * <!ELEMENT konfigurationsAenderung  (#PCDATA | %zeichenauszeichnung.klasse; | %absatzauszeichnung.klasse;)* >
			 * <!ATTLIST konfigurationsAenderung
			 *		stand CDATA #REQUIRED
			 *		version CDATA #IMPLIED
			 *		autor CDATA #REQUIRED
			 *		grund CDATA #IMPLIED>
			 */
			for (KonfigurationsAenderungBeschreibung kaB: listeKAB){
				listeZeilen.add(new Object[]{kaB.getStand(), kaB.getVersion(),
						kaB.getAutor(), kaB.getGrund(), kaB.getInhalt()});
			}
			erzeugeTabelle(new String[] {"Stand", "Version", "Autor", "Grund", "Info"}, listeZeilen);

			System.out.println();
		}
		return null;
	}

	@Override
	public IStatus ausgebenMengenDefinitionBeschreibungListe(List<MengenDefinitionBeschreibung> listeMengenDefinitionBeschreibung){
		System.out.println("7 *************************************************************");
		System.out.println("Einzelbeschreibung: MengenDefinitionen");
		System.out.println();

		List<Object[]> listeZeilen=new ArrayList<Object[]>();

		for(MengenDefinitionBeschreibung mdb: listeMengenDefinitionBeschreibung){
			System.out.println();
			ausgebenUebersicht(mdb.getUebersicht());
			System.out.println();

			listeZeilen.add(new Object[]{mdb.getAenderbar(), mdb.getMindestens(), mdb.getHoechstens(), mdb.getReferenzierungsart()});

			erzeugeTabelle(new String[] {"nderbar", "mind.", "hchs.", "referenzierungsart"}, listeZeilen);

			System.out.println();

			System.out.println("Die Menge kann folgenede Elemente enthalten:");
			for(String element: mdb.getElemente()){
				System.out.println(element);
			}

			System.out.println();
		}
		return null;
	}

	@Override
	public IStatus ausgebenAttDefBeschreibungListe(
			List<AttributDefinitionBeschreibung> listeAttribbutDefinitionBeschreibung) {
		System.out.println("7 *************************************************************");
		System.out.println("Einzelbeschreibung: AttributDefinitionen");
		System.out.println();

		/*
		 * <!ELEMENT attributDefinition (
		 * 	info,
		 * 	( zeichenkette | ganzzahl | zeitstempel | objektReferenz | kommazahl ),
		 * 	default?
		 * ) >
		 * <!ATTLIST attributDefinition
		 * 			pid ID #REQUIRED
		 * 			name CDATA #IMPLIED
		 * >
		 */
		for(AttributDefinitionBeschreibung adb: listeAttribbutDefinitionBeschreibung){
			System.out.println();
			ausgebenUebersicht(adb.getUebersicht());
			if(adb.getDefault()!=null){
				System.out.println("Default Wert: "+adb.getDefault());
			}
			System.out.println();

			List<Object[]> listeZeilen=new ArrayList<Object[]>();
			for(String[] zeile: adb.getArrAttributTyp()){
				listeZeilen.add(new Object[]{zeile[0] ,zeile[1]});
			}
			erzeugeTabelleOhneKopfzeile(new int[] {20, 20}, 50, listeZeilen, true, true);


			/*
			 * <!ELEMENT bereich ( info? ) >
			 * <!ATTLIST bereich skalierung CDATA #IMPLIED >
			 * <!ATTLIST bereich minimum CDATA #REQUIRED >
			 * <!ATTLIST bereich maximum CDATA #IMPLIED >
			 * <!ATTLIST bereich einheit CDATA #IMPLIED >
			 */
			if(!adb.getBereiche().isEmpty()){
				System.out.println("\nBereiche:");
				listeZeilen=new ArrayList<Object[]>();

				for(BereichBeschreibung bereich: adb.getBereiche()){
					String skalierung=bereich.getSkalierung();
					if(skalierung==null){
						skalierung="";
					}
					String maximum=bereich.getMaximum();
					if(maximum==null){
						maximum="";
					}
					String einheit=bereich.getEinheit();
					if(einheit==null){
						einheit="";
					}

					listeZeilen.add(new Object[]{
							bereich.getMinimum(), maximum, skalierung, einheit, bereich.getInfo()});
				}
				erzeugeTabelle(new String[] {"Minimum", "Maximum", "Skalierung", "Einheit", "Info"}, listeZeilen);
			}

			/*
			 * <!ELEMENT zustand ( info? ) >
			 * <!ATTLIST zustand name CDATA #REQUIRED >
			 * <!ATTLIST zustand wert CDATA #REQUIRED >
			 */
			if(!adb.getZustaende().isEmpty()){
				System.out.println("\nZustnde:");
				listeZeilen=new ArrayList<Object[]>();

				for(ZustandBeschreibung zustand: adb.getZustaende()){
					listeZeilen.add(new Object[]{
							zustand.getName(), zustand.getWert(), zustand.getInfo()});
				}
				erzeugeTabelle(new String[] {"Name", "Wert", "Info"}, listeZeilen);
			}
			System.out.println();
		}
		return null;
	}

	@Override
	public IStatus ausgebenAttAuswahlDefBeschreibungListe(
			List<AttributauswahlDefinitionBeschreibung> listeAttributauswahlDefinitionBescreibung) {
		System.out.println("7 *************************************************************");
		System.out.println("Einzelbeschreibung: AttributauswahlDefinitionen");
		System.out.println();

		for(AttributauswahlDefinitionBeschreibung adb: listeAttributauswahlDefinitionBescreibung){
			System.out.println();
			ausgebenUebersicht(adb.getUebersicht());
			System.out.println("typ: "+adb.getTyp());
			ausgebenAuswahlBeschreibungListe(adb.getListeAuswahlBeschreibung());
		}
		return null;
	}

	@Override
	public IStatus ausgebenAspektDefBeschreibungListe(
			List<AspektDefinitionBeschreibung> listeAspektDefBeschreibung) {
		System.out.println("7 *************************************************************");
		System.out.println("Einzelbeschreibung: AspektDefinitionen");
		System.out.println();

		for(AspektDefinitionBeschreibung ab: listeAspektDefBeschreibung){
			System.out.println();
			ausgebenUebersicht(ab.getUebersicht());
			System.out.println("Code: "+ab.getCode());
		}

		return null;
	}

	@Override
	public IStatus ausgebenAtlDefBeschreibungListe(
			List<AttributlistenDefinitionBeschreibung> listeAtlDefBeschreibung) {
		System.out.println("7 *************************************************************");
		System.out.println("Einzelbeschreibung: AttributlistenDefinitionen");
		System.out.println();

		MultiStatus multiMeldung=new MultiStatus(Activator.PLUGIN_ID, 0,
				"Meldung fr die Attributlisten", null);

		for(AttributlistenDefinitionBeschreibung atlDefB: listeAtlDefBeschreibung){
			MultiStatus meldung=ausgebenAtlDefBeschreibung(atlDefB);
			if((meldung!=null)&&(meldung.getChildren().length>0)){
				multiMeldung.add(meldung);
			}
		}

		return multiMeldung;
	}

	@Override
	public IStatus ausgebenAtgDefBeschreibungListe(
			List<AttributgruppenDefinitionBeschreibung> listeAtgDefBeschreibung) {
		System.out.println("7 *************************************************************");
		System.out.println("Einzelbeschreibung: AttributgruppenDefinitionen");
		System.out.println();

		MultiStatus multiMeldung=new MultiStatus(Activator.PLUGIN_ID, 0,
				"Meldung fr die Attributgruppen", null);

		for(AttributgruppenDefinitionBeschreibung atgDefB: listeAtgDefBeschreibung){
			MultiStatus meldung=ausgebenAtgDefBeschreibung(atgDefB);
			if((meldung!=null)&&(meldung.getChildren().length>0)){
				multiMeldung.add(meldung);
			}
		}

		return multiMeldung;
	}

	@Override
	public IStatus ausgebenTypDefBeschreibungListe(
			List<TypDefinitionBeschreibung> listeTypDefBeschreibung) {
		System.out.println("7 *************************************************************");
		System.out.println("Einzelbeschreibung: TypDefinitionen");
		System.out.println();

		for(TypDefinitionBeschreibung typDefB: listeTypDefBeschreibung){
			ausgebenTypDefinitionBeschreibung(typDefB);
		}

		System.out.println();

		return null;
	}

	@Override
	public IStatus ausgebenKOBeschreibungListe(
			List<KonfigurationsObjektBeschreibung> listeKOB) {
		System.out.println("10 ************************************************************");
		System.out.println("Einzelbeschreibung: Konfigurationsobjekte");
		System.out.println();

		for(KonfigurationsObjektBeschreibung koB: listeKOB){
			ausgebenKOB(koB);
		}

		return null;
	}

// ------------------------------------------------------------------------
// SONSTIGE METHODEN
// ------------------------------------------------------------------------

	/**
	 * <!ELEMENT attributgruppenDefinition ( info, aspekt*, (attribut|attributliste|attributauswahl)+ ) >
	 * <!ATTLIST attributgruppenDefinition
	 *  	pid ID #REQUIRED
	 *  	name CDATA #IMPLIED
	 *  	konfigurierend (ja|nein) "nein"
	 *  	parametrierend (ja|nein) "nein"
	 *  	code CDATA #IMPLIED
	 * >
	 *
	 * @param atgDebB
	 */
	private MultiStatus ausgebenAtgDefBeschreibung(AttributgruppenDefinitionBeschreibung atgDefB){
 		System.out.println();
		ausgebenUebersicht(atgDefB.getUebersicht());

		System.out.println("Konfigurierend: "+atgDefB.getKonfigurierend());
		System.out.println("Parametrierend: "+atgDefB.getParametrierend());
		System.out.println("Code: "+atgDefB.getCode());
		System.out.println();

		// Ausgabe Aspekte:
		List<Object[]> listeZeilen=new ArrayList<Object[]>();
		for(AspektBeschreibung aspektB: atgDefB.getListeAspektBeschreibung()){
			listeZeilen.add(new Object[]{
					aspektB.getUebersicht().getPid(),
					aspektB.getKonfigurationsModus(), aspektB.getOnlineModus(),
					aspektB.getUebersicht().getInfo()});
		}
		if(!listeZeilen.isEmpty()){
			System.out.println("Aspekte:");
			erzeugeTabelle(new String[]{"PID", "konfigurationsModus", "onlineModus", "Info"}, listeZeilen);
		}

		// Ausgabe (attribut|attributliste|attributauswahl)+
		int[] arSpaltenGroesse = new int[]{40,40,20,10,10,120};
		int laenge = 200;
		erzeugeKopfzeile(arSpaltenGroesse,
				new String[]{"Name", "Attributtyp", "Typ", "Anzahl", "AnzahlIst", "Info"}, laenge);

		MultiStatus multiMeldung=ausgebenTeilAttOrAtlOrAta(atgDefB, 0, arSpaltenGroesse, laenge,
				new MultiStatus(Activator.PLUGIN_ID, 0,
						"Meldungen fr die Atributgruppe ["+atgDefB.getUebersicht().getPid()+"]", null));

		System.out.println();

		return multiMeldung;
	}

	private void ausgebenUebersicht(Uebersicht uebersicht){
		if(uebersicht.getName()!=null){
			System.out.println("Name: "+uebersicht.getName());
		}
		if(uebersicht.getPid()!=null){
			System.out.println("PID: "+uebersicht.getPid());
		}
		ausgebenInfo(uebersicht.getInfo());
	}

	private void ausgebenInfo(InfoBeschreibung info){
		if(info!=null){
			List<String> ausgabeListe;

			System.out.println("kurzInfo: ");
			ausgabeListe=zerlegeString(erzeugeAusgabeFuerInhalt(info.getKurzinfo()), 180);
			for(String ausgabe: ausgabeListe){
				System.out.println(ausgabe);
			}
			System.out.println();

			if(info.getBeschreibung()!=null){
				System.out.println("beschreibung: ");
				ausgabeListe=zerlegeString(erzeugeAusgabeFuerInhalt(info.getBeschreibung()), 180);
				for(String ausgabe: ausgabeListe){
					System.out.println(ausgabe);
				}
				System.out.println();
			}
		}
	}

	/**
	 * Zerlege den String. Als Zerlegkriterium wird '\n' benutzt.
	 * Zustzlich kontrolliere die Gre der Elementen:
	 * falls elementGroesse grer als 0 ist, teile grere Elemente
	 * in mehrere.
	 *
	 * @param ausgabe String, das zerlegt werden soll.
	 * @param elementGroesse falls elementGroesse grer als 0 ist,
	 * kontrolliere zustzlich die Gre der Elementen.
 	 * @return Zerlegtes String.
	 */
	private List<String> zerlegeString(String ausgabe, int elementGroesse){
		String text=ausgabe;
		List<String> liste=new ArrayList<String>();
		String element;
		// Suche den Zeilenumbruch und definiere die Elemente der Liste:
		int index=text.indexOf("\n");
		while(index>-1){
			element=text.substring(0, index);
			liste.add(element);

			text=text.substring(index+1);
			index=text.indexOf("\n");
		}

		if(!text.equals("")){
			liste.add(text);
		}
		// Falls die Gre der Elementen begrenzt werden soll:
		if(elementGroesse>0){
			List<String> listeGekuerzt=new ArrayList<String>();
			for(String el: liste){
				if(el.length()<=elementGroesse){
					listeGekuerzt.add(el);
				}
				else{
					while(el.length()>elementGroesse){
						element=el.substring(0, elementGroesse);
						listeGekuerzt.add(element);
						el=el.substring(elementGroesse);
					}
					if(!el.equals("")){
						listeGekuerzt.add(el);
					}
				}
			}
			liste=listeGekuerzt;
		}
		return liste;
	}

	/**
	 * Rekursive Methode.
	 * Erzeuge die Ausgabe fr die Instanz der Klasse Inhalt.
	 *
	 * @param inhalt Die Liste _listeInhalte vom Parameter inhalt wird untersucht
	 * und Textangabe von jedem Typ wird zu Ausgabe addiert.
	 *
	 * @return erzeugte Ausgabe.
	 */
	private String erzeugeAusgabeFuerInhalt(Inhalt inhalt){
		// TODO Formatierung
		/*
			tief, hoch, code, adresse, pre, wichtig, zitat, absatz,
			index, hinweis, kommentar,
			verweis, anker,
			abbildung, liste, numListe, referenz, tabelle,
			titel, listenPunkt, spalte, kopf, rumf, zeile, zelle,
			startInfo,
			unbekannt
		 */

		String ausgabe="";
		String ausgabe2="";
		if(inhalt!=null){
			InhaltTyp typ=inhalt.getTyp();

			switch (typ) {
			case string:
				// Alle Tabs und Leerzeichen am Anfang und Ende werden von Text gelscht.
				// (fr code, pre sollte die Formatierung bleiben.)
				ausgabe=ausgabe+inhalt.getText().replaceAll("\t", "").trim();
				break;
			case listenPunkt:
				ausgabe=ausgabe+"* ";
				break;
			case wichtig:
				ausgabe=ausgabe+" ";
				break;
			}

			for(Inhalt inhalt2: inhalt.getListeInhalte()){
				ausgabe2=erzeugeAusgabeFuerInhalt(inhalt2);
				ausgabe=ausgabe+ausgabe2;
			}

			// Keinen Zeilenumbruch fr folgende Typen: TODO
			if((typ!=InhaltTyp.tief)&&(typ!=InhaltTyp.hoch)
					&&(typ!=InhaltTyp.adresse)&&(typ!=InhaltTyp.wichtig)){
				if(!ausgabe2.equals("")){
					ausgabe=ausgabe+"\n";
				}
			}
		}
		return ausgabe;
	}

	/**
	 * Erzeuge die Kopfzeile einer Tabelle.
	 *
 	 * @param arSpaltenGroesse Die Gre jeder Spalte.
	 * @param arKopfzeile Titel jeder Spalte.
	 * @param laenge Die Lnge der TabellenTrennlinie.
	 */
	private void erzeugeKopfzeile(int[] arSpaltenGroesse, String[] arKopfzeile, int laenge){
		String trennlinie=erzeugeTrennlinie(laenge);

		int spaltenAnzahl=arSpaltenGroesse.length;
		if(spaltenAnzahl!=arKopfzeile.length){
			System.err.println("Die Lnge von Arrays arSpaltenGroesse und arKopfzeile sind nicht gleich!");
		}
		else{
			// Kopfzeile der Tabelle:
			String[] arFormat=erzeugeArrayFormat(arSpaltenGroesse);
			System.out.println(trennlinie);
			for (int i=0; i<spaltenAnzahl; i++){
				System.out.format(arFormat[i], arKopfzeile[i]);
			}
			System.out.println();
			System.out.println(trennlinie);
		}
	}

	/**
	 * Erzeuge ein Array von FormatString.
	 *
	 * @param arSpaltenGroesse Die Gre jeder Spalte.
	 * @return Array von FormatString.
	 */
	private String[] erzeugeArrayFormat(int[] arSpaltenGroesse){
		String[] arFormat=new String[arSpaltenGroesse.length];
		// Array von Formaten fr die Ausgabe von Zeilen:
		for (int i=0; i<arSpaltenGroesse.length; i++){
			if(i<arSpaltenGroesse.length-1){
				arFormat[i]=" %-"+arSpaltenGroesse[i]+"s"+SPALTEN_TRENNZEICHEN;
			}
			else{
				arFormat[i]=" %-"+arSpaltenGroesse[i]+"s";
			}
		}
		return arFormat;
	}

	/**
	 * Rekursive Methode.
	 * Ausgebe die Listen von Attributen, Attributauswahlen und Attributlisten (rekursiv).
	 *
	 * Die Spalte 'Attributtyp' referenziert auf Definition.
	 *
	 * (attribut|attributliste|attributauswahl)+
	 *
	 * @param atlDefBorAtgDefB Instanz der Klasse AttributlistenDefinitionBeschreibung
	 * oder AttributgruppenDefinitionBeschreibung, deren Listen ausgeben werden.
	 * @param niveau Definiert die Ebene von Aufruf.
  	 * @param arSpaltenGroesse Die Gre jeder Spalte.
	 * @param laenge Die Lnge der TabellenTrennlinie.

	 */
	private MultiStatus ausgebenTeilAttOrAtlOrAta(Object atlDefBorAtgDefB,
			int niveau, int[] arSpaltenGroesse, int laenge,
			MultiStatus multiMeldung){
		//TODO: falls Name oder Info oder Typ... null sind.

		String abstand="";
		for (int i=0; i<niveau; i++){
			abstand=abstand+NIVEAU_TRENNZEICHEN;
		}

		// Definiere Listen von Attributen, Attributauswahlen und Attributlisten:
		List<AttributBeschreibung> listeAttB = new ArrayList<AttributBeschreibung>();
		List<AttributauswahlBeschreibung> listeAttAuswahlB = new ArrayList<AttributauswahlBeschreibung>();
		List<AttributlistenBeschreibung> listeAtlB = new ArrayList<AttributlistenBeschreibung>();

		if (atlDefBorAtgDefB instanceof AttributlistenDefinitionBeschreibung) {
			AttributlistenDefinitionBeschreibung atlDefB = (AttributlistenDefinitionBeschreibung) atlDefBorAtgDefB;
			listeAttB = atlDefB.getListeAttributBeschreibung();
			listeAttAuswahlB = atlDefB.getListeAttributAuswahlBeschreibung();
			listeAtlB = atlDefB.getListeAttributListenBeschreibung();
		}
		else if (atlDefBorAtgDefB instanceof AttributgruppenDefinitionBeschreibung) {
			AttributgruppenDefinitionBeschreibung atgDefB = (AttributgruppenDefinitionBeschreibung) atlDefBorAtgDefB;
			listeAttB = atgDefB.getListeAttributBeschreibung();
			listeAttAuswahlB = atgDefB.getListeAttributAuswahlBeschreibung();
			listeAtlB = atgDefB.getListeAttributListenBeschreibung();
		}
		else{
			System.err.println("Falsche atlDefBorAtgDefB Parameter fr die Methode ausgebenTeilAttOrAtlOrAta");
		}

		List<Object[]> listeZeilen=new ArrayList<Object[]>();

		// Ausgabe von Attributen:
		System.out.println(abstand+" * Attribute:");
		for(AttributBeschreibung attb: listeAttB){
			AttributDefinitionBeschreibung attDefB=attb.getAttributDefinitionBeschreibung();
			String attTyp="unbekannt";
			String typ="";
			if(attDefB!=null){
				attTyp=attDefB.getUebersicht().getName();
				typ=attDefB.getArrAttributTyp()[0][1];
			}
			else{
				multiMeldung.add(new Status(IStatus.WARNING, Activator.PLUGIN_ID,
						"Keine Definition fr das Attribut ["+attb.getUebersicht().getPid()+"]"));
			}

			listeZeilen.add(new Object[]{abstand+attb.getUebersicht().getName(),attTyp, typ,
					attb.getAnzahl(), attb.getAnzahlIst(),
					attb.getUebersicht().getInfo()});
		}
		erzeugeTabelleOhneKopfzeile(arSpaltenGroesse, laenge, listeZeilen, false, true);

		// Ausgabe von Attributaswahl:
		System.out.println(abstand+" * Attributaswahl:");

		listeZeilen=new ArrayList<Object[]>();
		for(AttributauswahlBeschreibung ata:listeAttAuswahlB){
			AttributauswahlDefinitionBeschreibung ataDefB=ata.getAttAuswahlDefinitionBescreibung();
			String attTyp="unbekannt";
			String typ="";
			if(ataDefB!=null){
				attTyp=ataDefB.getUebersicht().getName();
				typ=ataDefB.getTyp();
			}
			else{
				multiMeldung.add(new Status(IStatus.WARNING, Activator.PLUGIN_ID,
						"Keine Definition fr die Attributauswahl ["+ata.getUebersicht().getPid()+"]"));
			}

			listeZeilen.add(new Object[]{abstand+ata.getUebersicht().getName(),attTyp, typ,
					"", "",ata.getUebersicht().getInfo()});
		}
		erzeugeTabelleOhneKopfzeile(arSpaltenGroesse, laenge, listeZeilen, false, true);

		// Ausgabe von AttributenListen:
		System.out.println(abstand+" * Attributlisten:");

		listeZeilen=new ArrayList<Object[]>();
		for(AttributlistenBeschreibung atlb: listeAtlB){
			AttributlistenDefinitionBeschreibung atlDefB_2=atlb.getAttListenDefinitionBeschreibung();
			String attTyp="unbekannt";
			String typ="";
			if(atlDefB_2!=null){
				attTyp=atlDefB_2.getUebersicht().getName();
			}
			else{
				multiMeldung.add(new Status(IStatus.WARNING, Activator.PLUGIN_ID,
						"Keine Definition fr das Attributliste ["+atlb.getUebersicht().getPid()+"]"));
			}

			listeZeilen.add(new Object[]{abstand+atlb.getUebersicht().getName(),attTyp, typ,
					atlb.getAnzahl(), atlb.getAnzahlIst(),
					atlb.getUebersicht().getInfo()});

			erzeugeTabelleOhneKopfzeile(arSpaltenGroesse, laenge, listeZeilen, false, true);

			listeZeilen.remove(listeZeilen.size()-1);

			// rekursiver Aufruf:
			if(atlDefB_2!=null){
				ausgebenTeilAttOrAtlOrAta(atlDefB_2, niveau+1, arSpaltenGroesse, laenge, multiMeldung);
			}
		}
		if(listeAtlB.isEmpty()){
			System.out.println(erzeugeTrennlinie(laenge));
		}

		return multiMeldung;
	}

	/**
	 * Erzeugt eine Tabelle ohne Kopfzeile.
	 *
	 * @param arSpaltenGroesse Die Gre jeder Spalte.
	 * @param laenge Die Lnge der TabellenTrennlinie.
	 * @param listeZeilen Liste der Zeilen. Jede Zeile ist ein Array von Objekten
	 * (jedes element kann von Typ String, oder KurzInfo sein).
	 * @param trennlinieOben Definiert ob die Tabelle eine Trennlinie oben hat.
	 * @param trennlinieUnten Definiert ob die Tabelle eine Trennlinie unten hat.
	 */
	private void erzeugeTabelleOhneKopfzeile(int[] arSpaltenGroesse, int laenge, List<Object[]> listeZeilen,
			boolean trennlinieOben, boolean trennlinieUnten){
		// TODO fr jede Verweisung kb und kv als ToolTip,
		//      Die Lnge der Spalte behalten (falls text grer wird, teilen...)

		int spaltenAnzahl=arSpaltenGroesse.length;

		String[] arFormat=erzeugeArrayFormat(arSpaltenGroesse);
		// Trennlinie oben:
		if(trennlinieOben){
			System.out.println(erzeugeTrennlinie(laenge));
		}
		// Ausgabe der Zeilen:
		for(Object[] zeile: listeZeilen){
			if(zeile.length!=spaltenAnzahl){
				System.err.println("Anzahl der Spalten in Zeile ist falsch!");
			}
			else{
				List<String> listeInfoZeilen;
				int infoSpalte=-1;
				for (int i=0; i<spaltenAnzahl; i++){
					listeInfoZeilen=new ArrayList<String>();
					// Ausgabe von Inhalt-Element:
					if (zeile[i] instanceof InfoBeschreibung) {
						InfoBeschreibung infoB = (InfoBeschreibung) zeile[i];
						infoSpalte=i;
						if(infoB!=null){
							String ausgabe;
							List<String> listeInhalt;
							// KurzInfo:
							if(infoB.getKurzinfo()!=null){
								ausgabe=erzeugeAusgabeFuerInhalt(infoB.getKurzinfo());
								listeInhalt=zerlegeString(ausgabe, MAX_INFO);
								if(listeInhalt.size()>0){
									System.out.format(arFormat[i], listeInhalt.get(0));
									listeInfoZeilen.addAll(listeInhalt);
									listeInfoZeilen.remove(0);
								}
							}
							// Beschreibung:
							if(infoB.getBeschreibung()!=null){
								ausgabe=erzeugeAusgabeFuerInhalt(infoB.getBeschreibung());
								listeInhalt=zerlegeString(ausgabe, MAX_INFO);
								listeInfoZeilen.addAll(listeInhalt);
							}
						}
					}
					// Ausgabe von Inhalt-Element:
					else if (zeile[i] instanceof Inhalt) {
						Inhalt inhalt = (Inhalt) zeile[i];
						infoSpalte=i;
						if(inhalt!=null){
							String ausgabe=erzeugeAusgabeFuerInhalt(inhalt);
							List<String> listeInhalt=zerlegeString(ausgabe, MAX_INFO);
							if(listeInhalt.size()>0){
								System.out.format(arFormat[i], listeInhalt.get(0));
								listeInfoZeilen.addAll(listeInhalt);
								listeInfoZeilen.remove(0);
							}
						}
					}
					else if (zeile[i]==null){
						System.out.format(arFormat[i], "");
					}
					// Ausgabe von Element der Zeile:
					else{
						System.out.format(arFormat[i], zeile[i]);
					}

					// Ausgabe InfoRest:
					for(String infoZeile: listeInfoZeilen){
						System.out.println();
						for(int j=0; j<infoSpalte; j++){
							System.out.format(arFormat[j],"");
						}
						System.out.format(arFormat[infoSpalte],infoZeile);
						for(int j=infoSpalte+1; j<spaltenAnzahl-1; j++){
							System.out.format(arFormat[j],"");
						}
					}
				}
				System.out.println();
			}
		}
		if(trennlinieUnten){
			System.out.println(erzeugeTrennlinie(laenge));
		}
	}

	/**
	 * Erzeuge eine Trennlinie.
	 * @param laenge Die Lnge dir Trennlinie.
	 * @return Trennlinie bestimmter Lnge.
	 */
	private String erzeugeTrennlinie(int laenge){
		String trennlinie="";
		for(int i=0; i<laenge; i++){
			trennlinie=trennlinie+TABELLE_TRENNZEICHEN;
		}
		return trennlinie;
	}

	/**
	 * <!ELEMENT attributlistenDefinition ( info, (attribut|attributliste|attributauswahl)+ ) >
	 * <!ATTLIST attributlistenDefinition
	 * 		pid ID #REQUIRED
	 * 		name CDATA #IMPLIED
	 * >
	 *
	 * @param atlDefB
	 */
	private MultiStatus ausgebenAtlDefBeschreibung(AttributlistenDefinitionBeschreibung atlDefB){
		System.out.println();
		ausgebenUebersicht(atlDefB.getUebersicht());
		System.out.println();

		int[] arSpaltenGroesse = new int[]{40,40,20,10,10,120};
		int laenge = 200;
		erzeugeKopfzeile(arSpaltenGroesse,
				new String[]{"Name", "Attributtyp", "Typ", "Anzahl", "AnzahlIst", "Info"}, laenge);

		MultiStatus multiMeldung=ausgebenTeilAttOrAtlOrAta(atlDefB, 0, arSpaltenGroesse, laenge,
				new MultiStatus(Activator.PLUGIN_ID, 0,
						"Meldungen fr die Atributliste ["+atlDefB.getUebersicht().getPid()+"]", null));

		System.out.println();

		return multiMeldung;
	}

	private void ausgebenAuswahlBeschreibungListe(List<AuswahlBeschreibung> listeAuswahlBeschreibung){
		if(!listeAuswahlBeschreibung.isEmpty()){
			List<Object[]> listeZeilen=new ArrayList<Object[]>();

			for(AuswahlBeschreibung ab: listeAuswahlBeschreibung){
				listeZeilen.add(new Object[]{ab.getUebersicht().getPid(),
						ab.getUebersicht().getName(), ab.getWert(), ab.getArt(), ab.getAnzahl(),
						ab.getAnzahlIst(), ab.getUebersicht().getInfo()});
			}

			erzeugeTabelle(new String[] {"pid", "Name", "wert", "art", "anzahl", "anzahlIst", "Info"}, listeZeilen);
			System.out.println();
		}
	}

	/**
	 * Erzeugt eine Tabelle mit der Kopfzeile.
	 * Die Breite der Spalten wird automatisch berechnet
	 * (fr Info wird MAX_INFO als maximale Gre genommen).
	 *
	 * @param arKopfzeile Titel jeder Spalte.
	 * @param listeZeilen Liste der Zeilen. Jede Zeile ist ein Array von Objekten
	 * (jedes element kann von Typ String, oder KurzInfo sein).
	 */
	private void erzeugeTabelle( String[] arKopfzeile, List<Object[]> listeZeilen){
		//
		int arLaenge=arKopfzeile.length;
		int[] arSpaltenGroesse = new int[arLaenge];

		// Initialisiere Array von Spaltengre:
		for(int j=0; j<arLaenge; j++){
			arSpaltenGroesse[j]=arKopfzeile[j].length();
		}

		// Array von Spaltengre wird fr maximale mgliche Lnge in der Spalte angepasst:
		for(Object[] zeile: listeZeilen){
			if(zeile.length!=arLaenge){
				System.err.println("Anzahl der Spalten in Zeile ist falsch!");
			}
			else{
				List<String> listeInfoZeilen;
				for (int i=0; i<arLaenge; i++){
					listeInfoZeilen=new ArrayList<String>();
					if (zeile[i] instanceof String) {
						String text = (String) zeile[i];
						if(text.length()>arSpaltenGroesse[i]){
							arSpaltenGroesse[i]=text.length();
						}
					}
					// Ausgabe von Inhalt-Element:
					else if (zeile[i] instanceof InfoBeschreibung) {
						InfoBeschreibung infoB = (InfoBeschreibung) zeile[i];
						if(infoB!=null){
							String ausgabe;
							List<String> listeInhalt;
							// KurzInfo:
							if(infoB.getKurzinfo()!=null){
								ausgabe=erzeugeAusgabeFuerInhalt(infoB.getKurzinfo());
								listeInhalt=zerlegeString(ausgabe, MAX_INFO);
								listeInfoZeilen.addAll(listeInhalt);
							}
							// Beschreibung:
							if(infoB.getBeschreibung()!=null){
								ausgabe=erzeugeAusgabeFuerInhalt(infoB.getBeschreibung());
								listeInhalt=zerlegeString(ausgabe, MAX_INFO);
								listeInfoZeilen.addAll(listeInhalt);
							}
						}
					}
					// Inhalt
					else if (zeile[i] instanceof Inhalt) {
						Inhalt inhalt = (Inhalt) zeile[i];
						if(inhalt!=null){
							List<String> listeInhalt=zerlegeString(erzeugeAusgabeFuerInhalt(inhalt), MAX_INFO);
							listeInfoZeilen.addAll(listeInhalt);
						}
					}
					else{
						if(zeile[i]!=null){
							System.err.println("Unbekannte Typ der Spalte "+zeile[i]);
						}
					}
					for(String infoZeile: listeInfoZeilen){
						if(infoZeile.length()>arSpaltenGroesse[i]){
							arSpaltenGroesse[i]=infoZeile.length();
						}
					}
				}
			}
		}
		// Definiere die Lnge der Trennlinie:
		int laenge=SPALTEN_TRENNZEICHEN.length()*arLaenge + 3;
		for(int j=0; j<arLaenge; j++){
			laenge=laenge+arSpaltenGroesse[j];
		}

		// Erzeuge die Tabelle:
		erzeugeTabelle(arSpaltenGroesse, arKopfzeile, laenge, listeZeilen);
	}

	private void ausgebenKOB(KonfigurationsObjektBeschreibung koB){
		System.out.println();
		System.out.println("************************************************************");
		ausgebenUebersicht(koB.getUebersicht());
		System.out.println("id: "+koB.getId());
		System.out.println("typ: "+koB.getTyp().getPid());
		TypDefinitionBeschreibung typDefB=koB.getTyp().getTypDefinitionBeschreibung();
		if(typDefB!=null){
			ausgebenInfo(koB.getTyp().getTypDefinitionBeschreibung().getUebersicht().getInfo());
		}
		if(!koB.getListeDatensatzBeschreibung().isEmpty()){
			System.out.println();
			System.out.println("Datensatzliste:");
			for(DatensatzBeschreibung datensatzB: koB.getListeDatensatzBeschreibung()){
				ausgebenDatensatzBeschreibung(datensatzB);
			}
		}

		if(!koB.getListeObjektMengeBeschreibung().isEmpty()){
			System.out.println();
			System.out.println("Objektmengeliste:");
			for(ObjektMengeBeschreibung objektMengeB: koB.getListeObjektMengeBeschreibung()){
				ausgebenObjektMengeBeschreibung(objektMengeB);
			}
		}

		if(!koB.getListeDefaultParameterBeschreibung().isEmpty()){
			System.out.println();
			System.out.println("Defaultparameterliste:");
			for(DefaultParameterBeschreibung defParB: koB.getListeDefaultParameterBeschreibung()){
				ausgebenDefaultParameterBeschreibung(defParB);
			}
		}
	}

	private void ausgebenDatensatzBeschreibung(DatensatzBeschreibung datensatzB){
		System.out.println();
		System.out.println("PID: "+datensatzB.getPid());
		System.out.println("Attributgruppe: "+datensatzB.getAtgB().getPid());
		if(datensatzB.getAtgB().getAttributgruppenDefinitionBeschreibung()!=null){
			ausgebenInfo(datensatzB.getAtgB().
					getAttributgruppenDefinitionBeschreibung().getUebersicht().getInfo());
		}
		System.out.println("Aspekt: "+datensatzB.getAspektB().getUebersicht().getPid());
		if(datensatzB.getAspektB().getAspektDefB()!=null){
			ausgebenInfo(datensatzB.getAspektB().getAspektDefB().getUebersicht().getInfo());
		}
		System.out.println();

		int[] arSpaltenGroesse=new int[]{120, 40};
		int laenge=160;
		erzeugeKopfzeile(arSpaltenGroesse, new String[]{"Name", "Wert"}, laenge);

		ausgebenTeilDatumOrDListeOrDFeld(datensatzB.getListeDatumBeschreibung(),
				datensatzB.getListeDatenlisteBeschreibung(),
				datensatzB.getListeDatenfeldBeschreibung(),
				0, arSpaltenGroesse, laenge);
	}

	private void ausgebenObjektMengeBeschreibung(ObjektMengeBeschreibung objektMengeB){
		System.out.println();
		System.out.println("Name: "+objektMengeB.getName());
		System.out.println("Verwaltung: "+objektMengeB.getVerwaltung());
		System.out.println();

		if(!objektMengeB.getListeElementBeschreibung().isEmpty()){
			List<Object[]> listeZeilen=new ArrayList<Object[]>();
			for(ElementBeschreibung elelmentB: objektMengeB.getListeElementBeschreibung()){
				InfoBeschreibung infoB=null;
				if(elelmentB.getUebersicht()!=null){
					infoB=elelmentB.getUebersicht().getInfo();
				}
				listeZeilen.add(new Object[]{elelmentB.getPid(), infoB});
			}
			erzeugeTabelle(new String[]{"Element PID", "Info"}, listeZeilen);
		}
	}

	/**
	 * <!ELEMENT typDefinition ( info, (erweitert+ | basis), (attributgruppe | menge)*, defaultParameter* ) >
	 * <!ATTLIST typDefinition
	 *  	pid ID #REQUIRED
	 *  	name CDATA #IMPLIED
	 *  	objektNamenPermanent (ja|nein) "nein"
	 *  	persistenzModus ( transient | persistent | persistentUndUngltigNachNeustart ) #IMPLIED
	 * >
	 *
	 * <!ELEMENT basis EMPTY >
	 * <!ATTLIST basis konfigurierend (ja|nein) #REQUIRED >
	 *
	 * @param typDefB
	 */
	private void ausgebenTypDefinitionBeschreibung(TypDefinitionBeschreibung typDefB){
		System.out.println();
		ausgebenUebersicht(typDefB.getUebersicht());

		System.out.println("ObjektNamenPermanent: "+typDefB.getObjektNamenPermanent());
		System.out.println("Persistenzmodus: "+typDefB.getPersistenzModus());
		System.out.println();

		List<Object[]> listeZeilen=new ArrayList<Object[]>();

		// (erweitert+ | basis)
		System.out.println("Vererbung:");
		if(!typDefB.getBasisKonfigurierend().equals("")){
			System.out.println("Basis Konfigurierend: "+typDefB.getBasisKonfigurierend());
		}
		if(!typDefB.getListeErweitert().isEmpty()){
			for(TypBeschreibung typB: typDefB.getListeErweitert()){
				String name="";
				InfoBeschreibung info=null;
				if(typB.getTypDefinitionBeschreibung()!=null){
					name=typB.getTypDefinitionBeschreibung().getUebersicht().getName();
					info=typB.getTypDefinitionBeschreibung().getUebersicht().getInfo();
				}
				listeZeilen.add(new Object[] {typB.getPid(), name, info});
			}
			System.out.println("Dieser Typ erweitert folgende Typen:");
			erzeugeTabelle(new String[]{"PID", "Name", "Info"}, listeZeilen);
		}
		System.out.println();

		listeZeilen=new ArrayList<Object[]>();

		// (attributgruppe | menge)*
		if(!typDefB.getListeAttributgruppenBeschreibung().isEmpty()){
			for(AttributgruppenBeschreibung atgB: typDefB.getListeAttributgruppenBeschreibung()){
				String name="";
				InfoBeschreibung info=null;
				if(atgB.getAttributgruppenDefinitionBeschreibung()!=null){
					name=atgB.getAttributgruppenDefinitionBeschreibung().getUebersicht().getName();
					info=atgB.getAttributgruppenDefinitionBeschreibung().getUebersicht().getInfo();
				}
				listeZeilen.add(new Object[]{atgB.getPid(), name, info});
			}
			System.out.println("Attributgruppen:");
			System.out.println("Folgende Attributgruppen sind bei diesem Objekttyp erlaubt:");
			erzeugeTabelle(new String[]{"PID", "Name", "Info"}, listeZeilen);
			System.out.println();
		}

		listeZeilen=new ArrayList<Object[]>();
		if(!typDefB.getListeMengenBeschreibung().isEmpty()){
			for(MengenBeschreibung mengenB: typDefB.getListeMengenBeschreibung()){
				String aenderbar="";
				String mind="";
				String hoech="";
				InfoBeschreibung info=null;
				if(mengenB.getMengenDefinitionBeschreibung()!= null){
					aenderbar=mengenB.getMengenDefinitionBeschreibung().getAenderbar();
					mind=mengenB.getMengenDefinitionBeschreibung().getMindestens();
					hoech=mengenB.getMengenDefinitionBeschreibung().getHoechstens();
					info=mengenB.getMengenDefinitionBeschreibung().getUebersicht().getInfo();
					//mengenB.getMengenDefinitionBeschreibung().getElemente()
				}
				listeZeilen.add(new Object[]{mengenB.getUebersicht().getPid(),
						mengenB.getUebersicht().getName(), mengenB.getErforderlich(),
						aenderbar, mind, hoech, info});
			}
			System.out.println("Mengen");
			System.out.println("Folgende Mengen mssen (knnen) bei diesem Objekttyp vorhanden sein:");
			erzeugeTabelle(new String[]{"PID", "Name", "erforderlich", "online nderbar", "Min.", "Max.", "Info"}, listeZeilen);
			System.out.println();
		}

		listeZeilen=new ArrayList<Object[]>();

		// defaultParameter*
		if(!typDefB.getListeDefaultParameterBeschreibung().isEmpty()){
			for(DefaultParameterBeschreibung defParB: typDefB.getListeDefaultParameterBeschreibung()){
				ausgebenDefaultParameterBeschreibung(defParB);
			}
		}
	}

	/**
	 * <!ELEMENT defaultParameter ((datum|datenliste|datenfeld)+) >
	 * <!ATTLIST defaultParameter
	 * 			typ NMTOKEN #IMPLIED
	 * 			attributgruppe NMTOKEN #REQUIRED
	 * >
	 *
	 * @param defParB
	 */
	private void ausgebenDefaultParameterBeschreibung(DefaultParameterBeschreibung defParB){
		System.out.println("DefaultParameter:");

		System.out.println("Typ: "+defParB.getTypUebersicht().getPid());
		ausgebenInfo(defParB.getTypUebersicht().getInfo());

		System.out.println("Attributgruppe: "+defParB.getAttributgruppe().getPid());
		if(defParB.getAttributgruppe().getAttributgruppenDefinitionBeschreibung()!=null){
			ausgebenInfo(defParB.getAttributgruppe().
					getAttributgruppenDefinitionBeschreibung().
					getUebersicht().getInfo());
		}
		System.out.println();

		int[] arSpaltenGroesse=new int[]{120, 40};
		int laenge=160;
		erzeugeKopfzeile(arSpaltenGroesse, new String[]{"Name", "Wert"}, laenge);

		ausgebenTeilDatumOrDListeOrDFeld(defParB.getListeDatumBeschreibung(),
				defParB.getListeDatenlisteBeschreibung(),
				defParB.getListeDatenfeldBeschreibung(),
				0, arSpaltenGroesse, laenge);
	}

	/**
	 * Rekursive Methode.
	 *
	 * Erzeugt die Tabelle mit den Daten der Art:
	 * (datum|datenliste|datenfeld)+
	 *
	 * <!ELEMENT datum EMPTY >
	 * <!ATTLIST datum
	 * 			name CDATA #REQUIRED
	 * 			wert CDATA #REQUIRED
	 * >
	 *
	 * <!ELEMENT datenliste ((datum|datenliste|datenfeld)+) >
	 * <!ATTLIST datenliste
	 * 			name CDATA #REQUIRED
	 * >
	 *
	 * <!ELEMENT datenfeld ((datum|datenliste)*) >
	 * <!ATTLIST datenfeld
	 * 			name CDATA #REQUIRED
	 * >
	 *
	 * @param listeDatumB
	 * @param listeDListeB
	 * @param listeDFeldB
	 * @param niveau
	 * @param arSpaltenGroesse
	 * @param laenge
	 */
	private void ausgebenTeilDatumOrDListeOrDFeld(List<DatumBeschreibung> listeDatumB,
			List<DatenlisteBeschreibung> listeDListeB, List<DatenfeldBeschreibung> listeDFeldB,
			int niveau, int[] arSpaltenGroesse, int laenge){
		String abstand="";
		for (int i=0; i<niveau; i++){
			abstand=abstand+NIVEAU_TRENNZEICHEN;
		}

		List<Object[]> listeZeilen=new ArrayList<Object[]>();

		// Ausgabe von datum:
		if(listeDatumB.size()>0){
			System.out.println(abstand+" * Datum");
			for(DatumBeschreibung datumB: listeDatumB){
				listeZeilen.add(new Object[] {abstand+datumB.getName(), datumB.getWert()});
			}
			erzeugeTabelleOhneKopfzeile(arSpaltenGroesse, laenge, listeZeilen, false, true);
		}

		// Ausgabe von datenliste:
		if(listeDListeB.size()>0){
			System.out.println(abstand+" * Datenliste");
			listeZeilen.clear();
			for(DatenlisteBeschreibung datenlisteB: listeDListeB){
				listeZeilen.add(new Object[] {abstand+datenlisteB.getName(), ""});
				erzeugeTabelleOhneKopfzeile(arSpaltenGroesse, laenge, listeZeilen, false, true);
				listeZeilen.clear();
				ausgebenTeilDatumOrDListeOrDFeld(datenlisteB.getListeDatumBeschreibung(),
						datenlisteB.getListeDatenlisteBeschreibung(),
						datenlisteB.getListeDatenfeldBeschreibung(),
						niveau+1, arSpaltenGroesse, laenge);
			}
		}

		// Ausgabe von datenfeld:
		if(listeDFeldB.size()>0){
			System.out.println(abstand+" * Datenfeld");
			listeZeilen.clear();
			for(DatenfeldBeschreibung datenfeldB: listeDFeldB){
				listeZeilen.add(new Object[] {abstand+datenfeldB.getName(), ""});
				erzeugeTabelleOhneKopfzeile(arSpaltenGroesse, laenge, listeZeilen, false, true);
				listeZeilen.clear();
				ausgebenTeilDatumOrDListeOrDFeld(datenfeldB.getListeDatumBeschreibung(),
						datenfeldB.getListeDatenlisteBeschreibung(),
						new ArrayList<DatenfeldBeschreibung>(),
						niveau+1, arSpaltenGroesse, laenge);
			}
		}
	}

	/**
	 * Erzeugt eine Tabelle mit der Kopfzeile.
	 *
	 * @param arSpaltenGroesse Die Gre jeder Spalte.
	 * @param arKopfzeile Titel jeder Spalte.
	 * @param laenge Die Lnge der TabellenTrennlinie.
	 * @param listeZeilen Liste der Zeilen. Jede Zeile ist ein Array von Objekten
	 * (jedes element kann von Typ String, oder KurzInfo sein).
	 */
	private void erzeugeTabelle(int[] arSpaltenGroesse, String[] arKopfzeile, int laenge, List<Object[]> listeZeilen){
		erzeugeKopfzeile( arSpaltenGroesse, arKopfzeile, laenge);
		erzeugeTabelleOhneKopfzeile(arSpaltenGroesse, laenge, listeZeilen, false, true);
	}
}
