/*
 * Copyright 2008 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kni, Kln
 * <p>
 * This file is part of de.inovat.pat.datkat2html
 * <p>
 * de.inovat.pat.datkat2html is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * <p>
 * de.inovat.pat.datkat2html is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.pat.datkat2html; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * <p>
 * Contact Information:<br>
 * inovat, Dipl.-Ing. H. C. Kni<br>
 * Klner Strae 30<br>
 * 50859 Kln<br>
 * Phone: +49 2234 4301 800<br>
 * mailto: info@inovat.de
 */
package de.inovat.pat.datkat2html.ausgabe.beschreibung;

import java.util.ArrayList;
import java.util.List;

/**
 * Klasse verwaltet dtd-Element datensatz.
 * 
 * <!ELEMENT datensatz ((datum|datenliste|datenfeld)+) >
 * <!ATTLIST datensatz
 * 			pid NMTOKEN #IMPLIED
 * 			attributgruppe NMTOKEN #IMPLIED
 * 			aspekt NMTOKEN "asp.eigenschaften"
 * >
 * 
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 408 $ / $Date: 2010-04-06 08:14:07 +0200 (Di, 06. Apr 2010) $ / ($Author: HCK $)
 */

public class DatensatzBeschreibung {
// ------------------------------------------------------------------------
// VARIABLENDEKLARATIONEN
// ------------------------------------------------------------------------

// Private ----------------------------------------------------------------
	/** Pid (kann null sein). */
	private String _pid;

	/** Attributgruppe (kann null sein). */
	private AttributgruppenBeschreibung _atgB;

	/** Aspekt */
	private AspektBeschreibung _aspektB;

	/** Liste der DatumBeschreibungen. */
	private List<DatumBeschreibung> _listeDatumBeschreibung=new ArrayList<DatumBeschreibung>();

	/** Liste der DatenlisteBeschreibungen. */
	private List<DatenlisteBeschreibung> _listeDatenlisteBeschreibung=new ArrayList<DatenlisteBeschreibung>();

	/** Liste der DatenfeldBeschreibungen. */
	private List<DatenfeldBeschreibung> _listeDatenfeldBeschreibung=new ArrayList<DatenfeldBeschreibung>();

// ------------------------------------------------------------------------
// KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
// ------------------------------------------------------------------------

	/**
	 * Erzeugt eine Instanz der Klasse DatensatzBeschreibung.
	 *
	 * @param pid
	 * @param atg
	 * @param aspektB
	 * @param listeDatumBeschreibung
	 * @param listeDatenlisteBeschreibung
	 * @param listeDatenfeldBeschreibung
	 */
	public DatensatzBeschreibung(String pid, AttributgruppenBeschreibung atg,
			AspektBeschreibung aspektB, List<DatumBeschreibung> listeDatumBeschreibung,
			List<DatenlisteBeschreibung> listeDatenlisteBeschreibung,
			List<DatenfeldBeschreibung> listeDatenfeldBeschreibung) {
		_pid = pid;
		_atgB = atg;
		_aspektB = aspektB;

		if(listeDatumBeschreibung!=null){
			_listeDatumBeschreibung = listeDatumBeschreibung;
		}
		else{
			_listeDatumBeschreibung = new ArrayList<DatumBeschreibung>();
		}

		if(listeDatenlisteBeschreibung!=null){
			_listeDatenlisteBeschreibung = listeDatenlisteBeschreibung;
		}
		else{
			_listeDatenlisteBeschreibung = new ArrayList<DatenlisteBeschreibung>();
		}

		if(listeDatenfeldBeschreibung!=null){
			_listeDatenfeldBeschreibung = listeDatenfeldBeschreibung;
		}
		else{
			_listeDatenfeldBeschreibung = new ArrayList<DatenfeldBeschreibung>();
		}
	}

// ------------------------------------------------------------------------
// GETTER / SETTER METHODEN
// ------------------------------------------------------------------------

	/**
	 * Getter fr Eigenschaft 'aspektB'.
	 *
	 * @return Wert der Eigenschaft 'aspektB'.
	 *
	 * @see  {#link _aspektB}
	 */
	public AspektBeschreibung getAspektB() {
		return _aspektB;
	}

	/**
	 * Getter fr Eigenschaft 'atgB'.
	 *
	 * @return Wert der Eigenschaft 'atgB'.
	 *
	 * @see  {#link _atgB}
	 */
	public AttributgruppenBeschreibung getAtgB() {
		return _atgB;
	}

	/**
	 * Getter fr Eigenschaft 'listeDatenfeldBeschreibung'.
	 *
	 * @return Wert der Eigenschaft 'listeDatenfeldBeschreibung'.
	 *
	 * @see  {#link _listeDatenfeldBeschreibung}
	 */
	public List<DatenfeldBeschreibung> getListeDatenfeldBeschreibung() {
		return _listeDatenfeldBeschreibung;
	}

	/**
	 * Getter fr Eigenschaft 'listeDatenlisteBeschreibung'.
	 *
	 * @return Wert der Eigenschaft 'listeDatenlisteBeschreibung'.
	 *
	 * @see  {#link _listeDatenlisteBeschreibung}
	 */
	public List<DatenlisteBeschreibung> getListeDatenlisteBeschreibung() {
		return _listeDatenlisteBeschreibung;
	}

	/**
	 * Getter fr Eigenschaft 'listeDatumBeschreibung'.
	 *
	 * @return Wert der Eigenschaft 'listeDatumBeschreibung'.
	 *
	 * @see  {#link _listeDatumBeschreibung}
	 */
	public List<DatumBeschreibung> getListeDatumBeschreibung() {
		return _listeDatumBeschreibung;
	}

	/**
	 * Getter fr Eigenschaft 'pid'.
	 *
	 * @return Wert der Eigenschaft 'pid'.
	 *
	 * @see  {#link _pid}
	 */
	public String getPid() {
		return _pid;
	}
}
