/*
 * @(#)Kri2.java   30.09.10
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.gtm.datvew.tls;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.Data;

import de.inovat.buv.projektlib.dav.DavDatenVew;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Klasse beschreibt die KRI-Objekte von Typ typ.kri2_b und typ.kri2b
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class Kri2 {

    /** Die nach Pid sortierte Liste aller zugehrigen Systemobjekte vom Typ De. */
    private List<SystemObject> _listeDeSystemObj = new ArrayList<SystemObject>();

    /**
     * Die nach Pid sortierte Liste aller zugehrigen Systemobjekte vom Typ De
     * ohne die Elemente mit dem DEKanal=255.
     */
    private List<SystemObject> _listeDeSystemObjOhne255 = new ArrayList<SystemObject>();

    /** ID der zugehrigen EAK. */
    private String _eakId;

    /** ID der Kri2. */
    private String _id;

    /** Name der Kri2. */
    private String _name;

    /** SystemObject. */
    private SystemObject _soKri2;

    /** xWGS84-Koordinate der Zhlstelle oder -1, falls nicht ermittelbar ist. */
    private double _xWgs84;

    /** yWGS84-Koordinate der Zhlstelle oder -1, falls nicht ermittelbar ist. */
    private double _yWgs84;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse Kri2.
     *
     * @param soKri2 SystemObject.
     */
    public Kri2(SystemObject soKri2) {
        _soKri2 = soKri2;
        _id     = soKri2.getPidOrId();
        _name   = soKri2.getName();

        //
        DataModel davKonf = DavDatenVew.getInstanz().getDav().getDataModel();

        // Koordinaten:
        Data oData = soKri2.getConfigurationData(davKonf.getAttributeGroup("atg.punktKoordinaten"));

        // XY-Koordinaten:
        try {
            _xWgs84 = oData.getScaledValue("x").doubleValue();
            _yWgs84 = oData.getScaledValue("y").doubleValue();
        }
        catch (Exception e) {
            _xWgs84 = -1;
            _yWgs84 = -1;
        }

        // EAK und Listen DEs:
        ObjectSet objectSetEAK = ((ConfigurationObject) soKri2).getObjectSet("Eak");

        if ((objectSetEAK != null) && (objectSetEAK.getElements().size() == 1)) {
            _eakId = objectSetEAK.getElements().get(0).getPidOrId();

            ObjectSet objectSetDe = ((ConfigurationObject) objectSetEAK.getElements().get(0)).getObjectSet("De");

            if (objectSetDe != null) {
                for (SystemObject sysObjDe : objectSetDe.getElements()) {
                    _listeDeSystemObj.add(sysObjDe);
                    Collections.sort(_listeDeSystemObj, new Comparator<SystemObject>() {
                        @Override
                        public int compare(SystemObject s1, SystemObject s2) {
                            return s1.getPidOrId().compareTo(s2.getPidOrId());
                        }
                    });
                }
            }

            int deKanal;

            for (SystemObject so : _listeDeSystemObj) {
                deKanal = so.getConfigurationData(davKonf.getAttributeGroup("atg.de")).getScaledValue("DEKanal").intValue();

                if (deKanal != 255) {
                    _listeDeSystemObjOhne255.add(so);
                }
            }
        } else {
            _eakId = "";
        }
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'eakId'.
     *
     * @return Wert der Eigenschaft 'eakId'.
     *
     * @see #_eakId
     */
    public String getEakId() {
        return _eakId;
    }

    /**
     * Getter fr Eigenschaft 'id'.
     *
     * @return Wert der Eigenschaft 'id'.
     *
     * @see #_id
     */
    public String getId() {
        return _id;
    }

    /**
     * Getter fr Eigenschaft 'listeDeSystemObj'.
     *
     * @return Wert der Eigenschaft 'listeDeSystemObj'.
     *
     * @see #_listeDeSystemObj
     */
    public List<SystemObject> getListeDeSystemObj() {
        return _listeDeSystemObj;
    }

    /**
     * Getter fr Eigenschaft 'listeDeSystemObjOhne255'.
     *
     * @return Wert der Eigenschaft 'listeDeSystemObjOhne255'.
     *
     * @see #_listeDeSystemObjOhne255
     */
    public List<SystemObject> getListeDeSystemObjOhne255() {
        return _listeDeSystemObjOhne255;
    }

    /**
     * Getter fr Eigenschaft 'name'.
     *
     * @return Wert der Eigenschaft 'name'.
     *
     * @see #_name
     */
    public String getName() {
        return _name;
    }

    /**
     * Getter fr Eigenschaft 'soKri2'.
     *
     * @return Wert der Eigenschaft 'soKri2'.
     *
     * @see #_soKri2
     */
    public SystemObject getSoKri2() {
        return _soKri2;
    }

    //~ METHODEN ==============================================================

    /**
     * Getter fr Eigenschaft 'xWgs84'.
     *
     * @return Wert der Eigenschaft 'xWgs84'.
     *
     * @see #_xWgs84
     */
    public double getxWgs84() {
        return _xWgs84;
    }

    /**
     * Getter fr Eigenschaft 'yWgs84'.
     *
     * @return Wert der Eigenschaft 'yWgs84'.
     *
     * @see #_yWgs84
     */
    public double getyWgs84() {
        return _yWgs84;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 17.09.12
