/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.gtm.rw;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;

import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.projektlib.rechtevew.aktionen.AktionenVew;
import de.inovat.buv.projektlib.rwservice.RWService;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IStartup;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchListener;
import org.eclipse.ui.PlatformUI;

import static de.inovat.buv.inovat.lib.konstanten.Konstanten.NEWLINE;

//~ KLASSEN ===================================================================

/**
 * ber diese Klasse werden Projekt-PlugIns in RahmenWerk geladen.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class StartUp implements IStartup, DavVerbindungsListener {

    /**
     * Stellt fest, ob die Anwendung geschlossen wird (global. Aktion 'Beenden' aufgerufen wird).
     */
    private static boolean _globalActionBeendenAufgerufen = false;

    //~ METHODEN ==============================================================

    /**
     * Definiere IWorkbenchListener um die Perspective beim Schlieen zurckzusetzen.
     */
    private void addListenerRWGeschlossen() {
        PlatformUI.getWorkbench().addWorkbenchListener(new IWorkbenchListener() {
            @Override
            public void postShutdown(IWorkbench workbench) {}
            @Override
            public boolean preShutdown(IWorkbench workbench, boolean forced) {

                // Setze die Variable:
                _globalActionBeendenAufgerufen = true;

                // Setze die inovatGTM-Perspektive zurck:
                AktionenVew.getInstanz().perspektiveZuruecksetzen(Perspective.ID);

                return true;
            }
        });
    }

    @Override
    public void earlyStartup() {

        // Listener RW-geschlossen:
        addListenerRWGeschlossen();

        // berwache die DaV-Verbindung:
        RWService.getService().getRw().addDavVerbindungsListener(this);
    }

    @Override
    public void verbindungGetrennt(DavVerbindungsEvent event) {
        final Display display = Display.getDefault();

        // Falls DaV nicht wegen des Beenden der Anwendung geschlossen wird,
        // zeige die Fehlermeldungen:
        if (!_globalActionBeendenAufgerufen) {
            Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, "Es gibt keine Verbindung mit dem DaV."));
            display.syncExec(new Runnable() {
                @Override
                public void run() {
                    Shell shell = new Shell(display, SWT.APPLICATION_MODAL);

                    // MessageDialog.openError(shell, "Fehler", "Es gibt keine Verbindung mit dem DaV." + NEWLINE + "Die Anwendung befindet sich in einem inkonsistenten Zustand." + NEWLINE + NEWLINE + "Bitte beenden Sie die Anwendung und starten sie anschlieend erneut.");
                    MessageDialog.openWarning(shell, "Warnung", String.format("Es besteht keine Verbindung zum Datenverteiler (DaV).%s%sEventuell funktionieren einige Plug-in nicht korrekt.", NEWLINE, NEWLINE));
                }
            });
        }
    }

    @Override
    public boolean verbindungHalten(DavVerbindungsEvent event) {
        return false;
    }

    @Override
    public void verbindungHergestellt(DavVerbindungsEvent event) {}
}


//~Formatiert mit 'inovat Kodierkonvention' am 29.10.13
