/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.inovat.lib.debug;

//~ NICHT JDK IMPORTE =========================================================

import com.swtdesigner.ResourceManager;

import de.inovat.buv.inovat.lib.Activator;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;

//~ JDK IMPORTE ===============================================================

import java.io.FileWriter;
import java.io.IOException;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Klasse verwaltet das GUI 'Protokollmeldungen'.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 12 $ / $Date: 2013-09-24 13:23:48 +0200 (Di, 24. Sep 2013) $ / ($Author: LG $)
 */
public class ProtokollGUIVerwaltung {

    /** Liste der Meldungselementen. */
    private List<MeldungsElement> _listeMeldungElementen = new ArrayList<MeldungsElement>();

    /** GUI-Instanz. */
    private ProtokollGUI _gui;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse ProtokollGUIVerwaltung.
     *
     * @param gui GUI-Instanz.
     */
    protected ProtokollGUIVerwaltung(ProtokollGUI gui) {
        _gui = gui;
    }

    //~ METHODEN ==============================================================

    /** Button 'Lschen' wird selektiert. */
    protected void btnLoeschenSelektiert() {
        if (_listeMeldungElementen.size() > 0) {
            if (MessageDialog.openQuestion(_gui.getShell(), "Frage", "Mchten Sie alle Meldungen aus dem Protokoll lschen?" + Log.NEWLINE + Log.NEWLINE + "ACHTUNG: der Prozess kann nicht rckgngig gemacht werden!")) {
                loescheAlleMeldungen();
            }
        }
    }

    /** Button 'Speichern' wird selektiert. */
    protected void btnSpeichernSelektiert() {
        waehleDateiUndSpeichereAlleMeldungen();
    }

    /**
     * Ermittle das MeldungsElement als IStatus-Instanz.
     *
     * @param index Index des MeldungsElements in der Liste alle Elemente.
     *
     * @return MeldungsElement als IStatus-Instanz oder null, falls index falsch ist.
     */
    public IStatus ermittleMeldungsElementAlsIStatus(int index) {
        IStatus meldung = null;

        if ((index >= 0) && (index < _listeMeldungElementen.size())) {
            MeldungsElement me = _listeMeldungElementen.get(index);

            meldung = erstelleIStatusRekursiv(me);
        }

        return meldung;
    }

    /**
     * Ermittle aller Meldungen im Textform.
     *
     * @return Aller Meldungen im Textform.
     */
    public String ermittleTextAllerMeldungen() {
        String alleMeldungen = "";

        for (MeldungsElement me : _listeMeldungElementen) {
            alleMeldungen = alleMeldungen + erstelleMeldungsTextRekursiv(me, 0);
        }

        return alleMeldungen;
    }

    /**
     * Erstelle die Instanz von IStatus fr die Meldung und ihre Kinder.
     *
     * @param me MeldungsElement.
     *
     * @return Instanz von IStatus fr die Meldung und ihre Kinder.
     */
    @SuppressWarnings({ "ThrowableResultOfMethodCallIgnored" })
    private IStatus erstelleIStatusRekursiv(MeldungsElement me) {
        IStatus meldung;

        if (me._listeKinder.size() > 0) {

            // Erstelle die MultiMeldung:
            meldung = Log.erzeugeMultiMeldung(me._meldung.getPlugin(), me._meldung.getMessage(), me._meldung.getException());
        } else {

            // Erstelle einfache Meldung:
            meldung = Log.erzeugeMeldung(me._meldung.getSeverity(), me._meldung.getPlugin(), me._meldung.getMessage(), me._meldung.getException());
        }

        // Fge die Kinder ein:
        IStatus meldungKind;

        for (MeldungsElement kind : me._listeKinder) {
            meldungKind = erstelleIStatusRekursiv(kind);
            ((MultiStatus) meldung).add(meldungKind);
        }

        return meldung;
    }

    /**
     * Erstelle den Text fr die Meldung. Besteht aus der Meldung und Exception.
     *
     * @param me MeldungsElement.
     *
     * @return Text fr die Meldung.
     */
    private String erstelleMeldungsText(MeldungsElement me) {
        StringBuilder meldung = new StringBuilder();

        // hole den Text:
        meldung.append(me._meldung.getMessage());

        // hole die ExceptionInformation:
        // noinspection ThrowableResultOfMethodCallIgnored
        Throwable e = me._meldung.getException();

        if (e != null) {
            meldung.append(Log.NEWLINE).append(Log.NEWLINE);
            meldung.append(e.toString()).append(Log.NEWLINE).append(Log.NEWLINE);

            // noinspection ThrowableResultOfMethodCallIgnored
            for (StackTraceElement el : me._meldung.getException().getStackTrace()) {
                meldung.append(el.toString()).append(Log.NEWLINE);
            }
        }

        return meldung.append(Log.NEWLINE).toString();
    }

    /**
     * Erstelle den Text fr die Meldung und ihre Kinder. Besteht aus der Meldungen und Exception.
     *
     * @param me MeldungsElement.
     * @param i  Definiert die Ebene.
     *
     * @return Text fr die Meldung und ihre Kinder.
     */
    private String erstelleMeldungsTextRekursiv(MeldungsElement me, int i) {

        // Initialisiere die Abstand- und Trennzeichen:
        String einAbstand = "    ";
        int    laenge     = 120;
        String trennZeichen;

        if (me._listeKinder.size() > 0) {
            trennZeichen = "=";
        } else {
            trennZeichen = "-";
        }

        // Erstelle den Abstand und die Trennlinie:
        StringBuffer abstand = new StringBuffer();

        for (int j = 0; j < i; j++) {
            abstand.append(einAbstand);
        }

        StringBuffer trennLinie = new StringBuffer();

        for (int j = 0; j < laenge - i * einAbstand.length(); j++) {
            trennLinie.append(trennZeichen);
        }

        // Fge die Trennlinie zu der Ausgabe zu:
        StringBuilder meldung = new StringBuilder();

        meldung.append(String.format("%s%s%n", abstand, trennLinie));

        // Fge den Status zu:
        meldung.append(String.format("%s%s%n", abstand, Log.ermittleStatusText(me._meldung.getSeverity())));

        // Fge die Trennlinie zu:
        meldung.append(String.format("%s%s%n", abstand, trennLinie));

        // Fge den einzeln MeldungsText zu:
        String[] arZeile = erstelleMeldungsText(me).split(Log.NEWLINE);

        for (String zeile : arZeile) {
            meldung.append(String.format("%s%s%n", abstand, zeile));
        }

        // Fge den MeldungsText der Kinder zu:
        for (MeldungsElement kind : me._listeKinder) {
            meldung.append(erstelleMeldungsTextRekursiv(kind, i + 1));
        }

        return meldung.toString();
    }

    /**
     * Fge eine Meldung zum Protokoll zu.
     *
     * @param meldung Meldung.
     */
    public void hinzufuegenMeldung(IStatus meldung) {
        MeldungsElement me = new MeldungsElement(meldung);

        _listeMeldungElementen.add(me);
        initBaum();
    }

    /**
     * Fge Meldungen zum Protokoll zu.
     *
     * @param listeMeldungen Liste der Meldung.
     */
    public void hinzufuegenMeldungen(List<IStatus> listeMeldungen) {
        for (IStatus meldung : listeMeldungen) {
            MeldungsElement me = new MeldungsElement(meldung);

            _listeMeldungElementen.add(me);
        }

        initBaum();
    }

    /** Initialisiere den MeldungsBaum. */
    private void initBaum() {
        _gui.getDisplay().syncExec(new Runnable() {
            @Override
            public void run() {
                _gui.getTreeViewerMeldungen().setInput(_listeMeldungElementen);
            }
        });
    }

    /** Lsche den Inhalt des Protokolls. */
    public void loescheAlleMeldungen() {
        _listeMeldungElementen.clear();
        initBaum();
    }

    /** Selektion im Baum ist gendert. Zeige den Text der selektierten Meldung. */
    protected void selektionImBaumGeaendert() {
        String meldung;

        // Definiere das selektierte MeldungsElement:
        try {
            StructuredSelection ss = (StructuredSelection) _gui.getTreeViewerMeldungen().getSelection();
            MeldungsElement     me = (MeldungsElement) ss.getFirstElement();

            // erstelle den Text:
            meldung = erstelleMeldungsText(me);
        }
        catch (Exception e) {
            meldung = "";
        }

        // Setze den Text:
        _gui.getTxtMeldung().setText(meldung);
    }

    /**
     * Speichere alle Meldungen in die ASCII-Datei.
     *
     * @param dateiName Name der ASCII-Datei.
     *
     * @return true, falls die Methode erfolgreich beendet wurde, false - sonst.
     */
    public boolean speichereAlleMeldungenInDieDatei(String dateiName) {
        boolean erfolgreich = true;

        try {
            FileWriter file = new FileWriter(dateiName);

            file.write(ermittleTextAllerMeldungen());
            file.close();
        }
        catch (IOException e) {
            erfolgreich = false;
            new MultiMeldung(Activator.PLUGIN_ID, Log.FEHLER, "Die Datei <" + dateiName + "> kann nicht erstellt/berschrieben werden!", e).zeige();
        }

        return erfolgreich;
    }

    /** ffne den Auswahldialog fr die Datei und speichere alle Meldungen in die ausgewhlte Datei. */
    public void waehleDateiUndSpeichereAlleMeldungen() {

        // Auswahldialog fr die Datei anzeigen:
        FileDialog dialog = new FileDialog(_gui.getShell(), SWT.SAVE);

        dialog.setFileName("Protokoll.txt");
        dialog.setFilterExtensions(new String[] { "*.txt" });
        dialog.setText("Protokolldatei zum Speichern whlen");

        String datei;

        if ((datei = dialog.open()) != null) {

            // Meldungen in die Datei speichern:
            if (!speichereAlleMeldungenInDieDatei(datei)) {
                MessageDialog.openError(_gui.getShell(), "Fehler", "Es ist ein Fehler beim Speichern der Meldungen in die Datei aufgetreten (s. Meldung unten) !");
            }
        }
    }

    //~ INNERE KLASSEN ========================================================

    /** ContentProvider fr den MeldungsBaum. */
    static class LogTreeContentProvider implements ITreeContentProvider {
        @Override
        public void dispose() {}

        //~ GET METHODEN ======================================================

        @Override
        public Object[] getChildren(Object parentElement) {
            if (parentElement instanceof MeldungsElement) {
                MeldungsElement me = (MeldungsElement) parentElement;

                return me._listeKinder.toArray(new MeldungsElement[me._listeKinder.size()]);
            }

            return new Object[0];
        }

        @Override
        public Object[] getElements(Object inputElement) {

            // noinspection unchecked
            return ((List<MeldungsElement>) inputElement).toArray(new MeldungsElement[((List<MeldungsElement>) inputElement).size()]);
        }

        @Override
        public Object getParent(Object element) {
            if (element instanceof MeldungsElement) {
                MeldungsElement me = (MeldungsElement) element;

                return me._vorfahr;
            }

            return null;
        }

        @Override
        public boolean hasChildren(Object element) {
            if (element instanceof MeldungsElement) {
                MeldungsElement me = (MeldungsElement) element;

                return (me._meldung.getChildren().length > 0);
            }

            return false;
        }

        //~ METHODEN ==========================================================

        @Override
        public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}
    }


    /** LabelProvider fr den MeldungsBaum. */
    static class LogTreeLabelProvider extends LabelProvider implements ITableLabelProvider {
        private static final Image infoImage    = ResourceManager.getPluginImage(Activator.getDefault(), "icons/pde/info_st_obj.gif");
        private static final Image okImage      = ResourceManager.getPluginImage(Activator.getDefault(), "icons/pde/ok_st_obj.gif");
        private static final Image errorImage   = ResourceManager.getPluginImage(Activator.getDefault(), "icons/pde/error_st_obj.gif");
        private static final Image warningImage = ResourceManager.getPluginImage(Activator.getDefault(), "icons/pde/warning_st_obj.gif");

        //~ METHODEN ==========================================================

        /**
         * Formatiere das String. Falls die Lnge des Strings kleiner als formatLength ist, fge "(0)" for dem String.
         *
         * @param sDatum       Das zu formatierte String.
         * @param formatLength Die ntige Lnge des Strings.
         *
         * @return Das formatierte String.
         */
        private static String formatString(String sDatum, int formatLength) {
            int    l   = formatLength - sDatum.length();
            String nul = "";

            for (int i = 1; i <= l; i++) {
                nul = nul + "0";
            }

            return nul + sDatum;
        }

        //~ GET METHODEN ======================================================

        @Override
        public Image getColumnImage(Object element, int columnIndex) {
            if ((columnIndex == 0) && (element instanceof MeldungsElement)) {
                MeldungsElement me = (MeldungsElement) element;

                switch (me._meldung.getSeverity()) {
                case Log.FEHLER :
                    return errorImage;

                case Log.INFO :
                    return infoImage;

                case Log.OK :
                    return okImage;

                case Log.WARNUNG :
                    return warningImage;
                }
            }

            return null;
        }

        @Override
        public String getColumnText(Object element, int columnIndex) {
            if (element instanceof MeldungsElement) {
                MeldungsElement me = (MeldungsElement) element;

                switch (columnIndex) {
                case 0 :

                    // Falls die Meldung mehr Zeilen hat,
                    // wird nur die 1.Zeile im Baum angezeigt.
                    String   text   = me._meldung.getMessage();
                    String[] arTest = text.split(Log.NEWLINE);

                    if (arTest.length > 1) {
                        text = arTest[0] + "...";
                    }

                    return text;

                case 1 :
                    return me._meldung.getPlugin();

                case 2 :
                    if (me._meldungsZeit != 0) {
                        return getDatumZeitAlsString(me._meldungsZeit);
                    }
                }
            }

            return null;
        }

        /**
         * Stellt Datum und Uhrzeit als String dar.
         *
         * @param datumLong - Anzahl der Millisekunden.
         *
         * @return Datum als String "dd.mm.yyyy  hh:mm:ss".
         */
        private static String getDatumZeitAlsString(long datumLong) {
            Calendar calender = new GregorianCalendar();

            calender.setTimeInMillis(datumLong);

            String yyyy = "" + calender.get(Calendar.YEAR);

            // Monat+1, da fngt von 0 Monat an.
            String mm = "" + (calender.get(Calendar.MONTH) + 1);
            String dd = "" + calender.get(Calendar.DAY_OF_MONTH);
            String hh = "" + calender.get(Calendar.HOUR_OF_DAY);
            String mi = "" + calender.get(Calendar.MINUTE);
            String ss = "" + calender.get(Calendar.SECOND);

            yyyy = formatString(yyyy, 4);
            mm   = formatString(mm, 2);
            dd   = formatString(dd, 2);
            hh   = formatString(hh, 2);
            mi   = formatString(mi, 2);
            ss   = formatString(ss, 2);

            return dd + "." + mm + "." + yyyy + "  " + hh + ":" + mi + ":" + ss;
        }
    }


    /** Klasse beschreibt ein MeldungsElement, das im MeldungsBaum angezeigt wird. */
    public class MeldungsElement {

        /** Kinder der Meldung. */
        private List<MeldungsElement> _listeKinder = new ArrayList<MeldungsElement>();

        /** Meldung. */
        private IStatus _meldung;

        /** Zeit der letzten Meldungsnderung oder 0. */
        private long _meldungsZeit;

        /** Vorfahr der Meldung oder null. */
        private MeldungsElement _vorfahr;

        //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==========

        /**
         * Erzeugt eine Instanz der Klasse MeldungsElement.
         *
         * @param meldung Meldung.
         */
        public MeldungsElement(IStatus meldung) {
            this(meldung, null);

            // Falls Meldung eine MultiMeldung ist,
            // setze die Meldungszeit:
            if (meldung instanceof MultiMeldung) {
                MultiMeldung mm = (MultiMeldung) meldung;

                _meldungsZeit = mm.getMeldungsZeit();

                // Falls die MultiMeldung ein Kind mit dem gleichen Text hat,
                // wird die Meldung reduziert (nur das Kind angezeigt):
                if ((mm.getChildren().length == 1) && (mm.getMessage().equals(mm.getChildren()[0].getMessage()))) {
                    MeldungsElement me = new MeldungsElement(mm.getChildren()[0], null);

                    _meldung     = me._meldung;
                    _listeKinder = me._listeKinder;
                }
            } else {
                _meldungsZeit = System.currentTimeMillis();
            }
        }

        /**
         * Erzeugt eine Instanz der Klasse MeldungsElement.
         *
         * @param meldung Meldung.
         * @param vorfahr Vorfahr der Meldung oder null.
         */
        private MeldungsElement(IStatus meldung, MeldungsElement vorfahr) {
            _meldung = meldung;
            _vorfahr = vorfahr;

            if (meldung instanceof MultiStatus) {
                MultiStatus ms = (MultiStatus) meldung;

                for (IStatus mel : ms.getChildren()) {

                    // Bearbeite die Kinder rekursiv:
                    MeldungsElement me = new MeldungsElement(mel, this);

                    _listeKinder.add(me);
                }
            }
        }

        //~ GET METHODEN ======================================================

        /**
         * Getter fr Eigenschaft 'listeKinder'.
         *
         * @return Wert der Eigenschaft 'listeKinder'.
         *
         * @see #_listeKinder
         */
        public List<MeldungsElement> getListeKinder() {
            return _listeKinder;
        }

        /**
         * Getter fr Eigenschaft 'meldung'.
         *
         * @return Wert der Eigenschaft 'meldung'.
         *
         * @see #_meldung
         */
        public IStatus getMeldung() {
            return _meldung;
        }

        /**
         * Getter fr Eigenschaft 'meldungsZeit'.
         *
         * @return Wert der Eigenschaft 'meldungsZeit'.
         *
         * @see #_meldungsZeit
         */
        public long getMeldungsZeit() {
            return _meldungsZeit;
        }

        /**
         * Getter fr Eigenschaft 'vorfahr'.
         *
         * @return Wert der Eigenschaft 'vorfahr'.
         *
         * @see #_vorfahr
         */
        public MeldungsElement getVorfahr() {
            return _vorfahr;
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 28.10.14
