/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.bast;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.plugin.gtm.bast.aktionen.AktionenBast;
import de.inovat.buv.plugin.gtm.bast.exportimport.ExportEinstellungenBastBand;
import de.inovat.buv.plugin.gtm.bast.gui.ExportGUI;
import de.inovat.buv.plugin.gtm.bast.hilfe.HilfeVew;
import de.inovat.buv.projektlib.bundesland.BundeslandVew;
import de.inovat.buv.projektlib.dav.DavDatenVew;
import de.inovat.buv.projektlib.rechtevew.aktionen.AbstrakteSichtMitBerechtigung;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import static de.inovat.buv.inovat.lib.konstanten.Konstanten.NEWLINE;

//~ KLASSEN ===================================================================

/**
 * Sicht BASt-Band Export.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 */
public class ViewExport extends AbstrakteSichtMitBerechtigung {

    /** ID der Sicht. */
    public static final String ID = "de.inovat.buv.plugin.gtm.bast.export.view";

    //~ FELDER ================================================================

    /** GUI-Instanz. */
    private ExportGUI _gui;

    //~ METHODEN ==============================================================

    @Override
    public void dispose() {
        if (_gui != null) {
            _gui.getFormToolkit().dispose();

            // Sobald das Fenster geschlossen wird, werden alle beim Datenverteiler angemeldeten
            // Objekte vom Datenverteiler abgemeldet.
            _gui.abmeldenDaV();
        }

        super.dispose();
    }

    //~ GET METHODEN ==========================================================

    @Override
    public String getBerechtigungsFunktionId() {
        return AktionenBast.FUNKTION_ID_EXPORT;
    }

    @Override
    public String getHilfeContextId() {
        return HilfeVew.getInstanz().ermittleIdFuerAnker(HilfeVew.ANKER_EXPORT_BAST);
    }

    //~ METHODEN ==============================================================

    @Override
    public Control initGUI(Composite parent) {
        String par                          = KonstantenGTMBast.AUFRUFPARAMETER_BAST_IMPORT_EXPORT;
        String pidKonfigurationsObjBastBand = DavDatenVew.getInstanz().getAufrufParameterAlsString(par);

        if (pidKonfigurationsObjBastBand == null) {
            MessageDialog.openError(getViewSite().getShell(), "Fehlermeldung",                                                                                                           //
                                    String.format("Um die Sicht BASt Band Export ffnen zu knnen muss der Kommandozeilenparameter:%s%s=bastBand.%sDauz%s entsprechend gepflegt sein.",  //
                                                  NEWLINE, par, BundeslandVew.getInstanz().getBundesland().getAbkuerzung().toLowerCase(), NEWLINE));
        } else {

            // Erzeuge GUI:
            ExportEinstellungenBastBand einstellungen = new ExportEinstellungenBastBand();

            _gui = new ExportGUI(parent, ID, einstellungen, pidKonfigurationsObjBastBand);
        }

        return _gui;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 29.10.13
