/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.bast.gui;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.gtm.datvew.lib.ZaehlStelle;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Die Klasse reprsentiert die Exportbenutzereinstellungen. Zu den Benutzereinstellungen
 * gehren Angaben wie die Zugangsinformationen zum FTP-Server. Die URL des FTP-Servers, ob es sich
 * um einen Export der BASt-Band Version 2004 oder 2007 handelt, diese Exportbenutzereinstellungen
 * entsprechen den Einstellungen die der Benutzer im GUI speichern kann, so dass er diese nicht bei
 * jedem Aufruf neu eingeben muss.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 395 $ / $Date: 2013-06-19 14:41:05 +0200 (Mi, 19 Jun 2013) $ / ($Author: LG $)
 */
public class ExportBenutzerEinstellungen {
    private String            _ftpServerPort                          = "21";
    private List<ZaehlStelle> _listeAusgewaehlterLangZeitZaehlStellen = new ArrayList<ZaehlStelle>();
    private boolean           _istFtp                                 = true;
    private boolean           _axl2009;

    /** Gibt die mglichen BASt-Band Formate an. */
    private boolean _bast2004;
    private boolean _bast2007;
    private String  _ftpServerBenutzer;
    private String  _ftpServerPasswort;
    private String  _ftpServerURL;
    private boolean _passivmodus;
    private boolean _vorhandeneDateiErsetzten;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * WICHTIG, Methode wird vom XML-Decoder bentigt.
     *
     */
    public ExportBenutzerEinstellungen() {}

    /**
     * Erzeugt eine Instanz der Klasse ExportBenutzerEinstellungen.
     *
     * @param listeAusgewaehlterZs Liste der ausgewhlten LangZeitZaehlStellen.
     * @param bast2004
     * @param bast2007
     * @param axl2009
     * @param ftpServerBenutzer gewhlte ftpServer Benutzer.
     * @param ftpServerPasswort gewhlte ftpServer Passwort.
     * @param ftpServerURL gewhlte ftpServer URL.
     * @param ftpServerPort gewhlte ftpServer Port.
     * @param passivmodus Gibt an, ob FTP-Verbindung im passivmodus aufgebaut werden soll.
     * @param istFtp Servertyp: FTP (true) - SFTP (false).
     * @param vorhandeneDateiErsetzten boolesche Variable die angibt, ob die vorhandene Datei ersetzt werden soll.
     */
    public ExportBenutzerEinstellungen(List<ZaehlStelle> listeAusgewaehlterZs,               //
                                       boolean bast2004, boolean bast2007, boolean axl2009,  //
                                       String ftpServerBenutzer, String ftpServerPasswort,   //
                                       String ftpServerURL, String ftpServerPort,            //
                                       boolean passivmodus, boolean istFtp,                  //
                                       boolean vorhandeneDateiErsetzten) {
        _listeAusgewaehlterLangZeitZaehlStellen = listeAusgewaehlterZs;
        _bast2004                               = bast2004;
        _bast2007                               = bast2007;
        _axl2009                                = axl2009;
        _ftpServerBenutzer                      = ftpServerBenutzer;
        _ftpServerPasswort                      = ftpServerPasswort;
        _ftpServerURL                           = ftpServerURL;
        _ftpServerPort                          = ftpServerPort;
        _passivmodus                            = passivmodus;
        _istFtp                                 = istFtp;
        _vorhandeneDateiErsetzten               = vorhandeneDateiErsetzten;
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'ftpServerBenutzer'.
     *
     * @return Wert der Eigenschaft 'ftpServerBenutzer'.
     *
     * @see #_ftpServerBenutzer
     */
    public String getFtpServerBenutzer() {
        return _ftpServerBenutzer;
    }

    /**
     * Getter fr Eigenschaft 'ftpServerPasswort'.
     *
     * @return Wert der Eigenschaft 'ftpServerPasswort'.
     *
     * @see #_ftpServerPasswort
     */
    public String getFtpServerPasswort() {
        return _ftpServerPasswort;
    }

    /**
     * Getter fr Eigenschaft 'ftpServerPort'.
     *
     * @return Wert der Eigenschaft 'ftpServerPort'.
     *
     * @see #_ftpServerPort
     */
    public String getFtpServerPort() {
        return _ftpServerPort;
    }

    /**
     * Getter fr Eigenschaft 'ftpServerURL'.
     *
     * @return Wert der Eigenschaft 'ftpServerURL'.
     *
     * @see #_ftpServerURL
     */
    public String getFtpServerURL() {
        return _ftpServerURL;
    }

    /**
     * Getter fr Eigenschaft 'listeAusgewaehlterLangZeitZaehlStellen'.
     *
     * @return Wert der Eigenschaft 'listeAusgewaehlterLangZeitZaehlStellen'.
     *
     * @see #_listeAusgewaehlterLangZeitZaehlStellen
     */
    public List<ZaehlStelle> getListeAusgewaehlterLangZeitZaehlStellen() {
        return _listeAusgewaehlterLangZeitZaehlStellen;
    }

    /**
     * Getter fr Eigenschaft 'axl2009'.
     *
     * @return Wert der Eigenschaft 'axl2009'.
     *
     * @see #_axl2009
     */
    public boolean isAxl2009() {
        return _axl2009;
    }

    /**
     * Getter fr Eigenschaft 'bast2004'.
     *
     * @return Wert der Eigenschaft 'bast2004'.
     *
     * @see #_bast2004
     */
    public boolean isBast2004() {
        return _bast2004;
    }

    /**
     * Getter fr Eigenschaft 'bast2007'.
     *
     * @return Wert der Eigenschaft 'bast2007'.
     *
     * @see #_bast2007
     */
    public boolean isBast2007() {
        return _bast2007;
    }

    /**
     * Getter fr Eigenschaft 'istFtp'.
     *
     * @return Wert der Eigenschaft 'istFtp'.
     *
     * @see #_istFtp
     */
    public boolean isIstFtp() {
        return _istFtp;
    }

    /**
     *     Getter fr Eigenschaft 'passivmodus'.
     *
     *     @return Wert der Eigenschaft 'passivmodus'.
     *
     *     @see #_passivmodus
     */
    public boolean isPassivmodus() {
        return _passivmodus;
    }

    /**
     * Getter fr Eigenschaft 'vorhandeneDateiErsetzten'.
     *
     * @return Wert der Eigenschaft 'vorhandeneDateiErsetzten'.
     *
     * @see #_vorhandeneDateiErsetzten
     */
    public boolean isVorhandeneDateiErsetzten() {
        return _vorhandeneDateiErsetzten;
    }

    //~ SET METHODEN ==========================================================

    /**
     * Setter fr Eigenschaft 'axl2009'.
     *
     * @param axl2009 Wert zum Setzen der Eigenschaft 'axl2009'.
     *
     * @see #_axl2009
     */
    public void setAxl2009(boolean axl2009) {
        _axl2009 = axl2009;
    }

    /**
     * Setter fr Eigenschaft 'bast2004'.
     *
     * @param bast2004 Wert zum Setzen der Eigenschaft 'bast2004'.
     *
     * @see #_bast2004
     */
    public void setBast2004(boolean bast2004) {
        _bast2004 = bast2004;
    }

    /**
     * Setter fr Eigenschaft 'bast2007'.
     *
     * @param bast2007 Wert zum Setzen der Eigenschaft 'bast2007'.
     *
     * @see #_bast2007
     */
    public void setBast2007(boolean bast2007) {
        _bast2007 = bast2007;
    }

    /**
     * Setter fr Eigenschaft 'ftpServerBenutzer'.
     *
     * @param ftpServerBenutzer Wert zum Setzen der Eigenschaft 'ftpServerBenutzer'.
     *
     * @see #_ftpServerBenutzer
     */
    public void setFtpServerBenutzer(String ftpServerBenutzer) {
        _ftpServerBenutzer = ftpServerBenutzer;
    }

    /**
     * Setter fr Eigenschaft 'ftpServerPasswort'.
     *
     * @param ftpServerPasswort Wert zum Setzen der Eigenschaft 'ftpServerPasswort'.
     *
     * @see #_ftpServerPasswort
     */
    public void setFtpServerPasswort(String ftpServerPasswort) {
        _ftpServerPasswort = ftpServerPasswort;
    }

    /**
     * Setter fr Eigenschaft 'ftpServerPort'.
     *
     * @param ftpServerPort Wert zum Setzen der Eigenschaft 'ftpServerPort'.
     *
     * @see #_ftpServerPort
     */
    public void setFtpServerPort(String ftpServerPort) {
        _ftpServerPort = ftpServerPort;
    }

    /**
     * Setter fr Eigenschaft 'ftpServerURL'.
     *
     * @param ftpServerURL Wert zum Setzen der Eigenschaft 'ftpServerURL'.
     *
     * @see #_ftpServerURL
     */
    public void setFtpServerURL(String ftpServerURL) {
        _ftpServerURL = ftpServerURL;
    }

    /**
     * Setter fr Eigenschaft 'istFtp'.
     *
     * @param istFtp Wert zum Setzen der Eigenschaft 'istFtp'.
     *
     * @see #_istFtp
     */
    public void setIstFtp(boolean istFtp) {
        _istFtp = istFtp;
    }

    /**
     * Setter fr Eigenschaft 'listeAusgewaehlterLangZeitZaehlStellen'.
     *
     * @param listeAusgewaehlterLangZeitZaehlStellen Wert zum Setzen der Eigenschaft 'listeAusgewaehlterLangZeitZaehlStellen'.
     *
     * @see #_listeAusgewaehlterLangZeitZaehlStellen
     */
    public void setListeAusgewaehlterLangZeitZaehlStellen(List<ZaehlStelle> listeAusgewaehlterLangZeitZaehlStellen) {
        _listeAusgewaehlterLangZeitZaehlStellen = listeAusgewaehlterLangZeitZaehlStellen;
    }

    /**
     * Setter fr Eigenschaft 'passivmodus'.
     *
     * @param passivmodus Wert zum Setzen der Eigenschaft 'passivmodus'.
     *
     * @see #_passivmodus
     */
    public void setPassivmodus(boolean passivmodus) {
        _passivmodus = passivmodus;
    }

    /**
     * Setter fr Eigenschaft 'vorhandeneDateiErsetzten'.
     *
     * @param vorhandeneDateiErsetzten Wert zum Setzen der Eigenschaft 'vorhandeneDateiErsetzten'.
     *
     * @see #_vorhandeneDateiErsetzten
     */
    public void setVorhandeneDateiErsetzten(boolean vorhandeneDateiErsetzten) {
        _vorhandeneDateiErsetzten = vorhandeneDateiErsetzten;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 19.06.13
