/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.bast.gui;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.inovat.lib.funktionen.datum.DatumFunktionen;
import de.inovat.buv.inovat.lib.gui.widget.DatumZeitOhneSekunden;
import de.inovat.buv.plugin.gtm.bast.Activator;
import de.inovat.buv.projektlib.dav.DavArchivVew;
import de.inovat.buv.projektlib.dav.IDavArchiv;

import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

//~ KLASSEN ===================================================================

/**
 * Die Klasse beschreibt das GUI fr den Export.
 * In Konstruktor muss man die Instanz von IExportEinstellungen bergeben,
 * wo die Export-Methoden implementiert sind.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 394 $ / $Date: 2013-06-17 14:37:33 +0200 (Mo, 17 Jun 2013) $ / ($Author: LG $)
 */
public class ExportGUI extends Composite implements IParameterAenderung, IDavArchiv {
    private final FormToolkit formToolkit = new FormToolkit(Display.getDefault());

    /** Butten zum Abbrechen des Exportes. */
    private Button _btnAbbrechen;

    /** Button zum Exportieren. */
    private Button _btnExportieren;

    /** Button lschen der Exporteinstellung. */
    private Button _btnloeschenExporteinstellung;

    /** Button Speichern. */
    private Button _btnSpeichern;
    private Button _cbtnPassivmodus;

    /** Checkbox nderung der Datenstruktur. */
    private Button _checkAenderungDatenstruktur;

    /** Checkbox die angibt ob die ggf. schon erzeugte Datei auf dem Server berschrieben werden soll. */
    private Button _checkDateiErsetzen;

    /** Combobox mit den gespeicherten Exporteinstellungen. */
    private Combo _comboGespeicherteExporteinstellungen;

    /** Field description */
    private Composite _compositeAllgemein;

    /** Field description */
    private DatumZeitOhneSekunden _datumBis;

    /** Datum von. */
    private DatumZeitOhneSekunden _datumVon;

    /** Gruppe fr die Exporteinstellungen. */
    private Group _groupExport;

    /** Instanz der Klasse ExportGUIVerwaltung. */
    private ExportGUIVerwaltung _guiVew;

    /** Label fr die Anzeige der Langzeitzhlstellenauswahl. */
    private Label _lbLangZeitZaehlStellenAuswahl;

    /** Label Restzeit. */
    private Label _lbRestzeit;

    /** Field description */
    private ProgressBar _prgFortschrittsanzeige1;

    /** Radiobutton BASt-Export Axlastdaten 2009. */
    private Button _radioAXL2009;

    /** Radiobutton BASt-Export 2004. */
    private Button _radioBASt2004;

    /** Radiobutton BASt-Export 2007. */
    private Button _radioBASt2007;
    private Button _rbtnFtp;
    private Button _rbtnSftp;

    /** Text Benutzer. */
    private Text _txtBenutzer;

    /** Textfeld Meldungen. */
    private Text _txtMeldungen;

    /** Textfeld Passwort fr den FTP-Serer. */
    private Text _txtPasswd;
    private Text _txtPort;

    /** Textfeld URL des FTP-Servers. */
    private Text _txtURL;

    /** ID der Sicht. */
    private String _viewID;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse ExportGUI.
     *
     * @param parent Parent-Composite.
     * @param viewID ID der Sicht.
     * @param einstellungen Instanz der Klasse IExportEinstellungen.
     * @param pidKonfigurationsObjBastBand PID KonfigurationsObjBastBand.
     */
    public ExportGUI(Composite parent, String viewID, IExportEinstellungen einstellungen, String pidKonfigurationsObjBastBand) {
        super(parent, 0);
        setLayout(new GridLayout());

        // Setze Variable:
        _viewID = viewID;

        // Initialisiere GUI:
        initGUI();

        // Initialisiere GUI-Werte:
        _guiVew = new ExportGUIVerwaltung(einstellungen, this, pidKonfigurationsObjBastBand);
        _guiVew.initialisiereGUI();
        _guiVew.setzeGUIDatenMitAktParameter();
        _guiVew.initParameterMeldungUndFortschritt();
        _guiVew.initZeitbereich();
        _guiVew.erzeugeComboboxGespeicherteExporteinstellungen("");

        // Registriere die Klasse-Instanz als ParameteraenderungsListener:
        ParameterVerwaltung.addParameterListener(_viewID, this);

        // Lsche ParameterListener beim Schlieen vom GUI:
        final ExportGUI gui = this;

        parent.addDisposeListener(new DisposeListener() {
            public void widgetDisposed(final DisposeEvent e) {
                ParameterVerwaltung.removeParameterListener(_viewID, gui);
                DavArchivVew.getInstanz().removeArchivListener(gui);
            }
        });

        // Registrierung als Listener bzgl. der Archivverfgbarkeit.
        DavArchivVew.getInstanz().addArchivListener(gui);
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode abmeldenDav leitet die Abmeldung aus dem View an die Verwaltung weiter.
     * Beim Schlieen des GUI werden alle angemeldeten Datenverteilerverbindungen abgemeldet.
     */
    public void abmeldenDaV() {
        _guiVew.abmeldenDaVSenderExportImportSteuerung();
        _guiVew.abmeldenDaVEmpfaengerExportStatus();
    }

    @Override
    public void archivStatusGeaendert(final boolean archivVerfuegbar) {
        getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                try {
                    if (!archivVerfuegbar) {
                        _guiVew.aendereGUIAnzeige(1, "Das Archiv ist nicht verfgbar, daher kann kein Export durchgefhrt werden!", false);
                    } else {
                        _guiVew.aendereGUIAnzeige(1, "Das Archiv ist verfgbar...", false);
                        _guiVew.setzeExportSchaltflaeche();
                    }
                }
                catch (Exception e) {
                    Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
                }
            }
        });
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'btnAbbrechen'.
     *
     * @return Wert der Eigenschaft 'btnAbbrechen'.
     *
     * @see #_btnAbbrechen
     */
    public Button getBtnAbbrechen() {
        return _btnAbbrechen;
    }

    /**
     * Getter fr Eigenschaft 'btnExportieren'.
     *
     * @return Wert der Eigenschaft 'btnExportieren'.
     *
     * @see #_btnExportieren
     */
    public Button getBtnExportieren() {
        return _btnExportieren;
    }

    /**
     * Getter fr Eigenschaft 'btnloeschenExporteinstellung'.
     *
     * @return Wert der Eigenschaft 'btnloeschenExporteinstellung'.
     *
     * @see #_btnloeschenExporteinstellung
     */
    public Button getBtnloeschenExporteinstellung() {
        return _btnloeschenExporteinstellung;
    }

    /**
     * Getter fr Eigenschaft 'cbtnPassivmodus'.
     *
     * @return Wert der Eigenschaft 'cbtnPassivmodus'.
     *
     * @see #_cbtnPassivmodus
     */
    public Button getCbtnPassivmodus() {
        return _cbtnPassivmodus;
    }

    /**
     * Getter fr Eigenschaft 'checkAenderungDatenstruktur'.
     *
     * @return Wert der Eigenschaft 'checkAenderungDatenstruktur'.
     *
     * @see #_checkAenderungDatenstruktur
     */
    public Button getCheckAenderungDatenstruktur() {
        return _checkAenderungDatenstruktur;
    }

    /**
     * Getter fr Eigenschaft 'checkDateiErsetzen'.
     *
     * @return Wert der Eigenschaft 'checkDateiErsetzen'.
     *
     * @see #_checkDateiErsetzen
     */
    public Button getCheckDateiErsetzen() {
        return _checkDateiErsetzen;
    }

    /**
     * Getter fr Eigenschaft 'comboGespeicherteExporteinstellungen'.
     *
     * @return Wert der Eigenschaft 'comboGespeicherteExporteinstellungen'.
     *
     * @see #_comboGespeicherteExporteinstellungen
     */
    public Combo getComboGespeicherteExporteinstellungen() {
        return _comboGespeicherteExporteinstellungen;
    }

    /**
     * Getter fr Eigenschaft 'compositeAllgemein'.
     *
     * @return Wert der Eigenschaft 'compositeAllgemein'.
     *
     * @see #_compositeAllgemein
     */
    public Composite getCompositeAllgemein() {
        return _compositeAllgemein;
    }

    /**
     * Getter fr Eigenschaft 'datumBis'.
     *
     * @return Wert der Eigenschaft 'datumBis'.
     *
     * @see #_datumBis
     */
    public DatumZeitOhneSekunden getDatumBis() {
        return _datumBis;
    }

    /**
     * Getter fr Eigenschaft 'datumVon'.
     *
     * @return Wert der Eigenschaft 'datumVon'.
     *
     * @see #_datumVon
     */
    public DatumZeitOhneSekunden getDatumVon() {
        return _datumVon;
    }

    /**
     * Getter fr Eigenschaft 'formToolkit'.
     *
     * @return Wert der Eigenschaft 'formToolkit'.
     *
     * @see #formToolkit
     */
    public FormToolkit getFormToolkit() {
        return formToolkit;
    }

    /**
     * Getter fr Eigenschaft 'lbLangZeitZaehlStellenAuswahl'.
     *
     * @return Wert der Eigenschaft 'lbLangZeitZaehlStellenAuswahl'.
     *
     * @see #_lbLangZeitZaehlStellenAuswahl
     */
    public Label getLbLangZeitZaehlStellenAuswahl() {
        return _lbLangZeitZaehlStellenAuswahl;
    }

    /**
     * Getter fr Eigenschaft 'lbRestzeit'.
     *
     * @return Wert der Eigenschaft 'lbRestzeit'.
     *
     * @see #_lbRestzeit
     */
    public Label getLbRestzeit() {
        return _lbRestzeit;
    }

    /**
     * Getter fr Eigenschaft 'prgFortschrittsanzeige'.
     *
     * @return Wert der Eigenschaft 'prgFortschrittsanzeige'.
     *
     */
    public ProgressBar getPrgFortschrittsanzeige() {
        return _prgFortschrittsanzeige1;
    }

    /**
     * Getter fr Eigenschaft 'radioAXL2009'.
     *
     * @return Wert der Eigenschaft 'radioAXL2009'.
     *
     * @see #_radioAXL2009
     */
    public Button getRadioAXL2009() {
        return _radioAXL2009;
    }

    /**
     * Getter fr Eigenschaft 'radioBASt2004'.
     *
     * @return Wert der Eigenschaft 'radioBASt2004'.
     *
     * @see #_radioBASt2004
     */
    public Button getRadioBASt2004() {
        return _radioBASt2004;
    }

    /**
     * Getter fr Eigenschaft 'radioBASt2009'.
     *
     * @return Wert der Eigenschaft 'radioBASt2009'.
     *
     * @see #_radioBASt2007
     */
    public Button getRadioBASt2007() {
        return _radioBASt2007;
    }

    /**
     * Getter fr Eigenschaft 'rbtnFtp'.
     *
     * @return Wert der Eigenschaft 'rbtnFtp'.
     *
     * @see #_rbtnFtp
     */
    public Button getRbtnFtp() {
        return _rbtnFtp;
    }

    /**
     * Getter fr Eigenschaft 'rbtnSftp'.
     *
     * @return Wert der Eigenschaft 'rbtnSftp'.
     *
     * @see #_rbtnSftp
     */
    public Button getRbtnSftp() {
        return _rbtnSftp;
    }

    /**
     * Getter fr Eigenschaft 'txtBenutzer'.
     *
     * @return Wert der Eigenschaft 'txtBenutzer'.
     *
     * @see #_txtBenutzer
     */
    public Text getTxtBenutzer() {
        return _txtBenutzer;
    }

    /**
     * Getter fr Eigenschaft 'txtMeldungen'.
     *
     * @return Wert der Eigenschaft 'txtMeldungen'.
     *
     * @see #_txtMeldungen
     */
    public Text getTxtMeldungen() {
        return _txtMeldungen;
    }

    /**
     * Getter fr Eigenschaft 'txtPasswd'.
     *
     * @return Wert der Eigenschaft 'txtPasswd'.
     *
     * @see #_txtPasswd
     */
    public Text getTxtPasswd() {
        return _txtPasswd;
    }

    /**
     * Getter fr Eigenschaft 'txtPort'.
     *
     * @return Wert der Eigenschaft 'txtPort'.
     *
     * @see #_txtPort
     */
    public Text getTxtPort() {
        return _txtPort;
    }

    /**
     * Getter fr Eigenschaft 'txtURL'.
     *
     * @return Wert der Eigenschaft 'txtURL'.
     *
     * @see #_txtURL
     */
    public Text getTxtURL() {
        return _txtURL;
    }

    /**
     * Getter fr Eigenschaft 'viewID'.
     *
     * @return Wert der Eigenschaft 'viewID'.
     *
     * @see #_viewID
     */
    public String getViewID() {
        return _viewID;
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode erzeugt den GUI Bereich Exporteinstellungen.
     */
    private void initBereichExportEinstellungen() {
        _groupExport = new Group(_compositeAllgemein, SWT.NONE);
        _groupExport.setText("Export BASt-Band");
        _groupExport.setLayout(new GridLayout(1, false));
        _groupExport.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        formToolkit.adapt(_groupExport);
        formToolkit.paintBordersFor(_groupExport);

        Section sectionExportEinstellungen = formToolkit.createSection(_groupExport, Section.TWISTIE | Section.TITLE_BAR);

        sectionExportEinstellungen.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
        formToolkit.paintBordersFor(sectionExportEinstellungen);
        sectionExportEinstellungen.setText("Exporteinstellungen");

        Composite compEinstellungen = new Composite(sectionExportEinstellungen, SWT.NONE);

        formToolkit.adapt(compEinstellungen);
        formToolkit.paintBordersFor(compEinstellungen);
        sectionExportEinstellungen.setClient(compEinstellungen);

        GridLayout gl_compEinstellungen = new GridLayout(2, false);

        compEinstellungen.setLayout(gl_compEinstellungen);

        Label lbGespeicherteExporteinstellungen = new Label(compEinstellungen, SWT.NONE);

        formToolkit.adapt(lbGespeicherteExporteinstellungen, true, true);
        lbGespeicherteExporteinstellungen.setText("Gespeicherte Exporteinstellungen:");

        Composite compExportEinstellungen = new Composite(compEinstellungen, SWT.NONE);

        compExportEinstellungen.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
        formToolkit.adapt(compExportEinstellungen);
        formToolkit.paintBordersFor(compExportEinstellungen);
        compExportEinstellungen.setLayout(new GridLayout(3, false));
        _comboGespeicherteExporteinstellungen = new Combo(compExportEinstellungen, SWT.READ_ONLY);

        GridData gd_comboGespeicherteExporteinstellungen = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);

        gd_comboGespeicherteExporteinstellungen.widthHint = 259;
        _comboGespeicherteExporteinstellungen.setLayoutData(gd_comboGespeicherteExporteinstellungen);
        formToolkit.adapt(_comboGespeicherteExporteinstellungen);
        formToolkit.paintBordersFor(_comboGespeicherteExporteinstellungen);
        _comboGespeicherteExporteinstellungen.addFocusListener(new FocusListener() {
            @Override
            public void focusGained(FocusEvent e) {
                _guiVew.erzeugeComboboxGespeicherteExporteinstellungen("");
            }
            @Override
            public void focusLost(FocusEvent e) {}
        });
        _comboGespeicherteExporteinstellungen.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                if (_comboGespeicherteExporteinstellungen.getSelectionIndex() != -1) {
                    _btnloeschenExporteinstellung.setEnabled(true);
                    _guiVew.ladeGespeicherteExporteinstellungen(_comboGespeicherteExporteinstellungen.getItem(_comboGespeicherteExporteinstellungen.getSelectionIndex()));
                } else {
                    _btnloeschenExporteinstellung.setEnabled(false);
                }
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });
        _btnSpeichern = new Button(compExportEinstellungen, SWT.NONE);

        StringBuilder stringToolTipTextSpeichern = new StringBuilder();

        stringToolTipTextSpeichern.append("Gespeichert werden alle Exporteinstellungen bis auf das Datum.");
        stringToolTipTextSpeichern.append(System.getProperty("line.separator"));
        stringToolTipTextSpeichern.append("Das Von-Datum setzt sich immer automatisch auf den Anfang des letzten Monats, ");
        stringToolTipTextSpeichern.append(System.getProperty("line.separator"));
        stringToolTipTextSpeichern.append("und das Bis-Datum auf das Ende des letzten Monats.");
        _btnSpeichern.setToolTipText(stringToolTipTextSpeichern.toString());
        formToolkit.adapt(_btnSpeichern, true, true);
        _btnSpeichern.setText("Speichern");
        _btnSpeichern.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnSpeichern();
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });
        _btnloeschenExporteinstellung = new Button(compExportEinstellungen, SWT.NONE);
        _btnloeschenExporteinstellung.setEnabled(false);
        formToolkit.adapt(_btnloeschenExporteinstellung, true, true);
        _btnloeschenExporteinstellung.setText("Gespeicherte Exporteinstellung l\u00F6schen");
        _btnloeschenExporteinstellung.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                if (_comboGespeicherteExporteinstellungen.getSelectionIndex() != -1) {
                    String exporteinstellungsName = _comboGespeicherteExporteinstellungen.getItem(_comboGespeicherteExporteinstellungen.getSelectionIndex());

                    _guiVew.btnLoeschenExporteinstellung(exporteinstellungsName);
                }
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });

        Label lbAuswahlBAStFormat = new Label(compEinstellungen, SWT.NONE);

        formToolkit.adapt(lbAuswahlBAStFormat, true, true);
        lbAuswahlBAStFormat.setText("Auswahl BASt-Band Export Format:");

        Composite compBastBandExportFormat = new Composite(compEinstellungen, SWT.NONE);

        formToolkit.adapt(compBastBandExportFormat);
        formToolkit.paintBordersFor(compBastBandExportFormat);
        compBastBandExportFormat.setLayout(new GridLayout(3, false));
        _radioBASt2004 = new Button(compBastBandExportFormat, SWT.RADIO);
        _radioBASt2004.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.bastFormatSelektiert(_radioBASt2004);
            }
        });
        _radioBASt2004.setSelection(true);
        formToolkit.adapt(_radioBASt2004, true, true);
        _radioBASt2004.setText("BASt2004");
        _radioBASt2007 = new Button(compBastBandExportFormat, SWT.RADIO);
        formToolkit.adapt(_radioBASt2007, true, true);
        _radioBASt2007.setText("BASt2007");
        _radioAXL2009 = new Button(compBastBandExportFormat, SWT.RADIO);
        _radioBASt2007.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.bastFormatSelektiert(_radioBASt2007);
            }
        });
        _radioAXL2009.setGrayed(false);
        _radioAXL2009.setSelection(false);
        _radioAXL2009.setEnabled(true);
        formToolkit.adapt(_radioAXL2009, true, true);
        _radioAXL2009.setText("AXL2009");
        _radioAXL2009.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.bastFormatSelektiert(_radioAXL2009);
            }
        });

        Label lbZeit = new Label(compEinstellungen, SWT.NONE);

        formToolkit.adapt(lbZeit, true, true);
        lbZeit.setText("Zeitbereichsangabe:");

        Composite compZeitbereich = new Composite(compEinstellungen, SWT.NONE);

        compZeitbereich.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
        formToolkit.adapt(compZeitbereich);
        formToolkit.paintBordersFor(compZeitbereich);
        compZeitbereich.setLayout(new GridLayout(4, false));

        Label lbVon1 = new Label(compZeitbereich, SWT.NONE);

        lbVon1.setVisible(true);
        lbVon1.setText("Von:");
        formToolkit.adapt(lbVon1, true, true);
        _datumVon = new DatumZeitOhneSekunden(compZeitbereich, SWT.BORDER);
        _datumVon.setzeDatumZeit(DatumFunktionen.getAktuelleDateTime());
        _datumVon.setVisible(true);
        formToolkit.adapt(_datumVon);
        formToolkit.paintBordersFor(_datumVon);

        Label lbBis1 = new Label(compZeitbereich, SWT.NONE);

        lbBis1.setText("Bis:");
        formToolkit.adapt(lbBis1, true, true);
        _datumBis = new DatumZeitOhneSekunden(compZeitbereich, SWT.BORDER);
        _datumBis.setzeDatumZeit(DatumFunktionen.getAktuelleDateTime());
        formToolkit.adapt(_datumBis);
        formToolkit.paintBordersFor(_datumBis);

        Label lbFTPServerZugangsdaten = new Label(compEinstellungen, SWT.NONE);

        formToolkit.adapt(lbFTPServerZugangsdaten, true, true);
        lbFTPServerZugangsdaten.setText("FTP Server Zugangsdaten:");

        Composite compFTPServerZugangsdaten = new Composite(compEinstellungen, SWT.NONE);

        formToolkit.adapt(compFTPServerZugangsdaten);
        formToolkit.paintBordersFor(compFTPServerZugangsdaten);
        compFTPServerZugangsdaten.setLayout(new GridLayout(2, false));

        Label lbServertyp = new Label(compFTPServerZugangsdaten, SWT.NONE);

        formToolkit.adapt(lbServertyp, true, true);
        lbServertyp.setText("Servertyp:");

        Group      grpServertyp    = new Group(compFTPServerZugangsdaten, SWT.NONE);
        GridLayout gl_grpServertyp = new GridLayout(2, false);

        gl_grpServertyp.marginHeight = 0;
        grpServertyp.setLayout(gl_grpServertyp);
        grpServertyp.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        formToolkit.adapt(grpServertyp);
        formToolkit.paintBordersFor(grpServertyp);
        _rbtnFtp = new Button(grpServertyp, SWT.RADIO);
        _rbtnFtp.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.rbtnServertypGeaendert();
            }
        });
        formToolkit.adapt(_rbtnFtp, true, true);
        _rbtnFtp.setText("FTP");
        _rbtnSftp = new Button(grpServertyp, SWT.RADIO);
        _rbtnFtp.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.rbtnServertypGeaendert();
            }
        });
        formToolkit.adapt(_rbtnSftp, true, true);
        _rbtnSftp.setText("SFTP");

        Label lbURL = new Label(compFTPServerZugangsdaten, SWT.NONE);

        formToolkit.adapt(lbURL, true, true);
        lbURL.setText("URL:");
        _txtURL = new Text(compFTPServerZugangsdaten, SWT.BORDER);

        GridData gd_txtURL = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);

        gd_txtURL.widthHint = 213;
        _txtURL.setLayoutData(gd_txtURL);
        formToolkit.adapt(_txtURL, true, true);
        _txtURL.addModifyListener(new ModifyListener() {
            @Override
            public void modifyText(ModifyEvent e) {
                _guiVew.setzeExportSchaltflaeche();
            }
        });

        Label lbPort = new Label(compFTPServerZugangsdaten, SWT.NONE);

        formToolkit.adapt(lbPort, true, true);
        lbPort.setText("Port:");
        _txtPort = new Text(compFTPServerZugangsdaten, SWT.BORDER);
        _txtPort.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                _guiVew.txtPortGeaendert();
            }
        });
        _txtPort.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        formToolkit.adapt(_txtPort, true, true);

        Label lbBenutzer = new Label(compFTPServerZugangsdaten, SWT.NONE);

        formToolkit.adapt(lbBenutzer, true, true);
        lbBenutzer.setText("Benutzer:");
        _txtBenutzer = new Text(compFTPServerZugangsdaten, SWT.BORDER);
        _txtBenutzer.setText("");
        _txtBenutzer.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        formToolkit.adapt(_txtBenutzer, true, true);
        _txtBenutzer.addModifyListener(new ModifyListener() {
            @Override
            public void modifyText(ModifyEvent e) {
                _guiVew.setzeExportSchaltflaeche();
            }
        });

        Label lbpasswd = new Label(compFTPServerZugangsdaten, SWT.NONE);

        formToolkit.adapt(lbpasswd, true, true);
        lbpasswd.setText("Passwort:");
        _txtPasswd = new Text(compFTPServerZugangsdaten, SWT.BORDER | SWT.PASSWORD);
        _txtPasswd.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        formToolkit.adapt(_txtPasswd, true, true);
        _txtPasswd.addModifyListener(new ModifyListener() {
            @Override
            public void modifyText(ModifyEvent e) {
                _guiVew.setzeExportSchaltflaeche();
            }
        });

        Label lbPassiv = new Label(compFTPServerZugangsdaten, SWT.NONE);

        formToolkit.adapt(lbPassiv, true, true);
        lbPassiv.setText("Passivmodus:");
        _cbtnPassivmodus = new Button(compFTPServerZugangsdaten, SWT.CHECK);
        formToolkit.adapt(_cbtnPassivmodus, true, true);

        //
        Label lbErsetzen = new Label(compEinstellungen, SWT.NONE);

        formToolkit.adapt(lbErsetzen, true, true);
        lbErsetzen.setText("Vorhandene Datei ersetzten:");
        _checkDateiErsetzen = new Button(compEinstellungen, SWT.CHECK);
        _checkDateiErsetzen.setSelection(false);
        formToolkit.adapt(_checkDateiErsetzen, true, true);
        _checkDateiErsetzen.setText("");

        Label _lbAenderungDatenstruktur = new Label(compEinstellungen, SWT.NONE);

        _lbAenderungDatenstruktur.setToolTipText("\u00C4ndert sich die Datenstruktur im Jahresverlauf z.B. durch Umstellung der Fahrzeugklassifizierung auf 8+1," + System.getProperty("line.separator") + " so ist dieses durch einen Eintrag an siebter und achter Stelle des Namens kenntlich zu machen  (z.B.: \u201ENW4606_1.974\u201C). " + System.getProperty("line.separator") + "Durch setzten dieses Flags, wird der BASt-Band Dateiname entsprechend mit der Kennung \"_1\" gesetzt.");
        formToolkit.adapt(_lbAenderungDatenstruktur, true, true);
        _lbAenderungDatenstruktur.setText("\u00C4nderung der Datenstruktur:");
        _checkAenderungDatenstruktur = new Button(compEinstellungen, SWT.CHECK);
        formToolkit.adapt(_checkAenderungDatenstruktur, true, true);

        Label lbLangzeitzaehlstellen = new Label(compEinstellungen, SWT.NONE);

        formToolkit.adapt(lbLangzeitzaehlstellen, true, true);
        lbLangzeitzaehlstellen.setText("Zu exportierende Langzeitzaehlstellen:");

        Composite compLangZeitZaehlstellenAuswahl = new Composite(compEinstellungen, SWT.NONE);

        compLangZeitZaehlstellenAuswahl.setLayout(new GridLayout(2, false));
        compLangZeitZaehlstellenAuswahl.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
        formToolkit.adapt(compLangZeitZaehlstellenAuswahl);
        formToolkit.paintBordersFor(compLangZeitZaehlstellenAuswahl);

        Button btnLangzeitzaehlstellenAuswahl = new Button(compLangZeitZaehlstellenAuswahl, SWT.NONE);

        btnLangzeitzaehlstellenAuswahl.setText("Langzeitzhlstellenauswahl...");
        btnLangzeitzaehlstellenAuswahl.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent evt) {
                _guiVew.btnLangzeitzaehlstellenAuswahlSelektiert();
            }
        });
        _lbLangZeitZaehlStellenAuswahl = new Label(compLangZeitZaehlstellenAuswahl, SWT.NONE);
        formToolkit.adapt(_lbLangZeitZaehlStellenAuswahl, true, true);
        sectionExportEinstellungen.setExpanded(true);
    }

    /**
     * Die Methode initialisiert den Bereich Fortschrittsanzeige.
     */
    private void initBereichFortschrittsanzeige() {
        Group grpFortschritt = new Group(_groupExport, SWT.NONE);

        grpFortschritt.setText("Fortschrittsanzeige");
        grpFortschritt.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        formToolkit.adapt(grpFortschritt);
        formToolkit.paintBordersFor(grpFortschritt);
        grpFortschritt.setLayout(new GridLayout(1, false));
        _lbRestzeit = new Label(grpFortschritt, SWT.NONE);
        _lbRestzeit.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        formToolkit.adapt(_lbRestzeit, true, true);
        _prgFortschrittsanzeige1 = new ProgressBar(grpFortschritt, SWT.NONE);
        _prgFortschrittsanzeige1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        _prgFortschrittsanzeige1.setSize(726, 21);
    }

    /**
     * Die Methode initialisiert den Meldungsbereich.
     */
    private void initBereichMeldungen() {
        Group grpMeldungsBereich = new Group(_groupExport, SWT.NONE);

        grpMeldungsBereich.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        grpMeldungsBereich.setText("Meldungen");
        formToolkit.adapt(grpMeldungsBereich);
        formToolkit.paintBordersFor(grpMeldungsBereich);
        grpMeldungsBereich.setLayout(new GridLayout(1, false));
        _txtMeldungen = new Text(grpMeldungsBereich, SWT.WRAP | SWT.H_SCROLL | SWT.V_SCROLL | SWT.CANCEL | SWT.MULTI);
        _txtMeldungen.setEditable(false);
        _txtMeldungen.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        formToolkit.adapt(_txtMeldungen, true, true);

        Composite compButtons = new Composite(_groupExport, SWT.NONE);

        compButtons.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        formToolkit.adapt(compButtons);
        formToolkit.paintBordersFor(compButtons);
        compButtons.setLayout(new GridLayout(3, false));
        _btnExportieren = new Button(compButtons, SWT.NONE);
        _btnExportieren.setText("Exportieren");
        _btnExportieren.setEnabled(false);
        _btnExportieren.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent evt) {
                _guiVew.btnExportierenSelektiert();
            }
        });
        _btnAbbrechen = new Button(compButtons, SWT.NONE);
        _btnAbbrechen.setEnabled(false);
        _btnAbbrechen.setText("Abbrechen");

        Button _btnSchliessen = new Button(compButtons, SWT.NONE);

        _btnSchliessen.setToolTipText("Die Bet\u00E4tigung der schlie\u00DFen Schaltfl\u00E4che schlie\u00DFt die aktuelle Sicht, bricht den laufenden Export aber nicht ab.");
        formToolkit.adapt(_btnSchliessen, true, true);
        _btnSchliessen.setText("Sicht schlie\u00DFen");
        _btnSchliessen.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent evt) {
                _guiVew.btnSchliessenSelektiert();
            }
        });
        _btnAbbrechen.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent evt) {
                _guiVew.btnAbbruchSelektiert();
            }
        });
    }

    /**
     * Initialisiere das GUI
     */
    private void initGUI() {
        final ScrolledComposite scrolledCompositeAllgemein = new ScrolledComposite(this, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);

        scrolledCompositeAllgemein.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        scrolledCompositeAllgemein.setExpandHorizontal(true);
        scrolledCompositeAllgemein.setExpandVertical(true);
        _compositeAllgemein = new Composite(scrolledCompositeAllgemein, SWT.NONE);
        _compositeAllgemein.setLocation(0, 0);
        _compositeAllgemein.setLayout(new GridLayout());
        initBereichExportEinstellungen();
        initBereichFortschrittsanzeige();
        initBereichMeldungen();
        scrolledCompositeAllgemein.setContent(_compositeAllgemein);
        scrolledCompositeAllgemein.setMinSize(_compositeAllgemein.computeSize(SWT.DEFAULT, SWT.DEFAULT));
    }

    @Override
    public void parameterGeaendert() {
        _guiVew.setzeGUIDatenMitAktParameter();
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 28.10.13
