/*
 * Copyright (c) 2012 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.pat.sftptest.SFTPFunktionen
 *
 * de.inovat.pat.sftptest.SFTPFunktionen is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.pat.sftptest.SFTPFunktionen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.pat.sftptest.SFTPFunktionen; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.buv.plugin.gtm.bast.gui;

//~ NICHT JDK IMPORTE =========================================================

import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.Session;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Klasse stellt statischen SFTP-Funktionen zu Verfgung.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 282 $ / $Date: 2012-08-30 13:42:53 +0200 (Do, 30 Aug 2012) $ / ($Author: LG $)
 */
public class SFTPFunktionen {
    private static final String SFTP = "sftp";

    //~ METHODEN ==============================================================

    /**
     * Ermittle alle Dateiennamen im angegebenen SFTP-Ordner.
     *
     * @param session Session.
     * @param sftpOrdner SFTP-Ordner.
     *
     * @return Liste der Dateiennamen im angegebenen SFTP-Ordner
     * oder null, falls nicht ermittelbar ist.
     */
    public static List<String> ermittleDateienImSFTPOrdner(Session session, String sftpOrdner) {
        List<String> listeDateien = new ArrayList<String>();
        ChannelSftp  sftpChannel  = null;

        try {

            // ---------------------------------------------------------
            // ffne den Kanal:
            Channel channel = session.openChannel(SFTP);

            channel.connect();
            sftpChannel = (ChannelSftp) channel;

            // ---------------------------------------------------------
            java.util.Vector vOrdnerInhalt;

            if (sftpOrdner.isEmpty()) {
                vOrdnerInhalt = sftpChannel.ls("/");
            } else {
                vOrdnerInhalt = sftpChannel.ls(sftpOrdner);
            }

            if (vOrdnerInhalt != null) {
                for (Object obj : vOrdnerInhalt) {
                    if (obj instanceof ChannelSftp.LsEntry) {
                        ChannelSftp.LsEntry lse = (ChannelSftp.LsEntry) obj;

                        if (!lse.getAttrs().isDir()) {
                            listeDateien.add(lse.getFilename());
                        }
                    }
                }
            }
        }
        catch (Exception e) {
            listeDateien = null;
        }

        schliesseSftpKanal(sftpChannel);

        return listeDateien;
    }

    /**
     * Schliesse den SFTP-Kanal.
     *
     * @param sftpChannel ChannelSftp
     */
    public static void schliesseSftpKanal(ChannelSftp sftpChannel) {

        // Versuche den Kanal zu schlieen:
        try {
            if (sftpChannel != null) {
                sftpChannel.disconnect();
            }
        }
        catch (Exception ignored) {}
    }

    /**
     * Stelle die Verbindung mit dem SFTP-Server her.
     *
     * @param url SFTP-Host.
     * @param port SFTP-Port.
     * @param benutzer SFTP-Benutzer.
     * @param passwort SFTP-Passwort
     *
     * @return SFTP-Session.
     */
    public static Session verbindeMitSFTPServer(String url, int port, String benutzer, String passwort) throws Exception {
        JSch jsch = new JSch();

        // -----------------------------------------
        // TODO Host-Datei definieren (ber Aufrufparameter)
        // -----------------------------------------
        // jsch.setKnownHosts("sftp_inovat\\key\\priv");
        //
        // -----------------------------------------
        // Session ffnen:
        // -----------------------------------------
        Session session = jsch.getSession(benutzer, url, port);

        session.setPassword(passwort);

        /*
        // -----------------------------------------
        // TODO Proxy-Einstellungen:
        // -----------------------------------------
        if (_sftpProxy) {
            // via HTTP proxy:
            session.setProxy(new ProxyHTTP(_sftpProxyHost, _sftpPort));
            // via SOCKS proxy
            session.setProxy(new ProxySOCKS5(_sftpProxyHost, _sftpPort));
        }
        */

        // -----------------------------------------
        // HostKey:
        // -----------------------------------------
        // TODO berprfung der HostKeys ausschalten <--- ber Aufrufparameter.
        // WICHTIG: es wird automatisch allen Hosts geglaubt, die HostKeys werden nicht berprft.
        // Durch den Eintrag <config.put("StrictHostKeyChecking", "no")> wird explizit eine Verbindung
        // zu jeden beliebigen Host erlaubt (die Strikte berprfung der HostKeys wird unterbinden).
        java.util.Properties config = new java.util.Properties();

        config.put("StrictHostKeyChecking", "no");
        session.setConfig(config);

        // -----------------------------------------
        // Verbindung:
        // -----------------------------------------
        session.connect();

        return session;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 20.06.13
