/*
 * @(#)TreeContentProviderMq.java   13.03.12
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.mqnavigation.gui;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.gtm.datvew.lib.DatenVew;
import de.inovat.buv.gtm.datvew.lib.Fahrbahnen;
import de.inovat.buv.gtm.datvew.lib.Messquerschnitt;
import de.inovat.buv.gtm.datvew.tls.De;
import de.inovat.buv.plugin.gtm.mqnavigation.lib.DatenVewMq;
import de.inovat.buv.plugin.gtm.mqnavigation.lib.DavSystemObjekt;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

//~ JDK IMPORTE ===============================================================

import java.util.List;

//~ KLASSEN ===================================================================

/**
 * TreeContentProviderMq definiert die Baum-Struktur fr MQs.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 375 $ / $Date: 2013-05-22 14:24:03 +0200 (Mi, 22 Mai 2013) $ / ($Author: LG $)
 */
public class TreeContentProviderMq implements ITreeContentProvider {

    /** Sicht, fr die das ContentProvider implementiert wird. */
    private SichtMq _sicht;

    //~ KONSTANTE AUFZAEHLUNGSTYPEN ===========================================

    /**
     * Darstellungsicht.
     */
    enum SichtMq { MQ_ALLGEMEIN, MQ_NACH_TYP }

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse TreeContentProviderMq.
     *
     * @param sicht Sicht, fr die das ContentProvider implementiert wird.
     */
    public TreeContentProviderMq(SichtMq sicht) {
        _sicht = sicht;
    }

    //~ METHODEN ==============================================================

    @Override
    public void dispose() {}

    //~ GET METHODEN ==========================================================

    @Override
    public Object[] getChildren(Object parentElement) {
        if (parentElement instanceof DavSystemObjekt) {
            DavSystemObjekt mqTyp = (DavSystemObjekt) parentElement;

            return DatenVewMq.getInstanz().getListeAlleMqVomTypSortiert(mqTyp).toArray();
        } else if (parentElement instanceof Messquerschnitt) {
            Messquerschnitt mq = (Messquerschnitt) parentElement;

            return mq.getArFahrbahnen();
        } else if (parentElement instanceof Fahrbahnen) {
            Fahrbahnen fs = (Fahrbahnen) parentElement;

            if (fs.getFsQuellePid() != null) {
                De de = DatenVew.getInstanz().ermittleDe(fs.getFsQuellePid());

                if (de != null) {
                    return new Object[] { de };
                }
            }
        }

        return null;
    }

    @SuppressWarnings("unchecked")
    @Override
    public Object[] getElements(Object inputElement) {
        if (inputElement instanceof List) {
            List root = (List) inputElement;

            return root.toArray();
        }

        return null;
    }

    @Override
    public Object getParent(Object element) {
        if (element instanceof DavSystemObjekt) {
            return null;
        } else if (element instanceof Messquerschnitt) {
            if (_sicht == SichtMq.MQ_NACH_TYP) {
                Messquerschnitt mq = (Messquerschnitt) element;

                return DatenVewMq.getInstanz().getTypVonMq(mq);
            } else {
                return null;
            }
        } else if (element instanceof Fahrbahnen) {
            Fahrbahnen fahrbahnen = (Fahrbahnen) element;

            return fahrbahnen.getParentMQZuEinerFahrbahn();
        } else if (element instanceof De) {
            De de = (De) element;

            return DatenVew.getInstanz().ermittleFahrbahnen(de.getSystemObjekt().getPidOrId());
        } else {
            return null;
        }
    }

    @Override
    public boolean hasChildren(Object element) {
        return (element instanceof DavSystemObjekt) ||  //
               (element instanceof Messquerschnitt) ||  //
               (element instanceof Fahrbahnen);
    }

    //~ METHODEN ==============================================================

    @Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}
}


//~Formatiert mit 'inovat Kodierkonvention' am 22.05.13
