/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.export;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.buv.rw.basislib.tabellen.ITabellenZeile;
import de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur;

import de.inovat.buv.gtm.datvew.konstanten.KonstantenGTM;
import de.inovat.buv.projektlib.speicher.OrdnerEinstellungen;
import de.inovat.buv.inovat.lib.funktionen.datum.DatumFunktionen;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

//~ JDK IMPORTE ===============================================================

import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;

import java.util.LinkedList;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Klasse die den HtmlExport durchfhrt.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class HtmlExport {

    /** Liste mit der aktuellen Spaltenauswahl. */
    private List<String> _spaltenAuswahl;

    /** TabelViewer mit dem Tabelleninhalt, der ausgedruckt werden soll. */
    private TableViewer _tabellenInhalt;

    /** Tabellenkopfzeile. */
    private String[] _tabellenKopfzeile;

    /** Tabellendatenstruktur. */
    private TabellenDatenStruktur _tabellenStruktur;

    /** Array mit allen Zusatzinformationen die in dem Html Dokument ausgegeben werden. */
    private String[][] _zusatzinformationen;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse CsvExport.
     * @param tabellenKopfzeile Tabellenkopfzeile.
     * @param tabellenInhalt  Tabelleninhalt.
     * @param tabellenStruktur Tabellendatenstruktur.
     * @param zusatzInformationen Zusatzinformationen.
     * @param spaltenAuswahl  Spaltenauswahl.
     */
    public HtmlExport(String[] tabellenKopfzeile, TableViewer tabellenInhalt, TabellenDatenStruktur tabellenStruktur, String[][] zusatzInformationen, List<String> spaltenAuswahl) {
        _tabellenKopfzeile   = tabellenKopfzeile;
        _tabellenInhalt      = tabellenInhalt;
        _tabellenStruktur    = tabellenStruktur;
        _zusatzinformationen = zusatzInformationen;
        _spaltenAuswahl      = spaltenAuswahl;
    }

    //~ METHODEN ==============================================================

    /**
     * Tabellenkopfzeile wie auch der Inhalt der Tabelle wird in
     * ein csv Dokument geschrieben, welches mittels file chooser ausgewhlt wurde.
     * @param isArchivAnfrage
     *
     * @return true wenn keine Ausnahme aufgetreten ist, sonst false.
     */
    public String erzeugeHTML(boolean isArchivAnfrage) {
        try {
            Shell      shell      = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
            FileDialog fileDialog = new FileDialog(shell, SWT.SAVE);

            fileDialog.setFilterExtensions(new String[] { "*.html" });
            fileDialog.setText("Exportieren des Tabelleninhaltes in html Format");
            fileDialog.setFilterPath(OrdnerEinstellungen.ermittleProjektAusgabenUnterOrdner(KonstantenGTM.ORDNER_EXPORT_HTML));

            String gewaehlteFileName = fileDialog.open();
            String datum             = DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.getAktuelleDateTime());

            try {
                if (gewaehlteFileName != null) {
                    if (!gewaehlteFileName.substring(gewaehlteFileName.length() - 5).toLowerCase().equals(".html")) {
                        gewaehlteFileName += ".html";
                    }

                    File exportFile = new File(gewaehlteFileName);

                    if (exportFile != null) {
                        PrintWriter exportSchreiber = new PrintWriter(new FileWriter(exportFile.getAbsolutePath()));

                        exportSchreiber.print("<html><head><title></title><style type=\"text/css\">");
                        exportSchreiber.print("body {");
                        exportSchreiber.print("font-family: Arial, Helvetica, sans-serif;");
                        exportSchreiber.print("color: #555555;");
                        exportSchreiber.print("} ");
                        exportSchreiber.print("h1 { ");
                        exportSchreiber.print("font-size: 1.5em; ");
                        exportSchreiber.print(" background: #CCCCCC;");
                        exportSchreiber.print("border-top: 2px solid #222222; ");
                        exportSchreiber.print(" border-bottom: 2px solid #222222;");
                        exportSchreiber.print("} ");
                        exportSchreiber.print("p {");
                        exportSchreiber.print("line-height: 1.5em;");
                        exportSchreiber.print("margin-top: 3pt;");
                        exportSchreiber.print("margin-bottom: 3pt;");
                        exportSchreiber.print("letter-spacing: 0.0mm;");
                        exportSchreiber.print("padding-left: 0;");
                        exportSchreiber.print("padding-right: 5px;");
                        exportSchreiber.print("word-spacing: 0.8mm;");
                        exportSchreiber.print("}");
                        exportSchreiber.print("div {");
                        exportSchreiber.print("font-size: 8pt;");
                        exportSchreiber.print("line-height: 1.5em;");
                        exportSchreiber.print("letter-spacing: 0.0mm;");
                        exportSchreiber.print("padding-left: 0;");
                        exportSchreiber.print("padding-right: 5px;");
                        exportSchreiber.print("word-spacing: 0.8mm;");
                        exportSchreiber.print("}");
                        exportSchreiber.print("table { ");
                        exportSchreiber.print("border-collapse: collapse; ");
                        exportSchreiber.print("border: none; ");
                        exportSchreiber.print("width: auto; ");
                        exportSchreiber.print("text-align: left;");
                        exportSchreiber.print("vertical-align: top; ");
                        exportSchreiber.print(" }");
                        exportSchreiber.print("td, th { ");
                        exportSchreiber.print("font-size: 8pt;");
                        exportSchreiber.print("margin-top: 3pt;");
                        exportSchreiber.print("margin-bottom: 3pt;");
                        exportSchreiber.print("letter-spacing: 0.0mm;");
                        exportSchreiber.print("padding-left: 5px;");
                        exportSchreiber.print("padding-right: 5px;");
                        exportSchreiber.print("word-spacing: 0.8mm;");
                        exportSchreiber.print("border-collapse: collapse; ");
                        exportSchreiber.print("border: thin solid; ");
                        exportSchreiber.print("} ");
                        exportSchreiber.print("table tr.keineDaten{background-color: #98fb98; }");
                        exportSchreiber.print("table tr.keineDatenKeineQuelle{ background-color: #ffd700;}");
                        exportSchreiber.print("table tr.unterbrechung{ background-color: #ff7d7d;} ");
                        exportSchreiber.print("table tr.start{ background-color: #c8ebc8;} ");
                        exportSchreiber.print("table tr.keineRechte{ background-color: #ff0000;}  ");
                        exportSchreiber.print("table tr.keineDatenUndefinierteObjektkodierung{ background-color: #ff0000;} ");
                        exportSchreiber.print("table tr.keineDatenPotentielleDatenluecke{ background-color:         #ff00ff;}  ");
                        exportSchreiber.print("table tr.fehlerhafteAnmeldung{ background-color: #ff0000;}  ");
                        exportSchreiber.print("table tr.geloeschteBereich{ background-color: #ff0000;}  ");
                        exportSchreiber.print("table tr.endeArchivanfragezeitraum{ background-color:        #d2ffff;}  ");
                        exportSchreiber.print("table tr.ausgelagerterBereich{ background-color: #ffff00;}  ");
                        exportSchreiber.print("table tr.geloeschteBereich{ background-color: #ff0000;}  ");
                        exportSchreiber.print("</style> ");
                        exportSchreiber.print("</head><body><table width = '100%' border = '1'>");

                        // Ueberschrift
                        exportSchreiber.print("<h1>");

                        if (isArchivAnfrage) {
                            exportSchreiber.print("Archivanfrage vom " + datum);
                        } else {
                            exportSchreiber.print("Protokoll vom " + datum);
                        }

                        exportSchreiber.print("</h1>");
                        exportSchreiber.print("<tr>");

                        for (int k = 1; k < _zusatzinformationen.length; k++) {
                            exportSchreiber.print("<tr>");

                            for (int i = 0; i < 2; i++) {
                                if (i == 0) {
                                    exportSchreiber.print("<td nowrap><b>" + _zusatzinformationen[k][i] + "</b></td>");
                                } else {
                                    exportSchreiber.print("<td>" + _zusatzinformationen[k][i] + "</td>");
                                }
                            }

                            exportSchreiber.print("</tr>");
                        }

                        exportSchreiber.print("</tr>");
                        exportSchreiber.println();
                        exportSchreiber.print("</table>");
                        exportSchreiber.print("<br />");

                        // Eigentliche Tabelle mit Inhalt
                        exportSchreiber.print("<table>");

                        int   spaltenAnzahl      = _tabellenInhalt.getTable().getColumnCount();
                        int   statusInfoFeld     = 3;
                        int[] spaltenreihenfolge = _tabellenInhalt.getTable().getColumnOrder();

                        exportSchreiber.print("<tr>");

                        for (int i = 0; i < spaltenAnzahl; i++) {
                            if (_spaltenAuswahl.contains(_tabellenKopfzeile[spaltenreihenfolge[i]])) {

                                // Tabellenueberschrift
                                exportSchreiber.print("<th>" + _tabellenKopfzeile[spaltenreihenfolge[i]] + "</th>");
                            }
                        }

                        exportSchreiber.print("</tr>");
                        exportSchreiber.println();

                        LinkedList<ITabellenZeile> alleZeilen = _tabellenStruktur.getZeilen();

                        for (ITabellenZeile zeilenObj : alleZeilen) {
                            if (isArchivAnfrage) {
                                statusInfoFeld = 4;
                            }

                            if (zeilenObj.getAttribut(statusInfoFeld).equals("keine Daten")) {
                                exportSchreiber.print("<tr class= \"keineDaten\">");
                            } else if (zeilenObj.getAttribut(statusInfoFeld).equals("keine Daten (keine Quelle)")) {
                                exportSchreiber.print("<tr class= \"keineDatenKeineQuelle\">");
                            } else if (zeilenObj.getAttribut(statusInfoFeld).equals("keine Daten (fehlerhafte Anmeldung)")) {
                                exportSchreiber.print("<tr class= \"fehlerhafteAnmeldung\">");
                            } else if (zeilenObj.getAttribut(statusInfoFeld).equals("keine Daten (keine Rechte)")) {
                                exportSchreiber.print("<tr class= \"keineRechte\">");
                            } else if (zeilenObj.getAttribut(statusInfoFeld).equals("keine Daten (potentielle Datenlcke)")) {
                                exportSchreiber.print("<tr class= \"keineDatenPotentielleDatenluecke\">");
                            } else if (zeilenObj.getAttribut(statusInfoFeld).equals("keine Daten (Undefinierte Objektkodierung)")) {
                                exportSchreiber.print("<tr class= \"keineDatenUndefinierteObjektkodierung\">");
                            } else if (zeilenObj.getAttribut(statusInfoFeld).equals("Start durch den Benutzer")) {
                                exportSchreiber.print("<tr class= \"start\">");
                            } else if (zeilenObj.getAttribut(statusInfoFeld).equals("Unterbrechung durch den Benutzer")) {
                                exportSchreiber.print("<tr class= \"unterbrechung\">");
                            } else if (zeilenObj.getAttribut(statusInfoFeld).equals("Gelschter Bereich")) {
                                exportSchreiber.print("<tr class= \"geloeschteBereich\">");
                            } else if (zeilenObj.getAttribut(statusInfoFeld).equals("Ausgelagerter Bereich")) {
                                exportSchreiber.print("<tr class= \"ausgelagerterBereich\">");
                            } else if (zeilenObj.getAttribut(statusInfoFeld).equals("Ende des Archivanfragezeitraums")) {
                                exportSchreiber.print("<tr class= \"endeArchivanfragezeitraum\">");
                            } else {

                                // Normales Datenfeld
                                exportSchreiber.print("<tr>");
                            }

                            for (int j = 0; j < _spaltenAuswahl.size(); j++) {
                                exportSchreiber.print("<td>" + zeilenObj.getAttribut(spaltenreihenfolge[j]) + "</td>");
                            }

                            exportSchreiber.print("</tr>");
                        }

                        exportSchreiber.print("</table></body></html>");
                        exportSchreiber.close();
                    }
                }
            }
            catch (NullPointerException ex) {
                ex.printStackTrace();

                return "";
            }

            return gewaehlteFileName;
        }
        catch (Exception ex) {
            ex.printStackTrace();

            return "";
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 07.10.10
