/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.inovat.buv.plugin.gtm.navigation.filter;

import java.util.ArrayList;
import java.util.List;


/**
 * Die Klasse enthlt die Filtereintrge.
 * 
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */

public class Filter {
	/** Map FilterEintrag, Name des Filtereintrages mit allen Elementen dazu, d.h. Datenidentifikation,
	 * Liste von Zeilenobjekten und das Tabellenobjekt.*/
	private List<FilterEintrag> _filterEintraege = new ArrayList<FilterEintrag>();
	
	/** Name vom Filter.*/
	private String _filterName;
	/**
     * WICHTIG_: Die Methode darf nicht aufgerufen werden. Die wird von XMLDecoder bentigt.
     *
     * Erzeugt eine Instanz der Klasse Filter.
     */
    public Filter() {}

    /**
     * Erzeugt eine Instanz der Klasse Filter.
     *
     * @param filterName Name vom Filter.
     * @param filterEintraege FilterEinstrag.
     */
    public Filter(String filterName, List<FilterEintrag> filterEintraege) {
        _filterName      = filterName;
        _filterEintraege = filterEintraege;
    }

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime
				* result
				+ ((_filterEintraege == null) ? 0 : _filterEintraege.hashCode());
		result = prime * result
				+ ((_filterName == null) ? 0 : _filterName.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof Filter)) {
			return false;
		}
		Filter other = (Filter) obj;
		if (_filterEintraege == null) {
			if (other._filterEintraege != null) {
				return false;
			}
		} else if (!_filterEintraege.equals(other._filterEintraege)) {
			return false;
		}
		if (_filterName == null) {
			if (other._filterName != null) {
				return false;
			}
		} else if (!_filterName.equals(other._filterName)) {
			return false;
		}
		return true;
	}

	/**
	 * Getter fr Eigenschaft 'filterEintraege'.
	 * 
	 * @return Wert der Eigenschaft 'filterEintraege'.
	 * 
	 * @see #_filterEintraege
	 */
	public List<FilterEintrag> getFilterEintraege() {
		return _filterEintraege;
	}

	/**
	 * Setter fr Eigenschaft 'filterEintraege'.
	 *
	 * @param filterEintraege Wert zum Setzen der Eigenschaft 'filterEintraege'. 
	 *
	 * @see #_filterEintraege
	 */
	public void setFilterEintraege(List<FilterEintrag> filterEintraege) {
		_filterEintraege = filterEintraege;
	}

	/**
	 * Getter fr Eigenschaft 'filterName'.
	 * 
	 * @return Wert der Eigenschaft 'filterName'.
	 * 
	 * @see #_filterName
	 */
	public String getFilterName() {
		return _filterName;
	}

	/**
	 * Setter fr Eigenschaft 'filterName'.
	 *
	 * @param filterName Wert zum Setzen der Eigenschaft 'filterName'. 
	 *  WICHTIG! Verwendung fr die Speicherung im XML.
	 * @see #_filterName
	 */
	public void setFilterName(String filterName) {
		_filterName = filterName;
	}

	
}
