/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.filter;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.buv.rw.basislib.tabellen.ContentProviderTabellenDatenStruktur;
import de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur;

import de.inovat.buv.inovat.lib.gui.vewdlg.IRahmenVerwaltungsDialogGUI;
import de.inovat.buv.inovat.lib.gui.vewdlg.RahmenVerwaltungsDialogGUI;
import de.inovat.buv.plugin.gtm.navigation.datenidentifikation.Datenidentifikation;
import de.inovat.buv.plugin.gtm.navigation.datenidentifikation.DatenidentifikationGUIVew;
import de.inovat.buv.plugin.gtm.navigation.tabellen.LabelProviderTabellenDatenStrukturGTM;
import de.inovat.buv.plugin.gtm.navigation.tabellen.SpaltenObjekt;
import de.inovat.buv.plugin.gtm.navigation.tabellen.TabellenObjekt;
import de.inovat.buv.plugin.gtm.navigation.tabellen.ZeilenObjekt;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.custom.ControlEditor;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.custom.TableCursor;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

//~ KLASSEN ===================================================================

/**
 * Die Klasse verwaltet den Filtereinstellungsdialog.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class FilterEinstellungen implements IRahmenVerwaltungsDialogGUI {

    /** Das Array enthlt die konstanten Spaltennamen. */
    private static final String[] _constSpaltenNamen = { "Art", "Datenzeit", "Archivzeit", "Objekt" };

    /** Die regex Konstante. */
    private static final String REGEX_DEFAULT = ".*";

    /** ToolTip fr die Korrektur der regulren Ausdrcken. */
    private static final String TOOL_TIP = "Doppelklick auf der Zelle startet den Korrektur Modus. Mit dem <Enter> wird die Eingabe besttigt.";

    //~ FELDER ================================================================

    /** Gibt an ob die Daten im GUI gendert worden sind. */
    private boolean _guiDatenEqualGUIBereich = true;

    /** FormToolkit. */
    private final FormToolkit formToolkit = new FormToolkit(Display.getDefault());

    /** Konstante die angibt, dass kein Filter ausgewhlt wurde. */
    private final String KEINEN_FILTER = "Kein Filter";

    /** Instanz der Klasse AlleFilter. */
    private AlleFilter _alleFilterInstanz;

    /** Datenindentifikation. */
    private Datenidentifikation _datenIdentifikation;

    /** GUI Datenidentifikation. */
    private DatenidentifikationGUIVew _datenIdentifikationGUIVew;

    /** Filterelement. */
    private Filter _filterElement;

    /** Filtername. */
    private String _filterName;

    /** GUI Filtereintrag. */
    private FilterEintrag              _guiFilterEintragVergleichsObj;
    private RahmenVerwaltungsDialogGUI _guiVew;

    /** Tabellendatenstruktur. */
    private TabellenDatenStruktur _tabellenDatenStruktur;

    /** Liste Zeilenobjekte. */
    private LinkedList<ZeilenObjekt> _tabellenZeilen;

    /** Tabelle fr die Filter. */
    private Table _table;

    /** TableViewer. */
    private TableViewer _tableViewer;

    /** Tabellenobjekt. */
    private TabellenObjekt _tbo;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Die Methode initialisiert das _alleFilterInstanz wie auch das GUI Vergleichsobjekt.
     */
    public FilterEinstellungen() {
        _alleFilterInstanz = AlleFilter.getInstanz();
        initialisiereFilterGUIVergleichsObj();
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode aenderungErfolgt wird immer aufgerufen, wenn Daten ggf. modifiziert worden sind.
     * Die Methode prft, basierend auf den equals Methoden der Objekte, ob wirklich nderungen vollzogen worden sind und
     * setzt dementsprechend die Schaltflchen bernehmen und Zurcksetzten.
     *
     */
    private void aenderungErfolgt() {
        boolean aenderung = true;

        guiVergleichsObjFuellen();

        if (!((_filterName == null) || (_filterName.equals(KEINEN_FILTER)))) {
            Filter filter = _alleFilterInstanz.getFilterObjekt(_filterName);

            if (filter != null) {
                List<FilterEintrag> filterEintraegeListe = filter.getFilterEintraege();

                // Prfe ob das aktuelle GUI Element gleich dem gespeicherten Filterobjekt ist.
                for (FilterEintrag gespeicherterFilterEintrag : filterEintraegeListe) {
                    aenderung                = !_guiFilterEintragVergleichsObj.equals(gespeicherterFilterEintrag);
                    _guiDatenEqualGUIBereich = !aenderung;
                }

                if (filterEintraegeListe.size() == 0) {
                    guiInitialObjFuellen();
                    aenderung                = false;
                    _guiDatenEqualGUIBereich = !aenderung;
                }
            }
        }

        if (_guiVew != null) {
            _guiVew.getBtnUebernehmen().setEnabled(aenderung);
            _guiVew.getBtnZuruecksetzen().setEnabled(aenderung);
        }
    }

    /**
     * Die Methode richtet den Inhalt der angegebenen Spalte links aus.
     * @param spalte  Spalte.
     */
    private void ausrichtungSpaltenInhaltLinks(int spalte) {
        TableColumn zeilenObjekt = _table.getColumn(spalte);

        zeilenObjekt.setAlignment(SWT.LEFT);
    }

    @Override
    public void einfuegenNeuesElement(String objektName) {
        _filterElement = new Filter(objektName, new ArrayList<FilterEintrag>());
        _alleFilterInstanz.addFilter(_filterElement);
        speichereMapInXML();
    }

    /**
     * Logische berprfung der Eingaben im GUI Bereich.
     *
     * @return true, falls Eingaben im GUI Bereich in Ordnung sind.
     */
    private boolean eingabePruefen() {
        int     zeilenAnzahl = _table.getItemCount();
        boolean eingabeOK    = false;

        if (!_guiDatenEqualGUIBereich) {
            for (int zeilenNummer = 0; zeilenNummer < zeilenAnzahl; zeilenNummer++) {
                String name  = _table.getItem(zeilenNummer).getText(1);
                String regEx = _table.getItem(zeilenNummer).getText(2);

                try {

                    // Es wird mittels dieses Patternaufrufs getestet, ob es sich bei der Eingabe
                    // um einen regulren Ausdruck handelt, ist dies nicht der fall, wird eine Fehlermeldung
                    // ausgegeben und der Benutzer gebeten seine Eingabe zu korrigieren.
                    @SuppressWarnings("unused") Pattern pattern = Pattern.compile(regEx);

                    eingabeOK = true;
                }
                catch (PatternSyntaxException e) {
                    MessageDialog.openError(_guiVew.getShell(), "Fehler", "Bitte kontrollieren Sie den regulren Ausruck fr das Attribut " + System.getProperty("line.separator") + name + System.getProperty("line.separator") + System.getProperty("line.separator") + e);
                    eingabeOK = false;

                    break;
                }
            }
        }

        return eingabeOK;
    }

    @Override
    public String[] ermittleInputArray() {
        return _alleFilterInstanz.getFilterArray();
    }

    /**
     * Kopiere die Daten aus GUI in den Filter.
     *
     * @param filterElement Filter, wohin die Daten aus GUI kopiert werden.
     */
    private void erzeugeFilterAusGUIDaten(Filter filterElement) {
        String              objTyp = _datenIdentifikationGUIVew.getSelektierteObjType();
        String              atg    = _datenIdentifikationGUIVew.getSelektierteAtg();
        Datenidentifikation datId  = new Datenidentifikation(objTyp, atg, _datenIdentifikationGUIVew.istArchivDaten(), _datenIdentifikationGUIVew.istKonfigurierendeDaten());

        filterElement.getFilterEintraege().add(new FilterEintrag(datId, _tbo, _tabellenZeilen));
        _guiDatenEqualGUIBereich = true;
    }

    /**
     *  Die Methode erzeugeModifizierbareZeilen sorgt dafr, dass die dritte Spalte der Tabelle fr den
     *  Anwender modifizierbar ist. Modifiziert werden kann sobald die dritte Spalte, die die regulren Ausdrcke enthlt
     *  mittels Doppelklick ausgewhlt wurde.
     */
    private void erzeugeModifizierbareZeilen() {

        // -------------------------------------------
        // Definiere Editor im Cursor:
        final TableCursor   cursor = new TableCursor(_table, SWT.NONE);
        final ControlEditor editor = new ControlEditor(cursor);

        editor.grabHorizontal = true;
        editor.grabVertical   = true;

        // -------------------------------------------
        // Erstelle Editor-Text-Control beim Doppelklick auf der letzten Spalte:
        cursor.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseDoubleClick(MouseEvent e) {
                final TableItem row    = cursor.getRow();
                final int       column = cursor.getColumn();

                _table.setSelection(new TableItem[] { row });

                // letzte Spalte:
                if (column == 2) {
                    final Text text = new Text(cursor, SWT.NONE);

                    text.setText(row.getText(column));
                    editor.setEditor(text);
                    text.setFocus();

                    // -------------------------------------------
                    // berwache, wenn Editor geschlossen werde soll:
                    text.addKeyListener(new KeyAdapter() {
                        public void keyPressed(KeyEvent e) {

                            // Wenn der Benutzer "ENTER" bettigt ist die
                            // Eingabe beendet, der Inhalt wird bernommen.
                            if (e.character == SWT.CR) {
                                row.setText(column, text.getText());
                                text.dispose();
                                _guiDatenEqualGUIBereich = false;
                                tabelleAuslesen();
                                aenderungErfolgt();
                            }

                            // Der Editor wird geschlossen wenn "ESCAPE" eingegeben wird.
                            if (e.character == SWT.ESC) {
                                text.dispose();
                            }
                        }
                    });
                }
            }
        });

        // -------------------------------------------
        // berwache, wenn Editor (falls nicht leer) geschlossen werde soll:
        cursor.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent arg0) {

                // Der Editor wird geschlossen, wenn andere Zelle selektiert wird:
                try {
                    editor.getEditor().dispose();
                }
                catch (Exception ignored) {}
            }
        });
    }

    /**
     * Die Methode fuelleTabelle fllt die Tabelle in Abhngigkeit zu den in der Datenidentifikation spezifizierten Werten.
     */
    public void fuelleTabelle() {
        int titelIndex = 0;
        int count      = 3;

        _tabellenZeilen.removeAll(_tabellenZeilen);
        initialisiereTabelle();

        SpaltenObjekt spo      = new SpaltenObjekt(titelIndex);
        ZeilenObjekt  zeileArt = _tbo.erzeugeZeilenObjekt(titelIndex, _constSpaltenNamen[0], REGEX_DEFAULT);

        _tabellenDatenStruktur.addZeile(zeileArt);
        titelIndex++;

        ZeilenObjekt zeileZeit = _tbo.erzeugeZeilenObjekt(titelIndex, _constSpaltenNamen[1], REGEX_DEFAULT);

        _tabellenDatenStruktur.addZeile(zeileZeit);
        titelIndex++;

        // Wenn es sich um einen Archivanfragenfilter handelt muss eine
        // Zustzliche Spalte fr die Archivzeit eingefgt werden, damit auch ber diese
        // gefiltert werden kann.
        if (_datenIdentifikationGUIVew.istArchivDaten()) {
            ZeilenObjekt zeileArchivZeit = _tbo.erzeugeZeilenObjekt(titelIndex, _constSpaltenNamen[2], REGEX_DEFAULT);

            _tabellenDatenStruktur.addZeile(zeileArchivZeit);
            titelIndex++;
        }

        ZeilenObjekt zeileObjekt = _tbo.erzeugeZeilenObjekt(titelIndex, _constSpaltenNamen[3], REGEX_DEFAULT);

        _tabellenDatenStruktur.addZeile(zeileObjekt);

        if (!(_datenIdentifikationGUIVew.getSelektierteAtg().equals(""))) {
            List<String> spaltenNamen;

            spaltenNamen = spo.getSpaltenNamen(_datenIdentifikationGUIVew.getSelektierteAtg());

            for (String sp : spaltenNamen) {
                ZeilenObjekt zeile = _tbo.erzeugeZeilenObjekt(count, sp, REGEX_DEFAULT);

                _tabellenDatenStruktur.addZeile(zeile);
                count++;
            }
        }

        ausrichtungSpaltenInhaltLinks(1);
        _tableViewer.setInput(_tabellenDatenStruktur);
        erzeugeModifizierbareZeilen();

        //
        aenderungErfolgt();
    }

    /**
     * Die Methode fllt das initiale GUI Objekt, das einen leeren Filter reprsentiert.
     */
    private void guiInitialObjFuellen() {
        Datenidentifikation datId = new Datenidentifikation();

        datId.setAtgGroup(_datenIdentifikationGUIVew.getSelektierteAtg());
        datId.setKonfigurierendeDaten(false);
        datId.setObjTyp(_datenIdentifikationGUIVew.getSelektierteObjType());
        datId.setArchivDaten(false);
        _filterElement.getFilterEintraege().add(new FilterEintrag(datId, new TabellenObjekt(), _tabellenZeilen));
        speichereMapInXML();
    }

    /**
     * Die Methode guiVergleichsObjFuellen fllt das Vergleichsobjekt mit den aktuellen Daten des GUI.
     */
    private void guiVergleichsObjFuellen() {
        _datenIdentifikation.setAtgGroup(_datenIdentifikationGUIVew.getSelektierteAtg());
        _datenIdentifikation.setKonfigurierendeDaten(_datenIdentifikationGUIVew.istKonfigurierendeDaten());
        _datenIdentifikation.setObjTyp(_datenIdentifikationGUIVew.getSelektierteObjType());
        _datenIdentifikation.setArchivDaten(_datenIdentifikationGUIVew.istArchivDaten());
        _guiFilterEintragVergleichsObj.setDatenidentifikation(_datenIdentifikation);
        _guiFilterEintragVergleichsObj.setTabellenZeilen(_tabellenZeilen);
        _guiFilterEintragVergleichsObj.setTbo(_tbo);
    }

    @Override
    public void initialisiereElement() {
        _filterElement = new Filter("", new ArrayList<FilterEintrag>());
    }

    /**
     * Die Methode initialisiereFilterGUIVergleichsObj erzeugt ein GUI Vergleichsobjekt fr den Filter.
     * Dieses Vergleichsobjekt reprsentiert eine Filterinstanz, die die aktuellen GUI Datenstze enthlt.
     */
    private void initialisiereFilterGUIVergleichsObj() {
        _datenIdentifikation           = new Datenidentifikation("", "", null, false, false);
        _guiFilterEintragVergleichsObj = new FilterEintrag(_datenIdentifikation, null, null);
    }

    /**
     * @wbp.parser.entryPoint
     */
    @Override
    public void initialisiereGUIBereich(Composite parent) {
        ScrolledComposite scrolledComposite = new ScrolledComposite(parent, SWT.H_SCROLL | SWT.V_SCROLL);

        scrolledComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        scrolledComposite.setExpandHorizontal(true);
        scrolledComposite.setExpandVertical(true);

        Composite composite = new Composite(scrolledComposite, SWT.NONE);

        composite.setLayout(new GridLayout(1, false));
        scrolledComposite.setContent(composite);
        scrolledComposite.setMinSize(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));

        // ----------------------------------------------------
        // Datenidentifikation GUI:
        _datenIdentifikationGUIVew = new DatenidentifikationGUIVew(composite, this);

        // ----------------------------------------------------
        // Section FilterBereich:
        Section sectionFilter = formToolkit.createSection(composite, Section.TWISTIE | Section.TITLE_BAR);

        sectionFilter.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, true, 1, 1));
        formToolkit.paintBordersFor(sectionFilter);
        sectionFilter.setText("Filter");

        Group group = new Group(sectionFilter, SWT.NONE);

        formToolkit.adapt(group);
        formToolkit.paintBordersFor(group);
        sectionFilter.setClient(group);
        group.setLayout(new GridLayout());

        // ----------------------------------------------------
        // Tabelle:
        _table = new Table(group, SWT.FULL_SELECTION);
        _table.setLinesVisible(true);
        _table.setHeaderVisible(true);

        GridData gd__table = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);

        gd__table.heightHint = 250;
        _table.setLayoutData(gd__table);
        formToolkit.adapt(_table);
        formToolkit.paintBordersFor(_table);
        _tableViewer = new TableViewer(_table);

        //
        sectionFilter.setExpanded(true);
    }

    /**
     * Die Methode initialisiereTabelle initialisiert die Tabelle.
     */
    private void initialisiereTabelle() {
        aenderungErfolgt();
        _guiDatenEqualGUIBereich = false;
        _tbo                     = new TabellenObjekt();
        _tableViewer             = new TableViewer(_table);

        int anzahlSpalten = _table.getColumnCount();

        _tabellenDatenStruktur = new TabellenDatenStruktur(anzahlSpalten);

        for (int i = 0; i < anzahlSpalten; i++) {
            _table.getColumn(0).dispose();
        }

        _tabellenDatenStruktur = _tbo.erzeugeTabellenDatenStrukturFilter();
        _tabellenDatenStruktur.erzeugeTabellenKopfzeile(_tableViewer, true);

        // Anbinden des ContentProviders an den TableViewer
        ContentProviderTabellenDatenStruktur contentProviderTabellenDatenStruktur = new ContentProviderTabellenDatenStruktur();

        _tableViewer.setContentProvider(contentProviderTabellenDatenStruktur);

        // Anbinden des LabelProviders an den TableViewer
        LabelProviderTabellenDatenStrukturGTM labelProviderTabellenDatenStruktur = new LabelProviderTabellenDatenStrukturGTM();

        _tableViewer.setLabelProvider(labelProviderTabellenDatenStruktur);

        if (!(_tabellenZeilen == null)) {
            for (ZeilenObjekt zeile : _tabellenZeilen) {
                _tabellenDatenStruktur.addZeile(zeile);
            }

            ausrichtungSpaltenInhaltLinks(1);
        }

        // ToolTip:
        _table.getColumn(2).setToolTipText(TOOL_TIP);

        // Input:
        _tableViewer.setInput(_tabellenDatenStruktur);
    }

    @Override
    public void kopiereElement(String objektName) {
        List<FilterEintrag> fe = new ArrayList<FilterEintrag>();

        fe.addAll(_filterElement.getFilterEintraege());
        _filterElement = new Filter(objektName, fe);
        _alleFilterInstanz.addFilter(_filterElement);
        speichereMapInXML();
        aenderungErfolgt();
        _guiDatenEqualGUIBereich = true;
    }

    @Override
    public void loescheElement(String objektName) {
        _alleFilterInstanz.loescheFilter(objektName);
        speichereMapInXML();
        aenderungErfolgt();
        _guiDatenEqualGUIBereich = true;
    }

    @Override
    public void selektiereElement(String objektName) {
        _filterElement = _alleFilterInstanz.getFilterObjekt(objektName);
        aenderungErfolgt();
        _guiDatenEqualGUIBereich = true;
    }

    //~ SET METHODEN ==========================================================

    /**
     * Setter fr Eigenschaft 'guiVew'.
     *
     * @param guiVew Wert zum Setzen der Eigenschaft 'guiVew'.
     *
     * @see #_guiVew
     */
    public void setGuiVew(RahmenVerwaltungsDialogGUI guiVew) {
        _guiVew = guiVew;
    }

    //~ METHODEN ==============================================================

    /**
     * @wbp.parser.entryPoint
     */
    @Override
    public void setzeGUIbereichMitElementDaten(String objektName) {
        _filterName = objektName;

        Filter filter = _alleFilterInstanz.getFilterObjekt(objektName);

        _tabellenZeilen = new LinkedList<ZeilenObjekt>();
        _datenIdentifikationGUIVew.initialisiereGUI();

        if (!(filter == null)) {
            List<FilterEintrag> filterEintrag = filter.getFilterEintraege();

            if (!filterEintrag.isEmpty()) {
                for (FilterEintrag fi : filterEintrag) {
                    if (!(fi.getDatenidentifikation() == null)) {
                        _datenIdentifikationGUIVew.setzeDaten(fi.getDatenidentifikation().isArchivDaten(), fi.getDatenidentifikation().isKonfigurierendeDaten(),  //
                                                              fi.getDatenidentifikation().getObjTyp(), fi.getDatenidentifikation().getAtgGroup(), "");
                        _datenIdentifikation.setKonfigurierendeDaten(fi.getDatenidentifikation().isKonfigurierendeDaten());
                        _datenIdentifikation.setArchivDaten(fi.getDatenidentifikation().isArchivDaten());
                        _datenIdentifikation.setObjTyp(fi.getDatenidentifikation().getObjTyp());
                        _datenIdentifikation.setAtgGroup(fi.getDatenidentifikation().getAtgGroup());
                        _tbo = fi.getTbo();
                        _tabellenZeilen.addAll(fi.getTabellenZeilen());
                        _guiFilterEintragVergleichsObj.setDatenidentifikation(_datenIdentifikation);
                        _guiFilterEintragVergleichsObj.setTabellenZeilen(fi.getTabellenZeilen());
                        _guiFilterEintragVergleichsObj.setTbo(fi.getTbo());
                        initialisiereTabelle();
                        erzeugeModifizierbareZeilen();
                    }
                }
            } else {
                _datenIdentifikationGUIVew.initialisiereGUI();
                initialisiereTabelle();
                erzeugeModifizierbareZeilen();
            }

            _guiDatenEqualGUIBereich = true;
            aenderungErfolgt();
        }
    }

    @Override
    public boolean sindGuiDatenMitElementDatenGleich() {
        aenderungErfolgt();

        return _guiDatenEqualGUIBereich;
    }

    @Override
    public void speichereElement() {
        if (_filterElement.getFilterName().equals("")) {
            MessageDialog.openError(_guiVew.getShell(), "Fehler", "Es ist kein Filter ausgewhlt!");
        } else {
            if (eingabePruefen()) {
                tabelleAuslesen();
                _filterElement.setFilterEintraege(new ArrayList<FilterEintrag>());
                erzeugeFilterAusGUIDaten(_filterElement);
                speichereMapInXML();
                aenderungErfolgt();
                _guiDatenEqualGUIBereich = true;
            } else {
                MessageDialog.openError(_guiVew.getShell(), "Fehler", "Die Eingabe wurde nicht gespeichert, bitte erst korrigieren!");
            }
        }
    }

    /**
     * Speichere Map von definierten Filtern in XML Datei.
     * Die XML Datei hngt von _modus ab.
     */
    private void speichereMapInXML() {
        _alleFilterInstanz.speichereAlleFilter();
    }

    /**
     * Die Methode tabelleAuslesen liest die Tabelle aus und aktualisiert damit
     * die Liste der TabellenZeilen.
     */
    private void tabelleAuslesen() {
        _tabellenZeilen = new LinkedList<ZeilenObjekt>();

        int zeilenAnzahl = _table.getItemCount();

        if (!_guiDatenEqualGUIBereich) {
            _tabellenZeilen.removeAll(_tabellenZeilen);

            for (int zeilenNummer = 0; zeilenNummer < zeilenAnzahl; zeilenNummer++) {
                ZeilenObjekt zeile = _tbo.erzeugeZeilenObjekt(zeilenNummer, _table.getItem(zeilenNummer).getText(1), _table.getItem(zeilenNummer).getText(2));

                _tabellenZeilen.add(zeile);
            }
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 16.10.13
