/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.filter;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.plugin.gtm.navigation.datenidentifikation.Datenidentifikation;
import de.inovat.buv.plugin.gtm.navigation.tabellen.TabellenObjekt;
import de.inovat.buv.plugin.gtm.navigation.tabellen.ZeilenObjekt;

//~ JDK IMPORTE ===============================================================

import java.util.LinkedList;

//~ KLASSEN ===================================================================

/**
 * Die Klasse beschreibt einen Eintrag im FilterObjekt.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class FilterEintrag {

    /** Datenidentifikation des Filtereintrages. */
    private Datenidentifikation _datenidentifikation;

    /** Tabellenzeilen. */
    private LinkedList<ZeilenObjekt> _tabellenZeilen;

    /** Tabellenobjekt. */
    private TabellenObjekt _tbo;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * WICHTIG_: Die Methode darf nicht aufgerufen werden. Die wird vom
     * XMLDecoder bentigt.
     */
    public FilterEintrag() {}

    /**
     * Erzeugt eine Instanz der Klasse FilterEintrag.
     * @param tabellenZeilen Tabellenzeile.
     * @param tbo  Tabellenobjekt.
     * @param datId  Datenidentifikation.
     */
    public FilterEintrag(Datenidentifikation datId, TabellenObjekt tbo, LinkedList<ZeilenObjekt> tabellenZeilen) {
        _tbo                 = tbo;
        _datenidentifikation = datId;
        _tabellenZeilen      = tabellenZeilen;
    }

    //~ METHODEN ==============================================================

    /*
     *  (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }

        if (obj == null) {
            return false;
        }

        if (!(obj instanceof FilterEintrag)) {
            return false;
        }

        FilterEintrag other = (FilterEintrag) obj;

        if (_datenidentifikation == null) {
            if (other._datenidentifikation != null) {
                return false;
            }
        } else if (!_datenidentifikation.equals(other._datenidentifikation)) {
            return false;
        }

        if (_tabellenZeilen == null) {
            if (other._tabellenZeilen != null) {
                return false;
            }
        } else if (!_tabellenZeilen.equals(other._tabellenZeilen)) {
            return false;
        }

        return true;
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'datenidentifikation'.
     *
     * @return Wert der Eigenschaft 'datenidentifikation'.
     *
     * @see #_datenidentifikation
     */
    public Datenidentifikation getDatenidentifikation() {
        return _datenidentifikation;
    }

    /**
     * @return the tabellenZeilen
     */
    public LinkedList<ZeilenObjekt> getTabellenZeilen() {
        return _tabellenZeilen;
    }

    /**
     * @return the tbo
     */
    public TabellenObjekt getTbo() {
        return _tbo;
    }

    //~ METHODEN ==============================================================

    /*
     *  (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        final int prime  = 31;
        int       result = 1;

        result = prime * result + ((_datenidentifikation == null) ? 0 : _datenidentifikation.hashCode());
        result = prime * result + ((_tabellenZeilen == null) ? 0 : _tabellenZeilen.hashCode());
        result = prime * result + ((_tbo == null) ? 0 : _tbo.hashCode());

        return result;
    }

    //~ SET METHODEN ==========================================================

    /**
     * Setter fr Eigenschaft 'datenidentifikation'.
     *
     * @param datenidentifikation Wert zum Setzen der Eigenschaft 'datenidentifikation'.
     *  WICHTIG! Verwendung fr die Speicherung im XML.
     * @see #_datenidentifikation
     */
    public void setDatenidentifikation(Datenidentifikation datenidentifikation) {
        _datenidentifikation = datenidentifikation;
    }

    /**
     * Setter fr Eigenschaft 'tabellenZeilen'.
     *
     * @param tabellenZeilen Wert zum Setzen der Eigenschaft 'tabellenZeilen'.
     *  WICHTIG! Verwendung fr die Speicherung im XML.
     * @see #_tabellenZeilen
     */
    public void setTabellenZeilen(LinkedList<ZeilenObjekt> tabellenZeilen) {
        _tabellenZeilen = tabellenZeilen;
    }

    /**
     * Setter fr Eigenschaft 'tbo'.
     *
     * @param tbo Wert zum Setzen der Eigenschaft 'tbo'.
     *  WICHTIG! Verwendung fr die Speicherung im XML.
     * @see #_tbo
     */
    public void setTbo(TabellenObjekt tbo) {
        _tbo = tbo;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 03.09.10
