/*
 * @(#)NavigationGUIVew.java   07.03.12
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.gui;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.buv.plugin.selektion.SelektionsTransfer;

import de.inovat.buv.gtm.datvew.lib.BaumHilfsFunktionen;
import de.inovat.buv.gtm.datvew.lib.DatenVew;
import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.inovat.lib.funktionen.datum.DatumFunktionen;
import de.inovat.buv.plugin.gtm.navigation.Activator;
import de.inovat.buv.plugin.gtm.navigation.filter.AlleFilter;
import de.inovat.buv.plugin.gtm.navigation.gui.lib.DarstellungenVew;
import de.inovat.buv.plugin.gtm.navigation.gui.lib.IDarstellung;
import de.inovat.buv.plugin.gtm.navigation.sortierung.AlleSortierungen;
import de.inovat.buv.plugin.gtm.navigation.tabellenverwaltung.AlleTabellenVerwaltungen;
import de.inovat.buv.projektlib.konstanten.KonstantenGUIResource;

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.internal.misc.StringMatcher;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * GUI-Verwaltung.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class NavigationGUIVew {

    /** Instanz DarstellungenVew. */
    private DarstellungenVew _darstellungenVew;

    /** GUI-Instanz. */
    private NavigationGUI _gui;

    /** Definiert, ob der Baum aufgeklappt ist. */
    private boolean _istAufgeklappt;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse NavigationGUIVew.
     *
     * @param gui GUI-Instanz.
     */
    public NavigationGUIVew(NavigationGUI gui) {
        _gui = gui;
    }

    //~ METHODEN ==============================================================

    /**
     * Auf/zuklappe den Navigationbaum.
     */
    protected void baumAufZuKlappen() {
        if (_istAufgeklappt) {
            _gui.getLbAufZuKlappen().setImage(KonstantenGUIResource.ICON_VEW_ALLE_AUFKLAPEN);
            _gui.getLbAufZuKlappen().setToolTipText("Alles aufklappen");
            _gui.getTreeViewer().collapseAll();
        } else {
            _gui.getLbAufZuKlappen().setImage(KonstantenGUIResource.ICON_VEW_ALLE_ZUKLAPEN);
            _gui.getLbAufZuKlappen().setToolTipText("Alles zuklappen");
            _gui.getTreeViewer().expandAll();
        }

        _istAufgeklappt = !_istAufgeklappt;
    }

    /**
     * Schaltflche 'Aktualisieren' wird selektiert.
     */
    protected void btnAktualisierenSelektiert() {
        setzeInput();
    }

    /**
     * Checkbutton 'Aktualisieren' wird selektiert.
     */
    protected void cbtnAktualisierenSelektiert() {
        boolean istAutomatisch = _gui.getCbtnAktualisieren().getSelection();

        _gui.getBtnAktualisieren().setEnabled(!istAutomatisch);

        if (istAutomatisch) {
            _gui.getBtnAktualisieren().setText("wird automatisch aktualisiert");
        } else {
            _gui.getBtnAktualisieren().setText("aktualisieren");
        }

        //
        _gui.getBtnAktualisieren().getParent().layout();
    }

    /**
     * ComboSicht wird selektiert.
     */
    protected void comboSichtSelektiert() {

        // ---------------------------------------------------------------
        // Icon Baum zugeklappt:
        initLbAufZuKlappen();

        // ---------------------------------------------------------------
        // Setze RadioButton Sichtbarkeit:
        IDarstellung sicht = ermittleSelektierteDarstellung();

        // Suche ist erlaubt:
        if ((sicht != null) && (sicht.istSucheEingeschaltet())) {
            _gui.getRbtSuche().setEnabled(true);
        }

        // Suche wird nicht eingeschaltet:
        else {
            _gui.getRbtSuche().setEnabled(false);
            _gui.getRbtSuche().setSelection(false);
            _gui.getRbtnFilter().setSelection(true);
        }

        // ---------------------------------------------------------------
        // Button Aktualisieren:
        if ((sicht != null) && (sicht.istDynamisch())) {
            _gui.getBtnAktualisieren().setVisible(true);

            if (sicht.istModellAenderungUeberwacht()) {
                _gui.getCbtnAktualisieren().setVisible(true);
            } else {
                _gui.getCbtnAktualisieren().setVisible(false);
                _gui.getCbtnAktualisieren().setSelection(false);
            }

            cbtnAktualisierenSelektiert();
        } else {
            _gui.getCbtnAktualisieren().setVisible(false);
            _gui.getBtnAktualisieren().setVisible(false);
        }

        // ---------------------------------------------------------------
        // Baum:
        definiereBaum();
    }

    /**
     * Definiere den NavigationBaum (Struktur, Beschriftung und Inhalt)
     * und starte Suche/Filter.
     */
    private void definiereBaum() {
        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                try {

                    //
                    IDarstellung sicht = ermittleSelektierteDarstellung();

                    if (sicht != null) {

                        // ---------------------------------------------------------------
                        // Baum-Struktur
                        _gui.getTreeViewer().setContentProvider(sicht.getContentProvider());
                        _gui.getTreeViewer().setLabelProvider(sicht.getLabelProvider());

                        // ---------------------------------------------------------------
                        // Baum-Input:
                        setzeInput();

                        // ---------------------------------------------------------------
                        // Suche/Filter:
                        rbtnSucheFilterSelektiert();
                    }
                }
                catch (Exception e) {
                    Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
                }
            }
        });
    }

    /**
     * Ermittle die im Combo selektierte Darstellung.
     *
     * @return Selektierte Darstellung oder null.
     */
    private IDarstellung ermittleSelektierteDarstellung() {
        return _darstellungenVew.ermittleDarstellung(_gui.getComboSicht().getSelectionIndex());
    }

    /**
     * Frbe das TextFeld entsprechend des Such-Ergebnises.
     *
     * @param erfolgreich true, falls die Suche erfolgreich ist,
     * false - sonst.
     */
    private void faerbeTxtSuche(boolean erfolgreich) {
        if (erfolgreich) {
            _gui.getTxtSucheFilter().setBackground(KonstantenGUIResource.FARBE_WEIS);
        } else {
            _gui.getTxtSucheFilter().setBackground(KonstantenGUIResource.FARBE_ROT_HELL);
        }
    }

    /**
     * Initialisiere das Combo Sicht.
     */
    private void initComboSicht() {

        // Combo-Elemente:
        _gui.getComboSicht().setItems(_darstellungenVew.ermittleAlleDarstellungen());
        _gui.getComboSicht().getParent().layout();

        // Selektion:
        _gui.getComboSicht().select(0);

        // Navigationbaum:
        comboSichtSelektiert();
    }

    /**
     * Initialisiere das GUI.
     */
    protected void initGui() {

        //
        DatenVew.getInstanz();
        _darstellungenVew = DarstellungenVew.getInstanz();

        // Suche/Filter:
        _gui.getRbtSuche().setSelection(true);
        _gui.getRbtnFilter().setSelection(false);

        // Automatische Aktualisierung:
        _gui.getCbtnAktualisieren().setSelection(false);

        // Initialisiere das Combo:
        initComboSicht();

        // Lade alle Einstellungen:
        AlleTabellenVerwaltungen._klasseLaden = true;
        AlleFilter._klasseLaden               = true;
        AlleSortierungen._klasseLaden         = true;
    }

    /**
     * Initialisiere das Label 'Baum auf/zu klappen'.
     */
    protected void initLbAufZuKlappen() {
        _istAufgeklappt = false;
        _gui.getLbAufZuKlappen().setText("");
        _gui.getLbAufZuKlappen().setImage(KonstantenGUIResource.ICON_VEW_ALLE_AUFKLAPEN);
        _gui.getLbAufZuKlappen().setToolTipText("Alles aufklappen");
    }

    /**
     * Reaktion auf Tasten-Druck.
     *
     * @param keyCode Code der gedruckten Taste.
     * @param istTextfeld true, falls die Taste im Textfeld gedruckt wird,
     * false - im Baum.
     */
    protected void keyListener(int keyCode, boolean istTextfeld) {

        // Weiter suchen:
        if (keyCode == SWT.F3) {
            sucheImBaum(false);
        }

        // Text lschen:
        else if (keyCode == SWT.ESC) {
            _gui.getTxtSucheFilter().setText("");
        }

        //
        else {

            // Textfeld:
            if (istTextfeld) {

                // Suche ab Anfang:
                if (keyCode == SWT.CR) {
                    sucheImBaum(true);
                }
            }

            // Baum:
            else {

                // Fokus auf dem Suchfeld:
                if (keyCode == SWT.F7) {
                    _gui.getTxtSucheFilter().setFocus();
                    _gui.getTxtSucheFilter().selectAll();
                }
            }
        }
    }

    /**
     * DatenModell wurde gedert.
     *
     * @param sicht Darstellung, deren Daten gendert wurden.
     */
    protected void modellGeaendert(final IDarstellung sicht) {
        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                try {

                    // Definiere gerade selektierte Sicht:
                    IDarstellung sichtSelektiert = ermittleSelektierteDarstellung();

                    // Falls automatisch aktualisieren und Sicht dynamisch ist:
                    if ((_gui.getCbtnAktualisieren().getSelection()) && (sichtSelektiert.istDynamisch())) {

                        // Falls die gerade selektierte Sicht gendert wurde:
                        if (sicht == sichtSelektiert) {

                            // ---------------------------------------------------------------
                            // Baum-Input:
                            setzeInput();

                            // ---------------------------------------------------------------
                            // Suche/Filter:
                            rbtnSucheFilterSelektiert();
                        }
                    }
                }
                catch (Exception e) {
                    Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
                }
            }
        });
    }

    /**
     * Stelle fest, ob das Element zur Suche passt.
     * Dabei wird ber die Representation des Elements
     * im Baum mit dem Text aus dem Suchfeld verglichen.
     *
     * @param element Das zu prfende Element.
     *
     * @return true, falls das Element zur Suche passt,
     * false - sonst.
     */
    private boolean passtZurSuche(Object element) {
        ILabelProvider providerLb = (ILabelProvider) _gui.getTreeViewer().getLabelProvider();
        String         pattern    = String.format("*%s*", _gui.getTxtSucheFilter().getText());
        StringMatcher  matcher    = new StringMatcher(pattern, true, false);

        return matcher.match(providerLb.getText(element));
    }

    /**
     * Radio-Button Suche bzw. Filter wird selektiert.
     */
    protected void rbtnSucheFilterSelektiert() {
        String txt = _gui.getTxtSucheFilter().getText();

        // Lse die Listener aus:
        _gui.getTxtSucheFilter().setText("");
        _gui.getTxtSucheFilter().setText(txt);

        // Suche:
        if (!txt.isEmpty()) {
            sucheImBaum(true);
        }
    }

    /**
     * Setze Info ber die letzte Aktualisierung.
     */
    private void setzeAktualisierungsInfo() {
        String info = String.format("letzte Aktualisierung: %s",  //
                                    DatumFunktionen.getUhrzeitAlsString(DatumFunktionen.getAktuelleDateTime()));

        _gui.getBtnAktualisieren().setToolTipText(info);

        if (!_gui.getBtnAktualisieren().getEnabled()) {
            _gui.getBtnAktualisieren().setText(info);
        }
    }

    /**
     * Setze das Input des Baums.
     */
    private void setzeInput() {
        IDarstellung sicht = ermittleSelektierteDarstellung();

        if (sicht != null) {

            // Baum-Input:
            Object input = sicht.getInput();

            if (input == null) {
                input = new ArrayList<Object>();
                Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, String.format("Fehler in der Sicht <%s>: Methode getInput() liefert null", sicht.getName())));
            }

            if (!(input instanceof List)) {
                input = new ArrayList<Object>();
                Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, String.format("Fehler in der Sicht <%s>: Methode getInput() muss eine Liste liefern", sicht.getName())));
            }

            _gui.getTreeViewer().setInput(input);

            // Info ber Aktualisierung:
            setzeAktualisierungsInfo();
        }
    }

    /**
     * Suche das passende Element im Baum und, falls gefunden,
     * selektiere das.
     *
     * @param alle Legt fest, welche Elemente durchgesucht werden solle.
     * Falls true- alle Elemente.
     * Beim false - ab selektiertem Elementen und weiter im Baum nach unten.
     */
    protected void sucheImBaum(boolean alle) {

        // -------------------------------------------------------------------
        faerbeTxtSuche(true);

        // -------------------------------------------------------------------
        // Falls die Suche nicht eingeschaltet ist:
        if (!_gui.istSucheEingeschaltet()) {
            return;
        }

        // Falls der zu suchende Text leer ist:
        if (_gui.getTxtSucheFilter().getText().isEmpty()) {
            return;
        }

        // -------------------------------------------------------------------
        // Ermittle Liste aller zu suchender Elemente (ohne Kinder):
        // (zwischen diesen Elementen und deren Kindern wird weiter gesucht)
        ITreeContentProvider providerCt    = (ITreeContentProvider) _gui.getTreeViewer().getContentProvider();
        List<Object>         listeElemente = new ArrayList<Object>();

        // Ermittle die Root-Liste des Baums:
        List<Object> inputListe;
        IDarstellung sicht = ermittleSelektierteDarstellung();

        try {
            inputListe = (List<Object>) sicht.getInput();
        }
        catch (Exception e) {
            inputListe = new ArrayList<Object>();
        }

        // Selektion:
        IStructuredSelection sel = (IStructuredSelection) _gui.getTreeViewer().getSelection();

        //
        boolean erstesElementBeruecksichtigen;

        // Falls alle durchgesucht werden sollen,
        // wird die inputListe hinzugefgt:
        if ((sel.size() != 1) || (alle)) {
            erstesElementBeruecksichtigen = true;
            listeElemente.addAll(inputListe);
        }

        // Falls Elemente ab dem Selektierten durchgesucht werden sollen:
        else {
            erstesElementBeruecksichtigen = false;

            Object elSel = sel.getFirstElement();

            listeElemente.add(elSel);

            Object parent = providerCt.getParent(elSel);
            int    tiefe  = 0;

            while (parent != null) {
                parent = providerCt.getParent(parent);
                tiefe++;
            }

            //
            parent = providerCt.getParent(elSel);

            Object[] arKinder;
            boolean  hinzufuegen;

            for (int i = 0; i <= tiefe; i++) {
                hinzufuegen = false;

                if (i < tiefe) {
                    arKinder = providerCt.getChildren(parent);
                } else {
                    arKinder = inputListe.toArray();
                }

                if (arKinder != null) {
                    for (Object kind : arKinder) {
                        if (hinzufuegen) {
                            listeElemente.add(kind);
                        }

                        if (kind == elSel) {
                            hinzufuegen = true;
                        }
                    }
                }

                //
                elSel  = parent;
                parent = providerCt.getParent(elSel);
            }
        }

        // -------------------------------------------------------------------
        // Suche in der Liste der Elemente zwischen den Elementen
        // und deren Kindern nach passendem Element:
        Object  elGefunden = null;
        boolean erstes     = true;

        for (Object el : listeElemente) {
            if (!erstesElementBeruecksichtigen && erstes) {
                elGefunden = sucheImBaum(el, false);
            } else {
                elGefunden = sucheImBaum(el, true);
            }

            if (elGefunden != null) {
                _gui.getTreeViewer().setSelection(new StructuredSelection(elGefunden));

                break;
            }

            erstes = false;
        }

        // -------------------------------------------------------------------
        if (elGefunden == null) {
            faerbeTxtSuche(false);
        }
    }

    /**
     * Suche rekursiv das passende Element im Baum.
     * Gesucht wird ab angegebenem Element (inklusive) zwischen seinen Kindern.
     *
     * @param element Element des Baums, ab dem die suche anfngt.
     * @param elementBeruecksichtigen Legt fest, ob das Element selber
     * bei der Suche bercksichtigt werden soll.
     * Falls false, wird das Element aus der Suche ausgeschlossen.
     *
     * @return Erstes passendes Element oder null,
     * falls nicht gefunden wird.
     */
    private Object sucheImBaum(Object element, boolean elementBeruecksichtigen) {

        // ---------------------------------------------------
        // Falls das Element bei der Suche bercksichtigt werden soll, prfe das:
        if (elementBeruecksichtigen) {

            // Falls das Element zu suchende ist:
            if (passtZurSuche(element)) {
                return element;
            }
        }

        // ---------------------------------------------------
        // Suche das passende Element zwischen Element-Kindern:
        ITreeContentProvider providerCt = (ITreeContentProvider) _gui.getTreeViewer().getContentProvider();
        Object[]             arKinder   = providerCt.getChildren(element);

        if (arKinder == null) {
            return null;
        } else {
            Object ergebnis;

            for (Object kind : arKinder) {
                ergebnis = sucheImBaum(kind, true);

                if (ergebnis != null) {
                    return ergebnis;
                }
            }
        }

        return null;
    }

    /**
     * Die Selektion in TreeViewer wird gendert.
     */
    protected void treeViewerSelektionGeaendert() {

        // -----------------------------------
        // Selektion speichern:
        IWorkbenchPart sicht = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActivePart();

        SelektionsTransfer.getInstanz().selectionChanged(sicht, _gui.getTreeViewer().getSelection());

        // -----------------------------------
        // Kontexmen:
        Menu menu = new Menu(_gui.getShell(), SWT.POP_UP);

        BaumHilfsFunktionen.erstelleKontextMenu(menu, AlleTabellenVerwaltungen.getTabellenVerwaltungInstanz().getTabellenVerwaltungArray());
        _gui.getTreeViewer().getTree().setMenu(menu);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 06.10.14
