/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.tabellen;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.config.Attribute;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeListDefinition;
import de.bsvrz.dav.daf.main.config.AttributeSet;

import de.inovat.buv.gtm.datvew.lib.BaumVerwaltung;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

//~ KLASSEN ===================================================================

/**
 * Die Klasse beschreibt ein Spaltenobjekt der Tabelle.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class SpaltenObjekt {

    /**
     * Die Liste _prefixAtt enthlt die Prfixe zu einem Attribut, wenn es sich um ein Listenattribut handelt.
     * Der key ist der Name des Attributes, der Wert ist der Prefix.
     */

    // private static Map<String, String> _prefixAtt = new HashMap<String, String>();

    /** Jedes SpaltenObjekt steht fr eine Tabellenspaltenstruktur mit einer eigenen Kopfzeile. */
    private List<String> _spaltenNamen = new ArrayList<String>();

    /** Liste mit dem Attributtyp der Spaltennamen. */
    private List<String> _spaltenNamenTyp = new ArrayList<String>();

    /**
     * Map speichert den Spaltennamen als Schlssel und Anzahl der Attributwerte, die zu dem Attribut, fr
     * das die Spalte erzeugt wurde, maximal angezeigt werden. 
     * Beispiel: 
     * FehlercodeTLS.Fehler[0] (Schlssel)
     * 4 (Wert)
     * Der Maximalwert resultiert aus der Eigenschaft, ob die Feldgre des Attributes beschrnkt ist oder nicht,
     * und ob die Feldgre des Attributes variieren kann. Wenn die Feldgre beschrnkt ist, wird diese
     * fr das Attribut als Maximalwert verwendet, ist dies nicht der fall, wird die Grenze maxAnzahlVarAtt
     * verwendet, die vom Benutzer im GUI Tabellendarstellungsdialog eingestellt werden kann.
     */
    private Map<String, Integer> _mapAttributMaxWertDarzustellenderAttribute = new HashMap<String, Integer>();

    /**
     * Map mit Schlssel Feldname und Wert gleich der Anzahl der Felder die ein Arrayelement beinhaltet. Z.B.
     * WVZAnzeige wird in der folgenden Art geplttet:
     * WVZAnzeige[i].Code, WVZAnzeige[i].Blinken.Status, WVZAnzeige[i].Status.Fehler, WVZAnzeige[i].Status.Programm,
     * daraus resultiert eine Feldlnge von 5 fr das Arrayelement.
     */
    private Map<String, Integer> _mapArrayElementNameWAnzahlFelderDesElementes = new HashMap<String, Integer>();

    /** Variable die die Anzahl der Variablen Attribute reprsentiert. */
    private int _anzahlVarAtt;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse SpaltenObjekt.
     */
    public SpaltenObjekt() {}

    public SpaltenObjekt(int anzahlVarAtt) {
        _anzahlVarAtt = anzahlVarAtt;
    }

    //~ METHODEN ==============================================================

    /**
     * Methode zur Erzeugung der Tabellenstruktur.
     * @param atgSet Menge der Attribute.
     * @param pref gibt den Prfix an.
     */
    public void bestimmeSpaltenNamenStruktur(AttributeSet atgSet, String pref) {
    	BaumVerwaltung verwaltung = BaumVerwaltung.getInstanz();

        // Maximal Anzahl der Schleifendurchlufe.
        int maxWert = 0;

        for (Attribute att : atgSet.getAttributes()) {

            // Maximale Feldgre des Attributes ist nicht beschrnkt oder variabel.
            if (att.getMaxCount() != 0) {
            	maxWert = att.getMaxCount();
            } 
            if((att.getMaxCount() == 0) || (att.isCountVariable())) {           
                maxWert = _anzahlVarAtt;
            }
            
            if (verwaltung.isArray(att)) {
                if (maxWert > 0) {
                    for (int i = 0; i < maxWert; i++) {
                        String                  zaehler  = "[" + i + "]";
                        AttributeListDefinition attListe = verwaltung.bestimmeAttributListe(att);
                        if(!(attListe==null)) {                        	
                        	bestimmeSpaltenNamenStruktur(attListe, pref + verwaltung.getSpaltenText(att) + zaehler);
                        }else {
                        	 String prefix;
                             prefix = verwaltung.erzeugePrefixFuerSpaltenNamen(pref, verwaltung.getSpaltenText(att) + zaehler);
                        	 _spaltenNamen.add(prefix);
                        	 String prefixFuerFuellenDerTabelle = verwaltung.erzeugePrefixFuerSpaltenNamen(pref, verwaltung.getSpaltenText(att));
                        	 _mapAttributMaxWertDarzustellenderAttribute.put(prefixFuerFuellenDerTabelle, maxWert);
                        }
                    }
                } else {
                    _spaltenNamen.add(pref + verwaltung.getSpaltenText(att));
                }
            } else if (verwaltung.isAttributeListeDefinition(att)) {
                String prefixAttListe;

                prefixAttListe = verwaltung.erzeugePrefixFuerSpaltenNamen(pref, verwaltung.getSpaltenText(att));

                AttributeListDefinition attListe = verwaltung.bestimmeAttributListe(att);

                bestimmeSpaltenNamenStruktur(attListe, prefixAttListe);
            } else if (verwaltung.isBasisType(att) || verwaltung.isRefrenceAttributeType(att)) {
                String prefixAttListe;

                prefixAttListe = verwaltung.erzeugePrefixFuerSpaltenNamen(pref, verwaltung.getSpaltenText(att));
                _spaltenNamen.add(prefixAttListe);
            }
        }
    }

    //~ GET METHODEN ==========================================================

    /**
	 * Setter fr Eigenschaft 'mapArrayElementNameWAnzahlFelderDesElementes'.
	 *
	 * @param mapArrayElementNameWAnzahlFelderDesElementes Wert zum Setzen der Eigenschaft 'mapArrayElementNameWAnzahlFelderDesElementes'. 
	 *
	 * @see #_mapArrayElementNameWAnzahlFelderDesElementes
	 */
	public void setMapArrayElementNameWAnzahlFelderDesElementes(
			Map<String, Integer> mapArrayElementNameWAnzahlFelderDesElementes) {
		_mapArrayElementNameWAnzahlFelderDesElementes = mapArrayElementNameWAnzahlFelderDesElementes;
	}

	/**
     *     Getter fr Eigenschaft 'mapArrayElementNameWAnzahlFelderDesElementes'.
     *    
     *     @return Wert der Eigenschaft 'mapArrayElementNameWAnzahlFelderDesElementes'.
     *    
     *     @see #_mapArrayElementNameWAnzahlFelderDesElementes
     */
    public Map<String, Integer> getMapArrayElementNameWAnzahlFelderDesElementes() {
        return _mapArrayElementNameWAnzahlFelderDesElementes;
    }

    /**
     * Getter fr Eigenschaft 'mapAttributMaxWertDarzustellenderAttribute'.
     *
     * @return Wert der Eigenschaft 'mapAttributMaxWertDarzustellenderAttribute'.
     *
     * @see #_mapAttributMaxWertDarzustellenderAttribute
     */
    public Map<String, Integer> getMapAttributMaxWertDarzustellenderAttribute() {
        return _mapAttributMaxWertDarzustellenderAttribute;
    }

    /**
     * Methode liefert die Spaltennamen der Tabelle zu einer Attributgruppe und einem Aspekt.
     * @param atgPid Pid der Attributgruppe.
     * @return _spaltenNamen Liste der Spaltennamen.
     */
    public List<String> getSpaltenNamen(String atgPid) {
        _spaltenNamen.removeAll(_spaltenNamen);

        AttributeGroup atg = BaumVerwaltung.getInstanz().bestimmeAttributGruppeZuPidTabellenstruktur(atgPid);
        if(atg!=null) {        	
        	bestimmeSpaltenNamenStruktur(atg, "");
        }

        return _spaltenNamen;
    }

    /**
     *     Getter fr Eigenschaft 'spaltenNamenTyp'.
     *
     *     @return Wert der Eigenschaft 'spaltenNamenTyp'.
     *
     *     @see #_spaltenNamenTyp
     */
    public List<String> getSpaltenNamenTyp() {
        return _spaltenNamenTyp;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 14.04.10
