/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.tabellenverwaltung;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.plugin.gtm.navigation.archiv.ArchivanfrageDaten;
import de.inovat.buv.plugin.gtm.navigation.datenidentifikation.Datenidentifikation;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 *  Beschreibung eines Eintrages im Tabellenverwaltungs Objekt.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class TabellenVerwaltungEintrag {

    /** Liste mit allen gewhlten Knoten im Baum. */
    private List<String> _gewaehlteKnotenBaum = new ArrayList<String>();

    /** Boolesche Variable die angibt ob es sich um eine nderungsdarstellung handelt. */
    private boolean _aenderungsdarstellung;

    /** Archivanfragedaten. */
    private ArchivanfrageDaten _archivanfrageDaten;

    /** Datenidentifikation. */
    private Datenidentifikation _datenidentifikation;

    /** Filtername. */
    private String _filterAuswahlName;

    /** Boolesche Variable die angibt ob es sich um eine Archivdaten Tabellenverwaltung handelt. */
    private boolean _isArchiv;

    /**
     * Legt fest, ob die in Verwaltung gespeicherte Selektion in die TabellenDarstellung
     * bernommen werden soll. Falls ja, wird die Selektion bernommen. Falls nein, kommt
     * die Nachfrage.
     */
    private boolean _selektionUebernehmen;

    /** Sortierauswahlname. */
    private String _sortierAuswahlName;

    /** Spaltenauswahlname. */
    private String _spaltenAuswahlName;

    /** Anzahl der variablen Attribute. */
    private int _varAtt;

    /** Anzahl der Tabellenzeilen. */
    private int _varTbl;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * WICHTIG_: Die Methode darf nicht aufgerufen werden. Die wird von XMLDecoder bentigt.
     *
     * Erzeugt eine Instanz der Klasse SortierungEintrag.
     */
    public TabellenVerwaltungEintrag() {}

    /**
     * Erzeugt eine Instanz der Klasse TabellenVerwaltungEintrag.
     *
     * @param aenderungsdarstellung  Flag ob es sich um eine nderungsdarstellung handelt.
     * @param datenidentifikation  Datenidentifikation.
     * @param filterName  Filtername.
     * @param sortierAuswahlName  Auswahlname der Sortierung.
     * @param spaltenAuswahlName Auswahlname Spalten.
     * @param varAtt Anzahl variabler Attribute.
     * @param varTbl Anzahl Tabellenzeilen.
     * @param gewaehlteKnotenBaum   Gewhlte Knoten im Baum.
     * @param isArchiv boolesche Variable die angibt ob es sich um einen Tabellenvewaltungsdialog fr Archivdaten handelt.
     * @param archivanfrageDaten   Archivanfragedaten.
     * @param selektionUebernehmen Ja, falls die gespeicherte Selektion bernommen werden sool,
     * nein - es soll nachgefragt werden.
     */
    public TabellenVerwaltungEintrag(boolean aenderungsdarstellung, Datenidentifikation datenidentifikation, String filterName, String sortierAuswahlName, String spaltenAuswahlName, int varAtt, int varTbl, List<String> gewaehlteKnotenBaum, boolean isArchiv, ArchivanfrageDaten archivanfrageDaten, boolean selektionUebernehmen) {
        _aenderungsdarstellung = aenderungsdarstellung;
        _datenidentifikation   = datenidentifikation;
        _filterAuswahlName     = filterName;
        _sortierAuswahlName    = sortierAuswahlName;
        _spaltenAuswahlName    = spaltenAuswahlName;
        _varAtt                = varAtt;
        _varTbl                = varTbl;
        _gewaehlteKnotenBaum   = gewaehlteKnotenBaum;
        _isArchiv              = isArchiv;
        _archivanfrageDaten    = archivanfrageDaten;
        _selektionUebernehmen  = selektionUebernehmen;
    }

    //~ METHODEN ==============================================================

    /*
     *  (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }

        if (obj == null) {
            return false;
        }

        if (!(obj instanceof TabellenVerwaltungEintrag)) {
            return false;
        }

        TabellenVerwaltungEintrag other = (TabellenVerwaltungEintrag) obj;

        if (_aenderungsdarstellung != other._aenderungsdarstellung) {
            return false;
        }

        if (_archivanfrageDaten == null) {
            if (other._archivanfrageDaten != null) {
                return false;
            }
        } else if (!_archivanfrageDaten.equals(other._archivanfrageDaten)) {
            return false;
        }

        if (_datenidentifikation == null) {
            if (other._datenidentifikation != null) {
                return false;
            }
        } else if (!_datenidentifikation.equals(other._datenidentifikation)) {
            return false;
        }

        if (_filterAuswahlName == null) {
            if (other._filterAuswahlName != null) {
                return false;
            }
        } else if (!_filterAuswahlName.equals(other._filterAuswahlName)) {
            return false;
        }

        if (_gewaehlteKnotenBaum == null) {
            if (other._gewaehlteKnotenBaum != null) {
                return false;
            }
        } else if (!_gewaehlteKnotenBaum.equals(other._gewaehlteKnotenBaum)) {
            return false;
        }

        if (_sortierAuswahlName == null) {
            if (other._sortierAuswahlName != null) {
                return false;
            }
        } else if (!_sortierAuswahlName.equals(other._sortierAuswahlName)) {
            return false;
        }

        if (_spaltenAuswahlName == null) {
            if (other._spaltenAuswahlName != null) {
                return false;
            }
        } else if (!_spaltenAuswahlName.equals(other._spaltenAuswahlName)) {
            return false;
        }

        if (_varAtt != other._varAtt) {
            return false;
        }

        if (_varTbl != other._varTbl) {
            return false;
        }

        if (_selektionUebernehmen != other._selektionUebernehmen) {
            return false;
        }

        return true;
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'archivanfrageDaten'.
     *
     * @return Wert der Eigenschaft 'archivanfrageDaten'.
     *
     * @see #_archivanfrageDaten
     */
    public ArchivanfrageDaten getArchivanfrageDaten() {
        return _archivanfrageDaten;
    }

    /**
     * Getter fr Eigenschaft 'datenidentifikation'.
     *
     * @return Wert der Eigenschaft 'datenidentifikation'.
     *
     * @see #_datenidentifikation
     */
    public Datenidentifikation getDatenidentifikation() {
        return _datenidentifikation;
    }

    /**
     *     Getter fr Eigenschaft 'filterAuswahlName'.
     *
     *     @return Wert der Eigenschaft 'filterAuswahlName'.
     *
     *     @see #_filterAuswahlName
     */
    public String getFilterAuswahlName() {
        return _filterAuswahlName;
    }

    /**
     *     @return the gewaehlteKnotenBaum
     */
    public List<String> getGewaehlteKnotenBaum() {
        return _gewaehlteKnotenBaum;
    }

    /**
     * Getter fr Eigenschaft 'sortierAuswahlName'.
     *
     * @return Wert der Eigenschaft 'sortierAuswahlName'.
     *
     * @see #_sortierAuswahlName
     */
    public String getSortierAuswahlName() {
        return _sortierAuswahlName;
    }

    /**
     * Getter fr Eigenschaft 'spaltenAuswahlName'.
     *
     * @return Wert der Eigenschaft 'spaltenAuswahlName'.
     *
     * @see #_spaltenAuswahlName
     */
    public String getSpaltenAuswahlName() {
        return _spaltenAuswahlName;
    }

    /**
     * Getter fr Eigenschaft 'varAtt'.
     *
     * @return Wert der Eigenschaft 'varAtt'.
     *
     * @see #_varAtt
     */
    public int getVarAtt() {
        return _varAtt;
    }

    /**
     * Getter fr Eigenschaft 'varTbl'.
     *
     * @return Wert der Eigenschaft 'varTbl'.
     *
     * @see #_varTbl
     */
    public int getVarTbl() {
        return _varTbl;
    }

    //~ METHODEN ==============================================================

    /*
     *  (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        final int prime  = 31;
        int       result = 1;

        result = prime * result + (_aenderungsdarstellung ? 1231 : 1237);
        result = prime * result + ((_archivanfrageDaten == null) ? 0 : _archivanfrageDaten.hashCode());
        result = prime * result + ((_datenidentifikation == null) ? 0 : _datenidentifikation.hashCode());
        result = prime * result + ((_filterAuswahlName == null) ? 0 : _filterAuswahlName.hashCode());
        result = prime * result + ((_gewaehlteKnotenBaum == null) ? 0 : _gewaehlteKnotenBaum.hashCode());
        result = prime * result + (_isArchiv ? 1231 : 1237);
        result = prime * result + ((_sortierAuswahlName == null) ? 0 : _sortierAuswahlName.hashCode());
        result = prime * result + ((_spaltenAuswahlName == null) ? 0 : _spaltenAuswahlName.hashCode());
        result = prime * result + _varAtt;
        result = prime * result + _varTbl;

        return result;
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'aenderungsdarstellung'.
     *
     * @return Wert der Eigenschaft 'aenderungsdarstellung'.
     *
     * @see #_aenderungsdarstellung
     */
    public boolean isAenderungsdarstellung() {
        return _aenderungsdarstellung;
    }

    /**
     *     Getter fr Eigenschaft 'isArchiv'.
     *
     *     @return Wert der Eigenschaft 'isArchiv'.
     *
     *     @see #_isArchiv
     */
    public boolean isArchiv() {
        return _isArchiv;
    }

    /**
     *     Getter fr Eigenschaft 'selektionUebernehmen'.
     *
     *     @return Wert der Eigenschaft 'selektionUebernehmen'.
     *
     *     @see #_selektionUebernehmen
     */
    public boolean isSelektionUebernehmen() {
        return _selektionUebernehmen;
    }

    //~ SET METHODEN ==========================================================

    /**
     * Setter fr Eigenschaft 'aenderungsdarstellung'.
     *
     * @param aenderungsdarstellung Wert zum Setzen der Eigenschaft 'aenderungsdarstellung'.
     *  WICHTIG! Verwendung fr die Speicherung im XML.
     * @see #_aenderungsdarstellung
     */
    public void setAenderungsdarstellung(boolean aenderungsdarstellung) {
        _aenderungsdarstellung = aenderungsdarstellung;
    }

    /**
     * Setter fr Eigenschaft 'isArchiv'.
     *
     * @param isArchiv Wert zum Setzen der Eigenschaft 'isArchiv'.
     *
     * @see #_isArchiv
     */
    public void setArchiv(boolean isArchiv) {
        _isArchiv = isArchiv;
    }

    /**
     * Setter fr Eigenschaft 'archivanfrageDaten'.
     *
     * @param archivanfrageDaten Wert zum Setzen der Eigenschaft 'archivanfrageDaten'.
     *
     * @see #_archivanfrageDaten
     */
    public void setArchivanfrageDaten(ArchivanfrageDaten archivanfrageDaten) {
        _archivanfrageDaten = archivanfrageDaten;
    }

    /**
     * Setter fr Eigenschaft 'datenidentifikation'.
     *
     * @param datenidentifikation Wert zum Setzen der Eigenschaft 'datenidentifikation'.
     *  WICHTIG! Verwendung fr die Speicherung im XML.
     * @see #_datenidentifikation
     */
    public void setDatenidentifikation(Datenidentifikation datenidentifikation) {
        _datenidentifikation = datenidentifikation;
    }

    /**
     * Setter fr Eigenschaft 'filterAuswahlName'.
     *
     * @param filterAuswahlName Wert zum Setzen der Eigenschaft 'filterAuswahlName'.
     * WICHTIG! Verwendung fr die Speicherung im XML.
     * @see #_filterAuswahlName
     */
    public void setFilterAuswahlName(String filterAuswahlName) {
        _filterAuswahlName = filterAuswahlName;
    }

    /**
     * @param gewaehlteKnotenBaum the gewaehlteKnotenBaum to set
     */
    public void setGewaehlteKnotenBaum(List<String> gewaehlteKnotenBaum) {
        _gewaehlteKnotenBaum = gewaehlteKnotenBaum;
    }

    /**
     * Setter fr Eigenschaft 'selektionUebernehmen'.
     *
     * @param selektionUebernehmen Wert zum Setzen der Eigenschaft 'selektionUebernehmen'.
     *
     * @see #_selektionUebernehmen
     */
    public void setSelektionUebernehmen(boolean selektionUebernehmen) {
        _selektionUebernehmen = selektionUebernehmen;
    }

    /**
     * Setter fr Eigenschaft 'sortierAuswahlName'.
     *
     * @param sortierAuswahlName Wert zum Setzen der Eigenschaft 'sortierAuswahlName'.
     *   WICHTIG! Verwendung fr die Speicherung im XML.
     * @see #_sortierAuswahlName
     */
    public void setSortierAuswahlName(String sortierAuswahlName) {
        _sortierAuswahlName = sortierAuswahlName;
    }

    /**
     * Setter fr Eigenschaft 'spaltenAuswahlName'.
     *
     * @param spaltenAuswahlName Wert zum Setzen der Eigenschaft 'spaltenAuswahlName'.
     *  WICHTIG! Verwendung fr die Speicherung im XML.
     * @see #_spaltenAuswahlName
     */
    public void setSpaltenAuswahlName(String spaltenAuswahlName) {
        _spaltenAuswahlName = spaltenAuswahlName;
    }

    /**
     * Setter fr Eigenschaft 'varAtt'.
     *
     * @param varAtt Wert zum Setzen der Eigenschaft 'varAtt'.
     *  WICHTIG! Verwendung fr die Speicherung im XML.
     * @see #_varAtt
     */
    public void setVarAtt(int varAtt) {
        _varAtt = varAtt;
    }

    /**
     * Setter fr Eigenschaft 'varTbl'.
     *
     * @param varTbl Wert zum Setzen der Eigenschaft 'varTbl'.
     *  WICHTIG! Verwendung fr die Speicherung im XML.
     * @see #_varTbl
     */
    public void setVarTbl(int varTbl) {
        _varTbl = varTbl;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 07.02.12
