/*
 * @(#)AktionenBis.java   04.03.10
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.projektlib.aktionen;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.inovat.lib.internet.mail.MailFunktionen;
import de.inovat.buv.projektlib.Activator;
import de.inovat.buv.projektlib.konstanten.Zustaende;
import de.inovat.buv.projektlib.rechtevew.aktionen.AktionenVew;
import de.inovat.buv.projektlib.rwservice.RWService;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.Platform;
import org.eclipse.ui.branding.IProductConstants;

import static de.inovat.buv.inovat.lib.konstanten.Konstanten.FILESEPARATOR;
import static de.inovat.buv.inovat.lib.konstanten.Konstanten.NEWLINE;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Klasse implementiert die Projekt-Aktionen (Handler zu Commands).
 * </p>
 * Aktionen sind als Extensions org.eclipse.ui.commands in PlugIn eingetragen
 * und somit sind sie fr die Menerstellung im RahmenWerk verfgbar.
 * </p>
 * Handler zu Commands sind ber Extensions org.eclipse.ui.handlers registriert.
 * </p>
 * Es sind 2 Handler implementiert:
 * </p>
 * <ol>
 * <li> inovat-Projekt starten
 * (es wird die Perspektive mit ID {@link #PERSPECTIVE_INOVAT_START} geffnet, falls die in einem Plugin implementiert ist)
 * <li> Fehler an inovat melden
 * </ol>
 * </p>
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 76 $ / $Date: 2013-10-29 16:22:06 +0100 (Di, 29 Okt 2013) $ / ($Author: LG $)
 */
public class Aktionen extends AbstractHandler {
    public static final String COMMAND_ID_FEHLER_MELDEN = "de.inovat.buv.projektlib.aktionen.FehlerMelden";
    public static final String COMMAND_ID_PROJEKT_START = "de.inovat.buv.projektlib.aktionen.PrjStart";

    /**
     * Perspektive, die die Aktion 'inovatProjekt starten' ffnet.
     * TODO Perspektive in einem der Plugins definieren.
     */
    public static final String PERSPECTIVE_INOVAT_START = "de.inovat.buv.projektlib.PerspectiveInovatStart";

    //~ METHODEN ==============================================================

    /**
     * Fhre die Aktion 'FehlerMelden' aus.
     * Die Aktion sendet eine E-Mail mit ProduktInformation und ErrorLogDatei an Inovat.
     *
     * @return true, falls die Aktion erfolgreich ausgefhrt wurde,
     * false - sonst.
     */
    private boolean ausfuehrenAktionFehlerMelden() {

        // Text:
        String produkt    = Platform.getProduct().getProperty(IProductConstants.ABOUT_TEXT);
        String trennLinie = "-----------------------------------------------------";
        String text       = "Zustzliche Hinweise:" + NEWLINE + NEWLINE + "HIER BITTE ERGNZEN" + NEWLINE + NEWLINE + NEWLINE + NEWLINE + trennLinie + NEWLINE + produkt + NEWLINE + trennLinie;

        // Betreff:
        String benutzer = RWService.getService().getRw().getBenutzerName();
        String betreff  = Zustaende.PROJEKT_NAME + " - ErrorLog: Fehlermeldung von " + benutzer;

        // Attach-Datei:
        String       datei      = Platform.getLocation().toFile().getAbsolutePath() + FILESEPARATOR + ".metadata" + FILESEPARATOR + ".log";
        List<String> listeDatei = new ArrayList<String>();

        listeDatei.add(datei);

        // E-Mail:
        String email = Zustaende.PROJEKT_NAME + "@inovat.de";

        // Sende E-Mail:
        try {
            MailFunktionen.oeffneMail_Windows(betreff, new String[] { email }, text, listeDatei);

            return true;
        }
        catch (Exception e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);

            return false;
        }
    }

    /**
     * Fhre die StartProjekt-Aktion aus.
     * Diese Aktion ffnet die Projekt-Perspektive und setzt sie zurck.
     *
     * @return true, falls die Aktion erfolgreich ausgefhrt wurde,
     * false - sonst.
     */
    private boolean ausfuehrenAktionProjektStart() {
        return AktionenVew.getInstanz().perspektiveOeffnen(PERSPECTIVE_INOVAT_START, true);
    }

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        String commandId = event.getCommand().getId();

        if (commandId.equals(COMMAND_ID_FEHLER_MELDEN)) {
            ausfuehrenAktionFehlerMelden();
        } else if (commandId.equals(COMMAND_ID_PROJEKT_START)) {
            ausfuehrenAktionProjektStart();
        }

        return null;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 02.10.13
