/*
 * @(#)SpeicherListenerVew.java   08.10.13
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.projektlib.speicherlistener;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.projektlib.rwservice.RWService;

//~ KLASSEN ===================================================================

/**
 * Die Klasse stellt eine Zwischenschicht zum Rahmenwerk-Service 'Einstellungen'
 * {@link RWService#getEinstellungen()} dar.
 * </p>
 * Klasse verwaltet Listener fr den Speicher.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 97 $ / $Date: 2013-11-18 11:16:24 +0100 (Mo, 18. Nov 2013) $ / ($Author: LG $)
 */
public class SpeicherListenerVew {

    /** Einzige Instanz der Klasse. */
    private static SpeicherListenerVew _vewInstanz = new SpeicherListenerVew();

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse SpeicherVew.
     */
    private SpeicherListenerVew() {}

    //~ METHODEN ==============================================================

    /**
     * Registriert Klasse als Listener auf {@link ISpeicherAenderung}.
     *
     * @param klasse Klasse, die das Interface {@link ISpeicherAenderung}
     * implementiert und ber nderungen im Speicher informiert werden will.
     */
    public void addSpeicherAenderungListener(ISpeicherAenderung klasse) {
        RWService.getService().getEinstellungen().addEinstellungsListener(klasse);
    }

    /**
     * Registriert Klasse als Listener auf {@link ISpeicherVerfuegbarkeit}.
     *
     * @param klasse Klasse, die das Interface {@link ISpeicherVerfuegbarkeit}
     * implementiert und ber nderungen bezglich der Verfgbarkeit des Speichers
     * informiert werden will.
     */
    public void addSpeicherVerfuegbarkeitListener(ISpeicherVerfuegbarkeit klasse) {
        RWService.getService().getEinstellungen().addEinstellungsAvailabilityListener(klasse);
    }

    //~ GET METHODEN ==========================================================

    /**
     * Ermittle einzige Instanz der Klasse.
     *
     * @return Einzige Instanz der Klasse.
     */
    public static SpeicherListenerVew getInstanz() {
        return _vewInstanz;
    }

    //~ METHODEN ==============================================================

    /**
     * Lscht die Registrierung einer Klasse als Listener auf
     * {@link ISpeicherAenderung}.
     *
     * @param klasse Klasse, die sich zuvor mittels
     * {@link #addSpeicherAenderungListener(ISpeicherAenderung)} registriert hat.
     */
    public void removeSpeicherAenderungListener(ISpeicherAenderung klasse) {
        RWService.getService().getEinstellungen().removeEinstellungsListener(klasse);
    }

    /**
     * Lscht die Registrierung einer Klasse als Listener auf
     * {@link ISpeicherVerfuegbarkeit}.
     *
     * @param klasse Klasse, die sich zuvor mittels
     * {@link #addSpeicherVerfuegbarkeitListener(ISpeicherVerfuegbarkeit)} registriert hat.
     */
    public void removeSpeicheVerfuegbarkeitListener(ISpeicherVerfuegbarkeit klasse) {
        RWService.getService().getEinstellungen().removeEinstellungsAvailabilityListener(klasse);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 08.10.13
