/*
 * Copyright 2008 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kni, Kln
 * <p>
 * This file is part of de.inovat.pat.datkat2html
 * <p>
 * de.inovat.pat.datkat2html is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * <p>
 * de.inovat.pat.datkat2html is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.pat.datkat2html; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * <p>
 * Contact Information:<br>
 * inovat, Dipl.-Ing. H. C. Kni<br>
 * Klner Strae 30<br>
 * 50859 Kln<br>
 * Phone: +49 2234 4301 800<br>
 * mailto: info@inovat.de
 */
package de.inovat.pat.datkat2html.ausgabe;

import de.inovat.pat.datkat2html.Activator;
import de.inovat.pat.datkat2html.Log;
import de.inovat.pat.datkat2html.ausgabe.beschreibung.*;
import de.inovat.pat.datkat2html.ausgabe.beschreibung.Inhalt.InhaltTyp;
import de.inovat.sys.funclib.bsvrzxml.BSVRZXML;
import de.inovat.sys.funclib.bsvrzxml.binder.*;

import java.util.*;

/**
 * Die Klasse erfasst Definitionen fr alle Elemente aus alle KBDateien.
 * 
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 408 $ / $Date: 2010-04-06 08:14:07 +0200 (Di, 06. Apr 2010) $ / ($Author: HCK $)
 */

public class AlleDefinitionen {
// ------------------------------------------------------------------------
// VARIABLENDEKLARATIONEN
// ------------------------------------------------------------------------

// Private ----------------------------------------------------------------
	/**
	 * Map enthlt alle KonfigurationsBereichBeschreibungen fr KB,
	 * die in den Dateien aus der Liste aller KBDateien definiert sind.
	 * Diese KonfigurationsBereichBeschreibungen sind nicht komplett
	 * (die enthalten null statt KonfigurationsAenderungen, Modell und Objekte.).
	 *
	 * SChlssel: PID von KB, wert: KonfigurationsBereichBeschreibung.
	 */
	private Map<String, KonfigurationsBereichBeschreibung> _mapAlleKbBeschreibungen
		= new HashMap<String, KonfigurationsBereichBeschreibung>();

	/**
	 * Sortierte nach KonfigurationVerantwortliche und PId von KB Liste aller KonfigurationsBereichBeschreibungen.
	 */
	private List<KonfigurationsBereichBeschreibung> _listeKbBeschreibungenSortiert;

	/**
	 * Map aller Definitionen, die in den Dateien aus der Liste aller KBDateien liegen.
	 * Schlssel: PID des Elements, Wert: Definition des Elements.
	 */
	private Map<String, Object> _mapAlleDefinitionen = new HashMap<String, Object>();

// ------------------------------------------------------------------------
// KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
// ------------------------------------------------------------------------

	public AlleDefinitionen(List<BSVRZXML> listeKBDatei){
		for(BSVRZXML kbDatei: listeKBDatei){
			// Erstelle _mapAlleKbBeschreibungen:
			KonfigurationsBereichBeschreibung kb=
				erstelleKonfigurationsBereichBeschreibung(kbDatei.getKb());
			_mapAlleKbBeschreibungen.put(kb.getUebersicht().getPid(), kb);

			// Erstelle _mapAlleDefinitionen:
			addiereMapZuMap(kbDatei.getMapTypDefinition(), _mapAlleDefinitionen);
			addiereMapZuMap(kbDatei.getMapAttributgruppenDefinition(), _mapAlleDefinitionen);
			addiereMapZuMap(kbDatei.getMapAspektDefinition(), _mapAlleDefinitionen);
			addiereMapZuMap(kbDatei.getMapAttributDefinition(), _mapAlleDefinitionen);
			addiereMapZuMap(kbDatei.getMapAttributlistenDefinition(), _mapAlleDefinitionen);
			addiereMapZuMap(kbDatei.getMapAttributauswahlDefinition(), _mapAlleDefinitionen);
			addiereMapZuMap(kbDatei.getMapMengenDefinition(), _mapAlleDefinitionen);
			addiereMapZuMap(kbDatei.getMapKonfigurationsObjekt(), _mapAlleDefinitionen);
		}

		// Erstelle _listeKbBeschreibungenSortiert:
		_listeKbBeschreibungenSortiert=ermittleSortietreListeAlleKbBeschreibungen();
	}

	/**
	 * Aus der Instanz der Klasse KonfigurationsBereich erstelle
	 * eine nicht komplette KonfigurationsBereichBeschreibung.
	 * Diese KonfigurationsBereichBeschreibung enthlt null statt
	 * konfigurationsAenderung, Modell und Objekte.
	 *
	 * @param kb liefert die Parameter fr die KonfigurationsBereichBeschreibung.
	 *
	 * @return Erzeugte KonfigurationsBereichBeschreibung.
	 */
	private KonfigurationsBereichBeschreibung erstelleKonfigurationsBereichBeschreibung(KonfigurationsBereich kb){
		KonfigurationsBereichBeschreibung kbB=null;
		if(kb!=null){
			kbB=new KonfigurationsBereichBeschreibung(erstelleUebersicht(kb),
					kb.getVerantwortlich(),
					"",
					null,
					null,
					null);
		}
		return kbB;
	}

	/**
	 * Erstelle eine Instanz der Klasse Uebersicht.
	 * In Konstruktor werden die Werte von Parameter 'element' bernommen.
	 * Die erzeugte InfoBeschreibung kann null sein.
	 *
	 * @param element Element, dass PID, Name und Info fr den Konstruktor liefert.
	 *
	 * @return Instanz der Klasse Uebersicht.
	 */
	public Uebersicht erstelleUebersicht(Object element){
		String id="";
		String pid="";
		String name="";
		Info info=null;

		if (element instanceof KonfigurationsBereich) {
			KonfigurationsBereich kb = (KonfigurationsBereich) element;
			pid=kb.getPid();
			name=kb.getName();
			info=kb.getInfo();
		}
		else if (element instanceof KonfigurationsObjekt) {
			KonfigurationsObjekt ko = (KonfigurationsObjekt) element;
			id=ko.getId();
			pid=ko.getPid();
			name=ko.getName();
			info=ko.getInfo();
		}
		else if (element instanceof TypDefinition) {
			TypDefinition typDef = (TypDefinition) element;
			pid=typDef.getPid();
			name=typDef.getName();
			info=typDef.getInfo();
		}
		else if (element instanceof MengenDefinition) {
			MengenDefinition mengeDef = (MengenDefinition) element;
			pid=mengeDef.getPid();
			//name=mengeDef.getn;
			info=mengeDef.getInfo();
		}
		else if (element instanceof Menge) {
			Menge menge = (Menge) element;
			pid=menge.getPid();
			name=menge.getName();
			info=menge.getInfo();
		}
		else if (element instanceof AttributgruppenDefinition) {
			AttributgruppenDefinition atgDef = (AttributgruppenDefinition) element;
			pid=atgDef.getPid();
			name=atgDef.getName();
			info=atgDef.getInfo();
		}
		else if (element instanceof AttributlistenDefinition) {
			AttributlistenDefinition atlDef = (AttributlistenDefinition) element;
			pid=atlDef.getPid();
			name=atlDef.getName();
			info=atlDef.getInfo();
		}
		else if (element instanceof Attributliste) {
			Attributliste atl = (Attributliste) element;
			pid=atl.getPid();
			name=atl.getName();
			info=atl.getInfo();
		}
		else if (element instanceof AttributDefinition) {
			AttributDefinition attDef = (AttributDefinition) element;
			pid=attDef.getPid();
			name=attDef.getName();
			info=attDef.getInfo();
		}
		else if (element instanceof Attribut) {
			Attribut att = (Attribut) element;
			pid=att.getPid();
			name=att.getName();
			info=att.getInfo();
		}
		else if (element instanceof AttributauswahlDefinition) {
			AttributauswahlDefinition ataDef = (AttributauswahlDefinition) element;
			pid=ataDef.getPid();
			name=ataDef.getName();
			info=ataDef.getInfo();
		}
		else if (element instanceof Attributauswahl) {
			Attributauswahl ata = (Attributauswahl) element;
			pid=ata.getPid();
			name=ata.getName();
			info=ata.getInfo();
		}
		else if (element instanceof Auswahl) {
			Auswahl aus = (Auswahl) element;
			pid=aus.getPid();
			name=aus.getName();
			info=aus.getInfo();
		}
		else if (element instanceof AspektDefinition) {
			AspektDefinition aspektDef = (AspektDefinition) element;
			pid=aspektDef.getPid();
			name=aspektDef.getName();
			info=aspektDef.getInfo();
		}
		else if (element instanceof Aspekt) {
			Aspekt aspekt = (Aspekt) element;
			pid=aspekt.getPid();
			//name=aspekt.getn;
			info=aspekt.getInfo();
		}
		else{
			Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
					"Unbekannte Klasse fr die Methode erstelleUebersicht(): "+element));
		}
		//
		if(id==null){
			id="";
		}
		if(pid==null){
			pid="";
		}
		if(name==null){
			name="";
		}
		// ersetze name bzw. pid, falls leer ist durch pid bsw. id:
		if(pid.equals("")){
			pid=id;
		}
		if(name.equals("")){
			name=pid;
		}

		return new Uebersicht(pid, name, erstelleInfoBeschreibung(info));
	}

	/**
	 * Erstelle InfoBeschreibung aus der Instanz der Klasse Info.
	 * Falls Info gleich null ist , wird null zurck gegeben.
	 *
	 * @param info Liefert die Parameter fr die InfoBeschreibung.
	 *
	 * @return Erzeugte InfoBeschreibung oder null.
	 */
	public InfoBeschreibung erstelleInfoBeschreibung(Info info){
		InfoBeschreibung infoB = null;

		if(info!=null){
			Inhalt inhaltKurzinfo = null;
			Inhalt inhaltBeschreibung = null;

			Beschreibung beschreibung=info.getBeschreibung();

			inhaltKurzinfo=erstelleInhalt(info.getKurzinfo().getContent());

			if(beschreibung!=null){
				inhaltBeschreibung=erstelleInhalt(beschreibung.getContent());
			}

			infoB=new InfoBeschreibung(inhaltKurzinfo, inhaltBeschreibung);
		}
		return infoB;
	}

	/**
	 * Rufe die rekursive Methode mit Startwerten,
	 * um eine Instanz der Klasse Inhalt zu erstellen.
	 * @param content Liste von Objekten.
	 * @return Erzeugte Instanz der Klasse Inhalt.
	 */
	public Inhalt erstelleInhalt(List<Object> content){
		return erstelleInhalt(content, InhaltTyp.startInfo, "");
	}

	/**
	 * Addiere Map zum anderen Map.
	 * DoppelElemente werden nicht eingefgt.
	 *
	 * @param map Map, das addiert wird.
	 * @param mapAll Map, zum addiert wird.
	 */
	private void addiereMapZuMap(Map<String, ?> map, Map<String, Object> mapAll){
		if(map.size()>0){
			// Erstelle eine Kopie von map, um die Elemente lschen zu knnen:
			Map<String, Object> mapKopie=new HashMap<String, Object>();
			mapKopie.putAll(map);

			// Falls das Element schon im mapAll existiert,
			// wird das nicht eingefgt:
			for(String element: map.keySet()){
				if(mapAll.containsKey(element)){
					mapKopie.remove(element);
				}
			}

			mapAll.putAll(mapKopie);
		}
	}

	/**
	 * Erzeuge die Liste aus dem _mapAlleKbBeschreibungen
	 * und sortiere die nach KonfigurationVerantwortliche und KB_PID.
	 *
	 * @return Sortierte Liste aller KonfigurationsBereichBeschreibungen.
	 */
	private List<KonfigurationsBereichBeschreibung> ermittleSortietreListeAlleKbBeschreibungen(){
		List<KonfigurationsBereichBeschreibung> listeKbB=new ArrayList<KonfigurationsBereichBeschreibung>();
		for(KonfigurationsBereichBeschreibung kb:_mapAlleKbBeschreibungen.values()){
			listeKbB.add(kb);
		}
		KonfigurationsBereichBeschreibung[] arrKbB=listeKbB.toArray(new KonfigurationsBereichBeschreibung[0]);
		Arrays.sort(arrKbB, new Comparator<KonfigurationsBereichBeschreibung>(){
			@Override
			public int compare(KonfigurationsBereichBeschreibung o1,
					KonfigurationsBereichBeschreibung o2) {
				int sortKV=o1.getVerantwortlich().compareTo(o2.getVerantwortlich());
				if(sortKV!=0){
					return sortKV;
				}
				else{
					return o1.getUebersicht().getPid().compareTo(
							o2.getUebersicht().getPid());
				}
			}
		});
		return Arrays.asList(arrKbB);
	}

// ------------------------------------------------------------------------
// GETTER / SETTER METHODEN
// ------------------------------------------------------------------------

	/**
	 * Getter fr Eigenschaft '_listeKbBeschreibungenSortiert'.
	 *
	 * @return Wert der Eigenschaft '_listeKbBeschreibungenSortiert'.
	 *
	 * @see (#link _listeKbBeschreibungenSortiert)
	 */
	public List<KonfigurationsBereichBeschreibung> getListeKbBeschreibungenSortiert() {
		return _listeKbBeschreibungenSortiert;
	}

	/**
	 * Getter fr Eigenschaft '_mapAlleDefinitionen'.
	 *
	 * @return Wert der Eigenschaft '_mapAlleDefinitionen'.
	 *
	 * @see (#link _mapAlleDefinitionen)
	 */
	public Map<String, Object> getMapAlleDefinitionen() {
		return _mapAlleDefinitionen;
	}

	/**
	 * Getter fr Eigenschaft '_mapAlleKbBeschreibungen'.
	 *
	 * @return Wert der Eigenschaft '_mapAlleKbBeschreibungen'.
	 *
	 * @see (#link _mapAlleKbBeschreibungen)
	 */
	public Map<String, KonfigurationsBereichBeschreibung> getMapAlleKbBeschreibungen() {
		return _mapAlleKbBeschreibungen;
	}

// ------------------------------------------------------------------------
// SONSTIGE METHODEN
// ------------------------------------------------------------------------

	/**
	 * Rekursive Methode. Erzeugt eine Instanz der Klasse Inhalt.
	 *
	 * @param content Definiert die Liste von Objekten, die rekursiv in Inhalt umgewandelt werden.
	 * @param typ Typ von Inhalt.
	 * @param text Text von Inhalt.
	 * @return Erzeugte Instanz der Klasse Inhalt
	 */
	private Inhalt erstelleInhalt(List<Object> content, InhaltTyp typ, String text){
		Inhalt inhalt = null;

		List<Inhalt> listeInhalte=new ArrayList<Inhalt>();

		String text2="";

		InhaltTyp typ2;
		List<Object> content2;

		String indexSichtbar;
		String indexID;

		String hinweisPID;
		String hinweisTyp;

		String kommentarTitel;
		String kommentarStatus;

		String verweisPID;
		String verweisTxt;
		String verweisNum;
		String verweisKlammer;
		String verweisDirekt;
		String verweisTooltip;

		String ankerPID;
		String ankerTxt;

		String abbildungPID;
		String abbildungScr;
		String abbildungAlt;
		String abbildungHoehe;
		String abbildungWeite;

		String listeZeichen;

		String numListeTyp;
		String numListeStart;

		String referenzPID;
		String referenzLink;
		String referenzInReferenzverzeichnis;

		String tabellePID;
		String tabelleRahmen;
		String tabelleAusrichtung;
		String tabelleTransponiert;

		String spalteBreite;
		String spalteHintergrund;

		String zelleAusrichtung;
		String zelleVertikal;
		String zelleHintergrund;
		String zelleKopf;
		String zelleBreite;
		String zelleAnzSpalten;
		String zelleAnzZeilen;

		String sektionPid;

		// Untersuche jedes Objekt von der Liste content
		// und umwandele das in Inhalt:
		for(Object object: content){
			text2="";

			typ2=InhaltTyp.unbekannt;
			content2=new ArrayList<Object>();

			indexSichtbar="";
			indexID="";

			hinweisPID="";
			hinweisTyp="";

			kommentarTitel="";
			kommentarStatus="";

			verweisPID="";
			verweisTxt="";
			verweisNum="";
			verweisKlammer="";
			verweisDirekt="";
			verweisTooltip="";

			ankerPID="";
			ankerTxt="";

			abbildungPID="";
			abbildungScr="";
			abbildungAlt="";
			abbildungHoehe="";
			abbildungWeite="";

			listeZeichen="";

			numListeTyp="";
			numListeStart="";

			referenzPID="";
			referenzLink="";
			referenzInReferenzverzeichnis="";

			tabellePID="";
			tabelleRahmen="";
			tabelleAusrichtung="";
			tabelleTransponiert="";

			spalteBreite="";
			spalteHintergrund="";

			zelleAusrichtung="";
			zelleVertikal="";
			zelleHintergrund="";
			zelleKopf="";
			zelleBreite="";
			zelleAnzSpalten="";
			zelleAnzZeilen="";

			sektionPid="";

			// 1. Typ #PCDATA:
			if (object instanceof String) {
				typ2=InhaltTyp.string;

				text2 = (String) object;
			}
			// 2. Einfache Typen der Art (#PCDATA| %zeichenauszeichnung.klasse;)*
			// 		tief, hoch, code, adresse, pre, wichtig, zitat, absatz
			else if (object instanceof Tief) {
				Tief tief = (Tief) object;
				typ2=InhaltTyp.tief;
				content2=tief.getContent();
			}
			else if (object instanceof Hoch) {
				Hoch hoch = (Hoch) object;
				typ2=InhaltTyp.hoch;
				content2=hoch.getContent();
			}
			else if (object instanceof Code) {
				Code code = (Code) object;
				typ2=InhaltTyp.code;
				content2=code.getContent();
			}
			else if (object instanceof Adresse) {
				Adresse adresse = (Adresse) object;
				typ2=InhaltTyp.adresse;
				content2=adresse.getContent();
			}
			else if (object instanceof Pre) {
				Pre pre = (Pre) object;
				typ2=InhaltTyp.pre;
				content2=pre.getContent();
			}
			else if (object instanceof Wichtig) {
				Wichtig wichtig = (Wichtig) object;
				typ2=InhaltTyp.wichtig;
				content2=wichtig.getContent();
			}
			else if (object instanceof Zitat) {
				Zitat zitat = (Zitat) object;
				typ2=InhaltTyp.zitat;
				content2=zitat.getContent();
			}
			else if (object instanceof Absatz) {
				Absatz absatz = (Absatz) object;
				typ2=InhaltTyp.absatz;
				content2=absatz.getContent();
			}
			// 3. Komplizierte Typen der Art ((#PCDATA| %zeichenauszeichnung.klasse;)*
			//    oder (#PCDATA | %zeichenauszeichnung.klasse; | %absatzauszeichnung.klasse;)*
			//    und mit Attributlisten):
			// 		index, hinweis, kommentar
			else if (object instanceof Index) {
				Index index = (Index) object;
				typ2=InhaltTyp.index;
				content2=index.getContent();
				indexSichtbar=index.getSichtbar();
				indexID=index.getId();
			}
			else if (object instanceof Hinweis) {
				Hinweis hinweis = (Hinweis) object;
				typ2=InhaltTyp.hinweis;
				content2=hinweis.getContent();
				hinweisPID=hinweis.getPid();
				hinweisTyp=hinweis.getTyp();
			}
			else if (object instanceof Kommentar) {
				Kommentar kommentar = (Kommentar) object;
				typ2=InhaltTyp.kommentar;
				content2=kommentar.getContent();
				kommentarTitel=kommentar.getTitel();
				kommentarStatus=kommentar.getStatus();
			}
			// 4. Komplizierte Typen (EMPTY und mit Attributlisten):
			//		verweis, anker
			else if (object instanceof Verweis) {
				Verweis verweis = (Verweis) object;
				typ2=InhaltTyp.verweis;
				verweisPID=verweis.getPid();
				verweisTxt=verweis.getTxt();
				verweisNum=verweis.getNum();
				verweisKlammer=verweis.getKlammer();
				verweisDirekt=verweis.getDirekt();
				verweisTooltip=verweis.getTooltip();
			}
			else if (object instanceof Anker) {
				Anker anker = (Anker) object;
				typ2=InhaltTyp.anker;
				ankerPID=anker.getPid();
				ankerTxt=anker.getTxt();
			}
			// 5. Komplizierte Typen (mit Attributlisten und Elementen):
			//		abbildung, liste, numListe, referenz, tabelle
			else if (object instanceof Abbildung) {
				Abbildung abbildung = (Abbildung) object;
				typ2=InhaltTyp.abbildung;
				if(abbildung.getTitel()!=null){
					content2.add(abbildung.getTitel());
				}
				abbildungPID=abbildung.getPid();
				abbildungScr=abbildung.getSrc();
				abbildungAlt=abbildung.getAlt();
				abbildungHoehe=abbildung.getHoehe();
				abbildungWeite=abbildung.getWeite();
			}
			else if (object instanceof Liste) {
				Liste liste = (Liste) object;
				typ2=InhaltTyp.liste;
				content2=liste.getListenPunktOrListeOrNumListe();
				listeZeichen=liste.getZeichen();
			}
			else if (object instanceof NumListe) {
				NumListe numListe = (NumListe) object;
				typ2=InhaltTyp.numListe;
				content2=numListe.getListenPunktOrListeOrNumListe();
				numListeTyp=numListe.getTyp();
				numListeStart=numListe.getStart();
			}
			else if (object instanceof Referenz) {
				Referenz referenz = (Referenz) object;
				typ2=InhaltTyp.referenz;
				if(referenz.getTitel()!=null){
					content2.add(referenz.getTitel());
				}
				Info info=referenz.getInfo();
				if(info!=null){
					content2.add(info.getKurzinfo());
					if(info.getBeschreibung()!=null){
						content2.add(info.getBeschreibung());
					}
				}
				referenzPID=referenz.getPid();
				referenzLink=referenz.getLink();
				referenzInReferenzverzeichnis=referenz.getInReferenzverzeichnis();
			}
			else if (object instanceof Tabelle) {
				Tabelle tabelle = (Tabelle) object;
				typ2=InhaltTyp.tabelle;
				if(tabelle.getTitel()!=null){
					content2.add(tabelle.getTitel());
				}
				if(tabelle.getSpalte()!=null){
					for(Spalte spalte: tabelle.getSpalte()){
						content2.add(spalte);
					}
				}
				if(tabelle.getKopf()!=null){
					content2.add(tabelle.getKopf());
				}
				content2.add(tabelle.getRumpf());
				tabellePID=tabelle.getPid();
				tabelleRahmen=tabelle.getRahmen();
				tabelleAusrichtung=tabelle.getAusrichtung();
				tabelleTransponiert=tabelle.getTransponiert();
			}
			// 6. Zustzliche Typen, die in komplizierten Typen benutzt werden:
			//		titel (2), listenPunkt (2), spalte (4),
			//		kopf (5*), rumf(5*), zeile(5*), zelle (3)
			else if (object instanceof Titel) {
				Titel titel = (Titel) object;
				typ2=InhaltTyp.titel;
				content2=titel.getContent();
			}
			else if (object instanceof ListenPunkt) {
				ListenPunkt listenPunkt = (ListenPunkt) object;
				typ2=InhaltTyp.listenPunkt;
				content2=listenPunkt.getContent();
			}
			else if (object instanceof Spalte) {
				Spalte spalte = (Spalte) object;
				typ2=InhaltTyp.spalte;
				spalteBreite=spalte.getBreite();
				spalteHintergrund=spalte.getHintergrund();
			}
			else if (object instanceof Kopf) {
				Kopf kopf = (Kopf) object;
				typ2=InhaltTyp.kopf;
				if(kopf.getZeile()!=null){
					for(Zeile zeile: kopf.getZeile()){
						content2.add(zeile);
					}
				}
			}
			else if (object instanceof Rumpf) {
				Rumpf rumf = (Rumpf) object;
				typ2=InhaltTyp.rumpf;
				if(rumf.getZeile()!=null){
					for(Zeile zeile: rumf.getZeile()){
						content2.add(zeile);
					}
				}
			}
			else if (object instanceof Zeile) {
				Zeile zeile = (Zeile) object;
				typ2=InhaltTyp.zeile;
				if(zeile.getZelle()!=null){
					for(Zelle zelle: zeile.getZelle()){
						content2.add(zelle);
					}
				}
			}
			else if (object instanceof Zelle) {
				Zelle zelle = (Zelle) object;
				typ2=InhaltTyp.zelle;
				content2=zelle.getContent();
				zelleAusrichtung=zelle.getAusrichtung();
				zelleVertikal=zelle.getVertikal();
				zelleHintergrund=zelle.getHintergrund();
				zelleKopf=zelle.getKopf();
				zelleBreite=zelle.getBreite();
				zelleAnzSpalten=zelle.getAnzSpalten();
				zelleAnzZeilen=zelle.getAnzZeilen();
			}
			// 7. Komplizierte Typen der Art 3 und 5:
			//		sektion
			else if (object instanceof Sektion) {
				Sektion sektion=(Sektion)object;
				content2.add(sektion.getTitel());
				content2.addAll(sektion.getAnkerOrAbsatzOrAbbildung());
				sektionPid=sektion.getPid();
			}
			else if (object instanceof Ze) {
				// Wird ignoriert
			}
			// unbekannter Typ:
			else{
				Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
						"Folgender Typ aus Info wird nicht untersttzt: "+object.getClass()));
			}

			Inhalt inhalt2=erstelleInhalt(content2, typ2, text2);
			// Setzte zustzliche Instanzvariable fr erzeugten inhalt2
			// fr die Typen, die Attributlisten haben:
			switch (typ2) {
			// 	index, hinweis, kommentar
			case index:
				inhalt2.setIndexID(indexID);
				inhalt2.setIndexSichtbar(indexSichtbar);
				break;
			case hinweis:
				inhalt2.setHinweisPID(hinweisPID);
				inhalt2.setHinweisTyp(hinweisTyp);
				break;
			case kommentar:
				inhalt2.setKommentarStatus(kommentarStatus);
				inhalt2.setKommentarTitel(kommentarTitel);
				break;
			//	verweis, anker
			case verweis:
				inhalt2.setVerweisDirekt(verweisDirekt);
				inhalt2.setVerweisKlammer(verweisKlammer);
				inhalt2.setVerweisNum(verweisNum);
				inhalt2.setVerweisPID(verweisPID);
				inhalt2.setVerweisTooltip(verweisTooltip);
				inhalt2.setVerweisTxt(verweisTxt);
				break;
			case anker:
				inhalt2.setAnkerPID(ankerPID);
				inhalt2.setAnkerTxt(ankerTxt);
				break;
			//	abbildung, liste, numListe, referenz, tabelle
			case abbildung:
				inhalt2.setAbbildungAlt(abbildungAlt);
				inhalt2.setAbbildungHoehe(abbildungHoehe);
				inhalt2.setAbbildungPID(abbildungPID);
				inhalt2.setAbbildungScr(abbildungScr);
				inhalt2.setAbbildungWeite(abbildungWeite);
				break;
			case liste:
				inhalt2.setListeZeichen(listeZeichen);
				break;
			case numListe:
				inhalt2.setNumListeStart(numListeStart);
				inhalt2.setNumListeTyp(numListeTyp);
				break;
			case referenz:
				inhalt2.setReferenzInReferenzverzeichnis(referenzInReferenzverzeichnis);
				inhalt2.setReferenzLink(referenzLink);
				inhalt2.setReferenzPID(referenzPID);
				break;
			case tabelle:
				inhalt2.setTabelleAusrichtung(tabelleAusrichtung);
				inhalt2.setTabellePID(tabellePID);
				inhalt2.setTabelleRahmen(tabelleRahmen);
				inhalt2.setTabelleTransponiert(tabelleTransponiert);
				break;
			// spalte, zelle
			case spalte:
				inhalt2.setSpalteBreite(spalteBreite);
				inhalt2.setSpalteHintergrund(spalteHintergrund);
				break;
			case zelle:
				inhalt2.setZelleAnzSpalten(zelleAnzSpalten);
				inhalt2.setZelleAnzZeilen(zelleAnzZeilen);
				inhalt2.setZelleAusrichtung(zelleAusrichtung);
				inhalt2.setZelleBreite(zelleBreite);
				inhalt2.setZelleHintergrund(zelleHintergrund);
				inhalt2.setZelleKopf(zelleKopf);
				inhalt2.setZelleVertikal(zelleVertikal);
				break;
			// sektion
			case sektion:
				inhalt2.setSektionPid(sektionPid);
				break;
			}
			listeInhalte.add(inhalt2);
		}

		inhalt = new Inhalt(typ, listeInhalte, text);

		return inhalt;
	}
}
