/*
 * Copyright 2008 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kni, Kln
 * <p>
 * This file is part of de.inovat.pat.datkat2html
 * <p>
 * de.inovat.pat.datkat2html is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * <p>
 * de.inovat.pat.datkat2html is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.pat.datkat2html; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * <p>
 * Contact Information:<br>
 * inovat, Dipl.-Ing. H. C. Kni<br>
 * Klner Strae 30<br>
 * 50859 Kln<br>
 * Phone: +49 2234 4301 800<br>
 * mailto: info@inovat.de
 */
package de.inovat.pat.datkat2html.ausgabe;

import de.inovat.pat.datkat2html.Activator;
import de.inovat.pat.datkat2html.Log;
import de.inovat.pat.datkat2html.ausgabe.beschreibung.*;
import de.inovat.sys.funclib.bsvrzxml.BSVRZXML;
import de.inovat.sys.funclib.bsvrzxml.binder.*;

import java.util.*;

/**
 * 
 * Die Klasse konvertiert die XML-Klassen (Binder) in das Beschreibung-Format.
 * Hier wird die Information fr den KonfigurationBerech gesammelt.
 * 
 * Beschreibung-Klassen kann man fr die Ausgabe der XML-Information benutzen.
 * 
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 408 $ / $Date: 2010-04-06 08:14:07 +0200 (Di, 06. Apr 2010) $ / ($Author: HCK $)
 */

public class KonvertierungXML2Beschreibung {
// ------------------------------------------------------------------------
// VARIABLENDEKLARATIONEN
// ------------------------------------------------------------------------

// Private ----------------------------------------------------------------
	/** Erfasst Definitionen fr alle Elemente aus alle KBDateien. */
	private AlleDefinitionen _alleDefinitionen;

	/** KonfigurationsBereichBeschreibung fr den KonfigurationsBereich*/
	private KonfigurationsBereichBeschreibung _kbB;

	/** bersichtliste der TypDefinitionen. */
	private List<Uebersicht> _listeUebersichtTypDefinition;

	/** bersichtliste der MengenDefinitionen. */
	private List<Uebersicht> _listeUebersichtMengenDefinition;

	/** bersichtliste der AttributGruppenDefinitionen. */
	private List<Uebersicht> _listeUebersichtAttributgruppenDefinition;

	/** bersichtliste der AttributlistenDefinitionen. */
	private List<Uebersicht> _listeUebersichtAttributlistenDefinition;

	/** bersichtliste der AttributDefinitionen. */
	private List<Uebersicht> _listeUebersichtAttributDefinition;

	/** bersichtliste der AttributauswahlDefinitionen. */
	private List<Uebersicht> _listeUebersichtAttributauswahlDefinition;

	/** bersichtliste der AspektDefinitionen. */
	private List<Uebersicht> _listeUebersichtAspektDefinition;

	/** bersichtliste der KonfigurationsObjektDefinitionen. */
	private List<Uebersicht> _listeUebersichtKonfigurationsObjektDefinition;

	/** Liste der TypDefinitionBeschreibungen. */
	private List<TypDefinitionBeschreibung> _listeTypDefinitionB;

	/** Liste der MengeDefinitionBeschreibungen. */
	private List<MengenDefinitionBeschreibung> _listeMengeDefinitionB;

	/** Liste der AttributgruppenDefinitionBeschreibungen. */
	private List<AttributgruppenDefinitionBeschreibung> _listeAttributgruppenDefinitionB;

	/** Liste der AttributlistenDefinitionBeschreibungen. */
	private List<AttributlistenDefinitionBeschreibung> _listeAttributlistenDefinitionB;

	/** Liste der AttributDefinitionBeschreibungen. */
	private List<AttributDefinitionBeschreibung> _listeAttributDefinitionB;

	/** Liste der AttributauswahlDefinitionBeschreibungen. */
	private List<AttributauswahlDefinitionBeschreibung> _listeAttributauswahlDefinitionB;

	/** Liste der AspektDefinitionBeschreibungen. */
	private List<AspektDefinitionBeschreibung> _listeAspektDefinitionB;

	/** Liste der KonfigurationObjektDefinitionBeschreibungen. */
	private List<KonfigurationsObjektBeschreibung> _listeKonfigurationsObjektB;

	/**
	 * Map aller erzeugten AttributgruppenDefinitionBeschreibungen.
	 * Wird in der Methode erstelleAttributgruppenDefinitionBeschreibung() gefllt.
	 * Falls die AttributgruppenDefinitionBeschreibungen schon erzeugt wurde,
	 * wird die fr die nchste Verwendung aus dem Map abgeholt.
	 */
	private Map<AttributgruppenDefinition, AttributgruppenDefinitionBeschreibung> _mapAtg=new HashMap<AttributgruppenDefinition, AttributgruppenDefinitionBeschreibung>();

	/**
	 * Map aller erzeugten AspektDefinitionBeschreibungen.
	 * Wird in der Methode erstelleAspektDefinitionBeschreibung() gefllt.
	 * Falls die AspektDefinitionBeschreibung schon erzeugt wurde,
	 * wird die fr die nchste Verwendung aus dem Map abgeholt.
	 */
	private Map<AspektDefinition, AspektDefinitionBeschreibung> _mapAspekt=new HashMap<AspektDefinition, AspektDefinitionBeschreibung>();

	/**
	 * Map aller erzeugten AttributDefinitionBeschreibungen.
	 * Wird in der Methode erstelleAttributDefinitionBeschreibung() gefllt.
	 * Falls die AttributDefinitionBeschreibung schon erzeugt wurde,
	 * wird die fr die nchste Verwendung aus dem Map abgeholt.
	 */
	private Map<AttributDefinition, AttributDefinitionBeschreibung> _mapAtt=new HashMap<AttributDefinition, AttributDefinitionBeschreibung>();

	/**
	 * Map aller erzeugten AttributlistenDefinitionBeschreibungen.
	 * Wird in der Methode erstelleAttributlistenDefinitionBeschreibung() gefllt.
	 * Falls die AttributlistenDefinitionBeschreibung schon erzeugt wurde,
	 * wird die fr die nchste Verwendung aus dem Map abgeholt.
	 */
	private Map<AttributlistenDefinition, AttributlistenDefinitionBeschreibung> _mapAtl=new HashMap<AttributlistenDefinition, AttributlistenDefinitionBeschreibung>();

	/**
	 * Map aller erzeugten MengenDefinitionBeschreibungen.
	 * Wird in der Methode erstelleMengenDefinitionBeschreibung() gefllt.
	 * Falls die MengenDefinitionBeschreibung schon erzeugt wurde,
	 * wird die fr die nchste Verwendung aus dem Map abgeholt.
	 */
	private Map<MengenDefinition, MengenDefinitionBeschreibung> _mapMenge=new HashMap<MengenDefinition, MengenDefinitionBeschreibung>();

	/**
	 * Map aller erzeugten TypDefinitionBeschreibungen.
	 * Wird in der Methode erstelleTypDefinitionBeschreibung() gefllt.
	 * Falls die TypDefinitionBeschreibung schon erzeugt wurde,
	 * wird die fr die nchste Verwendung aus dem Map abgeholt.
	 */
	private Map<TypDefinition, TypDefinitionBeschreibung> _mapTyp=new HashMap<TypDefinition, TypDefinitionBeschreibung>();

// ------------------------------------------------------------------------
// KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
// ------------------------------------------------------------------------

	/**
	 * Erzeugt eine Instanz der Klasse KonvertierungXML2Beschreibung.
	 *
	 * @param alleKB Definitionen fr alle Elemente aus alle KBDateien.
	 *
	 * @param kbDatei KBDatei.
	 */
	public KonvertierungXML2Beschreibung(AlleDefinitionen alleKB, BSVRZXML kbDatei) {
		_alleDefinitionen = alleKB;

		// erstelle alle Instanzen:
		_kbB=erstelleKonfigurationsBereichBeschreibung(kbDatei.getKb());

		_listeUebersichtTypDefinition=erstelleUebersichtListe(kbDatei.getMapTypDefinition().values());
		_listeUebersichtMengenDefinition=erstelleUebersichtListe(kbDatei.getMapMengenDefinition().values());
		_listeUebersichtAttributgruppenDefinition=erstelleUebersichtListe(kbDatei.getMapAttributgruppenDefinition().values());
		_listeUebersichtAttributlistenDefinition=erstelleUebersichtListe(kbDatei.getMapAttributlistenDefinition().values());
		_listeUebersichtAttributDefinition=erstelleUebersichtListe(kbDatei.getMapAttributDefinition().values());
		_listeUebersichtAttributauswahlDefinition=erstelleUebersichtListe(kbDatei.getMapAttributauswahlDefinition().values());
		_listeUebersichtAspektDefinition=erstelleUebersichtListe(kbDatei.getMapAspektDefinition().values());
		_listeUebersichtKonfigurationsObjektDefinition=erstelleUebersichtListe(kbDatei.getMapKonfigurationsObjekt().values());

		_listeTypDefinitionB=erstelleTypDefinitionBeschreibungListe(kbDatei.getMapTypDefinition().values());
		_listeMengeDefinitionB=erstelleMengenDefinitionBeschreibungListe(kbDatei.getMapMengenDefinition().values());
		_listeAttributgruppenDefinitionB=erstelleAttributgruppenDefinitionBeschreibungListe(kbDatei.getMapAttributgruppenDefinition().values());
		_listeAttributlistenDefinitionB=erstelleAttributlistenDefinitionBeschreibungListe(kbDatei.getMapAttributlistenDefinition().values());
		_listeAttributDefinitionB=erstelleAttribbutDefinitionBeschreibungListe(kbDatei.getMapAttributDefinition().values());
		_listeAttributauswahlDefinitionB=erstelleAttributauswahlDefinitionBeschreibungListe(kbDatei.getMapAttributauswahlDefinition().values());
		_listeAspektDefinitionB=erstelleAspektDefinitionBeschreibungListe(kbDatei.getMapAspektDefinition().values());
		_listeKonfigurationsObjektB=_kbB.getObjekte().getListeKObBeschreibung();
	}

	/**
	 * Aus der Instanz der Klasse KonfigurationsBereich erstelle eine KonfigurationsBereichBeschreibung.
	 *
	 * <!ELEMENT konfigurationsBereich (info?, konfigurationsAenderung*, modell?, objekte?) >
	 * <!ATTLIST konfigurationsBereich pid ID #REQUIRED >
	 * <!ATTLIST konfigurationsBereich name CDATA #IMPLIED >
	 * <!ATTLIST konfigurationsBereich verantwortlich NMTOKEN #REQUIRED>
	 * <!ATTLIST konfigurationsBereich xml:base CDATA #IMPLIED >
	 *
	 * @param kb liefert die Parameter fr die KonfigurationsBereichBeschreibung.
	 *
	 * @return erzeugte KonfigurationsBereichBeschreibung.
	 */
	private KonfigurationsBereichBeschreibung erstelleKonfigurationsBereichBeschreibung(KonfigurationsBereich kb){
		KonfigurationsBereichBeschreibung kbB=null;
		if(kb!=null){
			String xmlBase=""; // wird nicht benutzt.

			kbB=new KonfigurationsBereichBeschreibung(_alleDefinitionen.erstelleUebersicht(kb),
					kb.getVerantwortlich(), xmlBase, erstelleListeKAB(kb),
					erstelleModellBeschreibung(kb.getModell()),
					erstelleObjekteBeschreibung(kb.getObjekte()));
		}

		return kbB;
	}

	/**
	 * Aus der Instanz der KonfigurationsBereich erstelle eine Liste der KonfigurationsAenderungBeschreibung.
	 *
	 * @param kb
	 * @return erzeugte Liste von KonfigurationsAenderungBeschreibung.
	 */
	private List<KonfigurationsAenderungBeschreibung> erstelleListeKAB(KonfigurationsBereich kb){
		List<KonfigurationsAenderungBeschreibung> listeKAB=new ArrayList<KonfigurationsAenderungBeschreibung>();
		for(KonfigurationsAenderung ka: kb.getKonfigurationsAenderung()){
			listeKAB.add(erstelleKonfigurationsAenderungBeschreibung(ka));
		}
		return listeKAB;
	}

	/**
	 * Aus der Instanz der Klasse KonfigurationsAenderung erstelle eine KonfigurationsAenderungBeschreibung.
	 *
	 * <!ELEMENT konfigurationsAenderung  (#PCDATA | %zeichenauszeichnung.klasse; |
	 *                                           %absatzauszeichnung.klasse;)* >
	 * <!ATTLIST konfigurationsAenderung
	 * 			stand CDATA #REQUIRED
	 * 			version CDATA #IMPLIED
	 * 			autor CDATA #REQUIRED
	 * 			grund CDATA #IMPLIED
	 * >
	 *
	 * @param ka liefert die Parameter fr die KonfigurationsAenderungBeschreibung.
	 *
	 * @return erzeugte KonfigurationsAenderungBeschreibung.
	 */
	private KonfigurationsAenderungBeschreibung erstelleKonfigurationsAenderungBeschreibung(KonfigurationsAenderung ka){
		String version="";
		if(ka.getVersion()!=null){
			version=ka.getVersion();
		}

		String grund="";
		if(ka.getGrund()!=null){
			grund=ka.getGrund();
		}

		return new KonfigurationsAenderungBeschreibung(_alleDefinitionen.erstelleInhalt(ka.getContent()),
				ka.getStand(), version, ka.getAutor(), grund);
	}

	/**
	 * Erstelle ModellBeschreibung aus der Instanz der Klasse Modell.
	 *
	 * <!ELEMENT modell (
	 * 	(	typDefinition |
	 * 		attributgruppenDefinition |
	 * 		aspektDefinition |
	 * 		attributDefinition |
	 * 		attributlistenDefinition |
	 * 		attributauswahlDefinition |
	 *         mengenDefinition)*
	 * ) >
	 *
	 * @param modell liefert die Parameter fr die ModellBeschreibung.
	 * @return erzeugte ModellBeschreibung
	 */
	private ModellBeschreibung erstelleModellBeschreibung(Modell modell){
		ModellBeschreibung modellB=null;
		if(modell!=null){
			modellB=new ModellBeschreibung(
					_listeTypDefinitionB,
					_listeAttributgruppenDefinitionB,
					_listeAspektDefinitionB,
					_listeAttributDefinitionB,
					_listeAttributlistenDefinitionB,
					_listeAttributauswahlDefinitionB,
					_listeMengeDefinitionB
					);
		}
		return modellB;
	}

	/**
	 * erstelle ObjekteBeschreibung aus der Instanz der Klasse Objekte.
	 *
	 * <!ELEMENT objekte (konfigurationsObjekt*) >
	 *
	 * @param objekte liefert die Parameter fr die ObjekteBeschreibung.
	 * @return erzeugte ObjekteBeschreibung.
	 */
	private ObjekteBeschreibung erstelleObjekteBeschreibung(Objekte objekte){
		List<KonfigurationsObjektBeschreibung> listeKOB=new ArrayList<KonfigurationsObjektBeschreibung>();
		if(objekte != null){
		for (KonfigurationsObjekt ko: objekte.getKonfigurationsObjekt()){
			listeKOB.add(erstelleKonfigurationsObjektBeschreibung(ko));
		}
		}
		return new ObjekteBeschreibung(listeKOB);
	}

	/**
	 * erstelle KonfigurationsObjektBeschreibung aus der Instanz der Klasse KonfigurationsObjekt.
	 *
	 * <!ELEMENT konfigurationsObjekt (info?, (datensatz|objektMenge|defaultParameter)*) >
	 * <!ATTLIST konfigurationsObjekt
	 * 			pid ID #REQUIRED
	 * 			id CDATA #IMPLIED
	 * 			name CDATA #IMPLIED
	 * 			typ NMTOKEN #REQUIRED
	 * >
	 *
	 * @param ko liefert Parameter fr die KonfigurationsObjektBeschreibung.
	 * @return erzeugte KonfigurationsObjektBeschreibung.
	 */
	private KonfigurationsObjektBeschreibung erstelleKonfigurationsObjektBeschreibung(KonfigurationsObjekt ko){
		String id="";
		if(ko.getId()!=null){
			id=ko.getId();
		}
		List<DatensatzBeschreibung> listeDatensatzB=new ArrayList<DatensatzBeschreibung>();
		List<ObjektMengeBeschreibung> listeObjektMengeB=new ArrayList<ObjektMengeBeschreibung>();
		List<DefaultParameterBeschreibung> listeDefPar=new ArrayList<DefaultParameterBeschreibung>();

		for(Object o: ko.getDatensatzOrObjektMengeOrDefaultParameter()){
			if (o instanceof Datensatz) {
				Datensatz datensatz = (Datensatz) o;
				listeDatensatzB.add(erstelleDatensatzBeschreibung(datensatz));
			}
			else if (o instanceof ObjektMenge) {
				ObjektMenge objektMenge = (ObjektMenge) o;
				listeObjektMengeB.add(erstelleObjektMengeBeschreibung(objektMenge));
			}
			else if (o instanceof DefaultParameter) {
				DefaultParameter defPar = (DefaultParameter) o;
				listeDefPar.add(erstelleDefaultParameterBeschreibung(defPar));
			}
		}

		return new KonfigurationsObjektBeschreibung(_alleDefinitionen.erstelleUebersicht(ko),id,
				erstelleTypBeschreibung(ko.getTyp()),
				listeDatensatzB, listeObjektMengeB, listeDefPar);
	}

	/**
	 * erstelle DatensatzBeschreibung aus der Instanz der Klasse Datensatz.
	 *
	 * <!ELEMENT datensatz ((datum|datenliste|datenfeld)+) >
	 * <!ATTLIST datensatz
	 * 			pid NMTOKEN #IMPLIED
	 * 			attributgruppe NMTOKEN #IMPLIED
	 * 			aspekt NMTOKEN "asp.eigenschaften"
	 * >
	 *
	 * @param datensatz liefert die Parameter fr die DatensatzBeschreibung.
	 * @return erzeugte DatensatzBeschreibung.
	 */
	private DatensatzBeschreibung erstelleDatensatzBeschreibung(Datensatz datensatz){
		String pid="";
		if(datensatz.getPid()!=null){
			pid=datensatz.getPid();
		}

		String atgPID="";
		if(datensatz.getAttributgruppe()!=null){
			atgPID=datensatz.getAttributgruppe();
		}

		String aspektPID="";
		if(datensatz.getAspekt()!=null){
			aspektPID=datensatz.getAspekt();
		}
		Aspekt aspekt=new Aspekt();
		aspekt.setKonfigurationsModus("");
		aspekt.setOnlineModus("");
		aspekt.setPid(aspektPID);

		List<DatumBeschreibung> listeDatumB = new ArrayList<DatumBeschreibung>();
		List<DatenlisteBeschreibung> listeDatenlisteB = new ArrayList<DatenlisteBeschreibung>();
		List<DatenfeldBeschreibung> listeDatenfeldB = new ArrayList<DatenfeldBeschreibung>();

		for(Object datumOrDlOrDf: datensatz.getDatumOrDatenlisteOrDatenfeld()){
			if (datumOrDlOrDf instanceof Datum) {
				Datum datum = (Datum) datumOrDlOrDf;
				listeDatumB.add(erstelleDatumBeschreibung(datum));
			}
			else if (datumOrDlOrDf instanceof Datenliste) {
				Datenliste datenliste = (Datenliste) datumOrDlOrDf;
				listeDatenlisteB.add(erstelleDatenlisteBeschreibung(datenliste));
			}
			else if (datumOrDlOrDf instanceof Datenfeld) {
				Datenfeld datenFeld = (Datenfeld) datumOrDlOrDf;
				listeDatenfeldB.add(erstelleDatenfeldBeschreibung(datenFeld));
			}
		}

		return new DatensatzBeschreibung(pid,
				erstelleAttributgruppenBeschreibung(atgPID),
				erstelleAspektBeschreibung(aspekt),
				listeDatumB, listeDatenlisteB, listeDatenfeldB);
	}

	/**
	 * erstelle eine Instanz der Klasse DatumBeschreibung aus Datum.
	 *
	 * <!ELEMENT datum EMPTY >
	 * <!ATTLIST datum
	 * 			name CDATA #REQUIRED
	 * 			wert CDATA #REQUIRED
	 * >
	 *
	 * @param datum liefert die Parameter fr die DatumBeschreibung.
	 *
	 * @return erzeugte DatumBeschreibung.
	 */
	private DatumBeschreibung erstelleDatumBeschreibung(Datum datum){
		return new DatumBeschreibung(datum.getName(), datum.getWert());
	}

	/**
	 * erstelle eine Instanz der Klasse DatenlisteBeschreibung aus Datenliste.
	 *
	 * <!ELEMENT datenliste ((datum|datenliste|datenfeld)+) >
 	 * <!ATTLIST datenliste name CDATA #REQUIRED
 	 * >
	 *
	 * @param datenliste liefert die Parameter fr die DatenlisteBeschreibung.
	 *
	 * @return erzeugte DatenlisteBeschreibung.
	 */
	private DatenlisteBeschreibung erstelleDatenlisteBeschreibung(Datenliste datenliste){
		List<DatumBeschreibung> listeDatumB = new ArrayList<DatumBeschreibung>();
		List<DatenlisteBeschreibung> listeDatenlisteB = new ArrayList<DatenlisteBeschreibung>();
		List<DatenfeldBeschreibung> listeDatenfeldB = new ArrayList<DatenfeldBeschreibung>();

		for(Object datumOrDlOrDf: datenliste.getDatumOrDatenlisteOrDatenfeld()){
			if (datumOrDlOrDf instanceof Datum) {
				Datum datum = (Datum) datumOrDlOrDf;
				listeDatumB.add(erstelleDatumBeschreibung(datum));
			}
			else if (datumOrDlOrDf instanceof Datenliste) {
				Datenliste datenliste2 = (Datenliste) datumOrDlOrDf;
				listeDatenlisteB.add(erstelleDatenlisteBeschreibung(datenliste2));
			}
			else if (datumOrDlOrDf instanceof Datenfeld) {
				Datenfeld datenFeld = (Datenfeld) datumOrDlOrDf;
				listeDatenfeldB.add(erstelleDatenfeldBeschreibung(datenFeld));
			}
		}

		return new DatenlisteBeschreibung(datenliste.getName(),
				listeDatumB, listeDatenlisteB, listeDatenfeldB);
	}

	/**
	 * erstelle eine Instanz der Klasse DatenfeldBeschreibung aus Datenfeld.
	 *
	 * <!ELEMENT datenfeld ((datum|datenliste)*) >
	 * <!ATTLIST datenfeld name CDATA #REQUIRED
	 *
	 * @param datenfeld liefert die Parameter fr die DatenfeldBeschreibung.
	 *
	 * @return erzeugte DatenfeldBeschreibung.
	 */
	private DatenfeldBeschreibung erstelleDatenfeldBeschreibung(Datenfeld datenfeld){
		List<DatumBeschreibung> listeDatumB = new ArrayList<DatumBeschreibung>();
		List<DatenlisteBeschreibung> listeDatenlisteB = new ArrayList<DatenlisteBeschreibung>();

		for(Object datumOrDl: datenfeld.getDatumOrDatenliste()){
			if (datumOrDl instanceof Datum) {
				Datum datum = (Datum) datumOrDl;
				listeDatumB.add(erstelleDatumBeschreibung(datum));
			}
			else if (datumOrDl instanceof Datenliste) {
				Datenliste datenliste2 = (Datenliste) datumOrDl;
				listeDatenlisteB.add(erstelleDatenlisteBeschreibung(datenliste2));
			}
		}

		return new DatenfeldBeschreibung(datenfeld.getName(), listeDatumB, listeDatenlisteB);
	}

	/**
	 * erstelle eine Instanz der Klasse AttributgruppenBeschreibung aus PID.
	 *
	 * Falls die entsprechende Instanz von AttributgruppenDefinitionBeschreibung
	 * nicht ber _referenzInfo gefunden wird,
	 * initialisiere die auf null.
	 *
	 * <!ELEMENT attributgruppe EMPTY>
	 * <!ATTLIST attributgruppe pid NMTOKEN #REQUIRED
	 * >
	 *
	 * @param pid PID der AttributgruppenBeschreibung.
	 *
	 * @return erzeugte AttributgruppenBeschreibung.
	 */
	private AttributgruppenBeschreibung erstelleAttributgruppenBeschreibung(String pid){
		// Suche die AttributgruppenDefinitionBeschreibung von atg:
		AttributgruppenDefinitionBeschreibung atgDefB=null;

		Object definition=_alleDefinitionen.getMapAlleDefinitionen().get(pid);
		if(definition!=null){
			if (definition instanceof AttributgruppenDefinition) {
				AttributgruppenDefinition atgDef = (AttributgruppenDefinition) definition;
				atgDefB=erstelleAttributgruppenDefinitionBeschreibung(atgDef);
			}
		}

		return new AttributgruppenBeschreibung(pid, atgDefB);
	}

	/**
	 * Suche eine Instanz der Klasse AttributgruppenDefinitionBeschreibung in Map _mapAtg.
	 * Falls finde, gebe die zurck.
	 * Sonst: erstelle eine Instanz der Klasse AttributgruppenDefinitionBeschreibung
	 * aus AttributgruppenDefinition.
	 *
	 * <!ELEMENT attributgruppenDefinition ( info, aspekt*, (attribut|attributliste|attributauswahl)+ ) >
	 * <!ATTLIST attributgruppenDefinition
 	 *  	pid ID #REQUIRED
	 *  	name CDATA #IMPLIED
 	 *  	konfigurierend (ja|nein) "nein"
 	 *  	parametrierend (ja|nein) "nein"
 	 *  	code CDATA #IMPLIED
 	 * >
 	 *
	 * @param atgDef liefert die Parameter fr die AttributgruppenDefinitionBeschreibung.
	 *
	 * @return erzeugte AttributgruppenDefinitionBeschreibung.
	 */
	private AttributgruppenDefinitionBeschreibung erstelleAttributgruppenDefinitionBeschreibung(AttributgruppenDefinition atgDef){
		if(_mapAtg.containsKey(atgDef)){
			return _mapAtg.get(atgDef);
		}
		else{
			// Listen:
			List<AspektBeschreibung> listeAspektBeschreibung=new ArrayList<AspektBeschreibung>();
			List<AttributBeschreibung> listeAttBeschreibung=new ArrayList<AttributBeschreibung>();
			List<AttributlistenBeschreibung> listeAtlBeschreibung=new ArrayList<AttributlistenBeschreibung>();
			List<AttributauswahlBeschreibung> listeAtaBeschreibung=new ArrayList<AttributauswahlBeschreibung>();

			for(Aspekt aspekt: atgDef.getAspekt()){
				listeAspektBeschreibung.add(erstelleAspektBeschreibung(aspekt));
			}
			for(Object attOrAttLOrAttA: atgDef.getAttributOrAttributlisteOrAttributauswahl()){
				if (attOrAttLOrAttA instanceof Attribut) {
					Attribut att = (Attribut) attOrAttLOrAttA;
					listeAttBeschreibung.add(erstelleAttribbutBeschreibung(att));
				}
				else if (attOrAttLOrAttA instanceof Attributliste) {
					Attributliste atl = (Attributliste) attOrAttLOrAttA;
					listeAtlBeschreibung.add(erstelleAttributlistenBeschreibung(atl));
				}
				else if (attOrAttLOrAttA instanceof Attributauswahl) {
					Attributauswahl ata = (Attributauswahl) attOrAttLOrAttA;
					listeAtaBeschreibung.add(erstelleAttributauswahlBeschreibung(ata));
				}
			}
			//
			String code="";
			if(atgDef.getCode()!=null){
				code=atgDef.getCode();
			}
			String konfigurierend="";
			if(atgDef.getKonfigurierend()!=null){
				konfigurierend=atgDef.getKonfigurierend();
			}
			String parametrierend="";
			if(atgDef.getParametrierend()!=null){
				parametrierend=atgDef.getParametrierend();
			}
			AttributgruppenDefinitionBeschreibung atgDefB =
				new AttributgruppenDefinitionBeschreibung(_alleDefinitionen.erstelleUebersicht(atgDef),
						konfigurierend, parametrierend, code);

			atgDefB.setListeAspektBescreibung(listeAspektBeschreibung);
			atgDefB.setListeAttributAuswahlBeschreibung(listeAtaBeschreibung);
			atgDefB.setListeAttributBeschreibung(listeAttBeschreibung);
			atgDefB.setListeAttributListenBeschreibung(listeAtlBeschreibung);

			_mapAtg.put(atgDef, atgDefB);

			return atgDefB;
		}
	}

	/**
	 * erstelle eine Instnz der Klasse AspektBeschreibung aus Aspekt.
	 *
	 *  <!ELEMENT aspekt (info?)>
	 * <!ATTLIST aspekt
	 * 	pid NMTOKEN #REQUIRED
	 * 	konfigurationsModus ( datensatzNotwendig | datensatzNotwendigUndnderbar | datensatzOptional | datensatzOptionalUndnderbar ) #IMPLIED
	 * 	onlineModus (quelle | senke | quelleUndSenke ) #IMPLIED
	 * >
	 *
	 * @param aspekt liefert die Parameter fr die AspektBeschreibung.
	 *
	 * @return erzeugte AspektBeschreibung.
	 */
	private AspektBeschreibung erstelleAspektBeschreibung(Aspekt aspekt){
		String konfigurationsModus="";
		if(aspekt.getKonfigurationsModus()!=null){
			konfigurationsModus=aspekt.getKonfigurationsModus();
		}
		String onlineModus="";
		if(aspekt.getOnlineModus()!=null){
			onlineModus=aspekt.getOnlineModus();
		}

		AspektDefinitionBeschreibung aspektDefB=null;
		Object definition=_alleDefinitionen.getMapAlleDefinitionen().get(aspekt.getPid());
		if(definition!=null){
			if (definition instanceof AspektDefinition) {
				AspektDefinition aspektDef = (AspektDefinition) definition;
				aspektDefB=erstelleAspektDefinitionBeschreibung(aspektDef);
			}
		}

		AspektBeschreibung aspektB=new AspektBeschreibung(
				_alleDefinitionen.erstelleUebersicht(aspekt),
				konfigurationsModus, onlineModus,
				aspektDefB);

		return aspektB;
	}

	/**
	 * erstelle ObjektMengeBeschreibung aus der Instanz der klasse ObjektMenge.
	 *
	 * <!ELEMENT objektMenge (element*) >
	 * <!ATTLIST objektMenge
 	 * 			name CDATA #REQUIRED
 	 * 			verwaltung NMTOKEN #IMPLIED
 	 * >
 	 *
	 * @param objektMenge liefert die Parameter fr die ObjektMengeBeschreibung.
	 * @return erzeugte ObjektMengeBeschreibung.
	 */
	private ObjektMengeBeschreibung erstelleObjektMengeBeschreibung(ObjektMenge objektMenge){
		String verwaltung="";
		if(objektMenge.getVerwaltung()!=null){
			verwaltung=objektMenge.getVerwaltung();
		}
		List<ElementBeschreibung> listeElementB=new ArrayList<ElementBeschreibung>();
		for( Element element:objektMenge.getElement()){
			Uebersicht uebersicht=null;

			Object definition=_alleDefinitionen.getMapAlleDefinitionen().get(element.getPid());
			if(definition!=null){
				uebersicht=_alleDefinitionen.erstelleUebersicht(definition);
			}
			listeElementB.add(new ElementBeschreibung(element.getPid(), uebersicht));
		}

		return new ObjektMengeBeschreibung(objektMenge.getName(), verwaltung, listeElementB);
	}

	/**
	 * erstelle eine Instanz der Klasse DefaultParameterBeschreibung aus DefaultParameter.
	 *
	 * <!ELEMENT defaultParameter ((datum|datenliste|datenfeld)+) >
	 * <!ATTLIST defaultParameter
	 * 			typ NMTOKEN #IMPLIED
	 * 			attributgruppe NMTOKEN #REQUIRED
	 * >
	 *
	 * @param defPar liefert die Parameter fr die DefaultParameterBeschreibung.
	 *
	 * @return erzeugte DefaultParameterBeschreibung.
	 */
	private DefaultParameterBeschreibung erstelleDefaultParameterBeschreibung(DefaultParameter defPar){
		String typPID="";
		if(defPar.getTyp()!=null){
			typPID=defPar.getTyp();
		}
		Uebersicht typUebersicht=new Uebersicht(typPID, "", null);
		Object definition=_alleDefinitionen.getMapAlleDefinitionen().get(typPID);
		if(definition!=null){
			typUebersicht=_alleDefinitionen.erstelleUebersicht(definition);
		}

		List<DatumBeschreibung> listeDatumB = new ArrayList<DatumBeschreibung>();
		List<DatenlisteBeschreibung> listeDatenlisteB = new ArrayList<DatenlisteBeschreibung>();
		List<DatenfeldBeschreibung> listeDatenfeldB = new ArrayList<DatenfeldBeschreibung>();

		for(Object datumOrDlOrDf: defPar.getDatumOrDatenlisteOrDatenfeld()){
			if (datumOrDlOrDf instanceof Datum) {
				Datum datum = (Datum) datumOrDlOrDf;
				listeDatumB.add(erstelleDatumBeschreibung(datum));
			}
			else if (datumOrDlOrDf instanceof Datenliste) {
				Datenliste datenliste = (Datenliste) datumOrDlOrDf;
				listeDatenlisteB.add(erstelleDatenlisteBeschreibung(datenliste));
			}
			else if (datumOrDlOrDf instanceof Datenfeld) {
				Datenfeld datenFeld = (Datenfeld) datumOrDlOrDf;
				listeDatenfeldB.add(erstelleDatenfeldBeschreibung(datenFeld));
			}
		}

		return new DefaultParameterBeschreibung(typUebersicht,
				erstelleAttributgruppenBeschreibung(defPar.getAttributgruppe()),
				listeDatumB, listeDatenlisteB, listeDatenfeldB);
	}

	/**
	 * erstelle eine Instanz der Klasse TypBeschreibung aus Pid.
	 *
	 * Falls die entsprechende Instanz von TypDefinitionBeschreibung
	 * nicht ber _referenzInfo gefunden wird,
	 * initialisiere die auf null.
	 *
	 * <!ELEMENT erweitert EMPTY >
	 * <!ATTLIST erweitert pid NMTOKEN #REQUIRED >
	 *
	 * @param pid PID von TypBeschreibung.
	 *
	 * @return erzeugte TypBeschreibung.
	 */
	private TypBeschreibung erstelleTypBeschreibung(String pid){
		// Suche die TypDefinition von erweitert:
		TypDefinitionBeschreibung typDefB=null;

		Object definition=_alleDefinitionen.getMapAlleDefinitionen().get(pid);
		if(definition!=null){
			if (definition instanceof TypDefinition) {
				TypDefinition typDef = (TypDefinition) definition;
				typDefB=erstelleTypDefinitionBeschreibung(typDef);
			}
		}

		return new TypBeschreibung(pid, typDefB);
	}

	/**
	 * Suche eine Instanz der Klasse TypDefinitionBeschreibung in Map _mapTyp.
	 * Falls finde, gebe die zurck.
	 * Sonst: erstelle eine Instanz der Klasse TypDefinitionBeschreibung aus TypDefinition.
	 *
	 * <!ELEMENT typDefinition ( info, (erweitert+ | basis), (attributgruppe | menge)*, defaultParameter* ) >
	 * <!ATTLIST typDefinition
	 *  	pid ID #REQUIRED
	 *  	name CDATA #IMPLIED
	 *  	objektNamenPermanent (ja|nein) "nein"
	 *  	persistenzModus ( transient | persistent | persistentUndUngltigNachNeustart ) #IMPLIED
	 * >
	 *
	 * @param typDef liefert die Parameter fr die TypDefinitionBeschreibung.
	 *
	 * @return erzeugte TypDefinitionBeschreibung.
	 */
	private TypDefinitionBeschreibung erstelleTypDefinitionBeschreibung(TypDefinition typDef){
		if(_mapTyp.containsKey(typDef)){
			return _mapTyp.get(typDef);
		}
		else{
			List<TypBeschreibung> listeErweitert=new ArrayList<TypBeschreibung>();
			for (Erweitert erweitert: typDef.getErweitert()){
				listeErweitert.add(erstelleTypBeschreibung(erweitert.getPid()));
			}

			List<AttributgruppenBeschreibung> listeAtgB=new ArrayList<AttributgruppenBeschreibung>();
			List<MengenBeschreibung> listeMengenB=new ArrayList<MengenBeschreibung>();
			for(Object atgOrMenge: typDef.getAttributgruppeOrMenge()){
				if (atgOrMenge instanceof Attributgruppe) {
					Attributgruppe atg = (Attributgruppe) atgOrMenge;
					listeAtgB.add(erstelleAttributgruppenBeschreibung(atg.getPid()));
				}
				else if (atgOrMenge instanceof Menge) {
					Menge menge = (Menge) atgOrMenge;
					listeMengenB.add(erstelleMengenBeschreibung(menge));
				}
			}

			List<DefaultParameterBeschreibung> listeDefParB=new ArrayList<DefaultParameterBeschreibung>();
			for(DefaultParameter defPar: typDef.getDefaultParameter()){
				listeDefParB.add(erstelleDefaultParameterBeschreibung(defPar));
			}

			String persistenzModus="";
			if(typDef.getPersistenzModus()!=null){
				persistenzModus=typDef.getPersistenzModus();
			}

			String basisKonfigurierend="";
			if(typDef.getBasis()!=null){
				basisKonfigurierend=typDef.getBasis().getKonfigurierend();
			}

			TypDefinitionBeschreibung typDefB=new TypDefinitionBeschreibung(_alleDefinitionen.erstelleUebersicht(typDef),
					typDef.getObjektNamenPermanent(), persistenzModus,
					listeErweitert, basisKonfigurierend, listeAtgB, listeMengenB, listeDefParB);

			_mapTyp.put(typDef, typDefB);

			return typDefB;
		}
	}

	/**
	 * erstelle eine Instanz der Klasse MengenBeschreibung aus Menge.
	 *
	 * Falls die entsprechende Instanz von MengenDefinitionBeschreibung
	 * nicht ber _referenzInfo gefunden wird,
	 * initialisiere die auf null.
	 *
	 * <!ELEMENT menge  ( info? )>
	 * <!ATTLIST menge
	 *      pid NMTOKEN #REQUIRED
	 * 		name CDATA #REQUIRED
	 * 		erforderlich (ja|nein) "ja"
	 * >
	 *
	 * @param menge liefert die Parameter fr die MengenBeschreibung.
	 *
	 * @return erzeugte MengenBeschreibung.
	 */
	private MengenBeschreibung erstelleMengenBeschreibung(Menge menge){
		// Suche die MengenDefinition von menge:
		MengenDefinitionBeschreibung mengenDefB=null;

		Object definition=_alleDefinitionen.getMapAlleDefinitionen().get(menge.getPid());
		if(definition!=null){
			if (definition instanceof MengenDefinition) {
				MengenDefinition mengenDef = (MengenDefinition) definition;
				mengenDefB=erstelleMengenDefinitionBeschreibung(mengenDef, menge.getName());
			}
		}

		return new MengenBeschreibung(_alleDefinitionen.erstelleUebersicht(menge), menge.getErforderlich(), mengenDefB);
	}

	/**
	 * erstelle die bersichtListe.
	 * Jedes Element der Liste besteht aus dem Namen, PID und Info.
	 *
	 * @param collObject Collection von Elementen, die den Namen, PID und Info liefern.
	 *
	 * @return bersichtListe.
	 */
	private List<Uebersicht> erstelleUebersichtListe(Collection<?> collObject){
		List<Uebersicht> listeUebersicht = new ArrayList<Uebersicht>();
		if(!collObject.isEmpty()){
			for(Object element: collObject){
				listeUebersicht.add(_alleDefinitionen.erstelleUebersicht(element));
			}
		}
		return listeUebersicht;
	}

	/**
	 * erstelle die Liste von TypDefinitionBeschreibung aus der Collection von TypDefinition.
	 *
	 * @param collTypDef Collection von TypDefinition, aus deren
	 * die Liste von TypDefinitionBeschreibung erzeugt wird.
	 * @return erzeugte Liste von TypDefinitionBeschreibung.
	 */
	private List<TypDefinitionBeschreibung> erstelleTypDefinitionBeschreibungListe(Collection<TypDefinition> collTypDef){
		List<TypDefinitionBeschreibung> listeTypDefB=new ArrayList<TypDefinitionBeschreibung>();
		for(TypDefinition typDef: collTypDef){
			listeTypDefB.add(erstelleTypDefinitionBeschreibung(typDef));
		}
		return listeTypDefB;
	}

	/**
	 * erstelle die Liste von MengenDefinitionBeschreibung aus der Collection von MengenDefinition.
	 *
	 * @param collMengenDefinition Collection von MengenDefinition
	 * aus deren die Liste von MengenDefinitionBeschreibung erzeugt wird.
	 *
	 * @return erzeugte Liste von MengenDefinitionBeschreibung.
	 */
	private List<MengenDefinitionBeschreibung> erstelleMengenDefinitionBeschreibungListe(Collection<MengenDefinition> collMengenDefinition){
		List<MengenDefinitionBeschreibung> listeMengenDefBeschreibung=new ArrayList<MengenDefinitionBeschreibung>();
		for(MengenDefinition mdd: collMengenDefinition){
			listeMengenDefBeschreibung.add(erstelleMengenDefinitionBeschreibung(mdd, null));
		}
		return listeMengenDefBeschreibung;
	}

	/**
	 * Suche eine Instanz der Klasse MengenDefinitionBeschreibung in Map _mapMenge.
	 * Falls finde, gebe die zurck.
	 * Sonst: aus der Instanz der Klasse MengenDefinition erstelle MengenDefinitionBeschreibung.
	 *
	 * <!ELEMENT mengenDefinition (info)>
	 * <!ATTLIST mengenDefinition
	 *  	pid ID #REQUIRED
	 *  	elemente NMTOKENS #REQUIRED
	 *  	aenderbar (ja|nein) "nein"
	 *  	mindestens CDATA #IMPLIED
	 *  	hoechstens CDATA #IMPLIED
	 *  	referenzierungsart (assoziation|aggregation|komposition) #IMPLIED
	 *  >
	 *
	 * @param mengenDef liefert die Parameter fr die MengenDefinitionBeschreibung.
	 * @param name Name oder null.
	 *
	 * @return aus der mengenDefinition erzeugte MengenDefinitionBeschreibung.
	 */
	private MengenDefinitionBeschreibung erstelleMengenDefinitionBeschreibung(MengenDefinition mengenDef, String name){
		if(_mapMenge.containsKey(mengenDef)){
			return _mapMenge.get(mengenDef);
		}
		else{
			String mindestens="";
			String hoechstens="";
			String referenzierungsart="";
			if(mengenDef.getMindestens()!=null){
				mindestens=mengenDef.getMindestens();
			}
			if(mengenDef.getHoechstens()!=null){
				hoechstens=mengenDef.getHoechstens();
			}
			if(mengenDef.getReferenzierungsart()!=null){
				referenzierungsart=mengenDef.getReferenzierungsart();
			}
			// Suche bersicht fr jedes Element:
			List<Uebersicht> listeUebersichtElemente=new ArrayList<Uebersicht>();
			for(String pid: mengenDef.getElemente()){
				Object definition=_alleDefinitionen.getMapAlleDefinitionen().get(pid);
				if(definition!=null){
					Uebersicht uebersicht=_alleDefinitionen.erstelleUebersicht(definition);
					listeUebersichtElemente.add(uebersicht);
				}
			}

			MengenDefinitionBeschreibung mdb=new MengenDefinitionBeschreibung(
					_alleDefinitionen.erstelleUebersicht(mengenDef),
					mengenDef.getElemente(), listeUebersichtElemente,
					mengenDef.getAenderbar(), mindestens,
					hoechstens, referenzierungsart);

			if(name != null){
				mdb.getUebersicht().setName(name);
			}
			
			_mapMenge.put(mengenDef, mdb);

			return mdb;
		}
	}

	/**
	 * erstelle die Liste von AttributgruppenDefinitionBeschreibung aus der Collection von AttributgruppenDefinition.
	 *
	 * @param collAtgDef Collection von AttributgruppenDefinition, aus deren
	 * die Liste von AttributgruppenDefinitionBeschreibung erzeugt wird.
	 *
	 * @return erzeugte Liste von AttributgruppenDefinitionBeschreibung
	 */
	private List<AttributgruppenDefinitionBeschreibung> erstelleAttributgruppenDefinitionBeschreibungListe(Collection<AttributgruppenDefinition> collAtgDef){
		List<AttributgruppenDefinitionBeschreibung> listeAtgDefBeschreibung=new ArrayList<AttributgruppenDefinitionBeschreibung>();
		for(AttributgruppenDefinition atgDef: collAtgDef){
			listeAtgDefBeschreibung.add(erstelleAttributgruppenDefinitionBeschreibung(atgDef));
		}
		return listeAtgDefBeschreibung;
	}

	/**
	 * erstelle die Liste von AttributlistenDefinitionBeschreibung aus der Collection von AttributlistenDefinition.
	 *
	 * @param collAtlDef Collection von AttributlistenDefinition, aus deren
	 * die Liste von AttributlistenDefinitionBeschreibung erzeugt wird.
	 *
	 * @return erzeugte Liste von AttributlistenDefinitionBeschreibung.
	 */
	private List<AttributlistenDefinitionBeschreibung> erstelleAttributlistenDefinitionBeschreibungListe(Collection<AttributlistenDefinition> collAtlDef){
		List<AttributlistenDefinitionBeschreibung> listeAttListenDefBeschreibung=new ArrayList<AttributlistenDefinitionBeschreibung>();
		for(AttributlistenDefinition atlDef: collAtlDef){
			listeAttListenDefBeschreibung.add(erstelleAttributlistenDefinitionBeschreibung(atlDef));
		}
		return listeAttListenDefBeschreibung;
	}

	/**
	 * Suche eine Instanz der Klasse AttributlistenDefinitionBeschreibung in Map _mapAspekt.
	 * Falls finde, gebe die zurck.
	 * Sonst: erstelle eine Instanz der Klasse AttributlistenDefinitionBeschreibung
	 * aus AttributlistenDefinition.
	 *
	 * <!ELEMENT attributlistenDefinition ( info, (attribut|attributliste|attributauswahl)+ ) >
	 * <!ATTLIST attributlistenDefinition
	 * 		pid ID #REQUIRED
	 * 		name CDATA #IMPLIED
	 * >
	 *
	 * @param atlDef liefert die Parameter fr die AttributlistenDefinitionBeschreibung.
	 *
	 * @return erzeugte AttributlistenDefinitionBeschreibung.
	 */
	private AttributlistenDefinitionBeschreibung erstelleAttributlistenDefinitionBeschreibung(AttributlistenDefinition atlDef){
		if(_mapAtl.containsKey(atlDef)){
			return _mapAtl.get(atlDef);
		}
		else{
			// Listen:
			List<AttributBeschreibung> listeAttBeschreibung=new ArrayList<AttributBeschreibung>();
			List<AttributlistenBeschreibung> listeAtlBeschreibung=new ArrayList<AttributlistenBeschreibung>();
			List<AttributauswahlBeschreibung> listeAtaBeschreibung=new ArrayList<AttributauswahlBeschreibung>();

			for(Object attOrAttLOrAttA: atlDef.getAttributOrAttributlisteOrAttributauswahl()){
				if (attOrAttLOrAttA instanceof Attribut) {
					Attribut att = (Attribut) attOrAttLOrAttA;
					listeAttBeschreibung.add(erstelleAttribbutBeschreibung(att));
				}
				else if (attOrAttLOrAttA instanceof Attributliste) {
					Attributliste atl = (Attributliste) attOrAttLOrAttA;
					listeAtlBeschreibung.add(erstelleAttributlistenBeschreibung(atl));
				}
				else if (attOrAttLOrAttA instanceof Attributauswahl) {
					Attributauswahl ata = (Attributauswahl) attOrAttLOrAttA;
					listeAtaBeschreibung.add(erstelleAttributauswahlBeschreibung(ata));
				}
			}
			AttributlistenDefinitionBeschreibung atlDefB=
				new AttributlistenDefinitionBeschreibung(_alleDefinitionen.erstelleUebersicht(atlDef));

			atlDefB.setListeAttributBeschreibung(listeAttBeschreibung);
			atlDefB.setListeAttributListenBeschreibung(listeAtlBeschreibung);
			atlDefB.setListeAttributAuswahlBeschreibung(listeAtaBeschreibung);

			_mapAtl.put(atlDef, atlDefB);

			return atlDefB;
		}
	}

	/**
	 * erstelle eine Instanz der Klasse AttributBeschreibung aus Attribut.
	 *
	 * Falls die entsprechende Instanz von AttributDefinitionBeschreibung
	 * nicht ber _referenzInfo gefunden wird,
	 * initialisiere die auf null.
	 *
	 * <!ELEMENT attribut ( info?, default? ) >
	 * <!ATTLIST attribut
	 * 		pid NMTOKEN #REQUIRED
	 * 		anzahl NMTOKEN #IMPLIED
	 * 		anzahlIst (variabel|fest) "fest"
	 * 		name NMTOKEN #REQUIRED
	 * >
	 *
	 * @param attribut liefert die Parameter fr die AttributBeschreibung.
	 *
	 * @return erzeugte AttributBeschreibung.
	 */
	private AttributBeschreibung erstelleAttribbutBeschreibung(Attribut attribut){
		//
		String defaultWert="";
		if(attribut.getDefault()!=null){
			defaultWert=attribut.getDefault().getWert();
		}
		String anzahl="";
		if(attribut.getAnzahl()!=null){
			anzahl=attribut.getAnzahl();
		}
		// Suche die Definition von Attribut.
		AttributDefinitionBeschreibung attDefB=null;

		Object definition=_alleDefinitionen.getMapAlleDefinitionen().get(attribut.getPid());
		if(definition!=null){
			if (definition instanceof AttributDefinition) {
				AttributDefinition attributDef = (AttributDefinition) definition;
				attDefB=erstelleAttribbutDefinitionBeschreibung(attributDef);
			}
		}

		return new AttributBeschreibung(
				_alleDefinitionen.erstelleUebersicht(attribut), defaultWert, anzahl, attribut.getAnzahlIst(), attDefB);
	}

	/**
	 * erstelle eine Instanz der Klasse AttributlistenBeschreibung aus Attributliste.
	 *
	 * Falls die entsprechende Instanz von AttributlistenDefinitionBeschreibung
	 * nicht ber _referenzInfo gefunden wird,
	 * initialisiere die auf null.
	 *
	 * <!ELEMENT attributliste ( info? ) >
	 * <!ATTLIST attributliste
	 * 		pid NMTOKEN #REQUIRED
	 * 		anzahl NMTOKEN #IMPLIED
	 * 		anzahlIst (variabel|fest) "fest"
	 * 		name NMTOKEN #REQUIRED
	 * >
	 *
	 * @param atl liefert die Parameter fr die AttributlistenBeschreibung.
	 *
	 * @return erzeugte AttributlistenBeschreibung.
	 */
	private AttributlistenBeschreibung erstelleAttributlistenBeschreibung(Attributliste atl){
		AttributlistenDefinitionBeschreibung atlDefB=null;

		Object definition=_alleDefinitionen.getMapAlleDefinitionen().get(atl.getPid());
		if(definition!=null){
			if (definition instanceof AttributlistenDefinition) {
				AttributlistenDefinition atlDef = (AttributlistenDefinition) definition;
				atlDefB=erstelleAttributlistenDefinitionBeschreibung(atlDef);
			}
		}

		String anzahl="";
		if(atl.getAnzahl()!=null){
			anzahl=atl.getAnzahl();
		}
		return new AttributlistenBeschreibung(_alleDefinitionen.erstelleUebersicht(atl), anzahl, atl.getAnzahlIst(), atlDefB);
	}

	/**
	 * erstelle eine Instanz der Klasse AttributauswahlBeschreibung.
	 *
	 * Falls die entsprechende Instanz von AttributauswahlDefinitionBeschreibung
	 * nicht ber _referenzInfo gefunden wird,
	 * initialisiere die auf null.
	 *
	 * <!ELEMENT attributauswahl ( info? ) >
	 * <!ATTLIST attributauswahl
	 * 		pid NMTOKEN #REQUIRED
	 * 		name CDATA #IMPLIED
	 * >
	 *
	 * @param ata liefert die Parameter fr die AttributauswahlBeschreibung.
	 *
	 * @return erzeugte AttributauswahlBeschreibung.
	 */
	private AttributauswahlBeschreibung erstelleAttributauswahlBeschreibung(Attributauswahl ata){
		//
		AttributauswahlDefinitionBeschreibung atadb=null;

		Object definition=_alleDefinitionen.getMapAlleDefinitionen().get(ata.getPid());
		if(definition!=null){
			if (definition instanceof AttributauswahlDefinition) {
				AttributauswahlDefinition ataDef = (AttributauswahlDefinition) definition;
				atadb=erstelleAttributauswahlDefinitionBeschreibung(ataDef);
			}
		}

		return new AttributauswahlBeschreibung(_alleDefinitionen.erstelleUebersicht(ata), atadb);
	}

	/**
	 * erstelle die Liste von AttributDefinitionBeschreibung aus der Collection von AttributDefinition.
	 *
	 * @param collAttDefinition Collection von AttributDefinition, aus deren
	 * die Liste von AttributDefinitionBeschreibung erzeugt wird.
	 *
	 * @return erzeugte Liste von AttributDefinitionBeschreibung.
	 */
	private List<AttributDefinitionBeschreibung> erstelleAttribbutDefinitionBeschreibungListe(Collection<AttributDefinition> collAttDefinition){
		List<AttributDefinitionBeschreibung> listAttDefinitionBeschreibung=new ArrayList<AttributDefinitionBeschreibung>();
		for(AttributDefinition ad: collAttDefinition){
			listAttDefinitionBeschreibung.add(erstelleAttribbutDefinitionBeschreibung(ad));
		}
		return listAttDefinitionBeschreibung;
	}

	/**
	 * Suche eine Instanz der Klasse AttributDefinitionBeschreibung in Map _mapAtt.
	 * Falls finde, gebe die zurck.
	 * Sonst: erstelle eine Instanz der Klasse AttributDefinitionBeschreibung
	 * aus AttributDefinition.
	 *
	 * <!ELEMENT attributDefinition (
	 * info,
	 * 	( zeichenkette | ganzzahl | zeitstempel | objektReferenz | kommazahl ),
	 * 	default? )
	 * >
	 *
	 * <!ELEMENT default EMPTY >
	 * <!ATTLIST default
	 * 		wert CDATA #REQUIRED
	 * >
	 *
	 * @param attDefinition liefert die Parameter fr die AttributDefinitionBeschreibung.
	 *
	 * @return erzeugte AttributDefinitionBeschreibung.
	 */
	private AttributDefinitionBeschreibung erstelleAttribbutDefinitionBeschreibung(AttributDefinition attDefinition){
		if(_mapAtt.containsKey(attDefinition)){
			return _mapAtt.get(attDefinition);
		}
		else{
			// Listen von Bereichen und Zustnden:
			List<BereichBeschreibung> listeBereichB=new ArrayList<BereichBeschreibung>();
			List<ZustandBeschreibung> listeZastandB=new ArrayList<ZustandBeschreibung>();
			/*
			 * Das zwei-dimensionale  Array.
			 *
			 * Element [0][0] gleich 'typ' ist.
			 * Element [0][1] enthlt den TypNamen (siehe in dieser Klasse definierte Konstanten)
			 *
			 * Weitere Zeile enthalten die Information ber die Attributlisten (siehe DTD)
			 * und sind vom Typ abhngig, jeweils Paar (AttributName - Wert).
			 *
			 * Z.B. fr den Typ zeichenkette kann Array so aussehen:
			 *
			 * 'typ'		'zeichenkette'
			 * 'laenge'		'30'
			 * 'kodierung'	'ISO-8859-1'
			 */
			String[][] arrAttributTyp=null;
			/*
			 * 	<!ELEMENT zeichenkette EMPTY >
			 *		<!ATTLIST zeichenkette laenge CDATA #REQUIRED >
			 *		<!ATTLIST zeichenkette kodierung (ISO-8859-1|TLS-Wechseltextzeichen) "ISO-8859-1" >
			 *
			 */
			if(attDefinition.getZeichenkette()!=null){
				arrAttributTyp=new String[][]{{"Typ", AttributDefinitionBeschreibung.ZEICHENKETTE},
						{"Lnge", attDefinition.getZeichenkette().getLaenge()},
						{"Kodierung", attDefinition.getZeichenkette().getKodierung()}};
			}
			/*
			 * <!ELEMENT ganzzahl ( ( bereich | zustand )+ ) >
			 * <!ATTLIST ganzzahl bits CDATA #IMPLIED >
			 *
			 * <!ELEMENT bereich ( info? ) >
			 * <!ATTLIST bereich skalierung CDATA #IMPLIED >
			 * <!ATTLIST bereich minimum CDATA #REQUIRED >
			 * <!ATTLIST bereich maximum CDATA #IMPLIED >
			 * <!ATTLIST bereich einheit CDATA #IMPLIED >
			 *
			 * <!ELEMENT zustand ( info? ) >
			 * <!ATTLIST zustand name CDATA #REQUIRED >
			 * <!ATTLIST zustand wert CDATA #REQUIRED >
			 */
			else if(attDefinition.getGanzzahl()!=null){
				String bits=attDefinition.getGanzzahl().getBits();
				if(bits==null){
					bits="";
				}
				arrAttributTyp=new String[][]{{"Typ", AttributDefinitionBeschreibung.GANZZAHL},
						{"Anzahl Bits", bits}};
				// ( bereich | zustand )+
				for(Object bOrZ: attDefinition.getGanzzahl().getBereichOrZustand()){
					if (bOrZ instanceof Bereich) {
						Bereich bereich = (Bereich) bOrZ;
						listeBereichB.add(erstelleBereichBeschreibung(bereich));
					}
					else if (bOrZ instanceof Zustand) {
						Zustand zustand = (Zustand) bOrZ;
						listeZastandB.add(erstelleZustandBeschreibung(zustand));
					}
				}
			}
			/*
			 * <!ELEMENT zeitstempel  EMPTY >
			 * <!ATTLIST zeitstempel
			 * 	relativ (ja|nein) "nein"
			 * 	genauigkeit (sekunden|millisekunden) "millisekunden"
			 * >
			 */
			else if(attDefinition.getZeitstempel()!=null){
				arrAttributTyp=new String[][]{{"Typ", AttributDefinitionBeschreibung.ZEITSTEMPEL},
						{"Relativ", attDefinition.getZeitstempel().getRelativ()},
						{"Genauigkeit", attDefinition.getZeitstempel().getGenauigkeit()}};
			}
			/*
			 * <!ELEMENT objektReferenz EMPTY >
			 * <!ATTLIST objektReferenz typ NMTOKEN #IMPLIED>
			 * <!ATTLIST objektReferenz undefiniert (erlaubt|verboten) "verboten">
			 * <!ATTLIST objektReferenz referenzierungsart (assoziation|aggregation|komposition) "assoziation">
			 */
			else if(attDefinition.getObjektReferenz()!=null){
				String typ=attDefinition.getObjektReferenz().getTyp();
				if(typ==null){
					typ="";
				}
				arrAttributTyp=new String[][]{{"Typ", AttributDefinitionBeschreibung.OBJEKT_REFERENZ},
						{"Referenz auf", typ},
						{"Undefiniert", attDefinition.getObjektReferenz().getUndefiniert()},
						{"Referenzierungsart", attDefinition.getObjektReferenz().getReferenzierungsart()}};
			}
			/*
			 * <!ELEMENT kommazahl  EMPTY >
			 * <!ATTLIST kommazahl einheit CDATA #IMPLIED >
			 * <!ATTLIST kommazahl genauigkeit (einfach|doppelt) "doppelt" >
			 */
			else if(attDefinition.getKommazahl()!=null){
				String einheit=attDefinition.getKommazahl().getEinheit();
				if(einheit==null){
					einheit="";
				}
				arrAttributTyp=new String[][]{{"Typ", AttributDefinitionBeschreibung.KOMMA_ZAHL},
						{"Einheit", einheit},
						{"Genauigkeit", attDefinition.getKommazahl().getGenauigkeit()}};
			}
			// defaultWert:
			String defaultWert=null;
			if(attDefinition.getDefault()!=null){
				defaultWert=attDefinition.getDefault().getWert();
			}

			AttributDefinitionBeschreibung adb= new AttributDefinitionBeschreibung(
					_alleDefinitionen.erstelleUebersicht(attDefinition), arrAttributTyp,
					listeBereichB, listeZastandB, defaultWert);

			_mapAtt.put(attDefinition, adb);

			return adb;
		}
	}

	/**
	 * Aus der Instanz der Klasse Bereich erstelle die BereichBeschreibung.
	 *
	 * <!ELEMENT bereich ( info? ) >
	 * <!ATTLIST bereich skalierung CDATA #IMPLIED >
	 * <!ATTLIST bereich minimum CDATA #REQUIRED >
	 * <!ATTLIST bereich maximum CDATA #IMPLIED >
	 * <!ATTLIST bereich einheit CDATA #IMPLIED >
 	 *
	 * @param bereich liefert die Parameter fr die BereichBeschreibung.
	 *
	 * @return die erzeugte BereichBeschreibung.
	 */
	private BereichBeschreibung erstelleBereichBeschreibung(Bereich bereich){
		String skalierung="";
		if(bereich.getSkalierung()!=null){
			skalierung=bereich.getSkalierung();
		}

		String maximum="";
		if(bereich.getMaximum()!=null){
			maximum=bereich.getMaximum();
		}

		String einheit="";
		if(bereich.getEinheit()!=null){
			einheit=bereich.getEinheit();
		}

		return new BereichBeschreibung(_alleDefinitionen.erstelleInfoBeschreibung(bereich.getInfo()),
				skalierung, bereich.getMinimum(), maximum, einheit);
	}

	/**
	 * Aus der Instanz der Klasse Zustand erstelle die ZustandBeschreibung.
	 *
	 * <!ELEMENT zustand ( info? ) >
	 * <!ATTLIST zustand name CDATA #REQUIRED >
	 * <!ATTLIST zustand wert CDATA #REQUIRED >
	 *
	 * @param zustand liefert die Parameter fr die ZustandBeschreibung.
	 *
	 * @return erzeugte ZustandBeschreibung.
	 */
	private ZustandBeschreibung erstelleZustandBeschreibung(Zustand zustand){
		return new ZustandBeschreibung(_alleDefinitionen.erstelleInfoBeschreibung(zustand.getInfo()),
				zustand.getName(), zustand.getWert());
	}

	/**
	 * erstelle die Liste von AttributauswahlDefinitionBeschreibung aus der Collection von AttributauswahlDefinition.
	 *
	 * @param collAttributauswahlDefinition Collection von AttributauswahlDefinition, aus deren
	 * die Liste von AttributauswahlDefinitionBeschreibung erzeugt wird.
	 *
	 * @return erzeugte Liste von AttributauswahlDefinitionBeschreibung.
	 */
	private List<AttributauswahlDefinitionBeschreibung> erstelleAttributauswahlDefinitionBeschreibungListe(Collection<AttributauswahlDefinition> collAttributauswahlDefinition){
		List <AttributauswahlDefinitionBeschreibung> listeAttributauswahlDefinitionBescreibung=new ArrayList<AttributauswahlDefinitionBeschreibung>();
		for(AttributauswahlDefinition aad: collAttributauswahlDefinition){
			listeAttributauswahlDefinitionBescreibung.add(erstelleAttributauswahlDefinitionBeschreibung(aad));
		}
		return listeAttributauswahlDefinitionBescreibung;
	}

	/**
	 * erstelle die Instanz der Klasse AttributauswahlDefinitionBeschreibung aus AttributauswahlDefinition.
	 *
	 * <!ELEMENT attributauswahlDefinition ( info, (auswahl)+ ) >
	 * <!ATTLIST attributauswahlDefinition
	 * 		pid ID #REQUIRED
	 * 		name CDATA #IMPLIED
	 * 		typ (wert|id) #REQUIRED
	 * >
	 *
	 * @param attAusDef liefert die Parameter fr die AttributauswahlDefinitionBeschreibung.
	 *
	 * @return erzeugte AttributauswahlDefinitionBeschreibung.
	 */
	private AttributauswahlDefinitionBeschreibung erstelleAttributauswahlDefinitionBeschreibung(AttributauswahlDefinition attAusDef){
		List<AuswahlBeschreibung> listeAuswahlBeschreibung=new ArrayList<AuswahlBeschreibung>();
		for(Auswahl auswahl: attAusDef.getAuswahl()){
			listeAuswahlBeschreibung.add(erstelleAuswahlBeschreibung(auswahl));
		}
		AttributauswahlDefinitionBeschreibung attributauswahlDefinitionBescreibung=
			new AttributauswahlDefinitionBeschreibung(
				_alleDefinitionen.erstelleUebersicht(attAusDef), listeAuswahlBeschreibung, attAusDef.getTyp());

		return attributauswahlDefinitionBescreibung;
	}

	/**
	 * erstelle eine Instanz der Klasse AuswahlBeschreibung aus Auswahl.
	 *
	 * <!ELEMENT auswahl ( info? ) >
	 * <!ATTLIST auswahl
	 * 		pid NMTOKEN #IMPLIED
	 * 		wert CDATA #IMPLIED
	 * 		art (konfigurierendeAttributgruppe|onlineAttributgruppe|beliebigeAttributgruppe|beliebigesAttribut|beliebigeAttributliste) #IMPLIED
	 * 		anzahl NMTOKEN #IMPLIED
	 * 		anzahlIst (variabel|fest) "fest"
	 * 		name CDATA #IMPLIED
	 * >
	 *
	 * @param auswahl liefert die Parameter fr die AuswahlBeschreibung.
	 *
	 * @return erzeugte AuswahlBeschreibung.
	 */
	private AuswahlBeschreibung erstelleAuswahlBeschreibung(Auswahl auswahl){
		String wert="";
		String art="";
		String anzahl="";
		String anzahlIst="";
		if (auswahl.getWert()!=null){
			wert=auswahl.getWert();
		}
		if (auswahl.getArt()!=null){
			art=auswahl.getArt();
		}
		if (auswahl.getAnzahl()!=null){
			anzahl=auswahl.getAnzahl();
		}
		if (auswahl.getAnzahlIst()!=null){
			anzahlIst=auswahl.getAnzahlIst();
		}
		AuswahlBeschreibung auswahlBeschreibung=new AuswahlBeschreibung(_alleDefinitionen.erstelleUebersicht(auswahl),
				anzahl, anzahlIst, art, wert);

		return auswahlBeschreibung;
	}

	/**
	 * erstelle die Liste von AspektDefinitionBeschreibung aus der Collection von AspektDefinition.
	 *
	 * @param collAspektDefinition Collection von AspektDefinition aus deren
	 * die Liste von AspektDefinitionBeschreibung erzegt wird.
	 *
	 * @return erzeugte Liste von AspektDefinitionBeschreibung.
	 */
	private List<AspektDefinitionBeschreibung> erstelleAspektDefinitionBeschreibungListe(Collection<AspektDefinition> collAspektDefinition){
		List<AspektDefinitionBeschreibung> listeAspektDefBeschreibung=new ArrayList<AspektDefinitionBeschreibung>();
		for(AspektDefinition ad: collAspektDefinition){
			listeAspektDefBeschreibung.add(erstelleAspektDefinitionBeschreibung(ad));
		}
		return listeAspektDefBeschreibung;
	}

	/**
	 * Suche eine Instanz der Klasse AspektDefinitionBeschreibung in Map _mapAspekt.
	 * Falls finde, gebe die zurck.
	 * Sonst: erstelle AspektDefinitionBeschreibung aus der Instanz der Klasse AspektDefinition.
	 *
	 * <!ELEMENT aspektDefinition (info)>
	 * <!ATTLIST aspektDefinition
	 * 		pid ID #REQUIRED
	 * 		name CDATA #IMPLIED
	 * 		code CDATA #IMPLIED
	 * >
	 *
	 * @param aspektDefinition liefert die Parameter fr die AspektDefinitionBeschreibung.
	 *
	 * @return erzeugte AspektDefinitionBeschreibung.
	 */
	private AspektDefinitionBeschreibung  erstelleAspektDefinitionBeschreibung(AspektDefinition aspektDefinition){
		if(_mapAspekt.containsKey(aspektDefinition)){
			return _mapAspekt.get(aspektDefinition);
		}
		else{
			String code="";
			if(aspektDefinition.getCode()!=null){
				code=aspektDefinition.getCode();
			}
			AspektDefinitionBeschreibung aspektDefBeschreibung=
				new AspektDefinitionBeschreibung(_alleDefinitionen.erstelleUebersicht(aspektDefinition), code);

			_mapAspekt.put(aspektDefinition, aspektDefBeschreibung);
			return aspektDefBeschreibung;
		}
	}

// ------------------------------------------------------------------------
// GETTER / SETTER METHODEN
// ------------------------------------------------------------------------

	//***************************************************************************************

	/**
	 * Getter fr Eigenschaft '_kbB'.
	 *
	 * @return Wert der Eigenschaft '_kbB'.
	 *
	 * @see (#link _kbB)
	 */
	public KonfigurationsBereichBeschreibung getKbB() {
		return _kbB;
	}

	/**
	 * Getter fr Eigenschaft '_listeAspektDefinitionB'.
	 *
	 * @return Wert der Eigenschaft '_listeAspektDefinitionB'.
	 *
	 * @see (#link _listeAspektDefinitionB)
	 */
	public List<AspektDefinitionBeschreibung> getListeAspektDefinitionB() {
		return _listeAspektDefinitionB;
	}

	/**
	 * Getter fr Eigenschaft '_listeAttributDefinitionB'.
	 *
	 * @return Wert der Eigenschaft '_listeAttributDefinitionB'.
	 *
	 * @see (#link _listeAttributDefinitionB)
	 */
	public List<AttributDefinitionBeschreibung> getListeAttributDefinitionB() {
		return _listeAttributDefinitionB;
	}

	/**
	 * Getter fr Eigenschaft '_listeAttributauswahlDefinitionB'.
	 *
	 * @return Wert der Eigenschaft '_listeAttributauswahlDefinitionB'.
	 *
	 * @see (#link _listeAttributauswahlDefinitionB)
	 */
	public List<AttributauswahlDefinitionBeschreibung> getListeAttributauswahlDefinitionB() {
		return _listeAttributauswahlDefinitionB;
	}

	/**
	 * Getter fr Eigenschaft '_listeAttributgruppenDefinitionB'.
	 *
	 * @return Wert der Eigenschaft '_listeAttributgruppenDefinitionB'.
	 *
	 * @see (#link _listeAttributgruppenDefinitionB)
	 */
	public List<AttributgruppenDefinitionBeschreibung> getListeAttributgruppenDefinitionB() {
		return _listeAttributgruppenDefinitionB;
	}

	/**
	 * Getter fr Eigenschaft '_listeAttributlistenDefinitionB'.
	 *
	 * @return Wert der Eigenschaft '_listeAttributlistenDefinitionB'.
	 *
	 * @see (#link _listeAttributlistenDefinitionB)
	 */
	public List<AttributlistenDefinitionBeschreibung> getListeAttributlistenDefinitionB() {
		return _listeAttributlistenDefinitionB;
	}

	/**
	 * Getter fr Eigenschaft '_listeKonfigurationsObjektB'.
	 *
	 * @return Wert der Eigenschaft '_listeKonfigurationsObjektB'.
	 *
	 * @see (#link _listeKonfigurationsObjektB)
	 */
	public List<KonfigurationsObjektBeschreibung> getListeKonfigurationsObjektB() {
		return _listeKonfigurationsObjektB;
	}

	/**
	 * Getter fr Eigenschaft '_listeMengeDefinitionB'.
	 *
	 * @return Wert der Eigenschaft '_listeMengeDefinitionB'.
	 *
	 * @see (#link _listeMengeDefinitionB)
	 */
	public List<MengenDefinitionBeschreibung> getListeMengeDefinitionB() {
		return _listeMengeDefinitionB;
	}

	/**
	 * Getter fr Eigenschaft '_listeTypDefinitionB'.
	 *
	 * @return Wert der Eigenschaft '_listeTypDefinitionB'.
	 *
	 * @see (#link _listeTypDefinitionB)
	 */
	public List<TypDefinitionBeschreibung> getListeTypDefinitionB() {
		return _listeTypDefinitionB;
	}

	/**
	 * Getter fr Eigenschaft '_listeUebersichtAspektDefinition'.
	 *
	 * @return Wert der Eigenschaft '_listeUebersichtAspektDefinition'.
	 *
	 * @see (#link _listeUebersichtAspektDefinition)
	 */
	public List<Uebersicht> getListeUebersichtAspektDefinition() {
		return _listeUebersichtAspektDefinition;
	}

	/**
	 * Getter fr Eigenschaft '_listeUebersichtAspektDefinition'.
	 *
	 * @return Wert der Eigenschaft '_listeUebersichtAspektDefinition'.
	 *
	 * @see (#link _listeUebersichtAspektDefinition)
	 */
	public List<Uebersicht> getListeUebersichtAspektDefinitionSortiert() {
		return sortiereUebersichtListe(_listeUebersichtAspektDefinition);
	}

	/**
	 * Getter fr Eigenschaft '_listeUebersichtAttributDefinition'.
	 *
	 * @return Wert der Eigenschaft '_listeUebersichtAttributDefinition'.
	 *
	 * @see (#link _listeUebersichtAttributDefinition)
	 */
	public List<Uebersicht> getListeUebersichtAttributDefinition() {
		return _listeUebersichtAttributDefinition;
	}

	/**
	 * Getter fr Eigenschaft '_listeUebersichtAttributDefinition'.
	 *
	 * @return Wert der Eigenschaft '_listeUebersichtAttributDefinition'.
	 *
	 * @see (#link _listeUebersichtAttributDefinition)
	 */
	public List<Uebersicht> getListeUebersichtAttributDefinitionSortiert() {
		return sortiereUebersichtListe(_listeUebersichtAttributDefinition);
	}

	/**
	 * Getter fr Eigenschaft '_listeUebersichtAttributauswahlDefinition'.
	 *
	 * @return Wert der Eigenschaft '_listeUebersichtAttributauswahlDefinition'.
	 *
	 * @see (#link _listeUebersichtAttributauswahlDefinition)
	 */
	public List<Uebersicht> getListeUebersichtAttributauswahlDefinition() {
		return _listeUebersichtAttributauswahlDefinition;
	}

	/**
	 * Getter fr Eigenschaft '_listeUebersichtAttributauswahlDefinition'.
	 *
	 * @return Wert der Eigenschaft '_listeUebersichtAttributauswahlDefinition'.
	 *
	 * @see (#link _listeUebersichtAttributauswahlDefinition)
	 */
	public List<Uebersicht> getListeUebersichtAttributauswahlDefinitionSortiert() {
		return sortiereUebersichtListe(_listeUebersichtAttributauswahlDefinition);
	}

	/**
	 * Getter fr Eigenschaft '_listeUebersichtAttributgruppenDefinition'.
	 *
	 * @return Wert der Eigenschaft '_listeUebersichtAttributgruppenDefinition'.
	 *
	 * @see (#link _listeUebersichtAttributgruppenDefinition)
	 */
	public List<Uebersicht> getListeUebersichtAttributgruppenDefinition() {
		return _listeUebersichtAttributgruppenDefinition;
	}

	/**
	 * Getter fr Eigenschaft '_listeUebersichtAttributgruppenDefinition'.
	 *
	 * @return Wert der Eigenschaft '_listeUebersichtAttributgruppenDefinition'.
	 *
	 * @see (#link _listeUebersichtAttributgruppenDefinition)
	 */
	public List<Uebersicht> getListeUebersichtAttributgruppenDefinitionSortiert() {
		return sortiereUebersichtListe(_listeUebersichtAttributgruppenDefinition);
	}

	/**
	 * Getter fr Eigenschaft '_listeUebersichtAttributlistenDefinition'.
	 *
	 * @return Wert der Eigenschaft '_listeUebersichtAttributlistenDefinition'.
	 *
	 * @see (#link _listeUebersichtAttributlistenDefinition)
	 */
	public List<Uebersicht> getListeUebersichtAttributlistenDefinition() {
		return _listeUebersichtAttributlistenDefinition;
	}

	/**
	 * Getter fr Eigenschaft '_listeUebersichtAttributlistenDefinition'.
	 *
	 * @return Wert der Eigenschaft '_listeUebersichtAttributlistenDefinition'.
	 *
	 * @see (#link _listeUebersichtAttributlistenDefinition)
	 */
	public List<Uebersicht> getListeUebersichtAttributlistenDefinitionSortiert() {
		return sortiereUebersichtListe(_listeUebersichtAttributlistenDefinition);
	}

	/**
	 * Getter fr Eigenschaft '_listeUebersichtKonfigurationsObjektDefinition'.
	 *
	 * @return Wert der Eigenschaft '_listeUebersichtKonfigurationsObjektDefinition'.
	 *
	 * @see (#link _listeUebersichtKonfigurationsObjektDefinition)
	 */
	public List<Uebersicht> getListeUebersichtKonfigurationsObjektDefinition() {
		return _listeUebersichtKonfigurationsObjektDefinition;
	}

	/**
	 * Getter fr Eigenschaft '_listeUebersichtKonfigurationsObjektDefinition'.
	 *
	 * @return Wert der Eigenschaft '_listeUebersichtKonfigurationsObjektDefinition'.
	 *
	 * @see (#link _listeUebersichtKonfigurationsObjektDefinition)
	 */
	public List<Uebersicht> getListeUebersichtKonfigurationsObjektDefinitionSortiert() {
		return sortiereUebersichtListe(_listeUebersichtKonfigurationsObjektDefinition);
	}

	/**
	 * Getter fr Eigenschaft '_listeUebersichtMengenDefinition'.
	 *
	 * @return Wert der Eigenschaft '_listeUebersichtMengenDefinition'.
	 *
	 * @see (#link _listeUebersichtMengenDefinition)
	 */
	public List<Uebersicht> getListeUebersichtMengenDefinition() {
		return _listeUebersichtMengenDefinition;
	}

	/**
	 * Getter fr Eigenschaft '_listeUebersichtMengenDefinition'.
	 *
	 * @return Wert der Eigenschaft '_listeUebersichtMengenDefinition'.
	 *
	 * @see (#link _listeUebersichtMengenDefinition)
	 */
	public List<Uebersicht> getListeUebersichtMengenDefinitionSortiert() {
		return sortiereUebersichtListe(_listeUebersichtMengenDefinition);
	}

	/**
	 * Getter fr Eigenschaft '_listeUebersichtTypDefinitionbB'.
	 *
	 * @return Wert der Eigenschaft '_listeUebersichtTypDefinition'.
	 *
	 * @see (#link _listeUebersichtTypDefinition)
	 */
	public List<Uebersicht> getListeUebersichtTypDefinition() {
		return _listeUebersichtTypDefinition;
	}

	/**
	 * Getter fr Eigenschaft '_listeUebersichtTypDefinition'.
	 *
	 * @return Wert der Eigenschaft '_listeUebersichtTypDefinition'.
	 *
	 * @see (#link _listeUebersichtTypDefinition)
	 */
	public List<Uebersicht> getListeUebersichtTypDefinitionSortiert() {
		return sortiereUebersichtListe(_listeUebersichtTypDefinition);
	}

	/**
	 * Sortiere bersichtListe.
	 * @param listeUebersicht
	 * @return
	 */
	private List<Uebersicht> sortiereUebersichtListe(List<Uebersicht> listeUebersicht){
		Uebersicht[] arUebersicht=listeUebersicht.toArray(new Uebersicht[0]);
		Arrays.sort(arUebersicht, ermittleComparator());
		return Arrays.asList(arUebersicht);
	}

	//***************************************************************************************

	/**
	 * erstelle Comporator.
	 *
	 * Vergleichen knnen die Instanzen der Klassen, die Uebersicht als Instanzvariable haben.
	 * Sortiert wird als String nach den Felde (Uebersicht._name und Uebersicht._pid).
	 *
	 * @return Comporator mit Sortierungkriterium Uebersicht._name und Uebersicht._pid.
	 */
	private Comparator<Object> ermittleComparator(){
		Comparator<Object> comparator=new Comparator<Object>(){
			@Override
			public int compare(Object o1, Object o2) {
				Uebersicht uebersicht1=null;
				Uebersicht uebersicht2=null;

				if (o1 instanceof Uebersicht) {
					uebersicht1 = (Uebersicht) o1;
					uebersicht2 = (Uebersicht) o2;
				}
				else if (o1 instanceof MengenDefinitionBeschreibung) {
					uebersicht1=((MengenDefinitionBeschreibung) o1).getUebersicht();
					uebersicht2=((MengenDefinitionBeschreibung) o2).getUebersicht();
				}
				else if (o1 instanceof AttributDefinitionBeschreibung) {
					uebersicht1=((AttributDefinitionBeschreibung) o1).getUebersicht();
					uebersicht2=((AttributDefinitionBeschreibung) o2).getUebersicht();
				}
				else if (o1 instanceof AttributauswahlDefinitionBeschreibung) {
					uebersicht1=((AttributauswahlDefinitionBeschreibung) o1).getUebersicht();
					uebersicht2=((AttributauswahlDefinitionBeschreibung) o2).getUebersicht();
				}
				else if (o1 instanceof AspektDefinitionBeschreibung) {
					uebersicht1=((AspektDefinitionBeschreibung) o1).getUebersicht();
					uebersicht2=((AspektDefinitionBeschreibung) o2).getUebersicht();
				}
				else if (o1 instanceof AttributlistenDefinitionBeschreibung) {
					uebersicht1=((AttributlistenDefinitionBeschreibung) o1).getUebersicht();
					uebersicht2=((AttributlistenDefinitionBeschreibung) o2).getUebersicht();
				}
				else if (o1 instanceof AttributgruppenDefinitionBeschreibung) {
					uebersicht1=((AttributgruppenDefinitionBeschreibung) o1).getUebersicht();
					uebersicht2=((AttributgruppenDefinitionBeschreibung) o2).getUebersicht();
				}
				else if (o1 instanceof TypDefinitionBeschreibung) {
					uebersicht1=((TypDefinitionBeschreibung) o1).getUebersicht();
					uebersicht2=((TypDefinitionBeschreibung) o2).getUebersicht();
				}
				else if (o1 instanceof KonfigurationsObjektBeschreibung) {
					uebersicht1=((KonfigurationsObjektBeschreibung) o1).getUebersicht();
					uebersicht2=((KonfigurationsObjektBeschreibung) o2).getUebersicht();
				}
				else {
					Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID,
							"unbekannte Klasse fr _comparator "+o1.getClass()));
				}

				if((uebersicht1!=null)&&(uebersicht2!=null)){
					int sortName= uebersicht1.getName().compareTo(uebersicht2.getName());
					if(sortName!=0){
						return sortName;
					}
					else{
						return uebersicht1.getPid().compareTo(uebersicht2.getPid());
					}
				}
				else{
					return 0;
				}
			}
		};
		return comparator;
	}

// ------------------------------------------------------------------------
// SONSTIGE METHODEN
// ------------------------------------------------------------------------

	/**
	 * Ermittelt die sortierte Liste der AspektDefinitionBeschreibungen.
	 *
	 * @return Sortierte Liste der AspektDefinitionBeschreibungen.
	 */
	public List<AspektDefinitionBeschreibung> getListeAspektDefinitionBSortiert(){
		AspektDefinitionBeschreibung[] arAspektDefBeschreibung=_listeAspektDefinitionB.toArray(new AspektDefinitionBeschreibung[0]);
		Arrays.sort(arAspektDefBeschreibung, ermittleComparator());
		return Arrays.asList(arAspektDefBeschreibung);
	}

	/**
	 * Ermittelt die sortierte Liste der MengenDefinitionBeschreibungen.
	 *
	 * @return Sortierte Liste der MengenDefinitionBeschreibungen.
	 */
	public List<AttributDefinitionBeschreibung> getListeAttributDefinitionBSortiert(){
		AttributDefinitionBeschreibung[] arAttributDefBeschreibung=_listeAttributDefinitionB.toArray(new AttributDefinitionBeschreibung[0]);
		Arrays.sort(arAttributDefBeschreibung, ermittleComparator());
		return Arrays.asList(arAttributDefBeschreibung);
	}

	/**
	 * Ermittelt die sortierte Liste der AttributauswahlDefinitionBeschreibungen.
	 *
	 * @return Sortierte Liste der AttributauswahlDefinitionBeschreibungen.
	 */
	public List<AttributauswahlDefinitionBeschreibung> getListeAttributauswahlDefinitionBSortiert(){
		AttributauswahlDefinitionBeschreibung[] arAttAuswahlDefBeschreibung=_listeAttributauswahlDefinitionB.toArray(new AttributauswahlDefinitionBeschreibung[0]);
		Arrays.sort(arAttAuswahlDefBeschreibung, ermittleComparator());
		return Arrays.asList(arAttAuswahlDefBeschreibung);
	}

	/**
	 * Ermittelt die sortierte Liste der AttributgruppenDefinitionBeschreibungen.
	 *
	 * @return Sortierte Liste der AttributgruppenDefinitionBeschreibungen.
	 */
	public List<AttributgruppenDefinitionBeschreibung> getListeAttributgruppenDefinitionBSortiert(){
		AttributgruppenDefinitionBeschreibung[] arAtgDefBeschreibung=_listeAttributgruppenDefinitionB.toArray(new AttributgruppenDefinitionBeschreibung[0]);
		Arrays.sort(arAtgDefBeschreibung, ermittleComparator());
		return Arrays.asList(arAtgDefBeschreibung);
	}

	/**
	 * Ermittelt die sortierte Liste der AttributlistenDefinitionBeschreibungen.
	 *
	 * @return Sortierte Liste der AttributlistenDefinitionBeschreibungen.
	 */
	public List<AttributlistenDefinitionBeschreibung> getListeAttributlistenDefinitionBSortiert(){
		AttributlistenDefinitionBeschreibung[] arAtlDefBeschreibung=_listeAttributlistenDefinitionB.toArray(new AttributlistenDefinitionBeschreibung[0]);
		Arrays.sort(arAtlDefBeschreibung, ermittleComparator());
		return Arrays.asList(arAtlDefBeschreibung);
	}

	/**
	 * Ermittelt die sortierte Liste der KonfigurationsObjektBeschreibungen.
	 *
	 * @return Sortierte Liste der KonfigurationsObjektBeschreibungen.
	 */
	public List<KonfigurationsObjektBeschreibung> getListeKonfigurationsObjektBSortiert(){
		KonfigurationsObjektBeschreibung[] arKOBeschreibung=_listeKonfigurationsObjektB.toArray(new KonfigurationsObjektBeschreibung[0]);
		Arrays.sort(arKOBeschreibung, ermittleComparator());
		return Arrays.asList(arKOBeschreibung);
	}

	/**
	 * Ermittelt die sortierte Liste der MengenDefinitionBeschreibungen.
	 *
	 * @return Sortierte Liste der MengenDefinitionBeschreibungen.
	 */
	public List<MengenDefinitionBeschreibung> getListeMengeDefinitionBSortiert(){
		MengenDefinitionBeschreibung[] arMengenDefBeschreibung=_listeMengeDefinitionB.toArray(new MengenDefinitionBeschreibung[0]);
		Arrays.sort(arMengenDefBeschreibung, ermittleComparator());
		return Arrays.asList(arMengenDefBeschreibung);
	}

	/**
	 * Ermittelt die sortierte Liste der MengenDefinitionBeschreibungen.
	 *
	 * @return Sortierte Liste der MengenDefinitionBeschreibungen.
	 */
	public List<TypDefinitionBeschreibung> getListeTypDefinitionBSortiert(){
		TypDefinitionBeschreibung[] arTypDefBeschreibung=_listeTypDefinitionB.toArray(new TypDefinitionBeschreibung[0]);
		Arrays.sort(arTypDefBeschreibung, ermittleComparator());
		return Arrays.asList(arTypDefBeschreibung);
	}
}
