/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.inovat.buv.inovat.lib.funktionen.datum;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.funktionen.datum.DatumFunktionen;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.*;
import org.junit.*;
import static org.junit.Assert.assertEquals;

import java.util.Calendar;
import java.util.GregorianCalendar;

//~ KLASSEN ===================================================================

/**
 * Klasse testet DatumFunktionen.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 12 $ / $Date: 2013-09-24 13:23:48 +0200 (Di, 24. Sep 2013) $ / ($Author: LG $)
 */
public class DatumFunktionenTest {

    /** Aktuelles Datum */
    DateTime dt1;

    /** Start-Datum */
    DateTime dtStart;
    Shell    shell;

    //~ SET METHODEN ==========================================================

    @Before        
    public void setUp() throws Exception {
        Display display = Display.getDefault();

        shell   = new Shell(display);
        dt1     = new DateTime(shell, SWT.NULL);
        dtStart = new DateTime(shell, SWT.NULL);
        dtStart.setYear(1970);
        dtStart.setMonth(0);
        dtStart.setDay(1);
        dtStart.setHours(1);
        dtStart.setMinutes(0);
        dtStart.setSeconds(0);
    }

    @BeforeClass
    public static void setUpBeforeClass() throws Exception {}

    //~ METHODEN ==============================================================

    @After
    public void tearDown() throws Exception {}

    @AfterClass
    public static void tearDownAfterClass() throws Exception {}

    @Test
    public final void test() {

        // TODO test.
    }

    @Test
    public final void testGetAktuelleDateTime() {
        long     millis   = DatumFunktionen.getAktuelleDateTime();
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(millis);
        assertEquals("MilliSecunden: 0", 0, calender.get(GregorianCalendar.MILLISECOND));
        assertEquals("Aktuelles Jahr:", dt1.getYear(), calender.get(GregorianCalendar.YEAR));
        assertEquals("Aktueller Monat:", dt1.getMonth(), calender.get(GregorianCalendar.MONTH));
        assertEquals("Aktueller Tag:", dt1.getDay(), calender.get(GregorianCalendar.DAY_OF_MONTH));
        assertEquals("Aktuelle Zeit / Stunden:", dt1.getHours(), calender.get(GregorianCalendar.HOUR_OF_DAY));
        assertEquals("Aktuelle Zeit / Minuten:", dt1.getMinutes(), calender.get(GregorianCalendar.MINUTE));
        assertEquals("Aktuelle Zeit / Secunden:", dt1.getSeconds(), calender.get(GregorianCalendar.SECOND));
    }

    @Test
    public final void testGetAnzahlDerBetroffenenMonate() {
        Calendar calender1 = new GregorianCalendar(2008, 1, 28, 12, 45, 40);
        Calendar calender2 = new GregorianCalendar(2009, 2, 18, 2, 40, 40);

        assertEquals("Es sind 14 Monate im Interval", 14, DatumFunktionen.getAnzahlDerBetroffenenMonate(calender1.getTimeInMillis(), calender2.getTimeInMillis()));
        assertEquals("Es sind -14 Monate im Interval", -14, DatumFunktionen.getAnzahlDerBetroffenenMonate(calender2.getTimeInMillis(), calender1.getTimeInMillis()));

        //
        calender1 = new GregorianCalendar(2009, 1, 28, 12, 45, 40);
        calender2 = new GregorianCalendar(2009, 2, 31, 2, 40, 40);
        assertEquals("Es sind 2 Monate im Interval", 2, DatumFunktionen.getAnzahlDerBetroffenenMonate(calender1.getTimeInMillis(), calender2.getTimeInMillis()));
        assertEquals("Es sind -2 Monate im Interval", -2, DatumFunktionen.getAnzahlDerBetroffenenMonate(calender2.getTimeInMillis(), calender1.getTimeInMillis()));

        //
        calender1 = new GregorianCalendar(2009, 1, 28, 12, 45, 40);
        calender2 = new GregorianCalendar(2009, 1, 28, 13, 40, 40);
        assertEquals("Es ist 1 Monat im Interval", 1, DatumFunktionen.getAnzahlDerBetroffenenMonate(calender1.getTimeInMillis(), calender2.getTimeInMillis()));
        assertEquals("Es ist -1 Monat im Interval", -1, DatumFunktionen.getAnzahlDerBetroffenenMonate(calender2.getTimeInMillis(), calender1.getTimeInMillis()));

        //
        calender1 = new GregorianCalendar(2009, 1, 28, 12, 45, 40);
        calender2 = new GregorianCalendar(2009, 1, 28, 12, 45, 40);
        assertEquals("Es ist 1 Monat im Interval", 1, DatumFunktionen.getAnzahlDerBetroffenenMonate(calender1.getTimeInMillis(), calender2.getTimeInMillis()));
        assertEquals("Es ist 1 Monat im Interval", 1, DatumFunktionen.getAnzahlDerBetroffenenMonate(calender2.getTimeInMillis(), calender1.getTimeInMillis()));
    }

    @Test
    public final void testGetDateTimeAsLongDateTime() {
        assertEquals("Start Datum ist 1970.01.01 01:00:00 (ist 0 Millisekunden)", 0, DatumFunktionen.getDateTimeAsLong(dtStart));
    }

    @Test
    public final void testGetDateTimeAsLongDateTimeDateTime() {

        /** Aktuelles Datum mit der Zeit 01:00:00 */
        DateTime dt0 = new DateTime(shell, SWT.NULL);

        dt0.setHours(1);
        dt0.setMinutes(0);
        dt0.setSeconds(0);

        long l = DatumFunktionen.getDateTimeAsLong(dt0);

        assertEquals("Aktuelles Datum mit Start-Zeit", l, DatumFunktionen.getDateTimeAsLong(dt1, dtStart));
    }

    @Test
    public final void testGetDatumAlsLong1() {
        assertEquals("01.01.2009", "01.01.2009  00:00:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.getDatumAlsLong("01.01.2009")));
        assertEquals("29.02.2008", "29.02.2008  00:00:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.getDatumAlsLong("29.02.2008")));
        assertEquals("28.02.2009", "28.02.2009  00:00:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.getDatumAlsLong("28.02.2009")));
        assertEquals("Flasches Datum: 29.02.2009", "01.01.1970  01:00:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.getDatumAlsLong("29.02.2009")));
        assertEquals("Flasches Datum: 01.02.09", "01.01.1970  01:00:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.getDatumAlsLong("01.02.09")));
        assertEquals("Flasches Datum: 01.13.09", "01.01.1970  01:00:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.getDatumAlsLong("01.13.09")));
        assertEquals("Flasches Datum: 1.12.09", "01.01.1970  01:00:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.getDatumAlsLong("1.12.09")));
    }

    @Test
    public final void testGetDatumAlsLong2() {
        assertEquals("01.01.2009", "01.01.2009  04:04:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.getDatumAlsLong("01.01.2009", "04:04")));
        assertEquals("29.02.2008", "29.02.2008  00:00:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.getDatumAlsLong("29.02.2008", "00:00")));
        assertEquals("28.02.2009", "28.02.2009  23:50:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.getDatumAlsLong("28.02.2009", "23:50")));
        assertEquals("Flasches Datum: 29.02.2009", "01.01.1970  01:00:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.getDatumAlsLong("29.02.2009", "00:00")));
        assertEquals("Flasches Datum: 01.02.09", "01.01.1970  01:00:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.getDatumAlsLong("01.02.09", "00:00")));
        assertEquals("Flasches Datum: 01.13.09", "01.01.1970  01:00:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.getDatumAlsLong("01.13.09", "00:00")));
        assertEquals("Flasches Datum: 1.12.09", "01.01.1970  01:00:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.getDatumAlsLong("1.12.09", "00:00")));
        assertEquals("Flasche Datum und Zeit: 1.12.09 25:12", "01.01.1970  01:00:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.getDatumAlsLong("1.12.09", "25:12")));
        assertEquals("Flasche Zeit: 01.12.09 25:12", "01.01.1970  01:00:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.getDatumAlsLong("01.12.09", "25:12")));
        assertEquals("Flasche Zeit: 01.12.09 20:60", "01.01.1970  01:00:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.getDatumAlsLong("01.12.09", "20:60")));
    }

    @Test
    public final void testGetDatumAlsString() {
        assertEquals("", "01.01.1970  01:00:00", DatumFunktionen.getDatumZeitAlsString(0));
    }

    @Test
    public final void testGetDatumMitInitZeit() {
        Calendar calender = new GregorianCalendar(2009, 1, 28, 12, 45, 40);
        long     datum    = DatumFunktionen.getDatumMitInitZeit(calender.getTimeInMillis() + 20);

        calender = new GregorianCalendar();
        calender.setTimeInMillis(datum);
        assertEquals("Jahr 2009", 2009, DatumFunktionen.getJahr(datum));
        assertEquals("Monat Februar", 1, calender.get(Calendar.MONTH));
        assertEquals("Tag 28", 28, calender.get(Calendar.DAY_OF_MONTH));
        assertEquals("Die Stunden werden auf 0 gesetzt", 0, calender.get(Calendar.HOUR));
        assertEquals("Die Minuten werden auf 0 gesetzt", 0, calender.get(Calendar.MINUTE));
        assertEquals("Die Sekunden werden auf 0 gesetzt", 0, calender.get(Calendar.SECOND));
        assertEquals("Die Millisekunden werden auf 0 gesetzt", 0, calender.get(Calendar.MILLISECOND));
    }

    @Test
    public final void testGetDatumMitInitZeitUndTag() {
        Calendar calender = new GregorianCalendar(2009, 11, 18, 12, 45, 40);
        long     datum    = DatumFunktionen.getMonatAnfang(calender.getTimeInMillis() + 30);

        calender = new GregorianCalendar();
        calender.setTimeInMillis(datum);
        assertEquals("Jahr 2009", 2009, DatumFunktionen.getJahr(datum));
        assertEquals("Monat Dezember", 11, calender.get(Calendar.MONTH));
        assertEquals("Der Tag wird auf 1 gesetzt", 1, calender.get(Calendar.DAY_OF_MONTH));
        assertEquals("Die Stunden werden auf 0 gesetzt", 0, calender.get(Calendar.HOUR));
        assertEquals("Die Minuten werden auf 0 gesetzt", 0, calender.get(Calendar.MINUTE));
        assertEquals("Die Sekunden werden auf 0 gesetzt", 0, calender.get(Calendar.SECOND));
        assertEquals("Die Millisekunden werden auf 0 gesetzt", 0, calender.get(Calendar.MILLISECOND));
    }

    @Test
    public final void testGetDatumZeitOhneSekunden() {
        Calendar calender = new GregorianCalendar(2009, 1, 28, 11, 45, 40);
        long     datum    = DatumFunktionen.getDatumZeitOhneSekunden(calender.getTimeInMillis() + 20);

        calender = new GregorianCalendar();
        calender.setTimeInMillis(datum);
        assertEquals("Jahr 2009", 2009, DatumFunktionen.getJahr(datum));
        assertEquals("Monat Februar", 1, calender.get(Calendar.MONTH));
        assertEquals("Tag 28", 28, calender.get(Calendar.DAY_OF_MONTH));
        assertEquals("11 Stuhden", 11, calender.get(Calendar.HOUR));
        assertEquals("45 Sekunden", 45, calender.get(Calendar.MINUTE));
        assertEquals("Die Sekunden werden auf 0 gesetzt", 0, calender.get(Calendar.SECOND));
        assertEquals("Die Millisekunden werden auf 0 gesetzt", 0, calender.get(Calendar.MILLISECOND));
    }

    @Test
    public final void testGetInitDateTime() {
        long     millis   = DatumFunktionen.getInitDateTime();
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(millis);
        assertEquals("Stunden: 0", 0, calender.get(GregorianCalendar.HOUR_OF_DAY));
        assertEquals("Minuten: 0", 0, calender.get(GregorianCalendar.MINUTE));
        assertEquals("Secunden: 0", 0, calender.get(GregorianCalendar.SECOND));
        assertEquals("MilliSecunden: 0", 0, calender.get(GregorianCalendar.MILLISECOND));
        assertEquals("Aktuelles Jahr:", dt1.getYear(), calender.get(GregorianCalendar.YEAR));
        assertEquals("Aktueller Monat:", dt1.getMonth(), calender.get(GregorianCalendar.MONTH));
        assertEquals("Aktueller Tag:", dt1.getDay(), calender.get(GregorianCalendar.DAY_OF_MONTH));
    }

    @Test
    public final void testGetMonat() {
        Calendar calender = new GregorianCalendar(2009, 2, 18, 12, 45, 40);

        assertEquals("Es ist Mrz", 3, DatumFunktionen.getMonat(calender.getTimeInMillis() + 30));
    }

    @Test
    public final void testGetNaechstenTag() {

        //
        long datum    = new GregorianCalendar(2008, 1, 28, 12, 45, 40).getTimeInMillis();
        long datumNeu = DatumFunktionen.getNaechstenTag(datum);

        assertEquals("Nchster Tag des Monats.", "29.02.2008  12:45:40", DatumFunktionen.getDatumZeitAlsString(datumNeu));

        //
        datum    = new GregorianCalendar(2008, 1, 29, 12, 45, 40).getTimeInMillis();
        datumNeu = DatumFunktionen.getNaechstenTag(datum);
        assertEquals("Ende des Monats.", "01.03.2008  12:45:40", DatumFunktionen.getDatumZeitAlsString(datumNeu));

        //
        datum    = new GregorianCalendar(2008, 11, 31, 12, 45, 40).getTimeInMillis();
        datumNeu = DatumFunktionen.getNaechstenTag(datum);
        assertEquals("Ende des Jahres.", "01.01.2009  12:45:40", DatumFunktionen.getDatumZeitAlsString(datumNeu));

        //
        datum    = new GregorianCalendar(2008, 9, 26, 3, 10, 10).getTimeInMillis();
        datumNeu = DatumFunktionen.getNaechstenTag(datum);
        assertEquals("Nchster Tag des Monats (Zeitumstellung?).", "27.10.2008  03:10:10", DatumFunktionen.getDatumZeitAlsString(datumNeu));

        //
        datum    = new GregorianCalendar(2008, 3, 27, 0, 10, 10).getTimeInMillis();
        datumNeu = DatumFunktionen.getNaechstenTag(datum);
        assertEquals("Nchster Tag des Monats (Zeitumstellung?).", "28.04.2008  00:10:10", DatumFunktionen.getDatumZeitAlsString(datumNeu));

        //
        // Sommerzeit: Sonntag, den 29. Mrz 2009 um 2:00 Uhr.
        // Die Uhren wurden von 02:00 Uhr auf 03:00 Uhr gestellt. Wir verlieren in dieser Nacht eine Stunde.
        datum    = new GregorianCalendar(2009, 2, 28, 2, 10, 10).getTimeInMillis();
        datumNeu = DatumFunktionen.getNaechstenTag(datum);
        assertEquals("Nchster Tag des Monats (Zeitumstellung?).", "29.03.2009  03:10:10", DatumFunktionen.getDatumZeitAlsString(datumNeu));

        //
        // Winterzeit: Sonntag den 25.10.2009 um 3:00 Uhr.
        // Die Uhr wurde um 1 Stunde zurckgestellt, die Nacht ist also 1 Stunde lnger.
        datum    = new GregorianCalendar(2009, 9, 25, 1, 10, 10).getTimeInMillis();
        datumNeu = DatumFunktionen.getNaechstenTag(datum);
        assertEquals("Nchster Tag des Monats (Zeitumstellung?).", "26.10.2009  01:10:10", DatumFunktionen.getDatumZeitAlsString(datumNeu));

        // Winterzeit:
        datum    = new GregorianCalendar(2009, 9, 25, 2, 10, 10).getTimeInMillis();
        datumNeu = DatumFunktionen.getNaechstenTag(datum);
        assertEquals("Nchster Tag des Monats (Zeitumstellung?).", "26.10.2009  02:10:10", DatumFunktionen.getDatumZeitAlsString(datumNeu));

        // Winterzeit:
        datum    = new GregorianCalendar(2009, 9, 25, 3, 10, 10).getTimeInMillis();
        datumNeu = DatumFunktionen.getNaechstenTag(datum);
        assertEquals("Nchster Tag des Monats (Zeitumstellung?).", "26.10.2009  03:10:10", DatumFunktionen.getDatumZeitAlsString(datumNeu));

        // Winterzeit:
        datum    = new GregorianCalendar(2009, 9, 25, 4, 10, 10).getTimeInMillis();
        datumNeu = DatumFunktionen.getNaechstenTag(datum);
        assertEquals("Nchster Tag des Monats (Zeitumstellung?).", "26.10.2009  04:10:10", DatumFunktionen.getDatumZeitAlsString(datumNeu));
    }

    @Test
    public final void testGetTag() {
        Calendar calender = new GregorianCalendar(2009, 2, 18, 12, 45, 40);

        assertEquals("Es ist 18.Tag", 18, DatumFunktionen.getTag(calender.getTimeInMillis() + 30));
    }

    @Test
    public final void testGetVerschobenenDateTime() {
        Calendar calender = new GregorianCalendar(2008, 1, 28, 12, 45, 40);
        long     datum    = calender.getTimeInMillis();
        long     datumNeu;

        datumNeu = DatumFunktionen.getVerschobenenDateTime(datum, 24, 0);
        assertEquals("Fge 24 Monate hinzu:", "28.02.2010  12:45:40", DatumFunktionen.getDatumZeitAlsString(datumNeu));
        datumNeu = DatumFunktionen.getVerschobenenDateTime(datum, 24, 3);
        assertEquals("Fge 24 Monate und 3 Tage hinzu:", "03.03.2010  12:45:40", DatumFunktionen.getDatumZeitAlsString(datumNeu));
        datumNeu = DatumFunktionen.getVerschobenenDateTime(datum, 0, 30);
        assertEquals("Fge 30 Tage hinzu:", "29.03.2008  12:45:40", DatumFunktionen.getDatumZeitAlsString(datumNeu));
        calender = new GregorianCalendar(2008, 1, 29, 12, 45, 40);
        datum    = calender.getTimeInMillis();
        datumNeu = DatumFunktionen.getVerschobenenDateTime(datum, 24, 0);
        assertEquals("Fge 24 Monate hinzu:", "28.02.2010  12:45:40", DatumFunktionen.getDatumZeitAlsString(datumNeu));
        datumNeu = DatumFunktionen.getVerschobenenDateTime(datum, 24, 3);
        assertEquals("Fge 24 Monate und 3 Tage hinzu:", "03.03.2010  12:45:40", DatumFunktionen.getDatumZeitAlsString(datumNeu));
        datumNeu = DatumFunktionen.getVerschobenenDateTime(datum, 0, 30);
        assertEquals("Fge 30 Tage hinzu:", "30.03.2008  12:45:40", DatumFunktionen.getDatumZeitAlsString(datumNeu));
    }

    @Test
    public final void testGetZeitdauerAlsString() {
        assertEquals("Das Jahr wird auf 2009 gesetztz", "249:56:09", DatumFunktionen.getZeitdauerAlsString(899768900));
        assertEquals("Das Jahr wird auf 2009 gesetztz", "00:00:09", DatumFunktionen.getZeitdauerAlsString(8900));
        assertEquals("Das Jahr wird auf 2009 gesetztz", "00:00:49", DatumFunktionen.getZeitdauerAlsString(48900));
        assertEquals("Das Jahr wird auf 2009 gesetztz", "00:02:29", DatumFunktionen.getZeitdauerAlsString(148900));
    }

    @Test
    public final void testKopiereZeit() {
        Calendar calender1 = new GregorianCalendar(2008, 1, 28, 12, 45, 40);
        Calendar calender2 = new GregorianCalendar(2009, 2, 18, 2, 40, 50);
        long     datum     = DatumFunktionen.kopiereZeit(calender1.getTimeInMillis(), calender2.getTimeInMillis());

        assertEquals("Die Zeit des ersten Datums wird gendert", "28.02.2008  02:40:50", DatumFunktionen.getDatumZeitAlsString(datum));
    }

    @Test
    public final void testSetzeDateTime() {

        /** Aktuelles Datum */
        DateTime dt2 = new DateTime(shell, SWT.NULL);

        // Setzte Datum auf 01.01.1970 01:00:00
        DatumFunktionen.setzeDateTime(dt2, 0);
        assertEquals("Das Jahr wird auf 1970 gesetztz", 1970, dt2.getYear());
        assertEquals("Der Monat wird auf 0 gesetztz", 0, dt2.getMonth());
        assertEquals("Der Tag wird auf 1 gesetztz", 1, dt2.getDay());
        assertEquals("Die Stunden werden auf 1 gesetztz", 1, dt2.getHours());
        assertEquals("Die Minuten werden auf 0 gesetztz", 0, dt2.getMinutes());
        assertEquals("Die Sekunden werden auf 0 gesetztz", 0, dt2.getSeconds());

        // Setzte Datum auf 30.01.2009 00:00:00
        DatumFunktionen.setzeDateTime(dt2, 1233270000000l);
        assertEquals("Das Jahr wird auf 2009 gesetztz", 2009, dt2.getYear());
        assertEquals("Der Monat wird auf 0 gesetztz", 0, dt2.getMonth());
        assertEquals("Der Tag wird auf 30 gesetztz", 30, dt2.getDay());
        assertEquals("Die Stunden werden auf 0 gesetztz", 0, dt2.getHours());
        assertEquals("Die Minuten werden auf 0 gesetztz", 0, dt2.getMinutes());
        assertEquals("Die Sekunden werden auf 0 gesetztz", 0, dt2.getSeconds());

        // Setzte Datum auf 02.02.2009 08:30:00
        DatumFunktionen.setzeDateTime(dt2, 1233559800000l);
        assertEquals("Das Jahr wird auf 2009 gesetztz", 2009, dt2.getYear());
        assertEquals("Der Monat wird auf 1 gesetztz", 1, dt2.getMonth());
        assertEquals("Der Tag wird auf 2 gesetztz", 2, dt2.getDay());
        assertEquals("Die Stunden werden auf 8 gesetztz", 8, dt2.getHours());
        assertEquals("Die Minuten werden auf 30 gesetztz", 30, dt2.getMinutes());
        assertEquals("Die Sekunden werden auf 0 gesetztz", 0, dt2.getSeconds());

        // Setzte Datum auf 30.01.2009 00:00:00
        DatumFunktionen.setzeDateTime(dt2, 1233270000000l);
        assertEquals("Das Jahr wird auf 2009 gesetztz", 2009, dt2.getYear());
        assertEquals("Der Monat wird auf 0 gesetztz", 0, dt2.getMonth());
        assertEquals("Der Tag wird auf 30 gesetztz", 30, dt2.getDay());
        assertEquals("Die Stunden werden auf 0 gesetztz", 0, dt2.getHours());
        assertEquals("Die Minuten werden auf 0 gesetztz", 0, dt2.getMinutes());
        assertEquals("Die Sekunden werden auf 0 gesetztz", 0, dt2.getSeconds());
    }

    @Test
    public final void testSetzeDateTimeVonBis() {}

    @Test
    public final void testSetzeZeit() {
        long heute = DatumFunktionen.getAktuelleDateTime();

        assertEquals("Neue Zeit 12:50:00", DatumFunktionen.getDatumAlsString(heute) + "  12:50:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.setzeZeit(heute, 12, 50)));
        assertEquals("Neue Zeit 23:03:00", DatumFunktionen.getDatumAlsString(heute) + "  23:03:00", DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.setzeZeit(heute, 23, 3)));
        assertEquals("Falsche Zeit 03:60:00", DatumFunktionen.getDatumZeitAlsString(heute), DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.setzeZeit(heute, 3, 60)));
        assertEquals("Falsche Zeit 25:03:00", DatumFunktionen.getDatumZeitAlsString(heute), DatumFunktionen.getDatumZeitAlsString(DatumFunktionen.setzeZeit(heute, 25, 3)));
    }

    @Test
    public final void testSpeichereDateTimeVonBis() {}

    @Test
    public final void testVergleicheDieUhrzeit() {

        //
        long datum1 = new GregorianCalendar(2009, 10, 1, 0, 0, 0).getTimeInMillis();
        long datum2 = new GregorianCalendar(2009, 1, 3, 0, 0, 0).getTimeInMillis();

        assertEquals("", true, DatumFunktionen.vergleicheDieUhrzeit(datum1, datum2) == 0);

        //
        datum1 = new GregorianCalendar(2009, 1, 5, 0, 0, 1).getTimeInMillis();
        datum2 = new GregorianCalendar(2000, 1, 31, 0, 2, 0).getTimeInMillis();
        assertEquals("", true, DatumFunktionen.vergleicheDieUhrzeit(datum1, datum2) < 0);

        //
        datum1 = new GregorianCalendar(2009, 10, 1, 3, 0, 1).getTimeInMillis();
        datum2 = new GregorianCalendar(2009, 1, 4, 0, 2, 0).getTimeInMillis();
        assertEquals("", true, DatumFunktionen.vergleicheDieUhrzeit(datum1, datum2) > 0);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 09.09.09
