/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.inovat.buv.inovat.lib.gui.pruefung;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.gui.pruefung.PruefungDialogFelder;

import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;

//~ KLASSEN ===================================================================

/**
 * Die Klasse Testet die Methode pruefeDialogFelder() aus PruefungDialogFelder.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 12 $ / $Date: 2013-09-24 13:23:48 +0200 (Di, 24. Sep 2013) $ / ($Author: LG $)
 */
public class PruefungDialogFelderTest1 {
    FocusEvent   evt;
    NewComposite klasseInstanz;

    //~ SET METHODEN ==========================================================

    @Before
    public void setUp() throws Exception {
        Display display = Display.getDefault();
        Shell   shell   = new Shell(display);

        klasseInstanz = new NewComposite(shell, SWT.NULL);

        Event e = new Event();

        e.widget = klasseInstanz.getText1();
        evt      = new FocusEvent(e);
    }

    //~ METHODEN ==============================================================

    @Test
    public final void testPruefeDialogFelder1() {
        FocusListenerKlass focusListenerInstanz1 = new FocusListenerKlass("text1", klasseInstanz);

        // Die Methode wird erfolgreich aufgerufen, da die Methode pruefe_text1 existiert:
        focusListenerInstanz1.pruefeDialogFelder(evt);
        assertFalse("Der Feld ist richtig, also FocusLos", klasseInstanz.text1.isFocusControl());

        FocusListenerKlass focusListenerInstanz3 = new FocusListenerKlass("text3", klasseInstanz);

        // Die Methode wird erfolgreich aufgerufen, da die Methode pruefe_text3 existiert:
        focusListenerInstanz3.pruefeDialogFelder(evt);

        // TODO hier sollte asserTrue sein (ich kann nicht berprfen, ob ein Feld Focus hat?):
        assertFalse("Der Feld ist falsh, also Focus bleibt im Feld", klasseInstanz.text3.isFocusControl());
    }

    @Test  // (expected=Exception.class)
    public final void testPruefeDialogFelder2() {
        FocusListenerKlass focusListenerInstanz2 = new FocusListenerKlass("text2", klasseInstanz);

        // NoSuchMethodException, da die Methode pruefe_text2 nicht existiert
        // (diese exception wird aber im try-catch Block abgefangen):
        focusListenerInstanz2.pruefeDialogFelder(evt);
    }

    //~ INNERE KLASSEN ========================================================

    /**
     * Innere Klasse von Typ PruefungDialogFelder.
     */
    class FocusListenerKlass extends PruefungDialogFelder {
        protected FocusListenerKlass(String variableName, Control guiInstanz) {
            super(variableName, guiInstanz);
        }

        //~ METHODEN ==========================================================

        public PruefungErgebnis pruefeFeld_text1() {
            PruefungErgebnis ergebnis = new PruefungErgebnis();

            return ergebnis;
        }

        public PruefungErgebnis pruefeFeld_text3() {
            PruefungErgebnis ergebnis = new PruefungErgebnis();

            ergebnis.setIsEingabeRichtig(false);

            return ergebnis;
        }
    }


    // ***************************************************************************************************

    /**
     * Innere Klasse von Typ org.eclipse.swt.widgets.Composite.
     */
    class NewComposite extends org.eclipse.swt.widgets.Composite {
        private Text text1;
        private Text text2;
        private Text text3;

        //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==========

        public NewComposite(org.eclipse.swt.widgets.Composite parent, int style) {
            super(parent, style);
            initGUI();
            getText2();
        }

        //~ GET METHODEN ======================================================

        public Text getText1() {
            return text1;
        }

        private Text getText2() {
            return text2;
        }

        //~ METHODEN ==========================================================

        private void initGUI() {
            try {
                GridLayout thisLayout = new GridLayout();

                thisLayout.makeColumnsEqualWidth = true;
                this.setLayout(thisLayout);
                this.setSize(434, 82);

                {
                    text1 = new Text(this, SWT.NONE);

                    GridData text1LData = new GridData();

                    text1LData.grabExcessHorizontalSpace = true;
                    text1LData.horizontalAlignment       = GridData.FILL;
                    text1.setLayoutData(text1LData);
                    text1.setText("text1");
                }

                {
                    text2 = new Text(this, SWT.NONE);

                    GridData text2LData = new GridData();

                    text2LData.grabExcessHorizontalSpace = true;
                    text2LData.horizontalAlignment       = GridData.FILL;
                    text2.setLayoutData(text2LData);
                    text2.setText("text2");
                }

                {
                    text3 = new Text(this, SWT.NONE);

                    GridData text1LData = new GridData();

                    text1LData.grabExcessHorizontalSpace = true;
                    text1LData.horizontalAlignment       = GridData.FILL;
                    text3.setLayoutData(text1LData);
                    text3.setText("text1");
                }

                this.layout();
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 29.05.09
